@@ -88,10 +88,6 @@ id label;
 	[NSProcessInfo initializeWithArguments:argv count:argc environment:env];
 #endif
 
-#ifndef NX_CURRENT_COMPILER_RELEASE
-	initialize_gnustep_backend();
-#endif
-
 	theApp = [NSApplication sharedApplication];
 
 	window = [[NSWindow alloc] init];
