@@ -88,9 +88,34 @@
 
 #ifndef NO_GNUSTEP
 
-#include <base/Coding.h>
-#include <Foundation/NSCoder.h>
-@interface NSCoder (GNU) <Encoding, Decoding>
+@interface NSCoder (GNUstep)
+/* Compatibility with libObjects */
+- (void) decodeArrayOfObjCType: (const char*)type
+		         count: (unsigned)count
+			    at: (void*)buf
+		      withName: (id*)name;
+- (void) decodeIndent;
+- (void) decodeObjectAt: (id*)anObject
+	       withName: (id*)name;
+- (void) decodeValueOfCType: (const char*)type
+			 at: (void*)buf
+		   withName: (id*)name;
+- (void) decodeValueOfObjCType: (const char*)type
+			    at: (void*)buf
+		      withName: (id*)name;
+- (void) encodeArrayOfObjCType: (const char*)type
+		         count: (unsigned)count
+			    at: (const void*)buf
+		      withName: (id)name;
+- (void) encodeIndent;
+- (void) encodeObject: (id)anObject
+	     withName: (id)name;
+- (void) encodeValueOfCType: (const char*)type
+			 at: (const void*)buf
+		   withName: (id)name;
+- (void) encodeValueOfObjCType: (const char*)type
+			    at: (const void*)buf
+		      withName: (id)name;
 @end
 
 #endif /* NO_GNUSTEP */
