@@ -24,8 +24,8 @@
 #ifndef __NSDebug_h_GNUSTEP_BASE_INCLUDE
 #define __NSDebug_h_GNUSTEP_BASE_INCLUDE
 
-#include <base/preface.h>
 #include <errno.h>
+#include <Foundation/NSObject.h>
 
 extern int	errno;
 
@@ -59,17 +59,17 @@ extern int	errno;
  */
 
 #ifndef	NDEBUG
-extern	void		GSDebugAllocationAdd(Class c);
-extern	void		GSDebugAllocationRemove(Class c);
+GS_EXPORT void		GSDebugAllocationAdd(Class c);
+GS_EXPORT void		GSDebugAllocationRemove(Class c);
 
-extern	BOOL		GSDebugAllocationActive(BOOL active);
-extern	int		GSDebugAllocationCount(Class c);
-extern	const char*	GSDebugAllocationList(BOOL changeFlag);
-extern	const char*	GSDebugAllocationListAll();
+GS_EXPORT BOOL		GSDebugAllocationActive(BOOL active);
+GS_EXPORT int		GSDebugAllocationCount(Class c);
+GS_EXPORT const char*	GSDebugAllocationList(BOOL changeFlag);
+GS_EXPORT const char*	GSDebugAllocationListAll();
 
-extern	NSString*	GSDebugFunctionMsg(const char *func, const char *file,
+GS_EXPORT NSString*	GSDebugFunctionMsg(const char *func, const char *file,
 				int line, NSString *fmt);
-extern	NSString*	GSDebugMethodMsg(id obj, SEL sel, const char *file,
+GS_EXPORT NSString*	GSDebugMethodMsg(id obj, SEL sel, const char *file,
 				int line, NSString *fmt);
 #endif
 
@@ -115,30 +115,30 @@ extern	NSString*	GSDebugMethodMsg(id obj, SEL sel, const char *file,
 
 #define NSDebugLLog(level, format, args...) \
   do { if (GSDebugSet(level) == YES) \
-    NSLog(format, ## args); } while (0)
+    NSLog(format , ## args); } while (0)
 #define NSDebugLog(format, args...) \
   do { if (GSDebugSet(@"dflt") == YES) \
-    NSLog(format, ## args); } while (0)
+    NSLog(format , ## args); } while (0)
 #define NSDebugFLLog(level, format, args...) \
   do { if (GSDebugSet(level) == YES) { \
     NSString *fmt = GSDebugFunctionMsg( \
 	__PRETTY_FUNCTION__, __FILE__, __LINE__, format); \
-    NSLog(fmt, ## args); }} while (0)
+    NSLog(fmt , ## args); }} while (0)
 #define NSDebugFLog(format, args...) \
   do { if (GSDebugSet(@"dflt") == YES) { \
     NSString *fmt = GSDebugFunctionMsg( \
 	__PRETTY_FUNCTION__, __FILE__, __LINE__, format); \
-    NSLog(fmt, ## args); }} while (0)
+    NSLog(fmt , ## args); }} while (0)
 #define NSDebugMLLog(level, format, args...) \
   do { if (GSDebugSet(level) == YES) { \
     NSString *fmt = GSDebugMethodMsg( \
 	self, _cmd, __FILE__, __LINE__, format); \
-    NSLog(fmt, ## args); }} while (0)
+    NSLog(fmt , ## args); }} while (0)
 #define NSDebugMLog(format, args...) \
   do { if (GSDebugSet(@"dflt") == YES) { \
     NSString *fmt = GSDebugMethodMsg( \
 	self, _cmd, __FILE__, __LINE__, format); \
-    NSLog(fmt, ## args); }} while (0)
+    NSLog(fmt , ## args); }} while (0)
 #else
 #define NSDebugLLog(level, format, args...)
 #define NSDebugLog(format, args...)
@@ -178,17 +178,17 @@ extern	NSString*	GSDebugMethodMsg(id obj, SEL sel, const char *file,
 
 #define NSWarnLog(format, args...) \
   do { \
-    NSLog(format, ## args); } while (0)
+    NSLog(format , ## args); } while (0)
 #define NSWarnFLog(format, args...) \
   do { \
     NSString *fmt = GSDebugFunctionMsg( \
 	__PRETTY_FUNCTION__, __FILE__, __LINE__, format); \
-    NSLog(fmt, ## args); } while (0)
+    NSLog(fmt , ## args); } while (0)
 #define NSWarnMLog(format, args...) \
   do { \
     NSString *fmt = GSDebugMethodMsg( \
 	self, _cmd, __FILE__, __LINE__, format); \
-    NSLog(fmt, ## args); } while (0)
+    NSLog(fmt , ## args); } while (0)
 #else
 #define NSWarnLog(format, args...)
 #define NSWarnFLog(format, args...)
