@@ -24,9 +24,7 @@
 #ifndef __NSGArray_h_GNUSTEP_BASE_INCLUDE
 #define __NSGArray_h_GNUSTEP_BASE_INCLUDE
 
-#include <base/preface.h>
 #include <Foundation/NSArray.h>
-#include <base/Array.h>
 
 @interface NSGArray : NSArray
 {
