@@ -24,10 +24,7 @@
 #ifndef __NSGSet_h_GNUSTEP_BASE_INCLUDE
 #define __NSGSet_h_GNUSTEP_BASE_INCLUDE
 
-#include <base/preface.h>
 #include <Foundation/NSSet.h>
-#include <base/Set.h>
-#include <base/Bag.h>
 
 @interface NSGSet : NSSet
 @end
