@@ -35,6 +35,8 @@
 #undef	NO_GNUSTEP
 #endif
 
+#include <Foundation/NSObjCRuntime.h>
+#include <base/preface.h>
 #include <GSConfig.h>
 #include <objc/objc.h>
 #include <objc/Protocol.h>
@@ -70,8 +72,8 @@
 - autorelease;
 - (oneway void) release;
 - (unsigned) retainCount;
-- (NSZone *) zone;
-- (NSString *) description;
+- (NSZone*) zone;
+- (NSString*) description;
 @end
 
 @protocol NSCopying
@@ -93,6 +95,9 @@
   Class isa;
 }
 
+#if	GS_WITH_GC
++ (BOOL) requiresTypedMemory;
+#endif
 + (void) initialize;
 + (id) allocWithZone: (NSZone*)z;
 + (id) alloc;
@@ -130,7 +135,7 @@
 - (id) replacementObjectForPortCoder: (NSPortCoder*)anEncoder;
 
 
-+ setVersion: (int)aVersion;
++ (id) setVersion: (int)aVersion;
 + (int) version;
 
 @end
@@ -166,24 +171,40 @@ enum {NSNotFound = 0x7fffffff};
 extern NSRecursiveLock *gnustep_global_lock;
 
 /*
- * The GNUDescriptionDestination protocol declares a single method used
- * to append a property-list description string to some output destination
+ * The GNUDescriptionDestination protocol declares methods used to
+ * append a property-list description string to some output destination
  * so that property-lists can be converted to strings in a stream avoiding
  * the use of ridiculous amounts of memory for deeply nested data structures.
  */
 @protocol       GNUDescriptionDestination
+- (void) appendFormat: (NSString*)str, ...;
 - (void) appendString: (NSString*)str;
 @end
 
 @interface NSObject (GNU)
-- (int) compare: anObject;
-- (void) descriptionTo: (id<GNUDescriptionDestination>)output;
-- (void) descriptionWithLocale: (NSDictionary*)aLocale
-			    to: (id<GNUDescriptionDestination>)output;
+- (int) compare: (id)anObject;
+/*
+ * Default description methods -
+ * [descriptionWithLocale:] calls [description]
+ * [descriptionWithLocale:indent:] calls [descriptionWithLocale:]
+ * [descriptionWithLocale:indent:to:] calls [descriptionWithLocale:indent:]
+ * So - to have working descriptions, it is only necessary to implement the
+ * [description] method, and to have efficient property-list generation, it
+ * is necessary to override [descriptionWithLocale:indent:to:]
+ */
+- (NSString*) descriptionWithLocale: (NSDictionary*)aLocale;
++ (NSString*) descriptionWithLocale: (NSDictionary*)aLocale;
+- (NSString*) descriptionWithLocale: (NSDictionary*)aLocale
+			     indent: (unsigned)level;
++ (NSString*) descriptionWithLocale: (NSDictionary*)aLocale
+			     indent: (unsigned)level;
 - (void) descriptionWithLocale: (NSDictionary*)aLocale
 			indent: (unsigned)level
 			    to: (id<GNUDescriptionDestination>)output;
-- (Class)transmuteClassTo:(Class)aClassObject;
++ (void) descriptionWithLocale: (NSDictionary*)aLocale
+			indent: (unsigned)level
+			    to: (id<GNUDescriptionDestination>)output;
+- (Class) transmuteClassTo: (Class)aClassObject;
 - subclassResponsibility: (SEL)aSel;
 - shouldNotImplement: (SEL)aSel;
 + (Class) autoreleaseClass;
