@@ -33,7 +33,7 @@
   NSMapTable	*_portMap;	/* Registered ports information.	*/
   NSMapTable	*_nameMap;	/* Registered names information.	*/
 }
-+ (id) defaultPortNameServer;
++ (id) systemDefaultPortNameServer;
 - (NSPort*) portForName: (NSString*)name;
 - (NSPort*) portForName: (NSString*)name
 		 onHost: (NSString*)host;
@@ -44,7 +44,9 @@
 
 #ifndef	NO_GNUSTEP
 @interface	NSPortNameServer (GNUstep)
+- (NSArray*) namesForPort: (NSPort*)port;	/* return all names for port */
 - (void) removePort: (NSPort*)port;		/* remove all names for port */
+- (void) removePort: (NSPort*)port forName: (NSString*)name;
 @end
 #endif
 
