@@ -86,7 +86,7 @@ NSLocationInRange(unsigned location, NSRange range)
 }
 
 /* Create an NSRange having the specified LOCATION and LENGTH. */
-extern NSRange
+GS_EXPORT NSRange
 NSMakeRange(unsigned int location, unsigned int length);
 
 GS_RANGE_SCOPE BOOL
@@ -134,8 +134,8 @@ NSIntersectionRange (NSRange aRange, NSRange bRange)
 
 @class NSString;
 
-extern NSString *NSStringFromRange(NSRange range);
-extern NSRange NSRangeFromString(NSString *aString);
+GS_EXPORT NSString *NSStringFromRange(NSRange range);
+GS_EXPORT NSRange NSRangeFromString(NSString *aString);
 
 #ifdef	GS_DEFINED_MAX
 #undef	GS_DEFINED_MAX
@@ -157,6 +157,11 @@ extern NSRange NSRangeFromString(NSString *aString);
     [NSException raise: NSRangeException \
                 format: @"in %s, range { %u, %u } extends beyond size (%u)", \
 		  sel_get_name(_cmd), RANGE.location, RANGE.length, SIZE]
+#define CHECK_INDEX_RANGE_ERROR(INDEX, OVER) \
+if (INDEX >= OVER) \
+  [NSException raise: NSRangeException \
+               format: @"in %s, index %d is out of range", \
+               sel_get_name (_cmd), INDEX]
 #endif
 
 #endif /* __NSRange_h_GNUSTEP_BASE_INCLUDE */
