@@ -24,8 +24,7 @@
 #ifndef __NSString_h_GNUSTEP_BASE_INCLUDE
 #define __NSString_h_GNUSTEP_BASE_INCLUDE
 
-#include <base/preface.h>
-
+#include <Foundation/NSObject.h>
 #include <Foundation/NSRange.h>
 
 typedef unsigned short unichar;
@@ -238,10 +237,11 @@ enum {
 - (BOOL) isAbsolutePath;
 - (NSArray*) pathComponents;
 - (NSArray*) stringsByAppendingPaths: (NSArray*)paths;
-+ (NSString*) stringWithString: (NSString*) aString;
 + (NSString*) localizedStringWithFormat: (NSString*) format, ...;
-+ (NSString*) stringWithFormat: (NSString*)format
-		     arguments: (va_list)argList;
+
++ (id) stringWithFormat: (NSString*)format
+	      arguments: (va_list)argList;
++ (id) stringWithString: (NSString*) aString;
 - (id) initWithFormat: (NSString*)format
 	       locale: (NSDictionary*)dictionary;
 - (id) initWithFormat: (NSString*)format
