@@ -24,7 +24,6 @@
 #ifndef __NSTask_h_GNUSTEP_BASE_INCLUDE
 #define __NSTask_h_GNUSTEP_BASE_INCLUDE
 
-#include <base/preface.h>
 #include <Foundation/NSObject.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSArray.h>
@@ -88,6 +87,6 @@
 - (void) waitUntilExit;
 @end
 
-extern	NSString*	NSTaskDidTerminateNotification;
+GS_EXPORT NSString*	NSTaskDidTerminateNotification;
 
 #endif /* __NSTask_h_GNUSTEP_BASE_INCLUDE */
