@@ -23,7 +23,7 @@
 #ifndef __NSZone_h_GNUSTEP_BASE_INCLUDE
 #define __NSZone_h_GNUSTEP_BASE_INCLUDE
 
-#include <objc/objc.h>
+#include <Foundation/NSObjCRuntime.h>
 
 @class NSString;
 
@@ -58,9 +58,17 @@ struct _NSZone
 
 void *GSOutOfMemory(size_t size, BOOL retry);
 
+#ifdef	IN_NSZONE_M
+#define	GS_ZONE_SCOPE	extern
+#define GS_ZONE_ATTR	
+#else
+#define	GS_ZONE_SCOPE	static inline
+#define GS_ZONE_ATTR	__attribute__((unused))
+#endif
+
 /* Default zone.  Name is hopelessly long so that no one will ever
    want to use it. ;) Private variable. */
-extern NSZone* __nszone_private_hidden_default_zone;
+GS_EXPORT NSZone* __nszone_private_hidden_default_zone;
 
 #ifndef	GS_WITH_GC
 #define	GS_WITH_GC	0
@@ -69,21 +77,21 @@ extern NSZone* __nszone_private_hidden_default_zone;
 
 #include <gc.h>
 
-extern NSZone* __nszone_private_hidden_atomic_zone;
+GS_EXPORT NSZone* __nszone_private_hidden_atomic_zone;
 
-extern inline NSZone* NSCreateZone (size_t start, size_t gran, BOOL canFree)
+GS_ZONE_SCOPE NSZone* NSCreateZone (size_t start, size_t gran, BOOL canFree)
 { return __nszone_private_hidden_default_zone; }
 
-extern inline NSZone* NSDefaultMallocZone (void)
+GS_ZONE_SCOPE NSZone* NSDefaultMallocZone (void)
 { return __nszone_private_hidden_default_zone; }
 
-extern inline NSZone* GSAtomicMallocZone (void)
+GS_ZONE_SCOPE NSZone* GSAtomicMallocZone (void)
 { return __nszone_private_hidden_atomic_zone; }
 
-extern inline NSZone* NSZoneFromPointer (void *ptr)
+GS_ZONE_SCOPE NSZone* NSZoneFromPointer (void *ptr)
 { return __nszone_private_hidden_default_zone; }
 
-extern inline void* NSZoneMalloc (NSZone *zone, size_t size)
+GS_ZONE_SCOPE void* NSZoneMalloc (NSZone *zone, size_t size)
 {
   void	*ptr;
 
@@ -97,7 +105,7 @@ extern inline void* NSZoneMalloc (NSZone *zone, size_t size)
   return ptr;
 }
 
-extern inline void* NSZoneCalloc (NSZone *zone, size_t elems, size_t bytes)
+GS_ZONE_SCOPE void* NSZoneCalloc (NSZone *zone, size_t elems, size_t bytes)
 {
   size_t	size = elems * bytes;
   void		*ptr;
@@ -113,7 +121,7 @@ extern inline void* NSZoneCalloc (NSZone *zone, size_t elems, size_t bytes)
   return ptr;
 }
 
-extern inline void* NSZoneRealloc (NSZone *zone, void *ptr, size_t size)
+GS_ZONE_SCOPE void* NSZoneRealloc (NSZone *zone, void *ptr, size_t size)
 {
   ptr = GC_REALLOC(ptr, size);
   if (ptr == 0)
@@ -121,37 +129,37 @@ extern inline void* NSZoneRealloc (NSZone *zone, void *ptr, size_t size)
   return ptr;
 }
 
-extern inline void NSRecycleZone (NSZone *zone)
+GS_ZONE_SCOPE void NSRecycleZone (NSZone *zone)
 {
 }
 
-extern inline void NSZoneFree (NSZone *zone, void *ptr)
+GS_ZONE_SCOPE void NSZoneFree (NSZone *zone, void *ptr)
 {
   GC_FREE(ptr);
 }
 
-extern inline void NSSetZoneName (NSZone *zone, NSString *name)
+GS_ZONE_SCOPE void NSSetZoneName (NSZone *zone, NSString *name)
 {
 }
 
-extern inline NSString* NSZoneName (NSZone *zone)
+GS_ZONE_SCOPE NSString* NSZoneName (NSZone *zone)
 {
   return nil;
 }
 
 #ifndef	NO_GNUSTEP
 
-extern inline void* NSZoneMallocAtomic (NSZone *zone, size_t size)
+GS_ZONE_SCOPE void* NSZoneMallocAtomic (NSZone *zone, size_t size)
 {
   return NSZoneMalloc(GSAtomicMallocZone(), size);
 }
 
-extern inline BOOL NSZoneCheck (NSZone *zone)
+GS_ZONE_SCOPE BOOL NSZoneCheck (NSZone *zone)
 {
   return YES;
 }
 
-extern inline struct NSZoneStats NSZoneStats (NSZone *zone)
+GS_ZONE_SCOPE struct NSZoneStats NSZoneStats (NSZone *zone)
 {
   struct NSZoneStats stats = { 0 };
   return stats;
@@ -160,53 +168,68 @@ extern inline struct NSZoneStats NSZoneStats (NSZone *zone)
 
 #else	/* GS_WITH_GC */
 
-extern NSZone* NSCreateZone (size_t start, size_t gran, BOOL canFree);
+GS_EXPORT NSZone* NSCreateZone (size_t start, size_t gran, BOOL canFree);
 
-extern inline NSZone* NSDefaultMallocZone (void)
+GS_ZONE_SCOPE NSZone* NSDefaultMallocZone (void) GS_ZONE_ATTR;
+
+GS_ZONE_SCOPE NSZone* NSDefaultMallocZone (void)
 {
   return __nszone_private_hidden_default_zone;
 }
 
-extern inline NSZone* GSAtomicMallocZone (void)
+GS_ZONE_SCOPE NSZone* GSAtomicMallocZone (void) GS_ZONE_ATTR;
+
+GS_ZONE_SCOPE NSZone* GSAtomicMallocZone (void)
 {
   return NSDefaultMallocZone();
 }
 
-extern NSZone* NSZoneFromPointer (void *ptr);
+GS_EXPORT NSZone* NSZoneFromPointer (void *ptr);
 
-extern inline void* NSZoneMalloc (NSZone *zone, size_t size)
+GS_ZONE_SCOPE void* NSZoneMalloc (NSZone *zone, size_t size) GS_ZONE_ATTR;
+
+GS_ZONE_SCOPE void* NSZoneMalloc (NSZone *zone, size_t size)
 {
   if (!zone)
     zone = NSDefaultMallocZone();
   return (zone->malloc)(zone, size);
 }
 
-extern void* NSZoneCalloc (NSZone *zone, size_t elems, size_t bytes);
+GS_EXPORT void* NSZoneCalloc (NSZone *zone, size_t elems, size_t bytes);
 
-extern inline void* NSZoneRealloc (NSZone *zone, void *ptr, size_t size)
+GS_ZONE_SCOPE void* 
+NSZoneRealloc (NSZone *zone, void *ptr, size_t size) GS_ZONE_ATTR;
+
+GS_ZONE_SCOPE void* NSZoneRealloc (NSZone *zone, void *ptr, size_t size)
 {
   if (!zone)
     zone = NSDefaultMallocZone();
   return (zone->realloc)(zone, ptr, size);
 }
 
-extern inline void NSRecycleZone (NSZone *zone)
+GS_ZONE_SCOPE void NSRecycleZone (NSZone *zone) GS_ZONE_ATTR;
+
+GS_ZONE_SCOPE void NSRecycleZone (NSZone *zone)
 {
   if (!zone)
     zone = NSDefaultMallocZone();
   (zone->recycle)(zone);
 }
 
-extern inline void NSZoneFree (NSZone *zone, void *ptr)
+GS_ZONE_SCOPE void NSZoneFree (NSZone *zone, void *ptr) GS_ZONE_ATTR;
+
+GS_ZONE_SCOPE void NSZoneFree (NSZone *zone, void *ptr)
 {
   if (!zone)
     zone = NSDefaultMallocZone();
   (zone->free)(zone, ptr);
 }
 
-extern void NSSetZoneName (NSZone *zone, NSString *name);
+GS_EXPORT void NSSetZoneName (NSZone *zone, NSString *name);
 
-extern inline NSString* NSZoneName (NSZone *zone)
+GS_ZONE_SCOPE NSString* NSZoneName (NSZone *zone) GS_ZONE_ATTR;
+
+GS_ZONE_SCOPE NSString* NSZoneName (NSZone *zone)
 {
   if (!zone)
     zone = NSDefaultMallocZone();
@@ -214,21 +237,28 @@ extern inline NSString* NSZoneName (NSZone *zone)
 }
 
 #ifndef	NO_GNUSTEP
-extern inline void* NSZoneMallocAtomic (NSZone *zone, size_t size)
+GS_ZONE_SCOPE void* 
+NSZoneMallocAtomic (NSZone *zone, size_t size) GS_ZONE_ATTR;
+
+GS_ZONE_SCOPE void* NSZoneMallocAtomic (NSZone *zone, size_t size)
 {
   if (!zone)
     zone = NSDefaultMallocZone();
   return (zone->malloc)(zone, size);
 }
 
-extern inline BOOL NSZoneCheck (NSZone *zone)
+GS_ZONE_SCOPE BOOL NSZoneCheck (NSZone *zone) GS_ZONE_ATTR;
+
+GS_ZONE_SCOPE BOOL NSZoneCheck (NSZone *zone)
 {
   if (!zone)
     zone = NSDefaultMallocZone();
   return (zone->check)(zone);
 }
 
-extern inline struct NSZoneStats NSZoneStats (NSZone *zone)
+GS_ZONE_SCOPE struct NSZoneStats NSZoneStats (NSZone *zone) GS_ZONE_ATTR;
+
+GS_ZONE_SCOPE struct NSZoneStats NSZoneStats (NSZone *zone)
 {
   if (!zone)
     zone = NSDefaultMallocZone();
@@ -239,22 +269,22 @@ extern inline struct NSZoneStats NSZoneStats (NSZone *zone)
 #endif	/* GS_WITH_GC */
 
 
-extern unsigned NSPageSize (void) __attribute__ ((const));
+GS_EXPORT unsigned NSPageSize (void) __attribute__ ((const));
 
-extern unsigned NSLogPageSize (void) __attribute__ ((const));
+GS_EXPORT unsigned NSLogPageSize (void) __attribute__ ((const));
 
-extern unsigned NSRoundDownToMultipleOfPageSize (unsigned bytes)
+GS_EXPORT unsigned NSRoundDownToMultipleOfPageSize (unsigned bytes)
   __attribute__ ((const));
 
-extern unsigned NSRoundUpToMultipleOfPageSize (unsigned bytes)
+GS_EXPORT unsigned NSRoundUpToMultipleOfPageSize (unsigned bytes)
   __attribute__ ((const));
 
-extern unsigned NSRealMemoryAvailable (void);
+GS_EXPORT unsigned NSRealMemoryAvailable (void);
 
-extern void* NSAllocateMemoryPages (unsigned bytes);
+GS_EXPORT void* NSAllocateMemoryPages (unsigned bytes);
 
-extern void NSDeallocateMemoryPages (void *ptr, unsigned bytes);
+GS_EXPORT void NSDeallocateMemoryPages (void *ptr, unsigned bytes);
 
-extern void NSCopyMemoryPages (const void *src, void *dest, unsigned bytes);
+GS_EXPORT void NSCopyMemoryPages (const void *src, void *dest, unsigned bytes);
 
 #endif /* not __NSZone_h_GNUSTEP_BASE_INCLUDE */
