@@ -261,6 +261,11 @@ static SEL eValSel = @selector(encodeValueOfObjCType:at:);
 	      type++;
 	    }
 
+	  if (_initialPass == NO)
+	    {
+	      (*_tagImp)(_dst, tagSel, _GSC_ARY_B);
+	    }
+
 	  [self encodeArrayOfObjCType: type count: count at: buf];
 	}
 	return;
@@ -354,7 +359,8 @@ static SEL eValSel = @selector(encodeValueOfObjCType:at:);
 		/*
 		 *	Second pass, write a cross-reference number.
 		 */
-		(*_xRefImp)(_dst, xRefSel, _GSC_PTR|_GSC_XREF, node->value.uint);
+		(*_xRefImp)(_dst, xRefSel, _GSC_PTR|_GSC_XREF,
+		  node->value.uint);
 	      }
 	  }
 	return;
@@ -420,8 +426,8 @@ static SEL eValSel = @selector(encodeValueOfObjCType:at:);
 		 *	of its subclasses are decoded and call
 		 *	[super initWithCoder:ccc]
 		 */
-		if (s == c || s == 0 ||
-			GSIMapNodeForKey(_clsMap, (GSIMapKey)(void*)s) != 0)
+		if (s == c || s == 0
+		  || GSIMapNodeForKey(_clsMap, (GSIMapKey)(void*)s) != 0)
 		  {
 		    done = YES;
 		  }
@@ -453,7 +459,7 @@ static SEL eValSel = @selector(encodeValueOfObjCType:at:);
 	    if (node == 0)
 	      {
 		node = GSIMapAddPair(_ptrMap,
-			(GSIMapKey)(void*)s, (GSIMapVal)++_xRefP);
+		  (GSIMapKey)(void*)s, (GSIMapVal)++_xRefP);
 		(*_xRefImp)(_dst, xRefSel, _GSC_SEL, node->value.uint);
 		/*
 		 *	Encode selector.
@@ -462,7 +468,8 @@ static SEL eValSel = @selector(encodeValueOfObjCType:at:);
 	      }
 	    else
 	      {
-		(*_xRefImp)(_dst, xRefSel, _GSC_SEL|_GSC_XREF, node->value.uint);
+		(*_xRefImp)(_dst, xRefSel, _GSC_SEL|_GSC_XREF,
+		  node->value.uint);
 	      }
 	  }
 	return;
@@ -705,14 +712,6 @@ static SEL eValSel = @selector(encodeValueOfObjCType:at:);
 	  (*_tagImp)(_dst, tagSel, _GSC_ID | _GSC_XREF, _GSC_X_0);
 	}
     }
-  else if (fastIsInstance(anObject) == NO)
-    {
-      /*
-       *	If the object we have been given is actually a class,
-       *	we encode it as a class instead.
-       */
-      (*_eValImp)(self, eValSel, @encode(Class), &anObject);
-    }
   else
     {
       GSIMapNode	node;
@@ -762,24 +761,35 @@ static SEL eValSel = @selector(encodeValueOfObjCType:at:);
 	    }
 
 	  obj = [anObject replacementObjectForArchiver: self];
-	  cls = [anObject classForArchiver];
-
-	  (*_xRefImp)(_dst, xRefSel, _GSC_ID, node->value.uint);
-	  if (_namMap->nodeCount)
+	  if (fastIsInstance(obj) == NO)
 	    {
-	      GSIMapNode	node;
+	      /*
+	       * If the object we have been given is actually a class,
+	       * we encode it as a special case.
+	       */
+	      (*_xRefImp)(_dst, xRefSel, _GSC_CID, node->value.uint);
+	      (*_eValImp)(self, eValSel, @encode(Class), &obj);
+	    }
+	  else
+	    {
+	      cls = [obj classForArchiver];
+	      if (_namMap->nodeCount)
+		{
+		  GSIMapNode	n;
 
-	      node = GSIMapNodeForKey(_namMap, (GSIMapKey)cls);
+		  n = GSIMapNodeForKey(_namMap, (GSIMapKey)cls);
 
-	      if (node)
-		{
-		  cls = (Class)node->value.ptr;
+		  if (n)
+		    {
+		      cls = (Class)n->value.ptr;
+		    }
 		}
+	      (*_xRefImp)(_dst, xRefSel, _GSC_ID, node->value.uint);
+	      (*_eValImp)(self, eValSel, @encode(Class), &cls);
+	      [obj encodeWithCoder: self];
 	    }
-	  (*_eValImp)(self, eValSel, @encode(Class), &cls);
-	  [obj encodeWithCoder: self];
 	}
-      else if(!_initialPass)
+      else
 	{
 	  (*_xRefImp)(_dst, xRefSel, _GSC_ID | _GSC_XREF, node->value.uint);
 	}
@@ -854,10 +864,10 @@ static SEL eValSel = @selector(encodeValueOfObjCType:at:);
       [NSException raise: NSInternalInconsistencyException
 		  format: @"attempt to remap object to nil"];
     }
-  node = GSIMapNodeForKey(_namMap, (GSIMapKey)object);
+  node = GSIMapNodeForKey(_repMap, (GSIMapKey)object);
   if (node == 0)
     {
-      GSIMapAddPair(_namMap, (GSIMapKey)object, (GSIMapVal)newObject);
+      GSIMapAddPair(_repMap, (GSIMapKey)object, (GSIMapVal)newObject);
     }
   else
     {
@@ -952,42 +962,5 @@ static SEL eValSel = @selector(encodeValueOfObjCType:at:);
     }
 }
 
-/* libObjects compatibility */
- 
-- (void) encodeArrayOfObjCType: (const char*) type
-		         count: (unsigned)count
-			    at: (const void*)buf
-		      withName: (id)name
-{
-  (*_eObjImp)(self, eObjSel, name);
-  [self encodeArrayOfObjCType: type count: count at: buf];
-}
-
-- (void) encodeIndent
-{
-}
-
-- (void) encodeValueOfCType: (const char*) type
-			 at: (const void*)buf
-		   withName: (id)name
-{
-  (*_eObjImp)(self, eObjSel, name);
-  (*_eValImp)(self, eValSel, type, buf);
-}
-
-- (void) encodeValueOfObjCType: (const char*) type
-			    at: (const void*)buf
-		      withName: (id)name
-{
-  (*_eObjImp)(self, eObjSel, name);
-  (*_eValImp)(self, eValSel, type, buf);
-}
-
-- (void) encodeObject: (id)anObject
-	     withName: (id)name
-{
-  (*_eObjImp)(self, eObjSel, name);
-  (*_eObjImp)(self, eObjSel, anObject);
-}
 @end
 
