@@ -37,6 +37,7 @@
 #include <Foundation/NSUtilities.h>
 #include <Foundation/NSException.h>
 #include <Foundation/NSAutoreleasePool.h>
+#include <Foundation/NSUserDefaults.h>
 #include <Foundation/NSDebug.h>
 
 #include <base/fast.x>
@@ -79,31 +80,16 @@ static SEL	rlSel = @selector(removeLastObject);
     }
 }
 
-+ (void) _setConcreteClass: (Class)c
-{
-  NSArray_concrete_class = c;
-}
-
-+ (void) _setMutableConcreteClass: (Class)c
-{
-  NSMutableArray_concrete_class = c;
-}
-
-+ (Class) _concreteClass
-{
-  return NSArray_concrete_class;
-}
-
-+ (Class) _mutableConcreteClass
-{
-  return NSMutableArray_concrete_class;
-}
-
 + (id) allocWithZone: (NSZone*)z
 {
   if (self == NSArray_abstract_class)
-    return NSAllocateObject(NSArray_concrete_class, 0, z);
-  return [super allocWithZone: z];
+    {
+      return NSAllocateObject(NSArray_concrete_class, 0, z);
+    }
+  else
+    {
+      return NSAllocateObject(self, 0, z);
+    }
 }
 
 + (id) array
@@ -626,26 +612,35 @@ static SEL	rlSel = @selector(removeLastObject);
   GS_RANGE_CHECK(aRange, c);
 
   if (aRange.length == 0)
-    return [NSArray array];
+    {
+      na = [NSArray array];
+    }
+  else
+    {
+      id	objects[aRange.length];
 
-  {
-    id	objects[aRange.length];
-
-    [self getObjects: objects range: aRange];
-    na = [NSArray arrayWithObjects: objects count: aRange.length];
-  }
+      [self getObjects: objects range: aRange];
+      na = [NSArray arrayWithObjects: objects count: aRange.length];
+    }
   return na;
 }
 
 - (NSEnumerator*) objectEnumerator
 {
-  return AUTORELEASE([[NSArrayEnumerator allocWithZone: NSDefaultMallocZone()]
-    initWithArray: self]);
+  id	e;
+
+  e = [NSArrayEnumerator allocWithZone: NSDefaultMallocZone()];
+  e = [e initWithArray: self];
+  return AUTORELEASE(e);
 }
 
 - (NSEnumerator*) reverseObjectEnumerator
 {
-  return AUTORELEASE([[NSArrayEnumeratorReverse allocWithZone: NSDefaultMallocZone()] initWithArray: self]);
+  id	e;
+
+  e = [NSArrayEnumeratorReverse allocWithZone: NSDefaultMallocZone()];
+  e = [e initWithArray: self];
+  return AUTORELEASE(e);
 }
 
 - (NSString*) description
@@ -663,7 +658,8 @@ static SEL	rlSel = @selector(removeLastObject);
 {
   NSMutableString	*result;
 
-  result = AUTORELEASE([[NSGMutableCString alloc] initWithCapacity: 20*[self count]]);
+  result = [[NSGMutableCString alloc] initWithCapacity: 20*[self count]];
+  result = AUTORELEASE(result);
   [self descriptionWithLocale: locale
 		       indent: level
 			   to: (id<GNUDescriptionDestination>)result];
@@ -690,64 +686,85 @@ static NSString	*indentStrings[] = {
 		        indent: (unsigned int)level
 			    to: (id<GNUDescriptionDestination>)result
 {
-  NSString		*iBaseString;
-  NSString		*iSizeString;
   unsigned		count = [self count];
+  unsigned		last = count - 1;
   NSString		*plists[count];
   unsigned		i;
   IMP			appImp;
 
   appImp = [(NSObject*)result methodForSelector: appSel];
 
-  if (level < sizeof(indentStrings)/sizeof(NSString*))
-    iBaseString = indentStrings[level];
-  else
-    iBaseString = indentStrings[sizeof(indentStrings)/sizeof(NSString*)-1];
-  level++;
-  if (level < sizeof(indentStrings)/sizeof(NSString*))
-    iSizeString = indentStrings[level];
-  else
-    iSizeString = indentStrings[sizeof(indentStrings)/sizeof(NSString*)-1];
-
-  (*appImp)(result, appSel, @"(\n");
-
   [self getObjects: plists];
-  for (i = 0; i < count; i++)
-    {
-      id	item = plists[i];
 
-      (*appImp)(result, appSel, iSizeString);
- 
-      if ([item respondsToSelector: 
-	      @selector(descriptionWithLocale:indent:)])
+  if (locale == nil)
+    {
+      (*appImp)(result, appSel, @"(");
+      for (i = 0; i < count; i++)
 	{
-	  [item descriptionWithLocale: locale indent: level to: result];
-	}
-      else if ([item respondsToSelector: 
-	      @selector(descriptionWithLocale:)])
-	{
-	  [item descriptionWithLocale: locale to: result];
-	}
-      else
-	{
-	  [item descriptionTo: result];
-	}
-      if (i == count - 1)
-	{
-	  (*appImp)(result, appSel, @"\n");
-	}
-      else
-	{
-	  (*appImp)(result, appSel, @",\n");
+	  id	item = plists[i];
+
+	  [item descriptionWithLocale: nil indent: 0 to: result];
+	  if (i != last)
+	    {
+	      (*appImp)(result, appSel, @", ");
+	    }
 	}
+      (*appImp)(result, appSel, @")");
+    }
+  else
+    {
+      NSString	*iBaseString;
+      NSString	*iSizeString;
+
+      if (level < sizeof(indentStrings)/sizeof(id))
+	{
+	  iBaseString = indentStrings[level];
+	}
+      else
+	{
+	  iBaseString = indentStrings[sizeof(indentStrings)/sizeof(id)-1];
+	}
+      level++;
+      if (level < sizeof(indentStrings)/sizeof(id))
+	{
+	  iSizeString = indentStrings[level];
+	}
+      else
+	{
+	  iSizeString = indentStrings[sizeof(indentStrings)/sizeof(id)-1];
+	}
+
+      (*appImp)(result, appSel, @"(\n");
+      for (i = 0; i < count; i++)
+	{
+	  id	item = plists[i];
+
+	  (*appImp)(result, appSel, iSizeString);
+     
+	  [item descriptionWithLocale: locale indent: level to: result];
+	  if (i == last)
+	    {
+	      (*appImp)(result, appSel, @"\n");
+	    }
+	  else
+	    {
+	      (*appImp)(result, appSel, @",\n");
+	    }
+	}
+      (*appImp)(result, appSel, iBaseString);
+      (*appImp)(result, appSel, @")");
     }
-  (*appImp)(result, appSel, iBaseString);
-  (*appImp)(result, appSel, @")");
 }
 
 - (BOOL) writeToFile: (NSString *)path atomically: (BOOL)useAuxiliaryFile
 {
-  return [[self description] writeToFile: path atomically: useAuxiliaryFile];
+  NSDictionary	*loc;
+  NSString	*desc;
+
+  loc = [[NSUserDefaults standardUserDefaults] dictionaryRepresentation];
+  desc = [self descriptionWithLocale: loc indent: 0];
+
+  return [desc writeToFile: path atomically: useAuxiliaryFile];
 }
 
 @end
@@ -767,8 +784,13 @@ static NSString	*indentStrings[] = {
 + (id) allocWithZone: (NSZone*)z
 {
   if (self == NSMutableArray_abstract_class)
-    return NSAllocateObject(NSMutableArray_concrete_class, 0, z);
-  return [super allocWithZone: z];
+    {
+      return NSAllocateObject(NSMutableArray_concrete_class, 0, z);
+    }
+  else
+    {
+      return NSAllocateObject(self, 0, z);
+    }
 }
 
 /* The NSCopying Protocol */
@@ -1315,3 +1337,130 @@ static NSString	*indentStrings[] = {
   return (*get)(array, oaiSel, --pos);
 }
 @end
+
+
+@implementation	NSArray (GNUstep)
+
+/*
+ *	The comparator function takes two items as arguments, the first is the
+ *	item to be added, the second is the item already in the array.
+ *      The function should return NSOrderedAscending if the item to be
+ *      added is 'less than' the item in the array, NSOrderedDescending
+ *      if it is greater, and NSOrderedSame if it is equal.
+ */
+- (unsigned) insertionPosition: (id)item
+		 usingFunction: (NSComparisonResult (*)(id, id))sorter
+{
+  unsigned	count = [self count];
+  unsigned	upper = count;
+  unsigned	lower = 0;
+  unsigned	index;
+  IMP		oai;
+
+  if (item == nil)
+    {
+      [NSException raise: NSGenericException
+		  format: @"Attempt to find position for nil object in array"];
+    }
+  if (sorter == 0)
+    {
+      [NSException raise: NSGenericException
+		  format: @"Attempt to find position with null comparator"];
+    }
+
+  oai = [self methodForSelector: oaiSel];
+  /*
+   *	Binary search for an item equal to the one to be inserted.
+   */
+  for (index = upper/2; upper != lower; index = lower+(upper-lower)/2)
+    {
+      NSComparisonResult comparison;
+
+      comparison = (*sorter)(item, (*oai)(self, oaiSel, index));
+      if (comparison == NSOrderedAscending)
+        {
+          upper = index;
+        }
+      else if (comparison == NSOrderedDescending)
+        {
+          lower = index + 1;
+        }
+      else
+        {
+          break;
+        }
+    }
+  /*
+   *	Now skip past any equal items so the insertion point is AFTER any
+   *	items that are equal to the new one.
+   */
+  while (index < count
+    && (*sorter)(item, (*oai)(self, oaiSel, index)) != NSOrderedAscending)
+    {
+      index++;
+    }
+  return index;
+}
+
+- (unsigned) insertionPosition: (id)item
+		 usingSelector: (SEL)comp
+{
+  unsigned	count = [self count];
+  unsigned	upper = count;
+  unsigned	lower = 0;
+  unsigned	index;
+  NSComparisonResult	(*imp)(id, SEL, id);
+  IMP		oai;
+
+  if (item == nil)
+    {
+      [NSException raise: NSGenericException
+		  format: @"Attempt to find position for nil object in array"];
+    }
+  if (comp == 0)
+    {
+      [NSException raise: NSGenericException
+		  format: @"Attempt to find position with null comparator"];
+    }
+  imp = (NSComparisonResult (*)(id, SEL, id))[item methodForSelector: comp];
+  if (imp == 0)
+    {
+      [NSException raise: NSGenericException
+		  format: @"Attempt to find position with unknown method"];
+    }
+
+  oai = [self methodForSelector: oaiSel];
+  /*
+   *	Binary search for an item equal to the one to be inserted.
+   */
+  for (index = upper/2; upper != lower; index = lower+(upper-lower)/2)
+    {
+      NSComparisonResult comparison;
+
+      comparison = (*imp)(item, comp, (*oai)(self, oaiSel, index));
+      if (comparison == NSOrderedAscending)
+        {
+          upper = index;
+        }
+      else if (comparison == NSOrderedDescending)
+        {
+          lower = index + 1;
+        }
+      else
+        {
+          break;
+        }
+    }
+  /*
+   *	Now skip past any equal items so the insertion point is AFTER any
+   *	items that are equal to the new one.
+   */
+  while (index < count
+    && (*imp)(item, comp, (*oai)(self, oaiSel, index)) != NSOrderedAscending)
+    {
+      index++;
+    }
+  return index;
+}
+@end
+
