@@ -39,31 +39,7 @@
 #include <Foundation/NSMapTable.h>
 #include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSFileManager.h>
-
-#include <sys/stat.h>
-
-#ifndef __WIN32__
 #include <unistd.h>
-#include <sys/param.h>		/* Needed by sys/stat */
-#endif
-
-/* For DIR and diropen() */
-#if HAVE_DIRENT_H
-# include <dirent.h>
-# define NAMLEN(dirent) strlen((dirent)->d_name)
-#else
-# define dirent direct
-# define NAMLEN(dirent) (dirent)->d_namlen
-# if HAVE_SYS_NDIR_H
-#  include <sys/ndir.h>
-# endif
-# if HAVE_SYS_DIR_H
-#  include <sys/dir.h>
-# endif
-# if HAVE_NDIR_H
-#  include <ndir.h>
-# endif
-#endif
 
 typedef enum {
   NSBUNDLE_BUNDLE = 1, NSBUNDLE_APPLICATION, NSBUNDLE_LIBRARY
@@ -73,6 +49,9 @@ typedef enum {
 static NSBundle*   _mainBundle = nil;
 static NSMapTable* _bundles = NULL;
 
+/* Keep the path to the executable file for finding the main bundle. */
+static NSString *_executable_path;
+
 /*
  * An empty strings file table for use when localization files can't be found.
  */
@@ -117,9 +96,6 @@ static NSString* library_combo =
   nil;
 #endif
 
-/* Declaration from find_exec.c */
-extern char *objc_find_executable(const char *name);
-
 /* This function is provided for objc-load.c, although I'm not sure it
    really needs it (So far only needed if using GNU dld library) */
 const char *
@@ -128,6 +104,26 @@ objc_executable_location( void )
   return [[[NSBundle mainBundle] bundlePath] cString];
 }
 
+static BOOL
+bundle_directory_readable(NSString *path)
+{
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  BOOL directory;
+
+  if (![mgr fileExistsAtPath: path isDirectory: &directory]
+      || !directory)
+    return NO;
+
+  return [mgr isReadableFileAtPath: path];
+}
+
+static BOOL
+bundle_file_readable(NSString *path)
+{
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  return [mgr isReadableFileAtPath: path];
+}
+
 /* Get the object file that should be located in the bundle of the same name */
 static NSString *
 bundle_object_name(NSString *path, NSString* executable)
@@ -183,35 +179,24 @@ _bundle_resource_path(NSString *primary, NSString* bundlePath, NSString *lang)
 
 /* Find the first directory entry with a given name (with any extension) */
 static NSString *
-_bundle_path_for_name(NSString* path, NSString* name)
+_bundle_name_first_match(NSString* directory, NSString* name)
 {
-#ifdef __WIN32__
-  return nil;
-#else
-  DIR *thedir;
-  struct dirent *entry;
-  NSString *fullname;
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  NSEnumerator *filelist;
+  NSString *path, *match, *cleanname;
 
-  fullname = NULL;
-  thedir = opendir([path cString]);
-  if(thedir) 
+  /* name might have a directory in it also, so account for this */
+  path = [[directory stringByAppendingPathComponent: name] 
+	  stringByDeletingLastPathComponent];
+  cleanname = [name lastPathComponent];
+  filelist = [[mgr directoryContentsAtPath: path] objectEnumerator];
+  while ((match = [filelist nextObject]))
     {
-      while ((entry = readdir(thedir))) 
-	{
-	  if (*(entry->d_name) != '.'
-	      && strncmp([name cString], entry->d_name, [name length]) == 0)
-	    {
-	      fullname = [NSString stringWithCString: entry->d_name];
-	      break;
-	    }
-	}
-      closedir(thedir);
+      if ([cleanname isEqual: [match stringByDeletingPathExtension]])
+	return [path stringByAppendingPathComponent: match];
     }
-  if (!fullname)
-    return nil;
 
-  return [path stringByAppendingPathComponent: fullname];
-#endif
+  return nil;
 }
 
 @interface NSBundle (Private)
@@ -275,6 +260,21 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 		    mutableCopy]);
 	  [system appendString: @"/Libraries"];
 
+	  _executable_path = nil;
+#ifdef HAVE_PROC_FS_EXE_LINK
+	  _executable_path = [[NSFileManager defaultManager]
+	    pathContentOfSymbolicLinkAtPath: @"/proc/self/exe"];
+#endif
+	  if (_executable_path == nil || [_executable_path length] == 0)
+	    {
+	      _executable_path =
+		[[[NSProcessInfo processInfo] arguments] objectAtIndex: 0];
+	      _executable_path = 
+		[NSBundle _absolutePathOfExecutable: _executable_path];
+	      NSAssert(_executable_path, NSInternalInconsistencyException);
+	    }
+
+	  RETAIN(_executable_path);
 	  _gnustep_bundle = RETAIN([NSBundle bundleWithPath: system]);
 	}
     }
@@ -287,6 +287,7 @@ _bundle_load_callback(Class theClass, Category *theCategory)
   void			*key;
   NSBundle		*bundle;
 
+  [load_lock lock];
   enumerate = NSEnumerateMapTable(_bundles);
   while (NSNextMapEnumeratorPair(&enumerate, &key, (void **)&bundle))
     {
@@ -296,6 +297,7 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 	  [array addObject: bundle];
 	}
     }
+  [load_lock unlock];
   return array;
 }
 
@@ -309,17 +311,10 @@ _bundle_load_callback(Class theClass, Category *theCategory)
   [load_lock lock];
   if ( !_mainBundle ) 
     {
-      char *output;
       NSString *path, *s;
       
-      path = [[[NSProcessInfo processInfo] arguments] objectAtIndex: 0];
-      output = objc_find_executable([path cString]);
-      NSAssert(output, NSInternalInconsistencyException);
-      path = [NSString stringWithCString: output];
-      OBJC_FREE(output);
-
       /* Strip off the name of the program */
-      path = [path stringByDeletingLastPathComponent];
+      path = [_executable_path stringByDeletingLastPathComponent];
 
       /* The executable may not lie in the main bundle directory
 	 so we need to chop off the extra subdirectories, the library
@@ -366,6 +361,7 @@ _bundle_load_callback(Class theClass, Category *theCategory)
   if (!aClass)
     return nil;
 
+  [load_lock lock];
   bundle = nil;
   enumerate = NSEnumerateMapTable(_bundles);
   while (NSNextMapEnumeratorPair(&enumerate, &key, (void **)&bundle))
@@ -376,6 +372,7 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 	break;
       bundle = nil;
     }
+  [load_lock unlock];
   if (!bundle) 
     {
       /* Is it in the main bundle? */
@@ -393,7 +390,6 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 
 - initWithPath:(NSString *)path;
 {
-  struct stat statbuf;
   [super init];
 
   if (!path || [path length] == 0) 
@@ -401,15 +397,24 @@ _bundle_load_callback(Class theClass, Category *theCategory)
       NSLog(@"No path specified for bundle");
       return nil;
     }
+  if ([path isAbsolutePath] == NO)
+    {
+      NSLog(@"WARNING: NSBundle -initWithPath: requires absolute path names!");
+      path = [[[NSFileManager defaultManager] currentDirectoryPath]
+        stringByAppendingPathComponent: path];
+    }
 
   /* Check if we were already initialized for this directory */
+  [load_lock lock];
   if (_bundles) 
     {
       NSBundle* bundle = (NSBundle *)NSMapGet(_bundles, path);
       if (bundle)
 	{
+	  RETAIN(bundle); /* retain - look as if we were alloc'ed */
+	  [load_lock unlock];
 	  [self dealloc];
-	  return RETAIN(bundle); /* retain - look as if we were alloc'ed */
+	  return bundle;
 	}
     }
   if (_releasedBundles)
@@ -419,18 +424,26 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 	{
 	  NSMapInsert(_bundles, path, loaded);
 	  NSMapRemove(_releasedBundles, path);
+	  RETAIN(loaded); /* retain - look as if we were alloc'ed */
+	  [load_lock unlock];
 	  [self dealloc];
-	  return RETAIN(loaded); /* retain - look as if we were alloc'ed */
+	  return loaded;
 	}
     }
+  [load_lock unlock];
 
-  if (stat([path cString], &statbuf) != 0) 
+  if (bundle_directory_readable(path) == NO)
     {
-      NSDebugMLLog(@"NSBundle", @"Could not access path %s for bundle", [path cString]);
+      NSDebugMLLog(@"NSBundle", @"Could not access path %@ for bundle", path);
       //[self dealloc];
       //return nil;
     }
 
+  _path = [path copy];
+  _bundleType = (unsigned int)NSBUNDLE_BUNDLE;
+  if (self == _mainBundle)
+    _bundleType = (unsigned int)NSBUNDLE_APPLICATION;
+
   [load_lock lock];
   if (!_bundles)
     {
@@ -439,14 +452,9 @@ _bundle_load_callback(Class theClass, Category *theCategory)
       _releasedBundles = NSCreateMapTable(NSObjectMapKeyCallBacks,
 				  NSNonOwnedPointerMapValueCallBacks, 0);
     }
+  NSMapInsert(_bundles, _path, self);
   [load_lock unlock];
 
-  _path = [path copy];
-  _bundleType = (unsigned int)NSBUNDLE_BUNDLE;
-  if (self == _mainBundle)
-    _bundleType = (unsigned int)NSBUNDLE_APPLICATION;
-
-  NSMapInsert(_bundles, _path, self);
   return self;
 }
 
@@ -460,15 +468,18 @@ _bundle_load_callback(Class theClass, Category *theCategory)
     {
       if ([self retainCount] == 1)
 	{
+	  [load_lock lock];
 	  if (self == NSMapGet(_releasedBundles, _path))
 	    {
+	      [load_lock unlock];
 	      [NSException raise: NSGenericException
 		format: @"Bundle for path %@ released too many times", _path];
 	    }
       
-	    NSMapRemove(_bundles, _path);
-	    NSMapInsert(_releasedBundles, _path, self);
-	    return;
+	  NSMapRemove(_bundles, _path);
+	  NSMapInsert(_releasedBundles, _path, self);
+	  [load_lock unlock];
+	  return;
 	}
     }
   [super release];
@@ -476,9 +487,11 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 
 - (void) dealloc
 {
-  if (_path)
+  if (_path != nil)
     {
+      [load_lock lock];
       NSMapRemove(_bundles, _path);
+      [load_lock unlock];
       RELEASE(_path);
     }
   TEST_RELEASE(_bundleClasses);
@@ -505,7 +518,7 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 	}
     }
 
-  if (self == _mainBundle) 
+  if (self == _mainBundle || self == _gnustep_bundle)
     {
       theClass = NSClassFromString(className);
       if (theClass && [[self class] bundleForClass:theClass] != _mainBundle)
@@ -530,7 +543,7 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 
   class_name = [[self infoDictionary] objectForKey: @"NSPrincipalClass"];
 
-  if (self == _mainBundle) 
+  if (self == _mainBundle || self == _gnustep_bundle) 
     {
       _codeLoaded = YES;
       if (class_name)
@@ -550,11 +563,22 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 
 - (BOOL) load
 {
+  if (self == _mainBundle || self == _gnustep_bundle) 
+    {
+      _codeLoaded = YES;
+      return YES;
+    }
+
   [load_lock lock];
   if (!_codeLoaded) 
     {
       NSString* object;
       object = [[self infoDictionary] objectForKey: @"NSExecutable"];
+      if (object == nil || [object length] == 0)
+	{
+	  [load_lock unlock];
+	  return NO;
+	}
       object = bundle_object_name(_path, object);
       _loadingBundle = self;
       _bundleClasses = RETAIN([NSMutableArray arrayWithCapacity: 2]);
@@ -604,7 +628,7 @@ _bundle_load_callback(Class theClass, Category *theCategory)
   enumerate = [languages objectEnumerator];
   while ((language = [enumerate nextObject]))
     [array addObject: _bundle_resource_path(primary, bundlePath, language)];
-    
+     
   primary = rootPath;
   [array addObject: _bundle_resource_path(primary, bundlePath, nil)];
   enumerate = [languages objectEnumerator];
@@ -620,9 +644,8 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 		inDirectory: (NSString *)bundlePath
 		withVersion: (int)version
 {
-  NSString *path;
-  NSArray* paths;
-  NSEnumerator* enumerate;
+  NSString *path, *fullpath;
+  NSEnumerator* pathlist;
     
   if (!name || [name length] == 0) 
     {
@@ -631,19 +654,19 @@ _bundle_load_callback(Class theClass, Category *theCategory)
       /* NOT REACHED */
     }
 
-  paths = [NSBundle _bundleResourcePathsWithRootPath: rootPath
-	     subPath: bundlePath];
-  enumerate = [paths objectEnumerator];
-  while((path = [enumerate nextObject]))
+  pathlist = [[NSBundle _bundleResourcePathsWithRootPath: rootPath
+	       subPath: bundlePath] objectEnumerator];
+  fullpath = nil;
+  while((path = [pathlist nextObject]))
     {
-      NSString* fullpath = nil;
+      if (!bundle_directory_readable(path))
+	continue;
 
       if (ext && [ext length] != 0)
 	{
-	  struct stat statbuf;
 	  fullpath = [path stringByAppendingPathComponent:
 		        [NSString stringWithFormat: @"%@.%@", name, ext]];
-	  if ( stat([fullpath cString], &statbuf) == 0) 
+	  if ( bundle_file_readable(fullpath) )
 	    {
 	      if (gnustep_target_os)
 		{
@@ -651,7 +674,7 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 		  platpath = [path stringByAppendingPathComponent:
 			      [NSString stringWithFormat: @"%@-%@.%@", 
 			       name, gnustep_target_os, ext]];
-		  if ( stat([platpath cString], &statbuf) == 0) 
+		  if ( bundle_file_readable(platpath) )
 		    fullpath = platpath;
 		}
 	    }
@@ -660,40 +683,22 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 	}
       else
 	{
-	  struct stat statbuf;
-	  fullpath = [path stringByAppendingPathComponent:
-		        [NSString stringWithFormat: @"%@", name]];
-	  if ( stat([fullpath cString], &statbuf) == 0) 
+	  fullpath = _bundle_name_first_match(path, name);
+	  if (fullpath && gnustep_target_os)
 	    {
-	      if (gnustep_target_os)
-		{
-		  NSString* platpath;
-		  platpath = [path stringByAppendingPathComponent:
-			      [NSString stringWithFormat: @"%@-%@", 
-			       name, gnustep_target_os]];
-		  if ( stat([platpath cString], &statbuf) == 0) 
-		    fullpath = platpath;
-		}
-	    }
-	  else
-	    {
-	      fullpath = _bundle_path_for_name(path, name);
-	      if (fullpath && gnustep_target_os)
-		{
-		  NSString* platpath;
-		  platpath = _bundle_path_for_name(path, 
-				 [NSString stringWithFormat: @"%@-%@", 
-					   name, gnustep_target_os]);
-		  if (platpath)
-		    fullpath = platpath;
-		}
+	      NSString* platpath;
+	      platpath = _bundle_name_first_match(path, 
+				   [NSString stringWithFormat: @"%@-%@", 
+				   name, gnustep_target_os]);
+	      if (platpath)
+		fullpath = platpath;
 	    }
 	}
       if (fullpath)
-	return fullpath;
+	break;
     }
 
-  return nil;
+  return fullpath;
 }
 
 + (NSString *) pathForResource: (NSString *)name
@@ -741,42 +746,30 @@ _bundle_load_callback(Class theClass, Category *theCategory)
 - (NSArray *) pathsForResourcesOfType: (NSString *)extension
 			  inDirectory: (NSString *)bundlePath
 {
+  BOOL allfiles;
   NSString *path;
-  NSArray* paths;
-  NSMutableArray* resources;
-  NSEnumerator* enumerate;
+  NSMutableArray *resources;
+  NSEnumerator *pathlist;
+  NSFileManager	*mgr = [NSFileManager defaultManager];
     
-  paths = [NSBundle _bundleResourcePathsWithRootPath: [self bundlePath]
-	    subPath: bundlePath];
-  enumerate = [paths objectEnumerator];
+  pathlist = [[NSBundle _bundleResourcePathsWithRootPath: [self bundlePath]
+	        subPath: bundlePath] objectEnumerator];
   resources = [NSMutableArray arrayWithCapacity: 2];
-#ifdef __WIN32__
-#else
-  while((path = [enumerate nextObject]))
-    {
-      DIR *thedir;
-      struct dirent *entry;
+  allfiles = (extension == nil || [extension length] == 0);
 
-      thedir = opendir([path cString]);
-      if (thedir) 
+  while((path = [pathlist nextObject]))
+    {
+      NSEnumerator *filelist;
+      NSString *match;
+
+      filelist = [[mgr directoryContentsAtPath: path] objectEnumerator];
+      while ((match = [filelist nextObject]))
 	{
-	  while ((entry = readdir(thedir))) 
-	    {
-	      if (*entry->d_name != '.') 
-		{
-		  char* ext;
-		  ext = strrchr(entry->d_name, '.');
-		  if (!extension || [extension length] == 0
-		      || (ext && strcmp(++ext, [extension cString]) == 0))
-		    [resources addObject: 
-		      [path stringByAppendingPathComponent:
-		        [NSString stringWithCString: entry->d_name]]];
-		}
-	    }
-	  closedir(thedir);
+	  if (allfiles || [extension isEqual: [match pathExtension]])
+	    [resources addObject: [path stringByAppendingPathComponent: match]];
 	}
     }
-#endif
+
   return resources;
 }
 
@@ -918,6 +911,59 @@ _bundle_load_callback(Class theClass, Category *theCategory)
    within the GNUstep directory structure specified by the environment
    variables. */
 
++ (NSString *) _absolutePathOfExecutable: (NSString *)path
+{
+  NSFileManager *mgr;
+  NSDictionary   *env;
+  NSString *pathlist, *prefix;
+  id patharr;
+
+  path = [path stringByStandardizingPath];
+  if ([path isAbsolutePath])
+    return path;
+
+  mgr = [NSFileManager defaultManager];
+  env = [[NSProcessInfo processInfo] environment];
+  pathlist = [env objectForKey:@"PATH"];
+#if defined(__MINGW__)
+  patharr = [pathlist componentsSeparatedByString:@";"];
+#else
+  patharr = [pathlist componentsSeparatedByString:@":"];
+#endif
+  /* Add . if not already in path */
+  if ([patharr indexOfObject: @"."] == NSNotFound)
+    {
+      patharr = AUTORELEASE([patharr mutableCopy]);
+      [patharr addObject: @"."];
+    }
+  patharr = [patharr objectEnumerator];
+  while ((prefix = [patharr nextObject]))
+    {
+      if ([prefix isEqual:@"."])
+	prefix = [mgr currentDirectoryPath];
+      prefix = [prefix stringByAppendingPathComponent: path];
+      if ([mgr isExecutableFileAtPath: prefix])
+	return [prefix stringByStandardizingPath];
+#if defined(__WIN32__)
+      /* Also add common executable extensions on windows */
+      if ([path pathExtension] == nil)
+	{
+	  NSString *wpath;
+	  wpath = [prefix stringByAppendingPathExtension: @"exe"];
+	  if ([mgr isExecutableFileAtPath: wpath])
+	    return [wpath stringByStandardizingPath];
+	  wpath = [prefix stringByAppendingPathExtension: @"com"];
+	  if ([mgr isExecutableFileAtPath: wpath])
+	    return [wpath stringByStandardizingPath];
+	  wpath = [prefix stringByAppendingPathExtension: @"cmd"];
+	  if ([mgr isExecutableFileAtPath: wpath])
+	    return [wpath stringByStandardizingPath];
+	}
+#endif
+    }
+  return nil;
+}
+
 + (NSBundle *) gnustepBundle
 {
   return _gnustep_bundle;
