@@ -24,6 +24,7 @@
 #include <config.h>
 #include <math.h>
 #include <objc/objc-api.h>
+#include <base/preface.h>
 #include <Foundation/NSDate.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSString.h>
@@ -31,15 +32,8 @@
 #include <Foundation/NSException.h>
 #include <Foundation/NSUserDefaults.h>
 #include <base/behavior.h>
-
-#ifndef __WIN32__
-#include <time.h>
-#endif /* !__WIN32__ */
 #include <stdio.h>
 #include <stdlib.h>
-#ifndef __WIN32__
-#include <sys/time.h>
-#endif /* !__WIN32__ */
 
 // Absolute Gregorian date for NSDate reference date Jan 01 2001
 //
@@ -53,6 +47,60 @@
 
 #define GREGORIAN_REFERENCE 730486
 
+static inline int
+lastDayOfGregorianMonth(int month, int year)
+{
+  switch (month)
+    {
+      case 2:
+	if ((((year % 4) == 0) && ((year % 100) != 0))
+	  || ((year % 400) == 0))
+	  return 29;
+	else
+	  return 28;
+      case 4:
+      case 6:
+      case 9:
+      case 11: return 30;
+      default: return 31;
+    }
+}
+
+static inline int
+absoluteGregorianDay(int day, int month, int year)
+{
+  int m, N;
+
+  N = day;   // day of month
+  for (m = month - 1;  m > 0; m--) // days in prior months this year
+    N = N + lastDayOfGregorianMonth(m, year);
+  return
+    (N                    // days this year
+     + 365 * (year - 1)   // days in previous years ignoring leap days
+     + (year - 1)/4       // Julian leap days before this year...
+     - (year - 1)/100     // ...minus prior century years...
+     + (year - 1)/400);   // ...plus prior years divisible by 400
+}
+
+/*
+ * External - so NSDate can use it.
+ */
+NSTimeInterval
+GSTime(int day, int month, int year, int h, int m, int s)
+{
+  NSTimeInterval	a;
+
+  a = (NSTimeInterval)absoluteGregorianDay(day, month, year);
+
+  // Calculate date as GMT
+  a -= GREGORIAN_REFERENCE;
+  a = (NSTimeInterval)a * 86400;
+  a += h * 3600;
+  a += m * 60;
+  a += s;
+  return a;
+}
+
 @interface NSCalendarDate (Private)
 
 - (void)getYear: (int *)year month: (int *)month day: (int *)day
@@ -186,416 +234,443 @@
        calendarFormat: (NSString *)fmt
                locale: (NSDictionary *)locale
 {
-  int		year = 0, month = 1, day = 1, hour = 0, min = 0, sec = 0;
-  NSTimeZone	*tz = [NSTimeZone localTimeZone];
-  BOOL		ampm = NO;
-  BOOL		twelveHrClock = NO; 
-  int		julianWeeks = -1, weekStartsMonday = 0, dayOfWeek = -1;
-  const char	*source = [description cString];
-  int		sourceLen = strlen(source);
-  const char	*format = [fmt cString];
-  int		formatLen = strlen(format);
-  int		formatIdx = 0;
-  int		sourceIdx = 0;
-  char		tmpStr[20];
-  int		tmpIdx;
-
-  if (locale == nil)
+  // If description does not match this format exactly, this method returns nil 
+  if ([description length] == 0)
     {
-      locale = [[NSUserDefaults standardUserDefaults] dictionaryRepresentation];
+      // Autorelease self because it isn't done by the calling function
+      // [[NSCalendarDate alloc] initWithString:calendarFormat:locale:];
+      AUTORELEASE(self);
+      return nil;
     }
-  if (fmt == nil)
+  else
     {
-      format = [[locale objectForKey: NSTimeDateFormatString] cString];
-      formatLen = strlen(format);
-    }
-
-  //
-  // WARNING:
-  //   -%c, %F, %x, %X do NOT work. (NSTimeDateFormatString isn't defined and
-  //    the underlying call has granularity to the second.
-  //   -Most locale stuff is dubious at best.
-  //   -Long day and month names depend on a non-alpha character after the last
-  //    digit to work.
-  //
-  // The strftime specifiers as used by OpenStep + %U.
-  //
-  // %%   literal % character
-  // %a   abbreviated weekday name according to locale
-  // %A   full weekday name according to locale
-  // %b   abbreviated month name according to locale
-  // %B   full month name according to locale
-  // %c   same as '%X %x'
-  // %d   day of month as decimal number
-  // %e   same as %d without leading zero (you get a leading space instead)
-  // %F   milliseconds as a decimal number
-  // %H   hour as a decimal number using 24-hour clock
-  // %I   hour as a decimal number using 12-hour clock
-  // %j   day of year as a decimal number
-  // %m   month as decimal number
-  // %M   minute as decimal number
-  // %p   'am' or 'pm'
-  // %S   second as decimal number
-  // %U   week of the current year as decimal number (Sunday first day)
-  // %W   week of the current year as decimal number (Monday first day)
-  // %w   day of the week as decimal number (Sunday = 0)
-  // %x   date with date representation for locale
-  // %X   time with time representation for locale
-  // %y   year as a decimal number without century 
-  // %Y   year as a decimal number with century
-  // %z   time zone offset in hours and minutes from GMT (HHMM)
-  // %Z   time zone abbreviation
-
-  while (formatIdx < formatLen)
-    {
-      if (format[formatIdx] != '%')
+      int		year = 0, month = 1, day = 1;
+      int		hour = 0, min = 0, sec = 0;
+      NSTimeZone	*tz = [NSTimeZone localTimeZone];
+      BOOL		ampm = NO;
+      BOOL		twelveHrClock = NO; 
+      int		julianWeeks = -1, weekStartsMonday = 0, dayOfWeek = -1;
+      const char	*source = [description cString];
+      int		sourceLen = strlen(source);
+      const char	*format = [fmt cString];
+      int		formatLen = strlen(format);
+      int		formatIdx = 0;
+      int		sourceIdx = 0;
+      char		tmpStr[20];
+      int		tmpIdx;
+      
+      if (locale == nil)
 	{
-	  // If it's not a format specifier, ignore it.
-	  sourceIdx++;
+	  locale
+	    = [[NSUserDefaults standardUserDefaults] dictionaryRepresentation];
 	}
-      else
+      if (fmt == nil)
 	{
-	  // Skip '%'
-	  formatIdx++;
+	  format = [[locale objectForKey: NSTimeDateFormatString] cString];
+	  formatLen = strlen(format);
+	}
+      
+      //
+      // WARNING:
+      //   -%c, %F, %x, %X do NOT work. (NSTimeDateFormatString isn't defined 
+      //    and the underlying call has granularity to the second.
+      //   -Most locale stuff is dubious at best.
+      //   -Long day and month names depend on a non-alpha character after the
+      //    last digit to work.
+      //
+      // The strftime specifiers as used by OpenStep + %U.
+      //
+      // %%   literal % character
+      // %a   abbreviated weekday name according to locale
+      // %A   full weekday name according to locale
+      // %b   abbreviated month name according to locale
+      // %B   full month name according to locale
+      // %c   same as '%X %x'
+      // %d   day of month as decimal number
+      // %e   same as %d without leading zero (you get a leading space instead)
+      // %F   milliseconds as a decimal number
+      // %H   hour as a decimal number using 24-hour clock
+      // %I   hour as a decimal number using 12-hour clock
+      // %j   day of year as a decimal number
+      // %m   month as decimal number
+      // %M   minute as decimal number
+      // %p   'am' or 'pm'
+      // %S   second as decimal number
+      // %U   week of the current year as decimal number (Sunday first day)
+      // %W   week of the current year as decimal number (Monday first day)
+      // %w   day of the week as decimal number (Sunday = 0)
+      // %x   date with date representation for locale
+      // %X   time with time representation for locale
+      // %y   year as a decimal number without century 
+      // %Y   year as a decimal number with century
+      // %z   time zone offset in hours and minutes from GMT (HHMM)
+      // %Z   time zone abbreviation
 
-	  switch (format[formatIdx])
+      while (formatIdx < formatLen)
+	{
+	  if (format[formatIdx] != '%')
 	    {
-	      case '%':
-		// skip literal %
-		sourceIdx++;
-		break;
-
-	      case 'a':
-		// Are Short names three chars in all locales?????
-		tmpStr[0] = toupper(source[sourceIdx++]);
-		tmpStr[1] = tolower(source[sourceIdx++]);
-		tmpStr[2] = tolower(source[sourceIdx++]);
-		tmpStr[3] = '\0';
-		{
-		  NSString	*currDay;
-		  NSArray	*dayNames;
-
-		  currDay = [NSString stringWithCString: tmpStr];
-		  dayNames = [locale objectForKey: NSShortWeekDayNameArray];
-		  for (tmpIdx = 0; tmpIdx < 7; tmpIdx++)
-		    {
-		      if ([[dayNames objectAtIndex: tmpIdx] isEqual: currDay]
-			== YES)
-			{
-			  break;
-			}
-		    }
-		  dayOfWeek = tmpIdx; 
-		}
-		break;
-
-	      case 'A':
-		for (tmpIdx = sourceIdx; tmpIdx < sourceLen; tmpIdx++)
-		  {
-		    if (isalpha(source[tmpIdx]))
-		      {
-			tmpStr[tmpIdx - sourceIdx] = source[tmpIdx];
-		      }
-		    else
-		      {
-			break;
-		      }
-		  }
-		tmpStr[tmpIdx - sourceIdx] = '\0';
-		sourceIdx += tmpIdx - sourceIdx;
-	        {
-		  NSString	*currDay;
-		  NSArray	*dayNames;
-
-		  currDay = [NSString stringWithCString: tmpStr];
-		  dayNames = [locale objectForKey: NSWeekDayNameArray];
-		  for (tmpIdx = 0; tmpIdx < 7; tmpIdx++)
-		    {
-		      if ([[dayNames objectAtIndex: tmpIdx] isEqual: currDay]
-			== YES)
-			{
-			  break;
-			}
-		    }
-		  dayOfWeek = tmpIdx;
-	        }
-		break;
-
-	      case 'b':
-		// Are Short names three chars in all locales?????
-		tmpStr[0] = toupper(source[sourceIdx++]);
-		tmpStr[1] = tolower(source[sourceIdx++]);
-		tmpStr[2] = tolower(source[sourceIdx++]);
-		tmpStr[3] = '\0';
-	        {
-		  NSString	*currMonth;
-		  NSArray	*monthNames;
-
-		  currMonth = [NSString stringWithCString: tmpStr];
-		  monthNames = [locale objectForKey: NSShortMonthNameArray];
-
-		  for (tmpIdx = 0; tmpIdx < 12; tmpIdx++)
-		    {
-		      if ([[monthNames objectAtIndex: tmpIdx]
-			isEqual: currMonth] == YES)
-			{
-			  break;
-			}
-		    }
-		  month = tmpIdx+1;
-	        }
-		break;
-
-	      case 'B':
-		for (tmpIdx = sourceIdx; tmpIdx < sourceLen; tmpIdx++)
-		  {
-		    if (isalpha(source[tmpIdx]))
-		      {
-			tmpStr[tmpIdx - sourceIdx] = source[tmpIdx];
-		      }
-		    else
-		      {
-			break;
-		      }
-		  }
-		tmpStr[tmpIdx - sourceIdx] = '\0';
-		sourceIdx += tmpIdx - sourceIdx;
-	        {
-		  NSString	*currMonth;
-		  NSArray	*monthNames;
-
-		  currMonth = [NSString stringWithCString: tmpStr];
-		  monthNames = [locale objectForKey: NSMonthNameArray];
-
-		  for (tmpIdx = 0; tmpIdx < 12; tmpIdx++)
-		    {
-		      if ([[monthNames objectAtIndex: tmpIdx]
-			isEqual: currMonth] == YES)
-			{
-			  break;
-			}
-		    }
-		  month = tmpIdx+1;
-	        }
-	        break;
-
-//	case 'c':
-//	break;
-
-	      case 'd': // fall through
-	      case 'e':
-		memcpy(tmpStr, &source[sourceIdx], 2);
-		tmpStr[2] = '\0';
-		sourceIdx += 2;
-		day = atoi(tmpStr);
-		break;
-
-//	case 'F':
-//	break;
-
-	      case 'I': // fall through
-		twelveHrClock = YES;
-	      case 'H':
-		memcpy(tmpStr, &source[sourceIdx], 2);
-		tmpStr[2] = '\0';
-		sourceIdx += 2;
-		hour = atoi(tmpStr);
-		break;
-
-	      case 'j':
-		memcpy(tmpStr, &source[sourceIdx], 3);
-		tmpStr[3] = '\0';
-		sourceIdx += 3;
-		day = atoi(tmpStr);
-		break;
-
-	      case 'm':
-		memcpy(tmpStr, &source[sourceIdx], 2);
-		tmpStr[2] = '\0';
-		sourceIdx += 2;
-		month = atoi(tmpStr);
-		break;
-
-	      case 'M':
-		memcpy(tmpStr, &source[sourceIdx], 2);
-		tmpStr[2] = '\0';
-		sourceIdx += 2;
-		min = atoi(tmpStr);
-		break;
-
-	      case 'p':
-		// Questionable assumption that all am/pm indicators are 2
-		// characters and in upper case....
-		tmpStr[0] = toupper(source[sourceIdx++]);
-		tmpStr[1] = toupper(source[sourceIdx++]);
-		tmpStr[2] = '\0';
-		{
-		  NSString	*currAMPM;
-		  NSArray	*amPMNames;
-
-		  currAMPM = [NSString stringWithCString: tmpStr];
-		  amPMNames = [locale objectForKey: NSAMPMDesignation];
-
-		  /*
-		   * The time addition is handled below because this
-		   * indicator only modifies the time on a 12hour clock.
-		   */
-		  if ([[amPMNames objectAtIndex: 1] isEqual: currAMPM] == YES)
-		    {
-		      ampm = YES;
-		    }
-		}
-		break;
-
-	      case 'S':
-		memcpy(tmpStr, &source[sourceIdx], 2);
-		tmpStr[2] = '\0';
-		sourceIdx += 2;
-		sec = atoi(tmpStr);
-		break;
-
-	      case 'w':
-		tmpStr[0] = source[sourceIdx++];
-		tmpStr[1] = '\0';
-		dayOfWeek = atoi(tmpStr);
-		break;
-
-	      case 'W': // Fall through
-		weekStartsMonday = 1;
-	      case 'U':
-		memcpy(tmpStr, &source[sourceIdx], 2);
-		tmpStr[2] = '\0';
-		sourceIdx += 2;
-		julianWeeks = atoi(tmpStr);
-		break;
-
-//	case 'x':
-//	break;
-
-//	case 'X':
-//	break;
-
-	      case 'y':
-		memcpy(tmpStr, &source[sourceIdx], 2);
-		tmpStr[2] = '\0';
-		sourceIdx += 2;
-		year = atoi(tmpStr);
-		if (year >= 70)
-		  {
-		    year += 1900;
-		  }
-		else
-		  {
-		    year += 2000;
-		  }
-		break;
-
-	      case 'Y':
-		memcpy(tmpStr, &source[sourceIdx], 4);
-		tmpStr[4] = '\0';
-		sourceIdx += 4;
-		year = atoi(tmpStr);
-		break;
-
-	      case 'z':
-		tmpStr[0] = toupper(source[sourceIdx++]);
-		tmpStr[1] = tolower(source[sourceIdx++]);
-		tmpStr[2] = tolower(source[sourceIdx++]);
-		tmpStr[3] = tolower(source[sourceIdx++]);
-		tmpStr[4] = '\0';
-	        {
-		  int zone = atoi(tmpStr);
-
-		  if ((tz = [NSTimeZone timeZoneForSecondsFromGMT: 
-		    (zone / 100 * 60 + (zone % 100)) * 60]) == nil)
-		    {
-		      tz = [NSTimeZone localTimeZone];
-		    }
-		}
-		break;
-
-	      case 'Z':
-		for (tmpIdx = sourceIdx; tmpIdx < sourceLen; tmpIdx++)
-		  {
-		    if (isalpha(source[tmpIdx]) || source[tmpIdx] == '-'
-		      || source[tmpIdx] == '+')
-		      {
-			tmpStr[tmpIdx - sourceIdx] = source[tmpIdx];
-		      }
-		    else
-		      {
-			break;
-		      }
-		  }
-		tmpStr[tmpIdx - sourceIdx] = '\0';
-		sourceIdx += tmpIdx - sourceIdx;
-	        {
-		  if ((tz = [NSTimeZone timeZoneWithAbbreviation: 
-		    [NSString stringWithCString: tmpStr]]) == nil)
-		    {
-		      tz = [NSTimeZone localTimeZone];
-		    }
-	        }
-		break;
-
-	      default:
-		[NSException raise: NSInvalidArgumentException format:
-		  @"Invalid NSCalendar date, specifier %c not recognized in format %s", format[formatIdx], format];
-	    }
-	} 
-      formatIdx++;
-    }
-
-  if (twelveHrClock == YES)
-    {
-      if (ampm == YES && hour != 12)
-	{
-	   hour += 12;
-        }
-    }
-
-  if (julianWeeks != -1)
-    {
-      NSTimeZone	*gmtZone;
-      NSCalendarDate	*d;
-      int		currDay;
-
-      gmtZone = [NSTimeZone timeZoneForSecondsFromGMT: 0];
-      d  = [NSCalendarDate dateWithYear: year
-				  month: 1
-				    day: 1
-				   hour: 0
-				 minute: 0
-				 second: 0
-			       timeZone: gmtZone];
-      currDay = [d dayOfWeek];
-
-      /*
-       * The julian weeks are either sunday relative or monday relative but all
-       * of the day of week specifiers are sunday relative.  This means that if
-       * no day of week specifier was used the week starts on monday.
-       */
-      if (dayOfWeek == -1)
-	{
-	  if (weekStartsMonday)
-	    {
-	      dayOfWeek = 1;
+	      // If it's not a format specifier, ignore it.
+	      sourceIdx++;
 	    }
 	  else
 	    {
-	      dayOfWeek = 0;
+	      // Skip '%'
+	      formatIdx++;
+
+	      switch (format[formatIdx])
+		{
+		  case '%':
+		    // skip literal %
+		    sourceIdx++;
+		    break;
+
+		  case 'a':
+		    // Are Short names three chars in all locales?????
+		    tmpStr[0] = toupper(source[sourceIdx++]);
+		    tmpStr[1] = tolower(source[sourceIdx++]);
+		    tmpStr[2] = tolower(source[sourceIdx++]);
+		    tmpStr[3] = '\0';
+		    {
+		      NSString	*currDay;
+		      NSArray	*dayNames;
+
+		      currDay = [NSString stringWithCString: tmpStr];
+		      dayNames = [locale objectForKey: NSShortWeekDayNameArray];
+		      for (tmpIdx = 0; tmpIdx < 7; tmpIdx++)
+			{
+			  if ([[dayNames objectAtIndex: tmpIdx] isEqual:
+			    currDay] == YES)
+			    {
+			      break;
+			    }
+			}
+		      dayOfWeek = tmpIdx; 
+		    }
+		    break;
+
+		  case 'A':
+		    for (tmpIdx = sourceIdx; tmpIdx < sourceLen; tmpIdx++)
+		      {
+			if (isalpha(source[tmpIdx]))
+			  {
+			    tmpStr[tmpIdx - sourceIdx] = source[tmpIdx];
+			  }
+			else
+			  {
+			    break;
+			  }
+		      }
+		    tmpStr[tmpIdx - sourceIdx] = '\0';
+		    sourceIdx += tmpIdx - sourceIdx;
+		    {
+		      NSString	*currDay;
+		      NSArray	*dayNames;
+
+		      currDay = [NSString stringWithCString: tmpStr];
+		      dayNames = [locale objectForKey: NSWeekDayNameArray];
+		      for (tmpIdx = 0; tmpIdx < 7; tmpIdx++)
+			{
+			  if ([[dayNames objectAtIndex: tmpIdx] isEqual:
+			    currDay] == YES)
+			    {
+			      break;
+			    }
+			}
+		      dayOfWeek = tmpIdx;
+		    }
+		    break;
+
+		  case 'b':
+		    // Are Short names three chars in all locales?????
+		    tmpStr[0] = toupper(source[sourceIdx++]);
+		    tmpStr[1] = tolower(source[sourceIdx++]);
+		    tmpStr[2] = tolower(source[sourceIdx++]);
+		    tmpStr[3] = '\0';
+		    {
+		      NSString	*currMonth;
+		      NSArray	*monthNames;
+
+		      currMonth = [NSString stringWithCString: tmpStr];
+		      monthNames = [locale objectForKey: NSShortMonthNameArray];
+
+		      for (tmpIdx = 0; tmpIdx < 12; tmpIdx++)
+			{
+			  if ([[monthNames objectAtIndex: tmpIdx]
+				    isEqual: currMonth] == YES)
+			    {
+			      break;
+			    }
+			}
+		      month = tmpIdx+1;
+		    }
+		    break;
+
+		  case 'B':
+		    for (tmpIdx = sourceIdx; tmpIdx < sourceLen; tmpIdx++)
+		      {
+			if (isalpha(source[tmpIdx]))
+			  {
+			    tmpStr[tmpIdx - sourceIdx] = source[tmpIdx];
+			  }
+			else
+			  {
+			    break;
+			  }
+		      }
+		    tmpStr[tmpIdx - sourceIdx] = '\0';
+		    sourceIdx += tmpIdx - sourceIdx;
+		    {
+		      NSString	*currMonth;
+		      NSArray	*monthNames;
+
+		      currMonth = [NSString stringWithCString: tmpStr];
+		      monthNames = [locale objectForKey: NSMonthNameArray];
+
+		      for (tmpIdx = 0; tmpIdx < 12; tmpIdx++)
+			{
+			  if ([[monthNames objectAtIndex: tmpIdx]
+				    isEqual: currMonth] == YES)
+			    {
+			      break;
+			    }
+			}
+		      month = tmpIdx+1;
+		    }
+		    break;
+
+		    //	case 'c':
+		    //	break;
+
+		  case 'd': // fall through
+		  case 'e':
+		    memcpy(tmpStr, &source[sourceIdx], 2);
+		    tmpStr[2] = '\0';
+		    sourceIdx += 2;
+		    day = atoi(tmpStr);
+		    break;
+
+		    //	case 'F':
+		    //	break;
+
+		  case 'I': // fall through
+		    twelveHrClock = YES;
+		  case 'H':
+		    memcpy(tmpStr, &source[sourceIdx], 2);
+		    tmpStr[2] = '\0';
+		    sourceIdx += 2;
+		    hour = atoi(tmpStr);
+		    break;
+
+		  case 'j':
+		    memcpy(tmpStr, &source[sourceIdx], 3);
+		    tmpStr[3] = '\0';
+		    sourceIdx += 3;
+		    day = atoi(tmpStr);
+		    break;
+
+		  case 'm':
+		    memcpy(tmpStr, &source[sourceIdx], 2);
+		    tmpStr[2] = '\0';
+		    sourceIdx += 2;
+		    month = atoi(tmpStr);
+		    break;
+
+		  case 'M':
+		    memcpy(tmpStr, &source[sourceIdx], 2);
+		    tmpStr[2] = '\0';
+		    sourceIdx += 2;
+		    min = atoi(tmpStr);
+		    break;
+
+		  case 'p':
+		    // Questionable assumption that all am/pm indicators are 2
+		    // characters and in upper case....
+		    tmpStr[0] = toupper(source[sourceIdx++]);
+		    tmpStr[1] = toupper(source[sourceIdx++]);
+		    tmpStr[2] = '\0';
+		    {
+		      NSString	*currAMPM;
+		      NSArray	*amPMNames;
+
+		      currAMPM = [NSString stringWithCString: tmpStr];
+		      amPMNames = [locale objectForKey: NSAMPMDesignation];
+
+		      /*
+		       * The time addition is handled below because this
+		       * indicator only modifies the time on a 12hour clock.
+		       */
+		      if ([[amPMNames objectAtIndex: 1] isEqual:
+			currAMPM] == YES)
+			{
+			  ampm = YES;
+			}
+		    }
+		    break;
+
+		  case 'S':
+		    memcpy(tmpStr, &source[sourceIdx], 2);
+		    tmpStr[2] = '\0';
+		    sourceIdx += 2;
+		    sec = atoi(tmpStr);
+		    break;
+
+		  case 'w':
+		    tmpStr[0] = source[sourceIdx++];
+		    tmpStr[1] = '\0';
+		    dayOfWeek = atoi(tmpStr);
+		    break;
+
+		  case 'W': // Fall through
+		    weekStartsMonday = 1;
+		  case 'U':
+		    memcpy(tmpStr, &source[sourceIdx], 2);
+		    tmpStr[2] = '\0';
+		    sourceIdx += 2;
+		    julianWeeks = atoi(tmpStr);
+		    break;
+
+		    //	case 'x':
+		    //	break;
+
+		    //	case 'X':
+		    //	break;
+
+		  case 'y':
+		    memcpy(tmpStr, &source[sourceIdx], 2);
+		    tmpStr[2] = '\0';
+		    sourceIdx += 2;
+		    year = atoi(tmpStr);
+		    if (year >= 70)
+		      {
+			year += 1900;
+		      }
+		    else
+		      {
+			year += 2000;
+		      }
+		    break;
+
+		  case 'Y':
+		    memcpy(tmpStr, &source[sourceIdx], 4);
+		    tmpStr[4] = '\0';
+		    sourceIdx += 4;
+		    year = atoi(tmpStr);
+		    break;
+
+		  case 'z':
+		    {
+		      int	sign = 1;
+		      int	zone;
+
+		      if (source[sourceIdx] == '+')
+			{
+			  sourceIdx++;
+			}
+		      else if (source[sourceIdx] == '-')
+			{
+			  sign = -1;
+			  sourceIdx++;
+			}
+		      tmpStr[0] = source[sourceIdx++];
+		      tmpStr[1] = source[sourceIdx++];
+		      tmpStr[2] = source[sourceIdx++];
+		      tmpStr[3] = source[sourceIdx++];
+		      tmpStr[4] = '\0';
+		      zone = atoi(tmpStr) * sign;
+
+		      if ((tz = [NSTimeZone timeZoneForSecondsFromGMT: 
+			(zone / 100 * 60 + (zone % 100)) * 60]) == nil)
+			{
+			  tz = [NSTimeZone localTimeZone];
+			}
+		    }
+		    break;
+
+		  case 'Z':
+		    for (tmpIdx = sourceIdx; tmpIdx < sourceLen; tmpIdx++)
+		      {
+			if (isalpha(source[tmpIdx]) || source[tmpIdx] == '-'
+			  || source[tmpIdx] == '+')
+			  {
+			    tmpStr[tmpIdx - sourceIdx] = source[tmpIdx];
+			  }
+			else
+			  {
+			    break;
+			  }
+		      }
+		    tmpStr[tmpIdx - sourceIdx] = '\0';
+		    sourceIdx += tmpIdx - sourceIdx;
+		    if ((tz = [NSTimeZone timeZoneWithAbbreviation: 
+		      [NSString stringWithCString: tmpStr]]) == nil)
+		      {
+			tz = [NSTimeZone localTimeZone];
+		      }
+		    break;
+
+		  default:
+		    [NSException raise: NSInvalidArgumentException
+				format: @"Invalid NSCalendar date, "
+			@"specifier %c not recognized in format %s",
+			format[formatIdx], format];
+		}
+	    } 
+	  formatIdx++;
+	}
+
+      if (twelveHrClock == YES)
+	{
+	  if (ampm == YES && hour != 12)
+	    {
+	      hour += 12;
 	    }
 	}
-      day = dayOfWeek + (julianWeeks * 7 - (currDay - 1));
-    }
 
-  return [[NSCalendarDate alloc] initWithYear: year
-					month: month
-					  day: day
-					 hour: hour
-				       minute: min
-				       second: sec
-				     timeZone: tz];
+      if (julianWeeks != -1)
+	{
+	  NSTimeZone	*gmtZone;
+	  NSCalendarDate	*d;
+	  int		currDay;
+
+	  gmtZone = [NSTimeZone timeZoneForSecondsFromGMT: 0];
+	  d  = [NSCalendarDate dateWithYear: year
+				      month: 1
+					day: 1
+				       hour: 0
+				     minute: 0
+				     second: 0
+				   timeZone: gmtZone];
+	  currDay = [d dayOfWeek];
+
+	  /*
+	   * The julian weeks are either sunday relative or monday relative
+	   * but all of the day of week specifiers are sunday relative.
+	   * This means that if no day of week specifier was used the week
+	   * starts on monday.
+	   */
+	  if (dayOfWeek == -1)
+	    {
+	      if (weekStartsMonday)
+		{
+		  dayOfWeek = 1;
+		}
+	      else
+		{
+		  dayOfWeek = 0;
+		}
+	    }
+	  day = dayOfWeek + (julianWeeks * 7 - (currDay - 1));
+	}
+
+      return [self initWithYear: year
+			  month: month
+			    day: day
+			   hour: hour
+			 minute: min
+			 second: sec
+		       timeZone: tz];
+    }
 }
 
 
@@ -607,18 +682,11 @@
 	     second: (unsigned int)second
 	   timeZone: (NSTimeZone *)aTimeZone
 {
-  int	a;
   int	c;
   NSTimeInterval s;
 
-  a = [self absoluteGregorianDay: day month: month year: year];
-
   // Calculate date as GMT
-  a -= GREGORIAN_REFERENCE;
-  s = (double)a * 86400;
-  s += hour * 3600;
-  s += minute * 60;
-  s += second;
+  s = GSTime(day, month, year, hour, minute, second);
 
   // Assign time zone detail
   _time_zone = RETAIN([aTimeZone
@@ -779,7 +847,7 @@
 	day: &d month: &m year: &y];
   days = d;
   for (i = m - 1;  i > 0; i--) // days in prior months this year
-    days = days + [self lastDayOfGregorianMonth: i year: y];
+    days = days + lastDayOfGregorianMonth(i, y);
 
   return days;
 }
@@ -965,7 +1033,7 @@
 	      if (ycent)
 		k = VSPRINTF_LENGTH(sprintf(&(buf[j]), "%04d", yd));
 	      else
-		k = VSPRINTF_LENGTH(sprintf(&(buf[j]), "%02d", (yd - 1900)));
+		k = VSPRINTF_LENGTH(sprintf(&(buf[j]), "%02d", yd % 100));
 	      j += k;
 	      break;
 
@@ -1219,34 +1287,12 @@
 
 - (int) lastDayOfGregorianMonth: (int)month year: (int)year
 {
-  switch (month) {
-    case 2:
-      if ((((year % 4) == 0) && ((year % 100) != 0))
-	  || ((year % 400) == 0))
-	return 29;
-      else
-	return 28;
-    case 4:
-    case 6:
-    case 9:
-    case 11: return 30;
-    default: return 31;
-  }
+  return lastDayOfGregorianMonth(month, year);
 }
 
 - (int) absoluteGregorianDay: (int)day month: (int)month year: (int)year
 {
-  int m, N;
-
-  N = day;   // day of month
-  for (m = month - 1;  m > 0; m--) // days in prior months this year
-      N = N + [self lastDayOfGregorianMonth: m year: year];
-  return
-    (N                    // days this year
-     + 365 * (year - 1)   // days in previous years ignoring leap days
-     + (year - 1)/4       // Julian leap days before this year...
-     - (year - 1)/100     // ...minus prior century years...
-     + (year - 1)/400);   // ...plus prior years divisible by 400
+  return absoluteGregorianDay(day, month, year);
 }
 
 - (void) gregorianDateFromAbsolute: (int)d
@@ -1256,15 +1302,14 @@
 {
   // Search forward year by year from approximate year
   *year = d/366;
-  while (d >= [self absoluteGregorianDay: 1 month: 1 year: (*year)+1])
+  while (d >= absoluteGregorianDay(1, 1, (*year)+1))
     (*year)++;
   // Search forward month by month from January
   (*month) = 1;
-  while (d > [self absoluteGregorianDay:
-		   [self lastDayOfGregorianMonth: *month year: *year]
-		   month: *month year: *year])
+  while (d > absoluteGregorianDay(lastDayOfGregorianMonth(*month, *year),
+    *month, *year))
     (*month)++;
-  *day = d - [self absoluteGregorianDay: 1 month: *month year: *year] + 1;
+  *day = d - absoluteGregorianDay(1, *month, *year) + 1;
 }
 
 @end
@@ -1468,7 +1513,7 @@
 	      tmpmonth += 12;
 	      tmpyear--;
 	    }
-          extra += [end lastDayOfGregorianMonth: tmpmonth year: tmpyear];
+          extra += lastDayOfGregorianMonth(tmpmonth, tmpyear);
         }
     }
 
