@@ -1,5 +1,5 @@
 /* Stream of bytes class for serialization and persistance in GNUStep
-   Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 2000 Free Software Foundation, Inc.
    
    Written by:  Andrew Kachites McCallum <mccallum@gnu.ai.mit.edu>
    Date: March 1995
@@ -74,6 +74,7 @@
 #include <Foundation/NSFileManager.h>
 #include <Foundation/NSPathUtilities.h>
 #include <Foundation/NSRange.h>
+#include <Foundation/NSZone.h>
 #include <string.h>		/* for memset() */
 #include <unistd.h>             /* SEEK_* on SunOS 4 */
 
@@ -106,8 +107,11 @@
  *	these are set up at process startup or in [NSData +initialize]
  */
 static SEL	appendSel = @selector(appendBytes:length:);
+static Class	dataStatic;
 static Class	dataMalloc;
 static Class	mutableDataMalloc;
+static Class	NSDataAbstract;
+static Class	NSMutableDataAbstract;
 static IMP	appendImp;
 
 static BOOL
@@ -118,6 +122,12 @@ readContentsOfFile(NSString* path, void** buf, unsigned* len, NSZone* zone)
   unsigned	fileLength;
   void		*tmp = 0;
   int		c;
+#if defined(__MINGW__)
+  HANDLE	fh;
+  DWORD		fileLength
+  DWORD		high;
+  DWORD		got;
+#endif
 
   if ([path getFileSystemRepresentation: thePath
 			      maxLength: sizeof(thePath)-1] == NO)
@@ -125,11 +135,66 @@ readContentsOfFile(NSString* path, void** buf, unsigned* len, NSZone* zone)
       NSDebugLog(@"Open (%s) attempt failed - bad path", thePath);
       return NO;
     }
-#if	defined(__WIN32__)
-  theFile = fopen(thePath, "rb");
+
+#if defined(__MINGW__)
+  fh = CreateFile(thePath, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING,
+    FILE_ATTRIBUTE_NORMAL, 0);
+  if (fh == INVALID_HANDLE_VALUE)
+    {
+      NSDebugLog(@"Open (%s) attempt failed", thePath);
+      return NO;
+    }
+
+  fileLength = GetFileSize(fh, &high);
+  if ((fileLength == 0xFFFFFFFF) && (GetLastError() != NO_ERROR))
+    {
+      CloseHandle(fh);
+      NSLog(@"Failed to determine size of - %s", thePath);
+      return NO;
+    }
+  if (high != 0)
+    {
+      CloseHandle(fh);
+      NSLog(@"File too big to handle - %s", thePath);
+      return NO;
+    }
+
+#if	GS_WITH_GC == 1
+  tmp = NSZoneMalloc(GSAtomicMallocZone(), fileLength);
 #else
-  theFile = fopen(thePath, "r");
+  tmp = NSZoneMalloc(zone, fileLength);
 #endif
+  if (tmp == 0)
+    {
+      CloseHandle(fh);
+      NSLog(@"Malloc failed for file (%s) of length %d - %s",
+	thePath, fileLength, strerror(errno));
+      return NO;
+    }
+  if (!ReadFile(fh, tmp, fileSize, &got, 0))
+    {
+      if (tmp != 0)
+	{
+	  NSZoneFree(zone, tmp);
+	  CloseHandle(fh);
+	  NSLog(@"File read operation failed for %s", thePath);
+	  return NO;
+	}
+    }
+  if (got != fileSize)
+    {
+      NSZoneFree(zone, tmp);
+      CloseHandle(fh);
+      NSLog(@"File read operation short for %s", thePath);
+      return NO;
+    }
+  CloseHandle(fh);
+  *buf = tmp;
+  *len = fileLength;
+  return YES;
+#endif
+
+  theFile = fopen(thePath, "rb");
 
   if (theFile == NULL)		/* We failed to open the file. */
     {
@@ -197,9 +262,9 @@ readContentsOfFile(NSString* path, void** buf, unsigned* len, NSZone* zone)
    *	Just in case the failure action needs to be changed.
    */
 failure:
-  if (tmp)
+  if (tmp != 0)
     NSZoneFree(zone, tmp);
-  if (theFile)
+  if (theFile != 0)
     fclose(theFile);
   return NO;
 }
@@ -263,16 +328,25 @@ failure:
 {
   if (self == [NSData class])
     {
+      NSDataAbstract = self;
+      NSMutableDataAbstract = [NSMutableData class];
       dataMalloc = [NSDataMalloc class];
+      dataStatic = [NSDataStatic class];
       mutableDataMalloc = [NSMutableDataMalloc class];
-      appendImp = [mutableDataMalloc
-	    instanceMethodForSelector: appendSel];
+      appendImp = [mutableDataMalloc instanceMethodForSelector: appendSel];
     }
 }
 
-+ (NSData*) allocWithZone: (NSZone*)z
++ (id) allocWithZone: (NSZone*)z
 {
-  return (NSData*)NSAllocateObject(dataMalloc, 0, z);
+  if (self == NSDataAbstract)
+    {
+      return NSAllocateObject(dataMalloc, 0, z);
+    }
+  else
+    {
+      return NSAllocateObject(self, 0, z);
+    }
 }
 
 + (id) data
@@ -336,10 +410,18 @@ failure:
   return AUTORELEASE(d);
 }
 
++ (id) new
+{
+  NSData	*d;
+
+  d = [dataMalloc allocWithZone: NSDefaultMallocZone()];
+  d = [d initWithBytesNoCopy: 0 length: 0 fromZone: NSDefaultMallocZone()];
+  return d;
+}
+
 - (id) init
 {
-   return [self initWithBytesNoCopy: 0
-			     length: 0];
+   return [self initWithBytesNoCopy: 0 length: 0];
 }
 
 - (id) initWithBytes: (const void*)aBuffer
@@ -376,8 +458,7 @@ failure:
 
 - (id) initWithData: (NSData*)data
 {
-  return [self initWithBytes: [data bytes]
-		      length: [data length]];
+  return [self initWithBytes: [data bytes] length: [data length]];
 }
 
 
@@ -431,19 +512,17 @@ failure:
   return AUTORELEASE(str);
 }
 
-- (void)getBytes: (void*)buffer
+- (void) getBytes: (void*)buffer
 {
   [self getBytes: buffer range: NSMakeRange(0, [self length])];
 }
 
-- (void)getBytes: (void*)buffer
-	  length: (unsigned)length
+- (void) getBytes: (void*)buffer length: (unsigned)length
 {
   [self getBytes: buffer range: NSMakeRange(0, length)];
 }
 
-- (void)getBytes: (void*)buffer
-	   range: (NSRange)aRange
+- (void) getBytes: (void*)buffer range: (NSRange)aRange
 {
   unsigned	size = [self length];
 
@@ -478,7 +557,32 @@ failure:
 
 - (unsigned) hash
 {
-  return [self length];
+  unsigned char	buf[64];
+  unsigned	l = [self length];
+  unsigned	ret =0;
+  
+  l = MIN(l,64);
+
+  /*
+   * hash for empty data matches hash for empty string
+   */
+  if (l == 0)
+    {
+      return 0xfffffffe;
+    }
+  
+  [self getBytes: &buf range: NSMakeRange(0, l)];
+
+  while (l-- > 0)
+    {
+      ret = (ret << 5 ) + ret + buf[l];
+    }
+  // Again, match NSString
+  if (ret == 0)
+    {
+       ret = 0xffffffff;
+    }
+  return ret;
 }
 
 - (BOOL) isEqual: anObject
@@ -497,7 +601,7 @@ failure:
   return (memcmp([self bytes], [other bytes], len) ? NO : YES);
 }
 
-- (unsigned)length;
+- (unsigned) length;
 {
   /* This is left to concrete subclasses to implement. */
   [self subclassResponsibility: _cmd];
@@ -507,13 +611,12 @@ failure:
 
 // Storing Data
 
-- (BOOL) writeToFile: (NSString *)path
-	  atomically: (BOOL)useAuxiliaryFile
+- (BOOL) writeToFile: (NSString*)path atomically: (BOOL)useAuxiliaryFile
 {
-  char thePath[BUFSIZ*2+8];
-  char theRealPath[BUFSIZ*2];
-  FILE *theFile;
-  int c;
+  char	thePath[BUFSIZ*2+8];
+  char	theRealPath[BUFSIZ*2];
+  FILE	*theFile;
+  int	c;
 
   if ([path getFileSystemRepresentation: theRealPath
 			      maxLength: sizeof(theRealPath)-1] == NO)
@@ -522,6 +625,38 @@ failure:
       return NO;
     }
 
+#if defined(__MINGW__)
+  HANDLE	fh;
+  DWORD		wroteBytes;
+
+  if (useAuxiliaryFile)
+    {
+      path = [path stringByAppendingPathExtension: @"tmp"];
+    }
+  if ([path getFileSystemRepresentation: thePath
+			      maxLength: sizeof(thePath)-1] == NO)
+    {
+      NSDebugLog(@"Open (%s) attempt failed - bad path", thePath);
+      return NO;
+    }
+  
+  fh = CreateFile(thePath, GENERIC_WRITE, 0, 0, CREATE_ALWAYS,
+    FILE_ATTRIBUTE_NORMAL, NULL);
+  if (fh == INVALID_HANDLE_VALUE)
+    {
+      NSLog(@"Create (%s) attempt failed", thePath);
+      return NO;
+    }
+
+  if (!WriteFile(fh, [self bytes], [self length], &wroteBytes, 0))
+    {
+      CloseHandle(fh);
+      NSLog(@"Write (%s) attempt failed", thePath);
+      goto failure;
+    }
+  CloseHandle(fh);
+#else
+
 #ifdef	HAVE_MKSTEMP
   if (useAuxiliaryFile)
     {
@@ -542,11 +677,7 @@ failure:
   else
     {
       strcpy(thePath, theRealPath);
-#if	defined(__WIN32__)
       theFile = fopen(thePath, "wb");
-#else
-      theFile = fopen(thePath, "w");
-#endif
     }
 #else
   if (useAuxiliaryFile)
@@ -568,11 +699,7 @@ failure:
     }
 
   /* Open the file (whether temp or real) for writing. */
-#if	defined(__WIN32__)
   theFile = fopen(thePath, "wb");
-#else
-  theFile = fopen(thePath, "w");
-#endif
 #endif
 
   if (theFile == NULL)          /* Something went wrong; we weren't
@@ -604,6 +731,7 @@ failure:
       NSLog(@"Fclose (%s) failed - %s", thePath, strerror(errno));
       goto failure;
     }
+#endif
 
   /* If we used a temporary file, we still need to rename() it be the
    * real file.  Also, we need to try to retain the file attributes of
@@ -657,7 +785,7 @@ failure:
   return YES;
 
   /* Just in case the failure action needs to be changed. */
- failure:
+failure:
   /*
    * Attempt to tidy up by removing temporary file on failure.
    */
@@ -720,15 +848,12 @@ failure:
 	  else
 	    {
 	      unsigned	len = (length+1)*sizeof(char);
-	      NSZone	*z = [self zone];
-	      NSData	*d;
 
-	      *(char**)data = (char*)NSZoneMalloc(z, len);
-	      d = [dataMalloc allocWithZone: z];
-	      d = [d initWithBytesNoCopy: *(void**)data
-				  length: len
-			        fromZone: z];
-	      IF_NO_GC(AUTORELEASE(d));
+#if	GS_WITH_GC == 0
+	      *(char**)data = (char*)NSZoneMalloc(NSDefaultMallocZone(), len);
+#else
+	      *(char**)data = (char*)NSZoneMalloc(NSAtomicMallocZone(), len);
+#endif
 	    }
 
 	  [self deserializeBytes: *(char**)data
@@ -790,15 +915,12 @@ failure:
       case _C_PTR:
 	{
 	  unsigned	len = objc_sizeof_type(++type);
-	  NSZone	*z = [self zone];
-	  NSData	*d;
 
-	  *(char**)data = (char*)NSZoneMalloc(z, len);
-	  d = [dataMalloc allocWithZone: z];
-	  d = [d initWithBytesNoCopy: *(void**)data
-			      length: len
-			    fromZone: z];
-	  IF_NO_GC(AUTORELEASE(d));
+#if	GS_WITH_GC == 0
+	  *(char**)data = (char*)NSZoneMalloc(NSDefaultMallocZone(), len);
+#else
+	  *(char**)data = (char*)NSZoneMalloc(NSAtomicMallocZone(), len);
+#endif
 	  [self deserializeDataAt: *(char**)data
 		       ofObjCType: type
 			 atCursor: cursor
@@ -1028,13 +1150,18 @@ failure:
 
 - (void) encodeWithCoder: (NSCoder*)coder
 {
-  [self subclassResponsibility: _cmd];
+  [coder encodeDataObject: self];
 }
 
 - (id) initWithCoder: (NSCoder*)coder
 {
-  [self subclassResponsibility: _cmd];
-  return nil;
+  id	obj = [coder decodeDataObject];
+
+  if (obj != self)
+    {
+      ASSIGN(self, obj);
+    }
+  return self;
 }
 
 @end
@@ -1151,9 +1278,16 @@ failure:
 
 
 @implementation NSMutableData
-+ (NSData*) allocWithZone: (NSZone*)z
++ (id) allocWithZone: (NSZone*)z
 {
-  return (NSData*)NSAllocateObject(mutableDataMalloc, 0, z);
+  if (self == NSMutableDataAbstract)
+    {
+      return NSAllocateObject(mutableDataMalloc, 0, z);
+    }
+  else
+    {
+      return NSAllocateObject(self, 0, z);
+    }
 }
 
 + (id) data
@@ -1230,17 +1364,72 @@ failure:
   return AUTORELEASE(d);
 }
 
++ (id) new
+{
+  NSMutableData	*d;
+
+  d = [mutableDataMalloc allocWithZone: NSDefaultMallocZone()];
+  d = [d initWithCapacity: 0];
+  return d;
+}
+
 - (const void*) bytes
 {
   return [self mutableBytes];
 }
 
+- (void) encodeWithCoder: (NSCoder*)aCoder
+{
+  unsigned	length = [self length];
+  void		*bytes = [self mutableBytes];
+
+  [aCoder encodeValueOfObjCType: @encode(unsigned long)
+			     at: &length];
+  if (length)
+    {
+      [aCoder encodeArrayOfObjCType: @encode(unsigned char)
+			      count: length
+				 at: bytes];
+    }
+}
+
 - (id) initWithCapacity: (unsigned)capacity
 {
   [self subclassResponsibility: _cmd];
   return nil;
 }
 
+- (id) initWithCoder: (NSCoder*)aCoder
+{
+  unsigned	l;
+  void		*b;
+  NSZone	*zone;
+
+#if	GS_WITH_GC
+  zone = GSAtomicMallocZone();
+#else
+  zone = [self zone];
+#endif
+
+  [aCoder decodeValueOfObjCType: @encode(unsigned long) at: &l];
+  if (l)
+    {
+      b = NSZoneMalloc(zone, l);
+      if (b == 0)
+	{
+	  NSLog(@"[NSDataMalloc -initWithCoder:] unable to get %lu bytes", l);
+	  RELEASE(self);
+	  return nil;
+        }
+      [aCoder decodeArrayOfObjCType: @encode(unsigned char) count: l at: b];
+    }
+  else
+    {
+      b = 0;
+    }
+  return [self initWithBytesNoCopy: b length: l fromZone: zone];
+}
+
 - (id) initWithLength: (unsigned)length
 {
   [self subclassResponsibility: _cmd];
@@ -1280,8 +1469,7 @@ failure:
 
 - (void) appendData: (NSData*)other
 {
-  [self appendBytes: [other bytes]
-	     length: [other length]];
+  [self appendBytes: [other bytes] length: [other length]];
 }
 
 
@@ -1314,14 +1502,14 @@ failure:
 
 // Serializing Data
 
-- (void)serializeAlignedBytesLength: (unsigned)length
+- (void) serializeAlignedBytesLength: (unsigned)length
 {
   [self serializeInt: length];
 }
 
-- (void)serializeDataAt: (const void*)data
-	     ofObjCType: (const char*)type
-	        context: (id <NSObjCTypeSerializationCallBack>)callback
+- (void) serializeDataAt: (const void*)data
+	      ofObjCType: (const char*)type
+		 context: (id <NSObjCTypeSerializationCallBack>)callback
 {
   if (!data || !type)
     return;
@@ -1637,7 +1825,7 @@ failure:
  */
 @implementation	NSDataStatic
 
-+ (NSData*) allocWithZone: (NSZone*)z
++ (id) allocWithZone: (NSZone*)z
 {
   return (NSData*)NSAllocateObject(self, 0, z);
 }
@@ -1657,13 +1845,13 @@ failure:
 - (id) mutableCopy
 {
   return [[mutableDataMalloc allocWithZone: NSDefaultMallocZone()]
-	initWithBytes: bytes length: length];
+    initWithBytes: bytes length: length];
 }
 
 - (id) mutableCopyWithZone: (NSZone*)z
 {
   return [[mutableDataMalloc allocWithZone: z]
-	initWithBytes: bytes length: length];
+    initWithBytes: bytes length: length];
 }
 
 - (void) dealloc
@@ -1706,18 +1894,6 @@ failure:
   return dataMalloc;		/* Will not be static data when decoded. */
 }
 
-- (void) encodeWithCoder: (NSCoder*)aCoder
-{
-  [aCoder encodeValueOfObjCType: @encode(unsigned long)
-			     at: &length];
-  if (length)
-    {
-      [aCoder encodeArrayOfObjCType: @encode(unsigned char)
-			      count: length
-				 at: bytes];
-    }
-}
-
 /* Basic methods	*/
 
 - (const void*) bytes
@@ -1793,14 +1969,10 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 	    }
 	  else
 	    {
-	      NSZone	*z = [self zone];
-
-	      *(char**)data = (char*)NSZoneMalloc(z, len+1);
-#if !GS_WITH_GC
-	      [[[dataMalloc allocWithZone: z]
-			   initWithBytesNoCopy: *(void**)data
-					length: len+1
-				      fromZone: z] autorelease];
+#if	GS_WITH_GC == 0
+	      *(char**)data = (char*)NSZoneMalloc(NSDefaultMallocZone(), len+1);
+#else
+	      *(char**)data = (char*)NSZoneMalloc(NSAtomicMallocZone(), len+1);
 #endif
 	    }
 	  getBytes(*(void**)data, bytes, len, length, cursor);
@@ -1860,14 +2032,11 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
       case _C_PTR:
 	{
 	  unsigned	len = objc_sizeof_type(++type);
-	  NSZone	*z = [self zone];
 
-	  *(char**)data = (char*)NSZoneMalloc(z, len);
-#if !GS_WITH_GC
-	  [[[dataMalloc allocWithZone: z]
-			 initWithBytesNoCopy: *(void**)data
-				      length: len
-				    fromZone: z] autorelease];
+#if	GS_WITH_GC == 0
+	  *(char**)data = (char*)NSZoneMalloc(NSDefaultMallocZone(), len);
+#else
+	  *(char**)data = (char*)NSZoneMalloc(NSAtomicMallocZone(), len);
 #endif
 	  [self deserializeDataAt: *(char**)data
 		       ofObjCType: type
@@ -2047,6 +2216,11 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 			      format: @"Range: (%u, 1) Size: %d",
 			  *cursor, length];
 		}
+#if NEED_WORD_ALIGNMENT
+	      if ((*cursor % __alignof__(gsu16)) != 0)
+		memcpy(&x, (bytes + *cursor), 2);
+	      else
+#endif
 	      x = *(gsu16*)(bytes + *cursor);
 	      *cursor += 2;
 	      *ref = (unsigned int)GSSwapBigI16ToHost(x);
@@ -2062,6 +2236,11 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 			      format: @"Range: (%u, 1) Size: %d",
 			  *cursor, length];
 		}
+#if NEED_WORD_ALIGNMENT
+	      if ((*cursor % __alignof__(gsu32)) != 0)
+		memcpy(&x, (bytes + *cursor), 4);
+	      else
+#endif
 	      x = *(gsu32*)(bytes + *cursor);
 	      *cursor += 4;
 	      *ref = (unsigned int)GSSwapBigI32ToHost(x);
@@ -2170,36 +2349,6 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   return self;
 }
 
-- (id) initWithCoder: (NSCoder*)aCoder
-{
-  unsigned	l;
-  void*		b;
-
-#if	GS_WITH_GC
-  zone = GSAtomicMallocZone();
-#else
-  zone = [self zone];
-#endif
-
-  [aCoder decodeValueOfObjCType: @encode(unsigned long) at: &l];
-  if (l)
-    {
-      b = NSZoneMalloc(zone, l);
-      if (b == 0)
-	{
-	  NSLog(@"[NSDataMalloc -initWithCoder:] unable to get %lu bytes", l);
-	  RELEASE(self);
-	  return nil;
-        }
-      [aCoder decodeArrayOfObjCType: @encode(unsigned char) count: l at: b];
-    }
-  else
-    {
-      b = 0;
-    }
-  return [self initWithBytesNoCopy: b length: l fromZone: zone];
-}
-
 - (id) initWithContentsOfFile: (NSString *)path
 {
 #if	GS_WITH_GC
@@ -2260,7 +2409,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
 #if	HAVE_MMAP
 @implementation	NSDataMappedFile
-+ (NSData*) allocWithZone: (NSZone*)z
++ (id) allocWithZone: (NSZone*)z
 {
   return (NSData*)NSAllocateObject([NSDataMappedFile class], 0, z);
 }
@@ -2332,7 +2481,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
 #if	HAVE_SHMCTL
 @implementation	NSDataShared
-+ (NSData*) allocWithZone: (NSZone*)z
++ (id) allocWithZone: (NSZone*)z
 {
   return (NSData*)NSAllocateObject([NSDataShared class], 0, z);
 }
@@ -2443,7 +2592,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
     }
 }
 
-+ (NSData*) allocWithZone: (NSZone*)z
++ (id) allocWithZone: (NSZone*)z
 {
   return (NSData*)NSAllocateObject(mutableDataMalloc, 0, z);
 }
@@ -2566,28 +2715,6 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   return self;
 }
 
-- (id) initWithCoder: (NSCoder*)aCoder
-{
-  unsigned	l;
-
-  [aCoder decodeValueOfObjCType: @encode(unsigned long) at: &l];
-  if (l)
-    {
-      [self initWithCapacity: l];
-      if (bytes == 0)
-	{
-	  NSLog(@"[NSMutableDataMalloc -initWithCoder:] unable to allocate %lu bytes", l);
-	  RELEASE(self);
-	  return nil;
-	}
-      [aCoder decodeArrayOfObjCType: @encode(unsigned char)
-			      count: l
-				 at: bytes];
-      length = l;
-    }
-  return self;
-}
-
 - (id) initWithLength: (unsigned)size
 {
   self = [self initWithCapacity: size];
@@ -2950,6 +3077,14 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 	  [self _grow: length + 3];
 	}
       *(gsu8*)(bytes + length++) = tag;
+#if NEED_WORD_ALIGNMENT
+      if ((length % __alignof__(gsu16)) != 0)
+	{
+	  x = GSSwapHostI16ToBig(x);
+	  memcpy((bytes + length), &x, 2);
+	}
+      else
+#endif
       *(gsu16*)(bytes + length) = GSSwapHostI16ToBig(x);
       length += 2;
     }
@@ -2963,6 +3098,14 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 	  [self _grow: length + 5];
 	}
       *(gsu8*)(bytes + length++) = tag;
+#if NEED_WORD_ALIGNMENT
+      if ((length % __alignof__(gsu32)) != 0)
+	{
+	  x = GSSwapHostI32ToBig(x);
+	  memcpy((bytes + length), &x, 4);
+	}
+      else
+#endif
       *(gsu32*)(bytes + length) = GSSwapHostI32ToBig(x);
       length += 4;
     }
@@ -3018,7 +3161,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
 #if	HAVE_SHMCTL
 @implementation	NSMutableDataShared
-+ (NSData*) allocWithZone: (NSZone*)z
++ (id) allocWithZone: (NSZone*)z
 {
   return (NSData*)NSAllocateObject([NSMutableDataShared class], 0, z);
 }
