@@ -36,14 +36,12 @@
 #include <base/preface.h>
 #include <base/behavior.h>
 #include <base/fast.x>
-#ifndef __WIN32__
+#if HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 #include <time.h>
-#endif /* !__WIN32__ */
 #include <stdio.h>
 #include <stdlib.h>
-#ifndef __WIN32__
-#include <sys/time.h>
-#endif /* !__WIN32__ */
 
 /* The number of seconds between 1/1/2001 and 1/1/1970 = -978307200. */
 /* This number comes from:
@@ -102,7 +100,7 @@ otherTime(NSDate* other)
   Class	c = fastClass(other);
 
   if (c == concreteClass || c == calendarClass)
-    return ((NSGDate*)other)->seconds_since_ref;
+    return ((NSGDate*)other)->_seconds_since_ref;
   else
     return [other timeIntervalSinceReferenceDate];
 }
@@ -110,7 +108,7 @@ otherTime(NSDate* other)
 NSTimeInterval
 GSTimeNow()
 {
-#if !defined(__WIN32__)
+#if !defined(__MINGW__)
   volatile NSTimeInterval interval;
   struct timeval tp;
 
@@ -118,15 +116,12 @@ GSTimeNow()
   gettimeofday (&tp, NULL);
   interval += tp.tv_sec;
   interval += (double)tp.tv_usec / 1000000.0;
-
-  /* There seems to be a problem with bad double arithmetic... */
-  NSCAssert(interval < 0, NSInternalInconsistencyException);
-
   return interval;
 #else
   SYSTEMTIME sys_time;
-  NSCalendarDate *d;
   NSTimeInterval t;
+#if 0
+  NSCalendarDate *d;
 
   // Get the system time
   GetLocalTime(&sys_time);
@@ -142,8 +137,16 @@ GSTimeNow()
      timeZone: [NSTimeZone localTimeZone]];
   t = otherTime(d);
   RELEASE(d);
+#else
+  /*
+   * Get current GMT time, convert to NSTimeInterval since reference date,
+   */
+  GetSystemTime(&sys_time);
+  t = GSTime(sys_time.eDay, sys_time.wMonth, sys_time.wYear,
+    sys_time.wHour, sys_time.wMinute, sys_time.wSecond); 
+#endif
   return t + sys_time.wMilliseconds / 1000.0;
-#endif /* __WIN32__ */
+#endif /* __MINGW__ */
 }
 
 /* The implementation of NSDate. */
@@ -634,10 +637,10 @@ GSTimeNow()
 		    break;
 
 		  default:
-		    dtoIndex++;
 		    if (debug)
 		      NSLog(@"odd char (unicode %d) in NSDateTimeOrdering.\n",
-			    [dto characterAtIndex: index]);
+			    [dto characterAtIndex: dtoIndex]);
+		    dtoIndex++;
 		    break;
 		}
 	    }
@@ -763,7 +766,7 @@ GSTimeNow()
 	      default:
 		if (debug)
 		  NSLog(@"unexpected char (unicode%d) in NSDateTimeOrdering.\n",
-		    [dto characterAtIndex: index]);
+		    [dto characterAtIndex: dtoIndex]);
 		break;
 	    }
 	}
@@ -867,19 +870,14 @@ GSTimeNow()
     return NSCopyObject(self, 0, zone);
 }
 
-- (Class) classForConnectedCoder: aRmc
+- (Class) classForPortCoder
 {
   /* Make sure that Connection's always send us bycopy,
      i.e. as our own class, not a Proxy class. */
   return abstractClass;
 }
 
-- (Class) classForPortCoder
-{
-  return abstractClass;
-}
-
-- replacementObjectForPortCoder: aRmc
+- (id) replacementObjectForPortCoder: (NSPortCoder*)aRmc
 {
   return self;
 }
@@ -912,9 +910,8 @@ GSTimeNow()
   // Easiest to just have NSCalendarDate do the work for us
   NSCalendarDate	*d = [calendarClass alloc];
 
-  [d initWithString: description];
-  [self initWithTimeIntervalSinceReferenceDate:
-	otherTime(d)];
+  d = [d initWithString: description];
+  self = [self initWithTimeIntervalSinceReferenceDate: otherTime(d)];
   RELEASE(d);
   return self;
 }
@@ -931,13 +928,13 @@ GSTimeNow()
 {
   // Get the current time, add the secs and init thyself
   return [self initWithTimeIntervalSinceReferenceDate:
-	       GSTimeNow() + secsToBeAdded];
+    GSTimeNow() + secsToBeAdded];
 }
 
 - (id)initWithTimeIntervalSince1970: (NSTimeInterval)seconds
 {
   return [self initWithTimeIntervalSinceReferenceDate:
-		       UNIX_REFERENCE_INTERVAL + seconds];
+    UNIX_REFERENCE_INTERVAL + seconds];
 }
 
 - (id) initWithTimeIntervalSinceReferenceDate: (NSTimeInterval)secs
@@ -1020,6 +1017,11 @@ GSTimeNow()
 
 - (NSTimeInterval) timeIntervalSinceDate: (NSDate*)otherDate
 {
+  if (otherDate == nil)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"nil argument for timeIntervalSinceDate:"];
+    }
   return otherTime(self) - otherTime(otherDate);
 }
 
@@ -1038,17 +1040,33 @@ GSTimeNow()
 
 - (NSComparisonResult) compare: (NSDate*)otherDate
 {
+  if (otherDate == self)
+    {
+      return NSOrderedSame;
+    }
+  if (otherDate == nil)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"nil argument for compare:"];
+    }
   if (otherTime(self) > otherTime(otherDate))
-    return NSOrderedDescending;
-
+    {
+      return NSOrderedDescending;
+    }
   if (otherTime(self) < otherTime(otherDate))
-    return NSOrderedAscending;
-
+    {
+      return NSOrderedAscending;
+    }
   return NSOrderedSame;
 }
 
 - (NSDate*) earlierDate: (NSDate*)otherDate
 {
+  if (otherDate == nil)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"nil argument for earlierDate:"];
+    }
   if (otherTime(self) > otherTime(otherDate))
     return otherDate;
   return self;
@@ -1056,6 +1074,8 @@ GSTimeNow()
 
 - (BOOL) isEqual: (id)other
 {
+  if (other == nil)
+    return NO;
   if ([other isKindOf: abstractClass]
       && 1.0 > ABS(otherTime(self) - otherTime(other)))
     return YES;
@@ -1064,6 +1084,8 @@ GSTimeNow()
 
 - (BOOL) isEqualToDate: (NSDate*)other
 {
+  if (other == nil)
+    return NO;
   if (1.0 > ABS(otherTime(self) - otherTime(other)))
     return YES;
   return NO;
@@ -1071,6 +1093,11 @@ GSTimeNow()
 
 - (NSDate*) laterDate: (NSDate*)otherDate
 {
+  if (otherDate == nil)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"nil argument for laterDate:"];
+    }
   if (otherTime(self)
     < otherTime(otherDate))
     return otherDate;
@@ -1094,96 +1121,121 @@ GSTimeNow()
   return [self class];
 }
 
-- replacementObjectForPortCoder: aRmc
+- (id) replacementObjectForPortCoder: (NSPortCoder*)aRmc
 {
   return self;
 }
 
 - (void) encodeWithCoder: (NSCoder*)coder
 {
-  [coder encodeValueOfObjCType: @encode(NSTimeInterval) at: &seconds_since_ref];
+  [coder encodeValueOfObjCType: @encode(NSTimeInterval)
+			    at: &_seconds_since_ref];
 }
 
 - (id) initWithCoder: (NSCoder*)coder
 {
-  [coder decodeValueOfObjCType: @encode(NSTimeInterval) at: &seconds_since_ref];
+  [coder decodeValueOfObjCType: @encode(NSTimeInterval)
+			    at: &_seconds_since_ref];
   return self;
 }
 
 - (id) initWithTimeIntervalSinceReferenceDate: (NSTimeInterval)secs
 {
-  seconds_since_ref = secs;
+  _seconds_since_ref = secs;
   return self;
 }
 
 
 // Adding and getting intervals
 
-- (id) addTimeInterval: (NSTimeInterval)seconds
-{
-  /* xxx We need to check for overflow? */
-  return [concreteClass dateWithTimeIntervalSinceReferenceDate:
-		       seconds_since_ref + seconds];
-}
-
 - (NSTimeInterval) timeIntervalSince1970
 {
-  return seconds_since_ref - UNIX_REFERENCE_INTERVAL;
+  return _seconds_since_ref - UNIX_REFERENCE_INTERVAL;
 }
 
 - (NSTimeInterval) timeIntervalSinceDate: (NSDate*)otherDate
 {
-  return seconds_since_ref - otherTime(otherDate);
+  if (otherDate == nil)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"nil argument for timeIntervalSinceDate:"];
+    }
+  return _seconds_since_ref - otherTime(otherDate);
 }
 
 - (NSTimeInterval) timeIntervalSinceNow
 {
-  return seconds_since_ref - GSTimeNow();
+  return _seconds_since_ref - GSTimeNow();
 }
 
 - (NSTimeInterval) timeIntervalSinceReferenceDate
 {
-  return seconds_since_ref;
+  return _seconds_since_ref;
 }
 
 // Comparing dates
 
 - (NSComparisonResult) compare: (NSDate*)otherDate
 {
-  if (seconds_since_ref > otherTime(otherDate))
-    return NSOrderedDescending;
-
-  if (seconds_since_ref < otherTime(otherDate))
-    return NSOrderedAscending;
-
+  if (otherDate == self)
+    {
+      return NSOrderedSame;
+    }
+  if (otherDate == nil)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"nil argument for compare:"];
+    }
+  if (_seconds_since_ref > otherTime(otherDate))
+    {
+      return NSOrderedDescending;
+    }
+  if (_seconds_since_ref < otherTime(otherDate))
+    {
+      return NSOrderedAscending;
+    }
   return NSOrderedSame;
 }
 
 - (NSDate*) earlierDate: (NSDate*)otherDate
 {
-  if (seconds_since_ref > otherTime(otherDate))
+  if (otherDate == nil)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"nil argument for earlierDate:"];
+    }
+  if (_seconds_since_ref > otherTime(otherDate))
     return otherDate;
   return self;
 }
 
 - (BOOL) isEqual: (id)other
 {
+  if (other == nil)
+    return NO;
   if ([other isKindOfClass: abstractClass]
-      && 1.0 > ABS(seconds_since_ref - otherTime(other)))
+      && 1.0 > ABS(_seconds_since_ref - otherTime(other)))
     return YES;
   return NO;
 }
 
 - (BOOL) isEqualToDate: (NSDate*)other
 {
-  if (1.0 > ABS(seconds_since_ref - otherTime(other)))
+  if (other == nil)
+    return NO;
+  if (1.0 > ABS(_seconds_since_ref - otherTime(other)))
     return YES;
   return NO;
 }
 
 - (NSDate*) laterDate: (NSDate*)otherDate
 {
-  if (seconds_since_ref < otherTime(otherDate))
+  if (otherDate == nil)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"nil argument for laterDate:"];
+    }
+  if (_seconds_since_ref < otherTime(otherDate))
     return otherDate;
   return self;
 }
@@ -1212,7 +1264,7 @@ GSTimeNow()
   return [self class];
 }
 
-- replacementObjectForPortCoder: aRmc
+- (id) replacementObjectForPortCoder: (NSPortCoder*)aRmc
 {
   return self;
 }
@@ -1282,7 +1334,7 @@ GSTimeNow()
 
 - (id) initWithTimeIntervalSinceReferenceDate: (NSTimeInterval)secs
 {
-  seconds_since_ref = DISTANT_PAST;
+  _seconds_since_ref = DISTANT_PAST;
   return self;
 }
 
@@ -1304,7 +1356,7 @@ GSTimeNow()
 
 - (id) initWithTimeIntervalSinceReferenceDate: (NSTimeInterval)secs
 {
-  seconds_since_ref = DISTANT_FUTURE;
+  _seconds_since_ref = DISTANT_FUTURE;
   return self;
 }
 
