@@ -32,6 +32,7 @@
 #include <Foundation/NSException.h>
 #include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSFileManager.h>
+#include <Foundation/NSUserDefaults.h>
 #include <Foundation/NSDebug.h>
 
 @interface NSDictionaryNonCore : NSDictionary
@@ -58,44 +59,29 @@ static SEL	remSel = @selector(removeObjectForKey:);
 static SEL	setSel = @selector(setObject:forKey:);
 static SEL	appSel = @selector(appendString:);
 
-+ (void) _setConcreteClass: (Class)c
-{
-  NSDictionary_concrete_class = c;
-}
-
-+ (void) _setMutableConcreteClass: (Class)c
-{
-  NSMutableDictionary_concrete_class = c;
-}
-
-+ (Class) _concreteClass
-{
-  return NSDictionary_concrete_class;
-}
-
-+ (Class) _mutableConcreteClass
-{
-  return NSMutableDictionary_concrete_class;
-}
-
 + (void) initialize
 {
   if (self == [NSDictionary class])
     {
+      behavior_class_add_class (self, [NSDictionaryNonCore class]);
       NSArray_class = [NSArray class];
       NSDictionary_abstract_class = [NSDictionary class];
       NSMutableDictionary_abstract_class = [NSMutableDictionary class];
       NSDictionary_concrete_class = [NSGDictionary class];
       NSMutableDictionary_concrete_class = [NSGMutableDictionary class];
-      behavior_class_add_class (self, [NSDictionaryNonCore class]);
     }
 }
 
 + (id) allocWithZone: (NSZone*)z
 {
   if (self == NSDictionary_abstract_class)
-    return NSAllocateObject(NSDictionary_concrete_class, 0, z);
-  return [super allocWithZone: z];
+    {
+      return NSAllocateObject(NSDictionary_concrete_class, 0, z);
+    }
+  else
+    {
+      return NSAllocateObject(self, 0, z);
+    }
 }
 
 /* This is the designated initializer */
@@ -328,43 +314,47 @@ static SEL	appSel = @selector(appendString:);
 - (id) initWithDictionary: (NSDictionary*)other copyItems: (BOOL)shouldCopy
 {
   unsigned	c = [other count];
-  id		os[c];
-  id		ks[c];
-  id		k;
-  NSEnumerator	*e = [other keyEnumerator];
-  unsigned	i = 0;
-  IMP		nxtObj = [e methodForSelector: nxtSel];
-  IMP		otherObj = [other methodForSelector: objSel];
 
-  if (shouldCopy)
+  if (c > 0)
     {
-      NSZone	*z = [self zone];
+      id		os[c];
+      id		ks[c];
+      id		k;
+      NSEnumerator	*e = [other keyEnumerator];
+      unsigned		i = 0;
+      IMP		nxtObj = [e methodForSelector: nxtSel];
+      IMP		otherObj = [other methodForSelector: objSel];
 
-      while ((k = (*nxtObj)(e, nxtSel)) != nil)
+      if (shouldCopy)
 	{
-	  ks[i] = k;
-	  os[i] = [(*otherObj)(other, objSel, k) copyWithZone: z];
-	  i++;
-	}
-      self = [self initWithObjects: os forKeys: ks count: i];
+	  NSZone	*z = [self zone];
+
+	  while ((k = (*nxtObj)(e, nxtSel)) != nil)
+	    {
+	      ks[i] = k;
+	      os[i] = [(*otherObj)(other, objSel, k) copyWithZone: z];
+	      i++;
+	    }
+	  self = [self initWithObjects: os forKeys: ks count: i];
 #if	!GS_WITH_GC
-      while (i > 0)
-	{
-	  [os[--i] release];
-	}
+	  while (i > 0)
+	    {
+	      [os[--i] release];
+	    }
 #endif
-      return self;
-    }
-  else
-    {
-      while ((k = (*nxtObj)(e, nxtSel)) != nil)
-	{
-	  ks[i] = k;
-	  os[i] = (*otherObj)(other, objSel, k);
-	  i++;
 	}
-      return [self initWithObjects: os forKeys: ks count: c];
+      else
+	{
+	  while ((k = (*nxtObj)(e, nxtSel)) != nil)
+	    {
+	      ks[i] = k;
+	      os[i] = (*otherObj)(other, objSel, k);
+	      i++;
+	    }
+	  self = [self initWithObjects: os forKeys: ks count: c];
+	}
     }
+  return self;
 }
 
 - (id) initWithContentsOfFile: (NSString*)path
@@ -417,24 +407,31 @@ static SEL	appSel = @selector(appendString:);
 
 - (BOOL) isEqualToDictionary: (NSDictionary*)other
 {
+  unsigned	count;
+
   if (other == self)
-    return YES;
-
-  if ([self count] == [other count])
     {
-      NSEnumerator	*e = [self keyEnumerator];
-      IMP		nxtObj = [e methodForSelector: nxtSel];
-      IMP		myObj = [self methodForSelector: objSel];
-      IMP		otherObj = [other methodForSelector: objSel];
-      id		k;
-
-      while ((k = (*nxtObj)(e, @selector(nextObject))) != nil)
+      return YES;
+    }
+  count = [self count];
+  if (count == [other count])
+    {
+      if (count > 0)
 	{
-	  id o1 = (*myObj)(self, objSel, k);
-	  id o2 = (*otherObj)(other, objSel, k);
+	  NSEnumerator	*e = [self keyEnumerator];
+	  IMP		nxtObj = [e methodForSelector: nxtSel];
+	  IMP		myObj = [self methodForSelector: objSel];
+	  IMP		otherObj = [other methodForSelector: objSel];
+	  id		k;
 
-	  if ([o1 isEqual: o2] == NO)
-	    return NO;
+	  while ((k = (*nxtObj)(e, @selector(nextObject))) != nil)
+	    {
+	      id o1 = (*myObj)(self, objSel, k);
+	      id o2 = (*otherObj)(other, objSel, k);
+
+	      if ([o1 isEqual: o2] == NO)
+		return NO;
+	    }
 	}
       return YES;
     }
@@ -577,12 +574,18 @@ compareIt(id o1, id o2, void* context)
 
 - (BOOL) writeToFile: (NSString *)path atomically: (BOOL)useAuxiliaryFile
 {
-  return [[self description] writeToFile: path atomically: useAuxiliaryFile];
+  NSDictionary	*loc;
+  NSString	*desc;
+
+  loc = [[NSUserDefaults standardUserDefaults] dictionaryRepresentation];
+  desc = [self descriptionWithLocale: loc indent: 0];
+
+  return [desc writeToFile: path atomically: useAuxiliaryFile];
 }
 
 - (NSString*) description
 {
-  return [self descriptionWithLocale: nil];
+  return [self descriptionWithLocale: nil indent: 0];
 }
 
 - (NSString*) descriptionInStringsFileFormat
@@ -601,11 +604,13 @@ compareIt(id o1, id o2, void* context)
       id val = (*myObj)(self, objSel, key);
 
       [key descriptionWithLocale: nil
+			  indent: 0
                               to: (id<GNUDescriptionDestination>)result];
       if (val != nil && [val isEqualToString: @""] == NO)
         {
 	  (*appImp)(result, appSel, @" = ");
           [val descriptionWithLocale: nil
+			      indent: 0
                                   to: (id<GNUDescriptionDestination>)result];
         }
       (*appImp)(result, appSel, @";\n");
@@ -653,161 +658,178 @@ static NSString	*indentStrings[] = {
 			    to: (id<GNUDescriptionDestination>)result
 {
   IMP			myObj = [self methodForSelector: objSel];
-  BOOL			canCompare = YES;
-  NSString		*iBaseString;
-  NSString		*iSizeString;
   unsigned		i;
   NSArray		*keyArray = [self allKeys];
   unsigned		numKeys = [keyArray count];
   NSString		*plists[numKeys];
   NSString		*keys[numKeys];
   IMP			appImp;
-  Class			lastClass = 0;
 
   appImp = [(NSObject*)result methodForSelector: appSel];
 
-  if (level < sizeof(indentStrings)/sizeof(NSString*))
-    iBaseString = indentStrings[level];
-  else
-    iBaseString = indentStrings[sizeof(indentStrings)/sizeof(NSString*)-1];
-  level++;
-  if (level < sizeof(indentStrings)/sizeof(NSString*))
-    iSizeString = indentStrings[level];
-  else
-    iSizeString = indentStrings[sizeof(indentStrings)/sizeof(NSString*)-1];
-
   [keyArray getObjects: keys];
-  for (i = 0; i < numKeys; i++)
+
+  if (locale == nil)
     {
-      if (fastClass(keys[i]) == lastClass)
-	continue;
-      if ([keys[i] respondsToSelector: @selector(compare:)] == NO)
+      for (i = 0; i < numKeys; i++)
 	{
-	  canCompare = NO;
-	  break;
+	  plists[i] = (*myObj)(self, objSel, keys[i]);
 	}
-      lastClass = fastClass(keys[i]);
+
+      (*appImp)(result, appSel, @"{");
+      for (i = 0; i < numKeys; i++)
+	{
+	  id	o = plists[i];
+
+	  [keys[i] descriptionWithLocale: nil indent: 0 to: result];
+	  (*appImp)(result, appSel, @" = ");
+	  [o descriptionWithLocale: nil indent: 0 to: result];
+	  (*appImp)(result, appSel, @"; ");
+	}
+      (*appImp)(result, appSel, @"}");
     }
-
-  if (canCompare)
+  else
     {
-/*
- * Shell sort algorithm taken from SortingInAction - a NeXT example
- * good value for stride factor is not well-understood
- * 3 is a fairly good choice (Sedgewick)
- */
-#define STRIDE_FACTOR 3
-      unsigned	c,d, stride;
-      BOOL	found;
-      NSComparisonResult	(*comp)(id, SEL, id);
-      int	count = numKeys;
-#ifdef	GSWARN
-      BOOL	badComparison = NO;
-#endif
+      NSString	*iBaseString;
+      NSString	*iSizeString;
+      BOOL	canCompare = YES;
+      Class	lastClass = 0;
 
-      stride = 1;
-      while (stride <= count)
+      if (level < sizeof(indentStrings)/sizeof(id))
 	{
-	  stride = stride * STRIDE_FACTOR + 1;
-	}
-      lastClass = 0;
-      while (stride > (STRIDE_FACTOR - 1))
-	{
-	  // loop to sort for each value of stride
-	  stride = stride / STRIDE_FACTOR;
-	  for (c = stride; c < count; c++)
-	    {
-	      found = NO;
-	      if (stride > c)
-		{
-		  break;
-		}
-	      d = c - stride;
-	      while (!found)	// move to left until correct place
-		{
-		  id			a = keys[d + stride];
-		  id			b = keys[d];
-		  Class			x;
-		  NSComparisonResult	r;
-
-		  x = fastClass(a);
-		  if (x != lastClass)
-		    {
-		      lastClass = x;
-		      comp = (NSComparisonResult (*)(id, SEL, id))
-			[a methodForSelector: @selector(compare:)];
-		    }
-		  r = (*comp)(a, @selector(compare:), b);
-		  if (r < 0)
-		    {
-#ifdef	GSWARN
-		      if (r != NSOrderedAscending)
-			{
-			  badComparison = YES;
-			}
-#endif
-		      keys[d + stride] = b;
-		      keys[d] = a;
-		      if (stride > d)
-			{
-			  break;
-			}
-		      d -= stride;		// jump by stride factor
-		    }
-		  else
-		    {
-#ifdef	GSWARN
-		      if (r != NSOrderedDescending && r != NSOrderedSame)
-			{
-			  badComparison = YES;
-			}
-#endif
-		      found = YES;
-		    }
-		}
-	    }
-	}
-#ifdef	GSWARN
-      if (badComparison == YES)
-	{
-	  NSWarnMLog(@"Detected bad return value from comparison", 0);
-	}
-#endif
-    }
-
-  for (i = 0; i < numKeys; i++)
-    {
-      plists[i] = (*myObj)(self, objSel, keys[i]);
-    }
-
-  (*appImp)(result, appSel, @"{\n");
-  for (i = 0; i < numKeys; i++)
-    {
-      id	item = plists[i];
-
-      (*appImp)(result, appSel, iSizeString);
-
-      [keys[i] descriptionTo: result];
-
-      (*appImp)(result, appSel, @" = ");
-
-      if ([item respondsToSelector: @selector(descriptionWithLocale:indent:)])
-	{
-	  [item descriptionWithLocale: locale indent: level to: result];
-	}
-      else if ([item respondsToSelector: @selector(descriptionWithLocale:)])
-	{
-	  [item descriptionWithLocale: locale to: result];
+	  iBaseString = indentStrings[level];
 	}
       else
 	{
-	  [item descriptionTo: result];
+	  iBaseString = indentStrings[sizeof(indentStrings)/sizeof(id)-1];
+	}
+      level++;
+      if (level < sizeof(indentStrings)/sizeof(id))
+	{
+	  iSizeString = indentStrings[level];
+	}
+      else
+	{
+	  iSizeString = indentStrings[sizeof(indentStrings)/sizeof(id)-1];
 	}
 
-      (*appImp)(result, appSel, @";\n");
+      for (i = 0; i < numKeys; i++)
+	{
+	  if (fastClass(keys[i]) == lastClass)
+	    continue;
+	  if ([keys[i] respondsToSelector: @selector(compare:)] == NO)
+	    {
+	      canCompare = NO;
+	      break;
+	    }
+	  lastClass = fastClass(keys[i]);
+	}
+
+      if (canCompare == YES)
+	{
+	  /*
+	   * Shell sort algorithm taken from SortingInAction - a NeXT example
+	   * good value for stride factor is not well-understood
+	   * 3 is a fairly good choice (Sedgewick)
+	   */
+#define STRIDE_FACTOR 3
+	  unsigned	c,d, stride;
+	  BOOL		found;
+	  NSComparisonResult	(*comp)(id, SEL, id);
+	  int		count = numKeys;
+#ifdef	GSWARN
+	  BOOL		badComparison = NO;
+#endif
+
+	  stride = 1;
+	  while (stride <= count)
+	    {
+	      stride = stride * STRIDE_FACTOR + 1;
+	    }
+	  lastClass = 0;
+	  while (stride > (STRIDE_FACTOR - 1))
+	    {
+	      // loop to sort for each value of stride
+	      stride = stride / STRIDE_FACTOR;
+	      for (c = stride; c < count; c++)
+		{
+		  found = NO;
+		  if (stride > c)
+		    {
+		      break;
+		    }
+		  d = c - stride;
+		  while (!found)	// move to left until correct place
+		    {
+		      id			a = keys[d + stride];
+		      id			b = keys[d];
+		      Class			x;
+		      NSComparisonResult	r;
+
+		      x = fastClass(a);
+		      if (x != lastClass)
+			{
+			  lastClass = x;
+			  comp = (NSComparisonResult (*)(id, SEL, id))
+			    [a methodForSelector: @selector(compare:)];
+			}
+		      r = (*comp)(a, @selector(compare:), b);
+		      if (r < 0)
+			{
+#ifdef	GSWARN
+			  if (r != NSOrderedAscending)
+			    {
+			      badComparison = YES;
+			    }
+#endif
+			  keys[d + stride] = b;
+			  keys[d] = a;
+			  if (stride > d)
+			    {
+			      break;
+			    }
+			  d -= stride;		// jump by stride factor
+			}
+		      else
+			{
+#ifdef	GSWARN
+			  if (r != NSOrderedDescending && r != NSOrderedSame)
+			    {
+			      badComparison = YES;
+			    }
+#endif
+			  found = YES;
+			}
+		    }
+		}
+	    }
+#ifdef	GSWARN
+	  if (badComparison == YES)
+	    {
+	      NSWarnMLog(@"Detected bad return value from comparison", 0);
+	    }
+#endif
+	}
+
+      for (i = 0; i < numKeys; i++)
+	{
+	  plists[i] = (*myObj)(self, objSel, keys[i]);
+	}
+
+      (*appImp)(result, appSel, @"{\n");
+      for (i = 0; i < numKeys; i++)
+	{
+	  id	o = plists[i];
+
+	  (*appImp)(result, appSel, iSizeString);
+	  [keys[i] descriptionWithLocale: nil indent: 0 to: result];
+	  (*appImp)(result, appSel, @" = ");
+	  [o descriptionWithLocale: locale indent: level to: result];
+	  (*appImp)(result, appSel, @";\n");
+	}
+      (*appImp)(result, appSel, iBaseString);
+      (*appImp)(result, appSel, @"}");
     }
-  (*appImp)(result, appSel, iBaseString);
-  (*appImp)(result, appSel, @"}");
 }
 
 @end
@@ -826,8 +848,13 @@ static NSString	*indentStrings[] = {
 + (id) allocWithZone: (NSZone*)z
 {
   if (self == NSMutableDictionary_abstract_class)
-    return NSAllocateObject(NSMutableDictionary_concrete_class, 0, z);
-  return [super allocWithZone: z];
+    {
+      return NSAllocateObject(NSMutableDictionary_concrete_class, 0, z);
+    }
+  else
+    {
+      return NSAllocateObject(self, 0, z);
+    }
 }
 
 - (id) copyWithZone: (NSZone*)z
@@ -918,7 +945,7 @@ static NSString	*indentStrings[] = {
 {
   unsigned	c = [keyArray count];
 
-  if (c)
+  if (c > 0)
     {
       id	keys[c];
       IMP	remObj = [self methodForSelector: remSel];
@@ -933,13 +960,16 @@ static NSString	*indentStrings[] = {
 
 - (void) addEntriesFromDictionary: (NSDictionary*)other
 {
-  id		k;
-  NSEnumerator	*e = [other keyEnumerator];
-  IMP		nxtObj = [e methodForSelector: nxtSel];
-  IMP		setObj = [self methodForSelector: setSel];
+  if (other != nil)
+    {
+      id		k;
+      NSEnumerator	*e = [other keyEnumerator];
+      IMP		nxtObj = [e methodForSelector: nxtSel];
+      IMP		setObj = [self methodForSelector: setSel];
 
-  while ((k = (*nxtObj)(e, nxtSel)) != nil)
-    (*setObj)(self, setSel, [other objectForKey: k], k);
+      while ((k = (*nxtObj)(e, nxtSel)) != nil)
+	(*setObj)(self, setSel, [other objectForKey: k], k);
+    }
 }
 
 - (void) setDictionary: (NSDictionary*)otherDictionary
