@@ -23,6 +23,7 @@
    */
 
 #include <config.h>
+#include <base/preface.h>
 #include <Foundation/DistributedObjects.h>
 #include <Foundation/NSLock.h>
 #include <Foundation/NSPort.h>
@@ -33,6 +34,20 @@ static int	debug_proxy = 0;
 static Class	placeHolder = 0;
 static Class	distantObjectClass = 0;
 
+typedef struct {
+  @defs(NSDistantObject)
+} NSDO;
+
+/* This is the proxy tag; it indicates where the local object is,
+   and determines whether the reply port to the Connection-where-the-
+   proxy-is-local needs to encoded/decoded or not. */
+enum
+{
+  PROXY_LOCAL_FOR_RECEIVER = 0,
+  PROXY_LOCAL_FOR_SENDER,
+  PROXY_REMOTE_FOR_BOTH
+};
+
 
 
 /*
@@ -43,6 +58,7 @@ static Class	distantObjectClass = 0;
  *	allocating the memory for a new instance unless absolutely necessary.
  */
 @interface	GSDistantObjectPlaceHolder
++ (id) initWithCoder: (NSCoder*)aCoder;
 + (id) initWithLocal: (id)anObject connection: (NSConnection*)aConnection;
 + (id) initWithTarget: (unsigned)target connection: (NSConnection*)aConnection;
 + (void) autorelease;
@@ -73,6 +89,176 @@ static Class	distantObjectClass = 0;
     }
 }
 
++ (id) initWithCoder: (NSCoder*)aCoder
+{
+  gsu8			proxy_tag;
+  unsigned		target;
+  id			decoder_connection;
+
+/*
+  if ([aCoder isKindOfClass: [NSPortCoder class]] == NO)
+    {
+      [NSException raise: NSGenericException
+		  format: @"NSDistantObject objects only decode with "
+			  @"NSPortCoder class"];
+    }
+*/
+
+  decoder_connection = [(NSPortCoder*)aCoder connection];
+  NSAssert(decoder_connection, NSInternalInconsistencyException);
+
+  /* First get the tag, so we know what values need to be decoded. */
+  [aCoder decodeValueOfObjCType: @encode(typeof(proxy_tag))
+			     at: &proxy_tag];
+
+  switch (proxy_tag)
+    {
+      case PROXY_LOCAL_FOR_RECEIVER:
+	/*
+	 *	This was a proxy on the other side of the connection, but
+	 *	here it's local.
+	 *	Lookup the target handle to ensure that it exists here.
+	 *	Return a retained copy of the local target object.
+	 */
+	[aCoder decodeValueOfObjCType: @encode(typeof(target))
+				   at: &target];
+
+        if (debug_proxy)
+	  NSLog(@"Receiving a proxy for local object 0x%x "
+		@"connection 0x%x\n", target, (gsaddr)decoder_connection);
+
+        if (![[decoder_connection class] includesLocalTarget: target])
+	  {
+	    [NSException raise: @"ProxyDecodedBadTarget"
+			format: @"No local object with given target (0x%x)",
+				target];
+	  }
+	else
+	  {
+	    NSDistantObject	*o;
+
+	    o = [decoder_connection includesLocalTarget: target];
+	    if (debug_proxy)
+	      {
+		NSLog(@"Local object is 0x%x (0x%x)\n",
+		  (gsaddr)o, (gsaddr)o ? ((NSDO*)o)->_object : 0);
+	      }
+	    return o ? RETAIN(((NSDO*)o)->_object) : nil;
+	  }
+
+      case PROXY_LOCAL_FOR_SENDER:
+        /*
+	 *	This was a local object on the other side of the connection,
+	 *	but here it's a proxy object.  Get the target address, and
+	 *	send [NSDistantObject +proxyWithTarget:connection:]; this will
+	 *	return the proxy object we already created for this target, or
+	 *	create a new proxy object if necessary.
+	 */
+	[aCoder decodeValueOfObjCType: @encode(typeof(target))
+				   at: &target];
+	if (debug_proxy)
+	  NSLog(@"Receiving a proxy, was local 0x%x connection 0x%x\n",
+		  target, (gsaddr)decoder_connection);
+	return [self initWithTarget: target
+			 connection: decoder_connection];
+
+      case PROXY_REMOTE_FOR_BOTH:
+        /*
+	 *	This was a proxy on the other side of the connection, and it
+	 *	will be a proxy on this side too; that is, the local version
+	 *	of this object is not on this host, not on the host the
+	 *	NSPortCoder is connected to, but on a *third* host.
+	 *	This is why I call this a "triangle connection".  In addition
+	 *	to decoding the target, we decode the NSPort object that we
+	 *	will use to talk directly to this third host.  We send
+	 *	[NSConnection +connectionWithReceivePort:sendPort:]; this
+	 *	will either return the connection already created for this
+	 *	inPort/outPort pair, or create a new connection if necessary.
+	 */
+	{
+	  NSConnection		*proxy_connection;
+	  NSPort		*proxy_connection_out_port = nil;
+	  unsigned		intermediary;
+
+	  /*
+	   *	There is an object on the intermediary host that is keeping
+	   *	that hosts proxy for the original object retained, thus
+	   *	ensuring that the original is not released.  We create a
+	   *	proxy for that intermediate proxy.  When we release this
+	   *	proxy, the intermediary will be free to release it's proxy
+	   *	and the original can then be released.  Of course, by that
+	   *	time we will have obtained our own proxy for the original
+	   *	object ...
+	   */
+	  [aCoder decodeValueOfObjCType: @encode(typeof(intermediary))
+				     at: &intermediary];
+	  AUTORELEASE([self initWithTarget: intermediary
+				connection: decoder_connection]);
+
+	  /*
+	   *	Now we get the target number and port for the orignal object
+	   *	and (if necessary) get the originating process to retain the
+	   *	object for us.
+	   */
+	  [aCoder decodeValueOfObjCType: @encode(typeof(target))
+				     at: &target];
+
+	  [aCoder decodeValueOfObjCType: @encode(id)
+				     at: &proxy_connection_out_port];
+
+	  NSAssert(proxy_connection_out_port, NSInternalInconsistencyException);
+	  /*
+	   #	If there already exists a connection for talking to the
+	   *	out port, we use that one rather than creating a new one from
+	   *	our listening port. 
+	   *
+	   *	First we try for a connection from our receive port,
+	   *	Then we try any connection to the send port
+	   *	Finally we resort to creating a new connection - we don't
+	   *	release the newly created connection - it will get released
+	   *	automatically when no proxies are left on it.
+	   */
+	  proxy_connection = [[decoder_connection class]
+	    connectionWithReceivePort: [decoder_connection receivePort]
+			     sendPort: proxy_connection_out_port];
+
+	  if (debug_proxy)
+	    NSLog(@"Receiving a triangle-connection proxy 0x%x "
+		  @"connection 0x%x\n", target, (gsaddr)proxy_connection);
+
+	  NSAssert(proxy_connection != decoder_connection,
+	    NSInternalInconsistencyException);
+	  NSAssert([proxy_connection isValid],
+	    NSInternalInconsistencyException);
+
+	  /*
+	   *	If we don't already have a proxy for the object on the
+	   *	remote system, we must tell the other end to retain its
+	   *	local object for our use.
+	   */
+	  if ([proxy_connection includesProxyForTarget: target] == NO)
+	    [proxy_connection retainTarget: target];
+
+	  /*
+	   *	Finally - we get a proxy via a direct connection to the
+	   *	originating server.  We have also created a proxy to an
+	   *	intermediate object - but this proxy has not been retained
+	   *	and will therefore go away when the current autorelease
+	   *	pool is destroyed.
+	   */
+	  return [self initWithTarget: target
+			   connection: proxy_connection];
+        }
+
+    default:
+      /* xxx This should be something different than NSGenericException. */
+      [NSException raise: NSGenericException
+		  format: @"Bad proxy tag"];
+    }
+  /* Not reached. */
+  return nil;
+}
+
 + (id) initWithLocal: (id)anObject connection: (NSConnection*)aConnection
 {
   NSDistantObject	*proxy;
@@ -85,7 +271,7 @@ static Class	distantObjectClass = 0;
    */
   if ((proxy = [aConnection localForObject: anObject]))
     {
-      return [proxy retain];
+      return RETAIN(proxy);
     }
 
   proxy = (NSDistantObject*)NSAllocateObject(distantObjectClass,
@@ -105,7 +291,7 @@ static Class	distantObjectClass = 0;
    */
   if ((proxy = [aConnection proxyForTarget: target]))
     {
-      return [proxy retain];
+      return RETAIN(proxy);
     }
 
   proxy = (NSDistantObject*)NSAllocateObject(distantObjectClass,
@@ -127,16 +313,6 @@ static Class	distantObjectClass = 0;
 
 @implementation NSDistantObject
 
-/* This is the proxy tag; it indicates where the local object is,
-   and determines whether the reply port to the Connection-where-the-
-   proxy-is-local needs to encoded/decoded or not. */
-enum
-{
-  PROXY_LOCAL_FOR_RECEIVER = 0,
-  PROXY_LOCAL_FOR_SENDER,
-  PROXY_REMOTE_FOR_BOTH
-};
-
 + (void) initialize
 {
   if (self == [NSDistantObject class])
@@ -153,15 +329,15 @@ enum
 + (NSDistantObject*) proxyWithLocal: (id)anObject
 			 connection: (NSConnection*)aConnection
 {
-  return [[placeHolder initWithLocal: anObject
-			  connection: aConnection] autorelease];
+  return AUTORELEASE([placeHolder initWithLocal: anObject
+				     connection: aConnection]);
 }
 
 + (NSDistantObject*) proxyWithTarget: (unsigned)anObject
 			  connection: (NSConnection*)aConnection
 {
-  return [[placeHolder initWithTarget: anObject
-			   connection: aConnection] autorelease];
+  return AUTORELEASE([placeHolder initWithTarget: anObject
+				      connection: aConnection]);
 }
 
 - (NSConnection*) connectionForProxy
@@ -181,10 +357,14 @@ enum
   gsu8		proxy_tag;
   NSConnection	*encoder_connection;
 
-  if ([aRmc class] != [PortEncoder class])
-    [NSException raise: NSGenericException
-		format: @"NSDistantObject objects only "
-			@"encode with PortEncoder class"];
+/*
+  if ([aRmc isKindOfClass: [NSPortCoder class]] == NO)
+    {
+      [NSException raise: NSGenericException
+		  format: @"NSDistantObject objects only "
+			  @"encode with NSPortCoder class"];
+    }
+*/
 
   encoder_connection = [(NSPortCoder*)aRmc connection];
   NSAssert(encoder_connection, NSInternalInconsistencyException);
@@ -210,13 +390,11 @@ enum
 	    NSLog(@"Sending a proxy, will be remote 0x%x connection 0x%x\n",
 			proxy_target, (gsaddr)_connection);
 
-	  [aRmc encodeValueOfCType: @encode(typeof(proxy_tag))
-				at: &proxy_tag
-			  withName: @"Proxy is local for sender"];
+	  [aRmc encodeValueOfObjCType: @encode(typeof(proxy_tag))
+				   at: &proxy_tag];
 
-	  [aRmc encodeValueOfCType: @encode(typeof(proxy_target))
-				at: &proxy_target
-			  withName: @"Proxy target"];
+	  [aRmc encodeValueOfObjCType: @encode(typeof(proxy_target))
+				   at: &proxy_target];
 	}
       else
 	{
@@ -229,13 +407,11 @@ enum
 	    NSLog(@"Sending a proxy, will be local 0x%x connection 0x%x\n",
 			proxy_target, (gsaddr)_connection);
 
-	  [aRmc encodeValueOfCType: @encode(typeof(proxy_tag))
-				at: &proxy_tag
-			  withName: @"Proxy is local for receiver"];
+	  [aRmc encodeValueOfObjCType: @encode(typeof(proxy_tag))
+				   at: &proxy_tag];
 
-	  [aRmc encodeValueOfCType: @encode(typeof(proxy_target))
-				at: &proxy_target
-			  withName: @"Proxy target"];
+	  [aRmc encodeValueOfObjCType: @encode(typeof(proxy_target))
+				   at: &proxy_target];
 	}
     }
   else
@@ -274,20 +450,16 @@ enum
        *	It's remote here, so we need to tell other side where to form
        *	triangle connection to
        */
-      [aRmc encodeValueOfCType: @encode(typeof(proxy_tag))
-			    at: &proxy_tag
-		      withName: @"Proxy remote for both sender and receiver"];
+      [aRmc encodeValueOfObjCType: @encode(typeof(proxy_tag))
+			       at: &proxy_tag];
 
-      [aRmc encodeValueOfCType: @encode(typeof(localProxy->_handle))
-			    at: &localProxy->_handle
-		      withName: @"Intermediary target"];
+      [aRmc encodeValueOfObjCType: @encode(typeof(localProxy->_handle))
+			       at: &localProxy->_handle];
 
-      [aRmc encodeValueOfCType: @encode(typeof(proxy_target))
-			    at: &proxy_target
-		      withName: @"Original target"];
+      [aRmc encodeValueOfObjCType: @encode(typeof(proxy_target))
+			       at: &proxy_target];
 
-      [aRmc encodeBycopyObject: proxy_connection_out_port
-		      withName: @"Original port"];
+      [aRmc encodeBycopyObject: proxy_connection_out_port];
     }
 }
 
@@ -306,21 +478,22 @@ enum
   unsigned	target;
   id		decoder_connection;
 
-  if ([aCoder class] != [PortDecoder class])
+/*
+  if ([aCoder isKindOfClass: [NSPortCoder class]] == NO)
     {
-      [self release];
+      RELEASE(self);
       [NSException raise: NSGenericException
 		  format: @"NSDistantObject objects only decode with "
-			  @"PortDecoder class"];
+			  @"NSPortCoder class"];
     }
+*/
 
   decoder_connection = [(NSPortCoder*)aCoder connection];
   NSAssert(decoder_connection, NSInternalInconsistencyException);
 
   /* First get the tag, so we know what values need to be decoded. */
-  [aCoder decodeValueOfCType: @encode(typeof(proxy_tag))
-			  at: &proxy_tag
-		    withName: NULL];
+  [aCoder decodeValueOfObjCType: @encode(typeof(proxy_tag))
+			     at: &proxy_tag];
 
   switch (proxy_tag)
     {
@@ -331,9 +504,8 @@ enum
 	 *	Lookup the target handle to ensure that it exists here.
 	 *	Return a retained copy of the local target object.
 	 */
-	[aCoder decodeValueOfCType: @encode(typeof(target))
-				at: &target
-			  withName: NULL];
+	[aCoder decodeValueOfObjCType: @encode(typeof(target))
+				   at: &target];
 
         if (debug_proxy)
 	  NSLog(@"Receiving a proxy for local object 0x%x "
@@ -341,7 +513,7 @@ enum
 
         if (![[decoder_connection class] includesLocalTarget: target])
 	  {
-	    [self release];
+	    RELEASE(self);
 	    [NSException raise: @"ProxyDecodedBadTarget"
 			format: @"No local object with given target (0x%x)",
 				target];
@@ -356,8 +528,8 @@ enum
 		NSLog(@"Local object is 0x%x (0x%x)\n",
 		  (gsaddr)o, (gsaddr)o ? o->_object : 0);
 	      }
-	    [self release];
-	    return o ? [o->_object retain] : nil;
+	    RELEASE(self);
+	    return o ? RETAIN(o->_object) : nil;
 	  }
 
       case PROXY_LOCAL_FOR_SENDER:
@@ -368,15 +540,14 @@ enum
 	 *	return the proxy object we already created for this target, or
 	 *	create a new proxy object if necessary.
 	 */
-	[aCoder decodeValueOfCType: @encode(typeof(target))
-				at: &target
-			  withName: NULL];
+	[aCoder decodeValueOfObjCType: @encode(typeof(target))
+				   at: &target];
 	if (debug_proxy)
 	  NSLog(@"Receiving a proxy, was local 0x%x connection 0x%x\n",
 		  target, (gsaddr)decoder_connection);
-        [self release];
-	return [[NSDistantObject proxyWithTarget: target
-				      connection: decoder_connection] retain];
+        RELEASE(self);
+	return RETAIN([NSDistantObject proxyWithTarget: target
+					    connection: decoder_connection]);
 
       case PROXY_REMOTE_FOR_BOTH:
         /*
@@ -407,9 +578,8 @@ enum
 	   *	time we will have obtained our own proxy for the original
 	   *	object ...
 	   */
-	  [aCoder decodeValueOfCType: @encode(typeof(intermediary))
-				  at: &intermediary
-			    withName: NULL];
+	  [aCoder decodeValueOfObjCType: @encode(typeof(intermediary))
+				     at: &intermediary];
 	  [NSDistantObject proxyWithTarget: intermediary
 				connection: decoder_connection];
 
@@ -418,12 +588,11 @@ enum
 	   *	and (if necessary) get the originating process to retain the
 	   *	object for us.
 	   */
-	  [aCoder decodeValueOfCType: @encode(typeof(target))
-				  at: &target
-			    withName: NULL];
+	  [aCoder decodeValueOfObjCType: @encode(typeof(target))
+				     at: &target];
 
-	  [aCoder decodeObjectAt: &proxy_connection_out_port
-		        withName: NULL];
+	  [aCoder decodeValueOfObjCType: @encode(id)
+				     at: &proxy_connection_out_port];
 
 	  NSAssert(proxy_connection_out_port, NSInternalInconsistencyException);
 	  /*
@@ -438,24 +607,8 @@ enum
 	   *	automatically when no proxies are left on it.
 	   */
 	  proxy_connection = [[decoder_connection class]
-				connectionByInPort:
-					[decoder_connection receivePort]
-				outPort:
-					proxy_connection_out_port];
-	  if (proxy_connection == nil)
-	    {
-	      proxy_connection = [[decoder_connection class]
-			      connectionByOutPort: proxy_connection_out_port];
-	    }
-	  if (proxy_connection == nil)
-	    {
-	      proxy_connection = [[decoder_connection class]
-			     newForInPort: [decoder_connection receivePort]
-				  outPort: proxy_connection_out_port
-		       ancestorConnection: decoder_connection];
-	      [proxy_connection setNotOwned];
-	      [proxy_connection autorelease];
-	    }
+	    connectionWithReceivePort: [decoder_connection receivePort]
+			     sendPort: proxy_connection_out_port];
 
 	  if (debug_proxy)
 	    NSLog(@"Receiving a triangle-connection proxy 0x%x "
@@ -474,9 +627,9 @@ enum
 	  if ([proxy_connection includesProxyForTarget: target] == NO)
 	    [proxy_connection retainTarget: target];
 
-	  result = [[NSDistantObject proxyWithTarget: target
-				          connection: proxy_connection] retain];
-	  [self release];
+	  result = RETAIN([NSDistantObject proxyWithTarget: target
+						connection: proxy_connection]);
+	  RELEASE(self);
 
 	  /*
 	   *	Finally - we have a proxy via a direct connection to the
@@ -490,7 +643,7 @@ enum
 
     default:
       /* xxx This should be something different than NSGenericException. */
-      [self release];
+      RELEASE(self);
       [NSException raise: NSGenericException
 		  format: @"Bad proxy tag"];
     }
@@ -510,8 +663,9 @@ enum
    */
   if ((new_proxy = [aConnection localForObject: anObject]))
     {
-      [self release];
-      return [new_proxy retain];
+      RETAIN(new_proxy);
+      RELEASE(self);
+      return new_proxy;
     }
 
   /*
@@ -524,7 +678,7 @@ enum
   /*
    *	We register this proxy with the connection using it.
    */
-  _connection = [aConnection retain];
+  _connection = RETAIN(aConnection);
   [_connection addLocalObject: self];
 
   if (debug_proxy)
@@ -546,8 +700,9 @@ enum
    */
   if ((new_proxy = [aConnection proxyForTarget: target]))
     {
-      [self release];
-      return [new_proxy retain];
+      RETAIN(new_proxy);
+      RELEASE(self);
+      return new_proxy;
     }
 
   _object = nil;
@@ -557,7 +712,7 @@ enum
    *	We retain our connection so it can't disappear while the app
    *	may want to use it.
    */
-  _connection = [aConnection retain];
+  _connection = RETAIN(aConnection);
 
   /*
    *	We register this object with the connection using it.
@@ -573,13 +728,13 @@ enum
 
 - (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector
 {
-  if (_object)
+  if (_object != nil)
     {
       return [_object methodSignatureForSelector: aSelector];
     }
   else
     {
-      if (_protocol)
+      if (_protocol != nil)
 	{
 	  const char	*types = 0;
 
@@ -602,13 +757,25 @@ enum
 	}
       else
 	{
+	  id		m;
+	  const char	*types;
 	  arglist_t	args;
+	  void		*retframe;
+
+	  id retframe_id (void *rframe)
+	    {
+	      __builtin_return (rframe);
+	    }
 
 	  /*
 	   *	No protocol - so try forwarding the message.
 	   */
 	  args = __builtin_apply_args();
-	  __builtin_return([self forward: _cmd : args]);
+	  retframe = [self forward: _cmd : args];
+	  m = retframe_id(retframe);
+	  types = [m methodType];
+
+	  return [NSMethodSignature signatureWithObjCTypes: types];
 	}
     }
 }
@@ -643,7 +810,7 @@ enum
        */
       if (_object == nil)
 	[_connection removeProxy: self];
-      [_connection release];
+      RELEASE(_connection);
     }
 }
 
@@ -750,7 +917,7 @@ static inline BOOL class_is_kind_of (Class self, Class aClassObject)
   return [self class];
 }
 
-- replacementObjectForPortCoder: (NSPortCoder*)aRmc;
+- (id) replacementObjectForPortCoder: (NSPortCoder*)aRmc;
 {
   if ([aRmc isBycopy])
     return self;
