@@ -26,6 +26,7 @@
 */
 
 #include <config.h>
+#include <string.h>
 #include <base/preface.h>
 #include <Foundation/NSFileManager.h>
 #include <Foundation/NSException.h>
@@ -61,7 +62,7 @@
 
 #define DIR_enum_state DIR
 
-#if	defined(__WIN32__)
+#if	defined(__MINGW__)
 #define	WIN32ERR	((DWORD)0xFFFFFFFF)
 #endif
 
@@ -71,7 +72,7 @@
 # include <limits.h>			/* for PATH_MAX */
 # include <utime.h>
 #else
-#ifdef __WIN32__
+#ifdef __MINGW__
 # include <limits.h>
 # include <sys/utime.h>
 #else
@@ -196,7 +197,7 @@ static NSFileManager* defaultManager = nil;
 
 - (void) dealloc
 {
-  TEST_RELEASE(lastError);
+  TEST_RELEASE(_lastError);
   [super dealloc];
 }
 
@@ -206,8 +207,8 @@ static NSFileManager* defaultManager = nil;
 {
   const char* cpath = [self fileSystemRepresentationWithPath: path];
     
-#if defined(__WIN32__)
-  return SetCurrentDirectory(cpath);
+#if defined(__MINGW__)
+  return SetCurrentDirectory(cpath) == TRUE ? YES : NO;
 #else
   return (chdir(cpath) == 0);
 #endif
@@ -216,12 +217,12 @@ static NSFileManager* defaultManager = nil;
 - (BOOL) createDirectoryAtPath: (NSString*)path
 		    attributes: (NSDictionary*)attributes
 {
-#if defined(__WIN32__)
+#if defined(__MINGW__)
   BOOL	ok;
 
   ok = CreateDirectory([self fileSystemRepresentationWithPath: path], NULL);
   if (ok == NO)
-    ASSIGN(lastError, @"Could not create directory");
+    ASSIGN(_lastError, @"Could not create directory");
   return ok;
 #else
   const char	*cpath;
@@ -246,13 +247,13 @@ static NSFileManager* defaultManager = nil;
   len = strlen(cpath);
   if (len > PATH_MAX) // name too long
     {
-      ASSIGN(lastError, @"Could not create directory - name too long");
+      ASSIGN(_lastError, @"Could not create directory - name too long");
       return NO;
     }
     
   if (strcmp(cpath, "/") == 0 || len == 0) // cannot use "/" or ""
     {
-      ASSIGN(lastError, @"Could not create directory - no name given");
+      ASSIGN(_lastError, @"Could not create directory - no name given");
       return NO;
     }
     
@@ -279,7 +280,7 @@ static NSFileManager* defaultManager = nil;
 	{
 	  if (cur == len)
 	    {
-	      ASSIGN(lastError, @"Could not create directory - already exists");
+	      ASSIGN(_lastError, @"Could not create directory - already exists");
 	      return NO;
 	    }
 	}
@@ -292,7 +293,7 @@ static NSFileManager* defaultManager = nil;
 
 	      s = [NSString stringWithFormat: @"Could not create '%s' - '%s'",
 		dirpath, strerror(errno)];
-	      ASSIGN(lastError, s);
+	      ASSIGN(_lastError, s);
 	      return NO;
 	    }
 	  // if last directory and attributes then change
@@ -321,14 +322,14 @@ static NSFileManager* defaultManager = nil;
   while (cur < len);
 
   return YES;
-#endif /* WIN32 */
+#endif /* !MINGW */
 }
 
 - (NSString*) currentDirectoryPath
 {
   char path[PATH_MAX];
 
-#if defined(__WIN32__)
+#if defined(__MINGW__)
   if (GetCurrentDirectory(PATH_MAX, path) > PATH_MAX)
     return nil;
 #else
@@ -339,7 +340,7 @@ static NSFileManager* defaultManager = nil;
   if (getwd(path) == NULL)
     return nil;
 #endif /* HAVE_GETCWD */
-#endif /* WIN32 */
+#endif /* !MINGW */
 
   return [self stringWithFileSystemRepresentation: path length: strlen(path)];
 }
@@ -380,7 +381,7 @@ static NSFileManager* defaultManager = nil;
 	      NSDictionary* errorInfo
 		  = [NSDictionary dictionaryWithObjectsAndKeys: 
 		      destination, @"Path",
-		      lastError, @"Error",
+		      _lastError, @"Error",
 		      nil];
 	      return [handler fileManager: self
 			      shouldProceedAfterError: errorInfo];
@@ -508,13 +509,19 @@ static NSFileManager* defaultManager = nil;
 {
   NSArray	*contents;
 
-  if (handler)
+  if (handler != nil)
     [handler fileManager: self willProcessPath: path];
 
   contents = [self directoryContentsAtPath: path];
   if (contents == nil)
     {
-      if (unlink([path fileSystemRepresentation]) < 0)
+      const char	*cpath = [path fileSystemRepresentation];
+
+#if defined(__MINGW__)
+      if (DeleteFile(cpath) == FALSE)
+#else
+      if (unlink(cpath) < 0)
+#endif
 	{
 	  BOOL	result;
 
@@ -544,10 +551,10 @@ static NSFileManager* defaultManager = nil;
 
       for (i = 0; i < count; i++)
 	{
-	  CREATE_AUTORELEASE_POOL(arp);
 	  NSString		*item;
 	  NSString		*next;
 	  BOOL			result;
+	  CREATE_AUTORELEASE_POOL(arp);
 
 	  item = [contents objectAtIndex: i];
 	  next = [path stringByAppendingPathComponent: item];
@@ -586,14 +593,43 @@ static NSFileManager* defaultManager = nil;
 		 contents: (NSData*)contents
 	       attributes: (NSDictionary*)attributes
 {
-  int fd, len, written;
+  int		len;
+  int		written;
+  const char	*cpath = [self fileSystemRepresentationWithPath: path];
 
-  fd = open ([self fileSystemRepresentationWithPath: path],
-                O_WRONLY|O_TRUNC|O_CREAT, 0644);
+#if	defined(__MINGW__)
+  HANDLE fh;
+
+  fh = CreateFile(cpath, GENERIC_WRITE, 0, 0, CREATE_ALWAYS,
+    FILE_ATTRIBUTE_NORMAL, 0);
+  if (fh == INVALID_HANDLE_VALUE)
+    {
+      return NO;
+    }
+  else
+    {
+      DWORD	len = [contents length];
+      DWORD	written = 0;
+
+      if (len > 0)
+	{
+	  WriteFile(fh, [contents bytes], len, &written, NULL);
+	}
+      CloseHandle(fh);
+      if ([self changeFileAttributes: attributes atPath: path] == NO)
+	{
+	  return NO;
+	}
+    }
+#else
+  int	fd;
+
+  fd = open (cpath, O_WRONLY|O_TRUNC|O_CREAT, 0644);
   if (fd < 0)
-    return NO;
-
-  if (![self changeFileAttributes: attributes atPath: path])
+    {
+      return NO;
+    }
+  if ([self changeFileAttributes: attributes atPath: path] == NO)
     {
       close (fd);
       return NO;
@@ -617,12 +653,16 @@ static NSFileManager* defaultManager = nil;
     }
 
   len = [contents length];
-  if (len)
-    written = write (fd, [contents bytes], len);
+  if (len > 0)
+    {
+      written = write(fd, [contents bytes], len);
+    }
   else
-    written = 0;
+    {
+      written = 0;
+    }
   close (fd);
-
+#endif
   return written == len;
 }
 
@@ -673,10 +713,10 @@ static NSFileManager* defaultManager = nil;
 
       for (index = 0; ok == YES && index < [a1 count]; index++)
 	{
-	  CREATE_AUTORELEASE_POOL(pool);
 	  NSString	*n = [a1 objectAtIndex: index];
 	  NSString	*p1;
 	  NSString	*p2;
+	  CREATE_AUTORELEASE_POOL(pool);
 
 	  p1 = [path1 stringByAppendingPathComponent: n];
 	  p2 = [path2 stringByAppendingPathComponent: n];
@@ -716,7 +756,7 @@ static NSFileManager* defaultManager = nil;
     }
   else
     {
-#if defined(__WIN32__)
+#if defined(__MINGW__)
       DWORD res;
 
       res = GetFileAttributes(cpath);
@@ -743,7 +783,7 @@ static NSFileManager* defaultManager = nil;
 	}
     
       return YES;
-#endif /* WIN32 */
+#endif /* MINGW */
     }
 }
 
@@ -755,7 +795,7 @@ static NSFileManager* defaultManager = nil;
     return NO;
   else
     {
-#if defined(__WIN32__)
+#if defined(__MINGW__)
       DWORD res= GetFileAttributes(cpath);
 
       if (res == WIN32ERR)
@@ -775,7 +815,7 @@ static NSFileManager* defaultManager = nil;
     return NO;
   else
     {
-#if defined(__WIN32__)
+#if defined(__MINGW__)
       DWORD res= GetFileAttributes(cpath);
 
       if (res == WIN32ERR)
@@ -795,13 +835,13 @@ static NSFileManager* defaultManager = nil;
     return NO;
   else
     {
-#if defined(__WIN32__)
+#if defined(__MINGW__)
       DWORD res= GetFileAttributes(cpath);
       int len = strlen(cpath);
 
       if (res == WIN32ERR)
         return NO;
-      if (len > 4 && strcmp(&path[len-4], ".exe") == 0)
+      if (len > 4 && strcmp(&cpath[len-4], ".exe") == 0)
 	return YES;
       return NO;
 #else
@@ -839,7 +879,7 @@ static NSFileManager* defaultManager = nil;
 
 - (NSDictionary*) fileSystemAttributesAtPath: (NSString*)path
 {
-#if defined(__WIN32__)
+#if defined(__MINGW__)
   long long totalsize, freesize;
   id  values[5];
   id	keys[5] = {
@@ -912,7 +952,7 @@ static NSFileManager* defaultManager = nil;
 #else
   return nil;
 #endif
-#endif /* WIN32 */
+#endif /* MINGW */
 }
 
 - (BOOL) changeFileAttributes: (NSDictionary*)attributes atPath: (NSString*)path
@@ -923,7 +963,7 @@ static NSFileManager* defaultManager = nil;
   NSDate	*date;
   BOOL		allOk = YES;
 
-#ifndef __WIN32__
+#ifndef __MINGW__
   num = [attributes objectForKey: NSFileOwnerAccountNumber];
   if (num)
     {
@@ -932,7 +972,7 @@ static NSFileManager* defaultManager = nil;
 	  allOk = NO;
 	  str = [NSString stringWithFormat:
 	    @"Unable to change NSFileOwnerAccountNumber to '%@'", num];
-	  ASSIGN(lastError, str);
+	  ASSIGN(_lastError, str);
 	}
     }
   else
@@ -954,7 +994,7 @@ static NSFileManager* defaultManager = nil;
 	      allOk = NO;
 	      str = [NSString stringWithFormat:
 		@"Unable to change NSFileOwnerAccountName to '%@'", str];
-	      ASSIGN(lastError, str);
+	      ASSIGN(_lastError, str);
 	    }
 	}
     }
@@ -967,7 +1007,7 @@ static NSFileManager* defaultManager = nil;
 	  allOk = NO;
 	  str = [NSString stringWithFormat:
 	    @"Unable to change NSFileGroupOwnerAccountNumber to '%@'", num];
-	  ASSIGN(lastError, str);
+	  ASSIGN(_lastError, str);
 	}
     }
   else if ((str=[attributes objectForKey: NSFileGroupOwnerAccountName]) != nil)
@@ -987,10 +1027,10 @@ static NSFileManager* defaultManager = nil;
 	  allOk = NO;
 	  str = [NSString stringWithFormat:
 	    @"Unable to change NSFileGroupOwnerAccountName to '%@'", str];
-	  ASSIGN(lastError, str);
+	  ASSIGN(_lastError, str);
 	}
     }
-#endif	/* __WIN32__ */
+#endif	/* __MINGW__ */
 
   num = [attributes objectForKey: NSFilePosixPermissions];
   if (num)
@@ -1000,7 +1040,7 @@ static NSFileManager* defaultManager = nil;
 	  allOk = NO;
 	  str = [NSString stringWithFormat:
 	    @"Unable to change NSFilePosixPermissions to '%o'", [num intValue]];
-	  ASSIGN(lastError, str);
+	  ASSIGN(_lastError, str);
 	}
     }
     
@@ -1034,7 +1074,7 @@ static NSFileManager* defaultManager = nil;
 	  allOk = NO;
 	  str = [NSString stringWithFormat:
 	    @"Unable to change NSFileModificationDate to '%@'", date];
-	  ASSIGN(lastError, str);
+	  ASSIGN(_lastError, str);
 	}
     }
     
@@ -1047,11 +1087,17 @@ static NSFileManager* defaultManager = nil;
 {
   NSDirectoryEnumerator	*direnum;
   NSMutableArray	*content;
-  BOOL			isDir;
   IMP			nxtImp;
   IMP			addImp;
-    
-  if (![self fileExistsAtPath: path isDirectory: &isDir] || !isDir)
+  NSDictionary		*attr;
+  NSString		*type;
+
+  /*
+   * See if this is a directory (don't follow links).
+   */
+  attr = [self fileAttributesAtPath: path traverseLink: NO];
+  type = [attr objectForKey: NSFileType];
+  if ([type isEqualToString: NSFileTypeDirectory] == NO)
     return nil;
     
   direnum = [[NSDirectoryEnumerator alloc] initWithDirectoryPath: path 
@@ -1061,10 +1107,10 @@ static NSFileManager* defaultManager = nil;
   content = [NSMutableArray arrayWithCapacity: 128];
 
   nxtImp = [direnum methodForSelector: @selector(nextObject)];
-  addImp = [content methodForSelector: @selector(addObject: )];
+  addImp = [content methodForSelector: @selector(addObject:)];
 
   while ((path = (*nxtImp)(direnum, @selector(nextObject))) != nil)
-    (*addImp)(content, @selector(addObject: ), path);
+    (*addImp)(content, @selector(addObject:), path);
 
   RELEASE(direnum);
 
@@ -1099,10 +1145,10 @@ static NSFileManager* defaultManager = nil;
   content = [NSMutableArray arrayWithCapacity: 128];
     
   nxtImp = [direnum methodForSelector: @selector(nextObject)];
-  addImp = [content methodForSelector: @selector(addObject: )];
+  addImp = [content methodForSelector: @selector(addObject:)];
 
   while ((path = (*nxtImp)(direnum, @selector(nextObject))) != nil)
-    (*addImp)(content, @selector(addObject: ), path);
+    (*addImp)(content, @selector(addObject:), path);
 
   RELEASE(direnum);
 
@@ -1117,7 +1163,7 @@ static NSFileManager* defaultManager = nil;
   const char* lpath = [self fileSystemRepresentationWithPath: path];
   const char* npath = [self fileSystemRepresentationWithPath: otherPath];
     
-#ifdef __WIN32__
+#ifdef __MINGW__
   return NO;
 #else
   return (symlink(lpath, npath) == 0);
@@ -1140,7 +1186,7 @@ static NSFileManager* defaultManager = nil;
 
 - (const char*) fileSystemRepresentationWithPath: (NSString*)path
 {
-#if 0 && defined(__WIN32__)
+#if 0 && defined(__MINGW__)
   unsigned	len = [path length];
   NSMutableData	*d = [NSMutableData dataWithLength: len + 5];
   char		*fspath = (char*)[d mutableBytes];
@@ -1186,7 +1232,7 @@ static NSFileManager* defaultManager = nil;
 
 /* 
   recurses into directory `path' 
-	- pushes relative path (relative to root of search) on pathStack
+	- pushes relative path (relative to root of search) on _pathStack
 	- pushes system dir enumerator on enumPath 
 */
 - (void) recurseIntoDirectory: (NSString*)path relativeName: (NSString*)name
@@ -1201,8 +1247,8 @@ static NSFileManager* defaultManager = nil;
     
   if (dir)
     {
-      [pathStack addObject: name];
-      [enumStack addObject: [NSValue valueWithPointer: dir]];
+      [_pathStack addObject: name];
+      [_enumStack addObject: [NSValue valueWithPointer: dir]];
     }
   else
     NSLog(@"Failed to recurse into directory '%@' - %s",
@@ -1211,18 +1257,18 @@ static NSFileManager* defaultManager = nil;
 
 /*
   backtracks enumeration to the previous dir
-  	- pops current dir relative path from pathStack
-	- pops system dir enumerator from enumStack
+  	- pops current dir relative path from _pathStack
+	- pops system dir enumerator from _enumStack
 	- sets currentFile* to nil
 */
 - (void) backtrack
 {
-  closedir((DIR*)[[enumStack lastObject] pointerValue]);
-  [enumStack removeLastObject];
-  [pathStack removeLastObject];
-  DESTROY(currentFileName);
-  DESTROY(currentFilePath);
-  DESTROY(fileAttributes);
+  closedir((DIR*)[[_enumStack lastObject] pointerValue]);
+  [_enumStack removeLastObject];
+  [_pathStack removeLastObject];
+  DESTROY(_currentFileName);
+  DESTROY(_currentFilePath);
+  DESTROY(_fileAttributes);
 }
 
 /*
@@ -1244,13 +1290,13 @@ static NSFileManager* defaultManager = nil;
   struct stat		statbuf;
   const char*		cpath;
     
-  DESTROY(currentFileName);
-  DESTROY(currentFilePath);
-  DESTROY(fileAttributes);
+  DESTROY(_currentFileName);
+  DESTROY(_currentFilePath);
+  DESTROY(_fileAttributes);
     
-  while ([pathStack count])
+  while ([_pathStack count])
     {
-      dir = (DIR*)[[enumStack lastObject] pointerValue];
+      dir = (DIR*)[[_enumStack lastObject] pointerValue];
       dirbuf = readdir(dir);
       if (dirbuf)
 	{
@@ -1259,20 +1305,20 @@ static NSFileManager* defaultManager = nil;
 	    strcmp(dirbuf->d_name, "..") == 0)
 	    continue;
 	  // Name of current file
-	  currentFileName = [manager
+	  _currentFileName = [manager
 		   stringWithFileSystemRepresentation: dirbuf->d_name
 		   length: strlen(dirbuf->d_name)];
-	  currentFileName = RETAIN([[pathStack lastObject]
-		stringByAppendingPathComponent: currentFileName]);
+	  _currentFileName = RETAIN([[_pathStack lastObject]
+		stringByAppendingPathComponent: _currentFileName]);
 	  // Full path of current file
-	  currentFilePath = RETAIN([topPath
-		stringByAppendingPathComponent: currentFileName]);
+	  _currentFilePath = RETAIN([_topPath
+		stringByAppendingPathComponent: _currentFileName]);
 	  // Check if directory
-	  cpath = [manager fileSystemRepresentationWithPath: currentFilePath];
-  	  if (flags.isRecursive == YES)
+	  cpath = [manager fileSystemRepresentationWithPath: _currentFilePath];
+  	  if (_flags.isRecursive == YES)
 	    {
 	      // Do not follow links
-	      if (!flags.isFollowing)
+	      if (!_flags.isFollowing)
 		{
 		  if (lstat(cpath, &statbuf) != 0)
 		    break;
@@ -1287,8 +1333,8 @@ static NSFileManager* defaultManager = nil;
 		}
 	      if (S_IFDIR == (S_IFMT & statbuf.st_mode))
 		{
-		  [self recurseIntoDirectory: currentFilePath 
-				relativeName: currentFileName];
+		  [self recurseIntoDirectory: _currentFilePath 
+				relativeName: _currentFileName];
 		}
 	    }
 	  break;	// Got a file name - break out of loop
@@ -1307,12 +1353,12 @@ static NSFileManager* defaultManager = nil;
 	      followSymlinks: (BOOL)follow
 		 prefixFiles: (BOOL)prefix
 {
-  pathStack = [NSMutableArray new];
-  enumStack = [NSMutableArray new];
-  flags.isRecursive = recurse;
-  flags.isFollowing = follow;
+  _pathStack = [NSMutableArray new];
+  _enumStack = [NSMutableArray new];
+  _flags.isRecursive = recurse;
+  _flags.isFollowing = follow;
   
-  topPath = RETAIN(path);
+  _topPath = RETAIN(path);
   [self recurseIntoDirectory: path relativeName: @""];
 
   return self;
@@ -1320,16 +1366,16 @@ static NSFileManager* defaultManager = nil;
 
 - (void) dealloc
 {
-  while ([pathStack count])
+  while ([_pathStack count])
     [self backtrack];
     
-  RELEASE(pathStack);
-  RELEASE(enumStack);
-  RELEASE(topPath);
-  TEST_RELEASE(currentFileName);
-  TEST_RELEASE(currentFilePath);
-  TEST_RELEASE(fileAttributes);
-  TEST_RELEASE(directoryAttributes);
+  RELEASE(_pathStack);
+  RELEASE(_enumStack);
+  RELEASE(_topPath);
+  TEST_RELEASE(_currentFileName);
+  TEST_RELEASE(_currentFilePath);
+  TEST_RELEASE(_fileAttributes);
+  TEST_RELEASE(_directoryAttributes);
   [super dealloc];
 }
 
@@ -1337,33 +1383,33 @@ static NSFileManager* defaultManager = nil;
 
 - (NSDictionary*) directoryAttributes
 {
-  if (directoryAttributes == nil)
+  if (_directoryAttributes == nil)
     {
-      directoryAttributes = [[NSFileManager defaultManager]
-	fileAttributesAtPath: topPath
-		traverseLink: flags.isFollowing];
-      RETAIN(directoryAttributes);
+      _directoryAttributes = [[NSFileManager defaultManager]
+	fileAttributesAtPath: _topPath
+		traverseLink: _flags.isFollowing];
+      IF_NO_GC(RETAIN(_directoryAttributes));
     }
-  return directoryAttributes;
+  return _directoryAttributes;
 }
 
 - (NSDictionary*) fileAttributes
 {
-  if (fileAttributes == nil)
+  if (_fileAttributes == nil)
     {
-      fileAttributes = [[NSFileManager defaultManager]
-	fileAttributesAtPath: currentFilePath
-		traverseLink: flags.isFollowing];
-      RETAIN(fileAttributes);
+      _fileAttributes = [[NSFileManager defaultManager]
+	fileAttributesAtPath: _currentFilePath
+		traverseLink: _flags.isFollowing];
+      IF_NO_GC(RETAIN(_fileAttributes));
     }
-  return fileAttributes;
+  return _fileAttributes;
 }
 
 // Skipping subdirectories
 
 - (void) skipDescendents
 {
-  if ([pathStack count])
+  if ([_pathStack count])
     [self backtrack];
 }
 
@@ -1372,7 +1418,7 @@ static NSFileManager* defaultManager = nil;
 - (id) nextObject
 {
   [self findNextFile];
-  return currentFileName;
+  return _currentFileName;
 }
 
 @end /* NSDirectoryEnumerator */
@@ -1381,8 +1427,8 @@ static NSFileManager* defaultManager = nil;
 - (NSDictionary*) _attributesForCopy
 {
   return [[NSFileManager defaultManager]
-    _attributesAtPath: currentFilePath
-	 traverseLink: flags.isFollowing
+    _attributesAtPath: _currentFilePath
+	 traverseLink: _flags.isFollowing
 	      forCopy: YES];
 }
 @end
@@ -1578,7 +1624,7 @@ static NSFileManager* defaultManager = nil;
 
 		  errorInfo = [NSDictionary dictionaryWithObjectsAndKeys: 
 		    destinationFile, @"Path",
-		    lastError, @"Error", nil];
+		    _lastError, @"Error", nil];
 		  if (![handler fileManager: self
 		    shouldProceedAfterError: errorInfo])
 		    return NO;
@@ -1629,7 +1675,7 @@ static NSFileManager* defaultManager = nil;
 
 	  s = [NSString stringWithFormat: @"cannot copy file type '%@'",
 	    fileType];
-	  ASSIGN(lastError, s);
+	  ASSIGN(_lastError, s);
 	  NSLog(@"%@: %@", sourceFile, s);
 	  continue;
 	}
