@@ -32,10 +32,9 @@
 #include <Foundation/NSDictionary.h>
 #include <Foundation/NSCharacterSet.h>
 #include <Foundation/NSRange.h>
+#include <Foundation/NSException.h>
 #include <base/NSGString.h>
 #include <base/NSGCString.h>
-#include <base/IndexedCollection.h>
-#include <base/IndexedCollectionPrivate.h>
 #include <Foundation/NSValue.h>
 #include <base/behavior.h>
 
@@ -102,7 +101,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
   return NSAllocateObject(self, 0, NSDefaultMallocZone());
 }
 
-- (void)dealloc
+- (void) dealloc
 {
   if (_zone)
     {
@@ -115,9 +114,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 - (unsigned) hash
 {
   if (_hash == 0)
-    {
-      _hash = _fastImp._NSString_hash(self, @selector(hash));
-    }
+    _hash = _fastImp._NSString_hash(self, @selector(hash));
   return _hash;
 }
 
@@ -133,7 +130,11 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 {
   _count = length;
   _contents_chars = (unsigned char*)byteString;
+#if	GS_WITH_GC
+  _zone = byteString ? GSAtomicMallocZone() : 0;
+#else
   _zone = byteString ? zone : 0;
+#endif
   return self;
 }
 
@@ -215,7 +216,11 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
 			     at: &_count];
   if (_count > 0)
     {
+#if	GS_WITH_GC
+      _zone = GSAtomicMallocZone();
+#else
       _zone = fastZone(self);
+#endif
       _contents_chars = NSZoneMalloc(_zone, _count);
       [aCoder decodeArrayOfObjCType: @encode(unsigned char)
 			      count: _count
@@ -553,10 +558,7 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
     return NO;
 }
 
-
-// FOR IndexedCollection SUPPORT;
-
-- objectAtIndex: (unsigned)index
+- (id) objectAtIndex: (unsigned)index
 {
   CHECK_INDEX_RANGE_ERROR(index, _count);
   return [NSNumber numberWithChar: _contents_chars[index]];
@@ -587,7 +589,9 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
   return [self initWithCStringNoCopy: buf length: length fromZone: z];
 }
 
-- (void) descriptionTo: (id<GNUDescriptionDestination>)output
+- (void) descriptionWithLocale: (NSDictionary*)aLocale
+			indent: (unsigned) level
+			    to: (id<GNUDescriptionDestination>)output
 {
   if (output == nil)
     return;
@@ -796,6 +800,77 @@ static	IMP	msInitImp;	/* designated initialiser for mutable	*/
     return strRangeCsNs(self, aString, mask, aRange);
 }
 
+- (BOOL) boolValue
+{
+  if (_count == 0)
+    {
+      return 0;
+    }
+  else if (_count == 3
+    && (_contents_chars[0] == 'Y' || _contents_chars[0] == 'y')
+    && (_contents_chars[1] == 'E' || _contents_chars[1] == 'e')
+    && (_contents_chars[2] == 'S' || _contents_chars[2] == 's'))
+    {
+      return YES;
+    }
+  else
+    {
+      char	buf[_count+1];
+
+      memcpy(buf, _contents_chars, _count);
+      buf[_count] = '\0';
+      return atoi(buf);
+    }
+}
+
+- (double) doubleValue
+{
+  if (_count == 0)
+    {
+      return 0;
+    }
+  else
+    {
+      char	buf[_count+1];
+
+      memcpy(buf, _contents_chars, _count);
+      buf[_count] = '\0';
+      return atof(buf);
+    }
+}
+
+- (float) floatValue
+{
+  if (_count == 0)
+    {
+      return 0;
+    }
+  else
+    {
+      char	buf[_count+1];
+
+      memcpy(buf, _contents_chars, _count);
+      buf[_count] = '\0';
+      return (float) atof(buf);
+    }
+}
+
+- (int) intValue
+{
+  if (_count == 0)
+    {
+      return 0;
+    }
+  else
+    {
+      char	buf[_count+1];
+
+      memcpy(buf, _contents_chars, _count);
+      buf[_count] = '\0';
+      return atoi(buf);
+    }
+}
+
 @end
 
 
@@ -841,39 +916,51 @@ static inline void
 stringIncrementCountAndMakeHoleAt(NSGMutableCStringStruct *self, 
 				  int index, int size)
 {
+  if (size > 0)
+    {
+      if (self->_count > 0)
+	{
 #ifndef STABLE_MEMCPY
-  {
-    unsigned i = self->_count;
+	  unsigned i = self->_count;
 
-    while (i-- > index)
-      self->_contents_chars[i+size] = self->_contents_chars[i];
-  }
+	  while (i-- > index)
+	    {
+	      self->_contents_chars[i+size] = self->_contents_chars[i];
+	    }
 #else
-  memcpy(self->_contents_chars + index, 
-	 self->_contents_chars + index + size,
-	 self->_count - index);
+	  memcpy(self->_contents_chars + index, 
+		 self->_contents_chars + index + size,
+		 self->_count - index);
 #endif /* STABLE_MEMCPY */
-  (self->_count) += size;
-  (self->_hash) = 0;
+	}
+      self->_count += size;
+      self->_hash = 0;
+    }
 }
 
 static inline void
 stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self, 
 				  int index, int size)
 {
-  (self->_count) -= size;
+  if (size > 0)
+    {
+      self->_count -= size;
 #ifndef STABLE_MEMCPY
-  {
-    int i;
-    for (i = index; i <= self->_count; i++)
-      self->_contents_chars[i] = self->_contents_chars[i+size];
-  }
+      {
+	int i;
+
+	for (i = index; i <= self->_count; i++)
+	  {
+	    self->_contents_chars[i] = self->_contents_chars[i+size];
+	  }
+      }
 #else
-  memcpy(self->_contents_chars + index + size,
-	 self->_contents_chars + index, 
-	 self->_count - index);
+      memcpy(self->_contents_chars + index + size,
+	     self->_contents_chars + index, 
+	     self->_count - index);
 #endif /* STABLE_MEMCPY */
-  (self->_hash) = 0;
+      self->_hash = 0;
+    }
 }
 
 /* This is the designated initializer for this class */
@@ -883,7 +970,11 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
   _capacity = capacity;
   if (capacity)
     {
+#if	GS_WITH_GC
+      _zone = GSAtomicMallocZone();
+#else
       _zone = fastZone(self);
+#endif
       _contents_chars = NSZoneMalloc(_zone, _capacity);
     }
   return self;
@@ -899,7 +990,11 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
       _count = length;
       _capacity = length;
       _contents_chars = (unsigned char*)byteString;
+#if	GS_WITH_GC
+      _zone = byteString ? GSAtomicMallocZone() : 0;
+#else
       _zone = byteString ? zone : 0;
+#endif
     }
   return self;
 }
@@ -1092,11 +1187,7 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
   return [self initWithCStringNoCopy: 0 length: 0 fromZone: 0];
 }
 
-/* For IndexedCollecting Protocol and other GNU libobjects conformity. */
-
-/* xxx This should be made to return void, but we need to change
-   IndexedCollecting and its conformers */
-- (void) removeRange: (IndexRange)range
+- (void) removeRange: (NSRange)range
 {
   stringDecrementCountAndFillHoleAt((NSGMutableCStringStruct*)self, 
 				    range.location, range.length);
@@ -1118,8 +1209,6 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
   return self;
 }
 
-/* For IndexedCollecting protocol */
-
 - (char) charAtIndex: (unsigned)index
 {
   CHECK_INDEX_RANGE_ERROR(index, _count);
@@ -1127,8 +1216,6 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
 }
 
 
-// FOR IndexedCollection and OrderedCollection SUPPORT;
-
 - (void) insertObject: newObject atIndex: (unsigned)index
 {
   CHECK_INDEX_RANGE_ERROR(index, _count+1);
@@ -1258,7 +1345,35 @@ stringDecrementCountAndFillHoleAt(NSGMutableCStringStruct *self,
 
 - (unsigned) hash
 {
-  return _fastImp._NSString_hash(self, @selector(hash));
+  unsigned ret = 0;
+
+  int len = _count;
+
+  if (len > NSHashStringLength)
+    len = NSHashStringLength;
+  if (len)
+    {
+      const unsigned char	*p;
+      unsigned	char_count = 0;
+
+      p = _contents_chars;
+      while (*p && char_count++ < NSHashStringLength)
+	{
+	  ret = (ret << 5) + ret + *p++;
+	}
+
+      /*
+       * The hash caching in our concrete string classes uses zero to denote
+       * an empty cache value, so we MUST NOT return a hash of zero.
+       */
+      if (ret == 0)
+	ret = 0xffffffff;
+    }
+  else
+    {
+      ret = 0xfffffffe;	/* Hash for an empty string.	*/
+    }
+  return ret;
 }
 
 - (BOOL) isEqual: (id)anObject
