@@ -39,8 +39,7 @@
 #include <Foundation/NSDictionary.h>
 #include <Foundation/NSCharacterSet.h>
 #include <Foundation/NSRange.h>
-#include <base/IndexedCollection.h>
-#include <base/IndexedCollectionPrivate.h>
+#include <Foundation/NSException.h>
 #include <Foundation/NSValue.h>
 #include <base/behavior.h>
 /* memcpy(), strlen(), strcmp() are gcc builtin's */
@@ -185,7 +184,11 @@
     {
       _count = length;
       _contents_chars = chars;
+#if	GS_WITH_GC
+      _zone = chars ? GSAtomicMallocZone() : 0;
+#else
       _zone = chars ? zone : 0;
+#endif
     }
   return self;
 }
@@ -202,7 +205,11 @@
       _contents_chars = chars;
       if (flag && chars)
 	{
+#if	GS_WITH_GC
+	  _zone = GSAtomicMallocZone();
+#else
 	  _zone = NSZoneFromPointer(chars);
+#endif
 	}
       else
 	{
@@ -299,7 +306,7 @@
   char *r = (char*)_fastMallocBuffer(_count+1);
 
   if (_count > 0)
-    ustrtostr(r,_contents_chars, _count);
+    ustrtostr(r, _contents_chars, _count);
   r[_count] = '\0';
   return r;
 }
@@ -350,7 +357,11 @@
   [aCoder decodeValueOfObjCType: @encode(unsigned) at: &_count];
   if (_count)
     {
+#if	GS_WITH_GC
+      _zone = GSAtomicMallocZone();
+#else
       _zone = fastZone(self);
+#endif
       _contents_chars = NSZoneMalloc(_zone, sizeof(unichar)*_count);
       [aCoder decodeArrayOfObjCType: @encode(unichar)
 			      count: _count
@@ -487,6 +498,81 @@
     return strRangeUsNs(self, aString, mask, aRange);
 }
 
+
+- (BOOL) boolValue
+{
+  if (_count == 0)
+    {
+      return 0;
+    }
+  else
+    {
+      char	buf[_count+1];
+
+      ustrtostr(buf, _contents_chars, _count);
+      buf[_count] = '\0';
+      if (_count == 3
+	&& (_contents_chars[0] == 'Y' || _contents_chars[0] == 'y')
+	&& (_contents_chars[1] == 'E' || _contents_chars[1] == 'e')
+	&& (_contents_chars[2] == 'S' || _contents_chars[2] == 's'))
+	{
+	  return YES;
+	}
+      else
+	{
+	  return atoi(buf);
+	}
+    }
+}
+
+- (double) doubleValue
+{
+  if (_count == 0)
+    {
+      return 0;
+    }
+  else
+    {
+      char	buf[_count+1];
+
+      ustrtostr(buf, _contents_chars, _count);
+      buf[_count] = '\0';
+      return atof(buf);
+    }
+}
+
+- (float) floatValue
+{
+  if (_count == 0)
+    {
+      return 0;
+    }
+  else
+    {
+      char	buf[_count+1];
+
+      ustrtostr(buf, _contents_chars, _count);
+      buf[_count] = '\0';
+      return (float) atof(buf);
+    }
+}
+
+- (int) intValue
+{
+  if (_count == 0)
+    {
+      return 0;
+    }
+  else
+    {
+      char	buf[_count+1];
+
+      ustrtostr(buf, _contents_chars, _count);
+      buf[_count] = '\0';
+      return atoi(buf);
+    }
+}
+
 @end
 
 
@@ -524,47 +610,58 @@ static inline void
 stringIncrementCountAndMakeHoleAt(NSGMutableStringStruct *self, 
 				  int index, int size)
 {
-  if (self->_count || size)
+  if (size > 0)
+    {
+      if (self->_count > 0)
 	{
 	  NSCAssert(index+size<=self->_count,@"index+size>length");
 	  NSCAssert(self->_count+size<=self->_capacity,@"length+size>capacity");
 #ifndef STABLE_MEMCPY
 	  {
-		int i;
-		for (i = self->_count; i >= index; i--)
-		  self->_contents_chars[i+size] = self->_contents_chars[i];
+	    int i;
+
+	    for (i = self->_count; i >= index; i--)
+	      {
+		self->_contents_chars[i+size] = self->_contents_chars[i];
+	      }
 	  }
 #else
-	  memcpy(self->_contents_chars + index, 
-			 self->_contents_chars + index + size,
-			 2*(self->_count - index));
+	  memcpy(self->_contents_chars + index,
+	    self->_contents_chars + index + size, 2*(self->_count - index));
 #endif /* STABLE_MEMCPY */
-	  (self->_count) += size;
-	};
-  (self->_hash) = 0;
+	  self->_count += size;
+	}
+      self->_hash = 0;
+    }
 }
 
 static inline void
 stringDecrementCountAndFillHoleAt(NSGMutableStringStruct *self, 
 				  int index, int size)
 {
-  if (self->_count || size)
+  if (size > 0)
+    {
+      if (self->_count > 0)
 	{
 	  NSCAssert(index+size<=self->_count,@"index+size>length");
-	  (self->_count) -= size;
+	  self->_count -= size;
 #ifndef STABLE_MEMCPY
 	  {
-		int i;
-		for (i = index; i <= self->_count; i++)
-		  self->_contents_chars[i] = self->_contents_chars[i+size];
+	    int i;
+
+	    for (i = index; i <= self->_count; i++)
+	      {
+		self->_contents_chars[i] = self->_contents_chars[i+size];
+	      }
 	  }
 #else
 	  memcpy(self->_contents_chars + index + size,
 			 self->_contents_chars + index, 
 			 2*(self->_count - index));
 #endif // STABLE_MEMCPY
-	};
-  (self->_hash) = 0;
+	}
+      self->_hash = 0;
+    }
 }
 
 // Initializing Newly Allocated Strings
@@ -585,7 +682,11 @@ stringDecrementCountAndFillHoleAt(NSGMutableStringStruct *self,
       _count = length;
       _capacity = length;
       _contents_chars = chars;
+#if	GS_WITH_GC
+      _zone = _zone ? GSAtomicMallocZone() : 0;
+#else
       _zone = zone;
+#endif
     }
   return self;
 }
@@ -602,7 +703,11 @@ stringDecrementCountAndFillHoleAt(NSGMutableStringStruct *self,
       _contents_chars = chars;
       if (flag && chars)
 	{
+#if	GS_WITH_GC
+	  _zone = GSAtomicMallocZone();
+#else
 	  _zone = NSZoneFromPointer(chars);
+#endif
 	}
       else
 	{
@@ -623,7 +728,11 @@ stringDecrementCountAndFillHoleAt(NSGMutableStringStruct *self,
 	}
       _count = 0;
       _capacity = capacity;
+#if	GS_WITH_GC
+      _zone = GSAtomicMallocZone();
+#else
       _zone = fastZone(self);
+#endif
       _contents_chars = NSZoneMalloc(_zone, sizeof(unichar)*capacity);
     }
   return self;
@@ -719,11 +828,7 @@ stringDecrementCountAndFillHoleAt(NSGMutableStringStruct *self,
   _hash = 0;
 }
 
-/* For IndexedCollecting Protocol and other GNU libobjects conformity. */
-
-/* xxx This should be made to return void, but we need to change
-   IndexedCollecting and its conformers */
-- (void) removeRange: (IndexRange)range
+- (void) removeRange: (NSRange)range
 {
   stringDecrementCountAndFillHoleAt((NSGMutableStringStruct*)self, 
 				    range.location, range.length);
@@ -745,8 +850,6 @@ stringDecrementCountAndFillHoleAt(NSGMutableStringStruct *self,
   return self;
 }
 
-/* For IndexedCollecting protocol */
-
 - (char) charAtIndex: (unsigned)index
 {
   CHECK_INDEX_RANGE_ERROR(index, _count);
