@@ -29,79 +29,174 @@
 #include <Foundation/NSHost.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSDictionary.h>
+#include <Foundation/NSSet.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSCoder.h>
 #include <netdb.h>
 /* #include <libc.h>*/
 
-#if	defined(__WIN32__) && !defined(__CYGWIN__)
-#include <Windows32/Sockets.h>
+#if defined(__MINGW__)
+#include <winsock.h>
 #else
 #include <unistd.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#endif /* __WIN32__*/
+#endif /* !__MINGW__*/
 
 #ifndef	INADDR_NONE
 #define	INADDR_NONE	-1
 #endif
 
+static NSString			*localHostName = @"GNUstep local host";
+static Class			hostClass;
 static NSLock			*_hostCacheLock = nil;
 static BOOL			_hostCacheEnabled = YES;
 static NSMutableDictionary	*_hostCache = nil;
+static NSString			*myHostName = nil;
+
 
 @interface NSHost (Private)
++ (struct hostent*) _entryForAddress: (NSString*)address;
 - (id) _initWithHostEntry: (struct hostent*)entry key: (NSString*)key;
++ (NSMutableSet*) _localAddresses;
 @end
 
 @implementation NSHost (Private)
 
++ (struct hostent*) _entryForAddress: (NSString*)address
+{
+  struct hostent	*h = 0;
+  struct in_addr	hostaddr;
+
+#ifndef	HAVE_INET_ATON
+  hostaddr.s_addr = inet_addr([address cString]);
+  if (hostaddr.s_addr == INADDR_NONE)
+    {
+      NSLog(@"Attempt to lookup host entry for bad IP address (%@)", address);
+    }
+#else
+  if (inet_aton([address cString], (struct in_addr*)&hostaddr.s_addr) == 0)
+    {
+      NSLog(@"Attempt to lookup host entry for bad IP address (%@)", address);
+    }
+#endif
+  else
+    {
+      h = gethostbyaddr((char*)&hostaddr, sizeof(hostaddr), AF_INET);
+      if (h == 0)
+	{
+	  NSLog(@"Host '%@' not found using 'gethostbyaddr()' - perhaps "
+	    @"the address is wrong or networking is not set up on your "
+	    @"machine", address);
+	}
+    }
+  return h;
+}
+
 - (id) _initWithHostEntry: (struct hostent*)entry key: (NSString*)name
 {
   int			i;
   char			*ptr;
   struct in_addr	in;
   NSString		*h_name;
+  NSMutableSet		*names;
+  NSMutableSet		*addresses;
+  NSMutableSet		*extra;
 
   if ((self = [super init]) == nil)
     {
       return nil;
     }
-  if (name == nil)
+  if (name != localHostName && entry == (struct hostent*)NULL)
     {
+      NSLog(@"Host '%@' init failed - perhaps the name/address is wrong or "
+	@"networking is not set up on your machine", name);
       RELEASE(self);
       return nil;
     }
-  if (entry == (struct hostent*)NULL)
+  else if (localHostName == nil && entry != (struct hostent*)NULL)
     {
+      NSLog(@"Nil hostname supplied but network database entry is not empty");
       RELEASE(self);
       return nil;
     }
 
-  _names = [NSMutableArray new];
-  _addresses = [NSMutableArray new];
+  names = [NSMutableSet new];
+  addresses = [NSMutableSet new];
 
-  h_name = [NSString stringWithCString: entry->h_name];
-  [_names addObject: h_name];
-
-  i = 0;
-  while ((ptr = entry->h_aliases[i++]) != 0)
+  if (name == localHostName)
     {
-      [_names addObject: [NSString stringWithCString: ptr]];
+      extra = [hostClass _localAddresses];
+    }
+  else
+    {
+      extra = nil;
     }
 
-  i = 0;
-  while ((ptr = entry->h_addr_list[i++]) != 0)
+  for (;;)
     {
-      NSString	*addr;
+      /*
+       * We remove all the IP addresses that we have added to the host so
+       * far from the set of extra addresses available on the current host.
+       * Then we try to find a new network database entry for one of the
+       * remaining extra addresses, and loop round to add all the names
+       * and addresses for that entry.
+       */
+      [extra minusSet: addresses];
+      while (entry == 0 && [extra count] > 0)
+	{
+	  NSString	*a = [extra anyObject];
 
-      memcpy((void*)&in.s_addr, (const void*)ptr, entry->h_length);
-      addr = [NSString stringWithCString: (char*)inet_ntoa(in)];
-      [_addresses addObject: addr];
+	  entry = [hostClass _entryForAddress: a];
+	  if (entry == 0)
+	    {
+	      /*
+	       * Can't find a database entry for this IP address, but since
+	       * we know the address is valid, we add it to the list of
+	       * addresses for this host anyway.
+	       */
+	      [addresses addObject: a];
+	      [extra removeObject: a];
+	    }
+	}
+      if (entry == 0)
+	{
+	  break;
+	}
+
+      h_name = [NSString stringWithCString: entry->h_name];
+      [names addObject: h_name];
+
+      if (entry->h_aliases != 0)
+	{
+	  i = 0;
+	  while ((ptr = entry->h_aliases[i++]) != 0)
+	    {
+	      [names addObject: [NSString stringWithCString: ptr]];
+	    }
+	}
+      if (entry->h_addr_list != 0)
+	{
+	  i = 0;
+	  while ((ptr = entry->h_addr_list[i++]) != 0)
+	    {
+	      NSString	*addr;
+
+	      memcpy((void*)&in.s_addr, (const void*)ptr, entry->h_length);
+	      addr = [NSString stringWithCString: (char*)inet_ntoa(in)];
+	      [addresses addObject: addr];
+	    }
+	}
+      entry = 0;
     }
 
+  _names = [names copy];
+  RELEASE(names);
+  _addresses = [addresses copy];
+  RELEASE(addresses);
+
   if (_hostCacheEnabled == YES)
     {
       [_hostCache setObject: self forKey: name];
@@ -110,6 +205,14 @@ static NSMutableDictionary	*_hostCache = nil;
   return self;
 }
 
++ (NSMutableSet*) _localAddresses
+{
+  NSMutableSet	*set;
+
+  set = [[self currentHost]->_addresses mutableCopy];
+  [set addObject: @"127.0.0.1"];
+  return AUTORELEASE(set);
+}
 @end
 
 @implementation NSHost
@@ -119,8 +222,6 @@ static NSMutableDictionary	*_hostCache = nil;
  */
 #define	GSMAXHOSTNAMELEN	255
 
-static NSString	*myHost = nil;
-
 + (void) initialize
 {
   if (self == [NSHost class])
@@ -128,24 +229,25 @@ static NSString	*myHost = nil;
       char	buf[GSMAXHOSTNAMELEN+1];
       int	res;
 
+      hostClass = self;
       res = gethostname(buf, GSMAXHOSTNAMELEN);
-      if (res < 0)
+      if (res < 0 || *buf == '\0')
 	{
 	  NSLog(@"Unable to get name of current host - using 'localhost'");
-	  myHost = @"localhost";
+	  myHostName = @"localhost";
 	}
       else
 	{
-	  myHost = [[NSString alloc] initWithCString: buf];
+	  myHostName = [[NSString alloc] initWithCString: buf];
 	}
-      _hostCacheLock = [[NSLock alloc] init];
+      _hostCacheLock = [[NSRecursiveLock alloc] init];
       _hostCache = [NSMutableDictionary new];
     }
 }
 
 + (NSHost*) currentHost
 {
-  return [self hostWithName: myHost];
+  return [self hostWithName: myHostName];
 }
 
 + (NSHost*) hostWithName: (NSString*)name
@@ -157,6 +259,11 @@ static NSString	*myHost = nil;
       NSLog(@"Nil host name sent to [NSHost +hostWithName:]");
       return nil;
     }
+  if ([name isEqual: @""] == YES)
+    {
+      NSLog(@"Empty host name sent to [NSHost +hostWithName:]");
+      return nil;
+    }
 
   [_hostCacheLock lock];
   if (_hostCacheEnabled == YES)
@@ -165,12 +272,32 @@ static NSString	*myHost = nil;
     }
   if (host == nil)
     {
-      struct hostent	*h;
+      if (name == localHostName)
+	{
+	  /*
+	   * Special GNUstep extension host - we try to have a host entry
+	   * with ALL the IP addresses of any interfaaces on the local machine
+	   */
+	  host = [[self alloc] _initWithHostEntry: 0 key: name];
+	  AUTORELEASE(host);
+	}
+      else
+	{
+	  struct hostent	*h = 0;
 
-      h = gethostbyname((char*)[name cString]);
-
-      host = [[self alloc] _initWithHostEntry: h key: name];
-      AUTORELEASE(host);
+	  h = gethostbyname((char*)[name cString]);
+	  if (h == 0)
+	    {
+	      NSLog(@"Host '%@' not found using 'gethostbyname()' - perhaps "
+		@"the hostname is wrong or networking is not set up on your "
+		@"machine", name);
+	    }
+	  else
+	    {
+	      host = [[self alloc] _initWithHostEntry: h key: name];
+	      AUTORELEASE(host);
+	    }
+	}
     }
   [_hostCacheLock unlock];
   return host;
@@ -185,6 +312,11 @@ static NSString	*myHost = nil;
       NSLog(@"Nil host address sent to [NSHost +hostWithName:]");
       return nil;
     }
+  if ([address isEqual: @""] == YES)
+    {
+      NSLog(@"Empty host address sent to [NSHost +hostWithName:]");
+      return nil;
+    }
 
   [_hostCacheLock lock];
   if (_hostCacheEnabled == YES)
@@ -195,24 +327,10 @@ static NSString	*myHost = nil;
   if (host == nil)
     {
       struct hostent	*h;
-      struct in_addr	hostaddr;
-      BOOL		addrOk = YES;
 
-#ifndef	HAVE_INET_ATON
-      hostaddr.s_addr = inet_addr([address cString]);
-      if (hostaddr.s_addr == INADDR_NONE)
+      h = [self _entryForAddress: address];
+      if (h != 0)
 	{
-	  addrOk = NO;
-	}
-#else
-      if (inet_aton([address cString], (struct in_addr*)&hostaddr.s_addr) == 0)
-	{
-	  addrOk = NO;
-	}
-#endif
-      if (addrOk == YES)
-	{
-	  h = gethostbyaddr((char*)&hostaddr, sizeof(hostaddr), AF_INET);
 	  host = [[self alloc] _initWithHostEntry: h key: address];
 	  AUTORELEASE(host);
 	}
@@ -230,13 +348,7 @@ static NSString	*myHost = nil;
 
 + (BOOL) isHostCacheEnabled;
 {
-  BOOL res;
-
-  [_hostCacheLock lock];
-  res = _hostCacheEnabled;
-  [_hostCacheLock unlock];
-
-  return res;
+  return _hostCacheEnabled;
 }
 
 + (void) flushHostCache
@@ -259,8 +371,21 @@ static NSString	*myHost = nil;
 
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  [super encodeWithCoder: aCoder];
-  [aCoder encodeObject: [self address]];
+  NSString	*address = [self address];
+
+  if ([address isEqual: @"127.0.0.1"] == YES)
+    {
+      NSEnumerator	*e = [_addresses objectEnumerator];
+
+      while ((address = [e nextObject]) != nil)
+	{
+	  if ([address isEqual: @"127.0.0.1"] == NO)
+	    {
+	      break;
+	    }
+	}
+    }
+  [aCoder encodeObject: address];
 }
 
 - (id) initWithCoder: (NSCoder*)aCoder
@@ -268,9 +393,16 @@ static NSString	*myHost = nil;
   NSString	*address;
   NSHost	*host;
 
-  self = [super initWithCoder: aCoder];
   address = [aCoder decodeObject];
-  host = RETAIN([NSHost hostWithAddress: address]);
+  if (address != nil)
+    {
+      host = [NSHost hostWithAddress: address];
+    }
+  else
+    {
+      host = [NSHost currentHost];
+    }
+  RETAIN(host);
   RELEASE(self);
   return host;
 }
@@ -306,52 +438,68 @@ static NSString	*myHost = nil;
 - (BOOL) isEqual: (id)other
 {
   if (other == self)
-    return YES;
+    {
+      return YES;
+    }
   if ([other isKindOfClass: [NSHost class]])
-    return [self isEqualToHost: (NSHost*)other];
+    {
+      return [self isEqualToHost: (NSHost*)other];
+    }
   return NO;
 }
 
 - (BOOL) isEqualToHost: (NSHost*)aHost
 {
-  NSArray*	a;
-  int		i;
+  NSEnumerator	*e;
+  NSString	*a;
 
   if (aHost == self)
-    return YES;
-
-  a = [aHost addresses];
-  for (i = 0; i < [a count]; i++)
-    if ([_addresses containsObject: [a objectAtIndex: i]])
+    {
       return YES;
-
+    }
+  e = [aHost->_addresses objectEnumerator];
+  while ((a = [e nextObject]) != nil)
+    {
+      if ([_addresses member: a] != nil)
+	{
+	  return YES;
+	}
+    }
   return NO;
 }
 
 - (NSString*) name
 {
-  return [_names objectAtIndex: 0];
+  return [_names anyObject];
 }
 
 - (NSArray*) names
 {
-  return _names;
+  return [_names allObjects];
 }
 
 - (NSString*) address
 {
-  return [_addresses objectAtIndex: 0];
+  return [_addresses anyObject];
 }
 
 - (NSArray*) addresses
 {
-  return _addresses;
+  return [_addresses allObjects];
 }
 
 - (NSString*) description
 {
   return [NSString stringWithFormat: @"Host %@ (%@ %@)",
-    [self name], [self names], [self addresses]];
+    [self name], _names, _addresses];
 }
 
 @end
+
+@implementation	NSHost (GNUstep)
++ (NSHost*) localHost
+{
+  return [self hostWithName: localHostName];
+}
+@end
+
