@@ -28,7 +28,6 @@
 #include <objc/Protocol.h>
 #include <objc/objc-api.h>
 #include <Foundation/NSMethodSignature.h>
-#include <base/Invocation.h>
 #include <Foundation/NSInvocation.h>
 #include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSString.h>
@@ -38,6 +37,9 @@
 #include <Foundation/NSPortCoder.h>
 #include <Foundation/NSDistantObject.h>
 #include <Foundation/NSZone.h>
+#include <Foundation/NSDebug.h>
+#include <Foundation/NSThread.h>
+#include <Foundation/NSNotification.h>
 #include <limits.h>
 
 #include <base/fast.x>
@@ -79,13 +81,13 @@ void	_fastBuildCache()
    */
 
   _fastImp._NSString_hash = (unsigned (*)())[_fastCls._NSString
-	    instanceMethodForSelector: @selector(hash)];
+    instanceMethodForSelector: @selector(hash)];
   _fastImp._NSString_isEqualToString_ = (BOOL (*)())[_fastCls._NSString
-	    instanceMethodForSelector: @selector(isEqualToString:)];
+    instanceMethodForSelector: @selector(isEqualToString:)];
   _fastImp._NSGString_isEqual_ = (BOOL (*)())[_fastCls._NSGString
-	    instanceMethodForSelector: @selector(isEqual:)];
+    instanceMethodForSelector: @selector(isEqual:)];
   _fastImp._NSGCString_isEqual_ = (BOOL (*)())[_fastCls._NSGCString
-	    instanceMethodForSelector: @selector(isEqual:)];
+    instanceMethodForSelector: @selector(isEqual:)];
 }
 
 
@@ -102,6 +104,12 @@ void	_fastBuildCache()
  *	correct zone to free memory very fast.
  */
 
+/*
+ * retain_counts_gate is needed when running multi-threaded for retain/release
+ * to work reliably.
+ */
+static objc_mutex_t retain_counts_gate = NULL;
+
 #if	GS_WITH_GC == 0
 #define	REFCNT_LOCAL	1
 #define	CACHE_ZONE	1
@@ -174,6 +182,7 @@ NSDecrementExtraRefCountWasZero(id anObject)
  *	depending on whether we are using local or global counting.
  */
 #if	defined(REFCNT_LOCAL)
+
 unsigned
 NSExtraRefCount(id anObject)
 {
@@ -183,23 +192,61 @@ NSExtraRefCount(id anObject)
 void
 NSIncrementExtraRefCount(id anObject)
 {
-  ((obj)anObject)[-1].retained++;
+  if (retain_counts_gate != 0)
+    {
+      objc_mutex_lock(retain_counts_gate);
+      ((obj)anObject)[-1].retained++;
+      objc_mutex_unlock (retain_counts_gate);
+    }
+  else
+    {
+      ((obj)anObject)[-1].retained++;
+    }
 }
 
-#define	NSIncrementExtraRefCount(X) \
-	((obj)(X))[-1].retained++
+#define	NSIncrementExtraRefCount(X) ({ \
+  if (retain_counts_gate != 0) \
+    { \
+      objc_mutex_lock(retain_counts_gate); \
+      ((obj)(X))[-1].retained++;            \
+      objc_mutex_unlock(retain_counts_gate); \
+    } \
+  else \
+    { \
+      ((obj)X)[-1].retained++; \
+    } \
+})
 
 BOOL
 NSDecrementExtraRefCountWasZero(id anObject)
 {
-  if (((obj)anObject)[-1].retained-- == 0)
-    return YES;
+  if (retain_counts_gate != 0)
+    {
+      objc_mutex_lock(retain_counts_gate);
+      if (((obj)anObject)[-1].retained-- == 0)
+	{
+	  objc_mutex_unlock(retain_counts_gate);
+	  return YES;
+	}
+      else
+	{
+	  objc_mutex_unlock(retain_counts_gate);
+	  return NO;
+	}
+    }
   else
-    return NO;
+    {
+      if (((obj)anObject)[-1].retained-- == 0)
+	{
+	  return YES;
+	}
+      else
+	{
+	  return NO;
+	}
+    }
 }
 
-#define	NSDecrementExtraRefCountWasZero(X) \
-	(((obj)(X))[-1].retained-- == 0 ? YES : NO)
 
 #define	NSExtraRefCount(X)	(((obj)(X))[-1].retained)
 
@@ -207,22 +254,31 @@ NSDecrementExtraRefCountWasZero(id anObject)
 
 /* The maptable of retain counts on objects */
 static o_map_t *retain_counts = NULL;
-/* The mutex lock to protect multi-threaded use of `retain_counts' */
-static objc_mutex_t retain_counts_gate = NULL;
 
 void
 NSIncrementExtraRefCount (id anObject)
 {
   o_map_node_t *node;
-  extern o_map_node_t *o_map_node_for_key (o_map_t *m, const void *k);
+  extern o_map_node_t *o_map_node_for_key(o_map_t *m, const void *k);
 
-  objc_mutex_lock (retain_counts_gate);
-  node = o_map_node_for_key (retain_counts, anObject);
-  if (node)
-    ((int)(node->value))++;
+  if (retain_counts_gate != 0)
+    {
+      objc_mutex_lock(retain_counts_gate);
+      node = o_map_node_for_key (retain_counts, anObject);
+      if (node)
+	((int)(node->value))++;
+      else
+	o_map_at_key_put_value_known_absent (retain_counts, anObject, (void*)1);
+      objc_mutex_unlock(retain_counts_gate);
+    }
   else
-    o_map_at_key_put_value_known_absent (retain_counts, anObject, (void*)1);
-  objc_mutex_unlock (retain_counts_gate);
+    {
+      node = o_map_node_for_key (retain_counts, anObject);
+      if (node)
+	((int)(node->value))++;
+      else
+	o_map_at_key_put_value_known_absent (retain_counts, anObject, (void*)1);
+    }
 }
 
 BOOL
@@ -232,17 +288,35 @@ NSDecrementExtraRefCountWasZero (id anObject)
   extern o_map_node_t *o_map_node_for_key (o_map_t *m, const void *k);
   extern void o_map_remove_node (o_map_node_t *node);
 
-  objc_mutex_lock (retain_counts_gate);
-  node = o_map_node_for_key (retain_counts, anObject);
-  if (!node)
+  if (retain_counts_gate != 0)
     {
-      objc_mutex_unlock (retain_counts_gate);
-      return YES;
+      objc_mutex_lock(retain_counts_gate);
+      node = o_map_node_for_key (retain_counts, anObject);
+      if (!node)
+	{
+	  objc_mutex_unlock(retain_counts_gate);
+	  return YES;
+	}
+      NSAssert((int)(node->value) > 0, NSInternalInconsistencyException);
+      if (!--((int)(node->value)))
+	{
+	  o_map_remove_node (node);
+	}
+      objc_mutex_unlock(retain_counts_gate);
+    }
+  else
+    {
+      node = o_map_node_for_key (retain_counts, anObject);
+      if (!node)
+	{
+	  return YES;
+	}
+      NSAssert((int)(node->value) > 0, NSInternalInconsistencyException);
+      if (!--((int)(node->value)))
+	{
+	  o_map_remove_node (node);
+	}
     }
-  NSAssert((int)(node->value) > 0, NSInternalInconsistencyException);
-  if (!--((int)(node->value)))
-    o_map_remove_node (node);
-  objc_mutex_unlock (retain_counts_gate);
   return NO;
 }
 
@@ -251,11 +325,26 @@ NSExtraRefCount (id anObject)
 {
   unsigned ret;
 
-  objc_mutex_lock (retain_counts_gate);
-  ret = (unsigned) o_map_value_at_key (retain_counts, anObject);
-  if (ret == (unsigned)o_map_not_a_key_marker(retain_counts) ||
-      ret == (unsigned)o_map_not_a_value_marker(retain_counts)) ret = 0;
-  objc_mutex_unlock (retain_counts_gate);
+  if (retain_counts_gate != 0)
+    {
+      objc_mutex_lock(retain_counts_gate);
+      ret = (unsigned) o_map_value_at_key(retain_counts, anObject);
+      if (ret == (unsigned)o_map_not_a_key_marker(retain_counts)
+	|| ret == (unsigned)o_map_not_a_value_marker(retain_counts))
+	{
+	  ret = 0;
+	}
+      objc_mutex_unlock(retain_counts_gate);
+    }
+  else
+    {
+      ret = (unsigned) o_map_value_at_key(retain_counts, anObject);
+      if (ret == (unsigned)o_map_not_a_key_marker(retain_counts)
+	|| ret == (unsigned)o_map_not_a_value_marker(retain_counts))
+	{
+	  ret = 0;
+	}
+    }
   return ret;	/* ExtraRefCount + 1	*/
 }
 
@@ -271,6 +360,9 @@ NSExtraRefCount (id anObject)
  */
 #if	GS_WITH_GC
 
+#include	<gc.h>
+#include	<gc_typed.h>
+
 inline NSZone *
 fastZone(NSObject *object)
 {
@@ -281,42 +373,59 @@ static void
 GSFinalize(void* object, void* data)
 {
   [(id)object gcFinalize];
-
-  /* Set the class of anObject to FREED_OBJECT. The further messages to this
-     object will cause an error to occur. */
-  ((id)object)->class_pointer = __freedObjectClass;
-
 #ifndef	NDEBUG
   GSDebugAllocationRemove(((id)object)->class_pointer);
 #endif
-  ((id)object)->class_pointer = (void*) 0xdeadface;
+  ((id)object)->class_pointer = (void*)0xdeadface;
 }
 
 inline NSObject *
-NSAllocateObject (Class aClass, unsigned extraBytes, NSZone *zone)
+NSAllocateObject(Class aClass, unsigned extraBytes, NSZone *zone)
 {
-  id new = nil;
-  int size = aClass->instance_size + extraBytes;
-  if (CLS_ISCLASS (aClass))
-    new = NSZoneMalloc (zone, size);
-  if (new != nil)
+  id	new = nil;
+  int	size = aClass->instance_size + extraBytes;
+
+  NSCAssert((CLS_ISCLASS(aClass)), @"Bad class for new object");
+  if (zone == GSAtomicMallocZone())
     {
-      memset (new, 0, size);
-      new->class_pointer = aClass;
+      new = NSZoneMalloc(zone, size);
     }
-  if ([new respondsToSelector: @selector(gcFinalize)])
+  else
     {
-#ifndef	NDEBUG
-      /*
-       *	We only do allocation counting for objects that can be
-       *	finalised - for other objects we have no way of decrementing
-       *	the count when the object is collected.
-       */
-      GSDebugAllocationAdd(aClass);
-#endif
-      GC_REGISTER_FINALIZER (new, GSFinalize, NULL, NULL, NULL);
+      GC_descr	gc_type = (GC_descr)aClass->gc_object_type;
+
+      if (gc_type == 0)
+	{
+	  new = NSZoneMalloc(zone, size);
+	  NSLog(@"No garbage collection information for '%s'", aClass->name);
+	}
+      else if ([aClass requiresTypedMemory])
+	{
+	  new = GC_CALLOC_EXPLICTLY_TYPED(1, size, gc_type);
+        }
+      else
+	{
+	  new = NSZoneMalloc(zone, size);
+	}
     }
 
+  if (new != nil)
+    {
+      memset(new, 0, size);
+      new->class_pointer = aClass;
+      if (__objc_responds_to(new, @selector(gcFinalize)))
+	{
+#ifndef	NDEBUG
+	  /*
+	   *	We only do allocation counting for objects that can be
+	   *	finalised - for other objects we have no way of decrementing
+	   *	the count when the object is collected.
+	   */
+	  GSDebugAllocationAdd(aClass);
+#endif
+	  GC_REGISTER_FINALIZER (new, GSFinalize, NULL, NULL, NULL);
+	}
+    }
   return new;
 }
 
@@ -474,25 +583,62 @@ static IMP autorelease_imp = 0;
    This does not need mutex protection. */
 static BOOL double_release_check_enabled = NO;
 
+
 
 @implementation NSObject
 
++ (void) _becomeMultiThreaded: (NSNotification)aNotification
+{
+  if (retain_counts_gate == 0)
+    {
+      retain_counts_gate = objc_mutex_allocate();
+    }
+}
+
+#if	GS_WITH_GC
++ (BOOL) requiresTypedMemory
+{
+  return NO;
+}
+#endif
+
 + (void) initialize
 {
   if (self == [NSObject class])
     {
+      extern void	GSBuildStrings();	// See externs.m
+#ifdef __FreeBSD__
+      // Manipulate the FPU to add the exception mask. (Fixes SIGFPE
+      // problems on *BSD)
+
+      volatile short cw;
+
+      __asm__ volatile ("fstcw (%0)" : : "g" (&cw));
+      cw |= 1; /* Mask 'invalid' exception */
+      __asm__ volatile ("fldcw (%0)" : : "g" (&cw));
+#endif
+
       // Create the global lock
       gnustep_global_lock = [[NSRecursiveLock alloc] init];
-#if !defined(REFCNT_LOCAL)
-      retain_counts = o_map_with_callbacks (o_callbacks_for_non_owned_void_p,
-					    o_callbacks_for_int);
-      retain_counts_gate = objc_mutex_allocate ();
-#endif
       autorelease_class = [NSAutoreleasePool class];
       autorelease_imp = [autorelease_class methodForSelector: autorelease_sel];
       fastMallocClass = [_FastMallocBuffer class];
+#if	GS_WITH_GC == 0
+#if	!defined(REFCNT_LOCAL)
+      retain_counts = o_map_with_callbacks (o_callbacks_for_non_owned_void_p,
+					    o_callbacks_for_int);
+#endif
       fastMallocOffset = fastMallocClass->instance_size % ALIGN;
+#else
+      fastMallocOffset = 0;
+#endif
       _fastBuildCache();
+      GSBuildStrings();
+      [[NSNotificationCenter defaultCenter]
+	addObserver: self
+	   selector: @selector(_becomeMultiThreaded:)
+	       name: NSWillBecomeMultiThreadedNotification
+	     object: nil];
     }
   return;
 }
@@ -527,10 +673,27 @@ static BOOL double_release_check_enabled = NO;
   NSDeallocateObject (self);
 }
 
-- free
+static BOOL deallocNotifications = NO;
+
+- (BOOL) deallocNotificationsActive
+{
+  return deallocNotifications;
+}
+
+- (void) setDeallocNotificationsActive: (BOOL)flag
+{
+  deallocNotifications = flag;
+}
+
+- (BOOL) _dealloc
+{
+  return YES;
+}
+
+- (id) free
 {
   [NSException raise: NSGenericException
-	       format: @"Use `dealloc' instead of `free' for %@.", self];
+	      format: @"Use `dealloc' instead of `free' for %@.", self];
   return nil;
 }
 
@@ -611,48 +774,76 @@ static BOOL double_release_check_enabled = NO;
 
 + (NSMethodSignature*) instanceMethodSignatureForSelector: (SEL)aSelector
 {
-    struct objc_method* mth = class_get_instance_method(self, aSelector);
-    return mth ? [NSMethodSignature signatureWithObjCTypes:mth->method_types]
-		: nil;
+  struct objc_method* mth = class_get_instance_method(self, aSelector);
+  return mth ? [NSMethodSignature signatureWithObjCTypes:mth->method_types]
+    : nil;
 }
   
 - (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector
 {
-    struct objc_method* mth =
-	    (object_is_instance(self) ?
-		  class_get_instance_method(self->isa, aSelector)
-		: class_get_class_method(self->isa, aSelector));
-    return mth ? [NSMethodSignature signatureWithObjCTypes:mth->method_types]
-		: nil;
+  const char	*types = aSelector->sel_types;
+
+  if (types == 0)
+    {
+      struct objc_method *mth;
+
+      mth = (object_is_instance(self)
+	? class_get_instance_method(self->isa, aSelector)
+	: class_get_class_method(self->isa, aSelector));
+      if (mth == 0)
+	{
+	  return nil;
+	}
+      types = mth->method_types;
+    }
+  return [NSMethodSignature signatureWithObjCTypes: types];
 }
 
 - (NSString*) description
 {
-  return [NSString stringWithFormat: @"<%s %lx>",
-	object_get_class_name(self), (unsigned long)self];
+  return [NSString stringWithFormat: @"<%s: %lx>",
+    object_get_class_name(self), (unsigned long)self];
 }
 
 + (NSString*) description
 {
-  return [NSString stringWithFormat: @"<%s>", object_get_class_name(self)];
+  return [NSString stringWithCString: object_get_class_name(self)];
 }
 
-- (void) descriptionTo: (id<GNUDescriptionDestination>)output
+- (NSString*) descriptionWithLocale: (NSDictionary*)locale
 {
-  [output appendString: [self description]];
+  return [self description];
 }
 
-- (void) descriptionWithLocale: (NSDictionary*)aLocale
-			    to: (id<GNUDescriptionDestination>)output
++ (NSString*) descriptionWithLocale: (NSDictionary*)locale
 {
-  [output appendString: [(id)self descriptionWithLocale: aLocale]];
+  return [self description];
+}
+
+- (NSString*) descriptionWithLocale: (NSDictionary*)locale indent: (unsigned)c
+{
+  return [self descriptionWithLocale: locale];
+}
+
++ (NSString*) descriptionWithLocale: (NSDictionary*)locale indent: (unsigned)c
+{
+  return [self descriptionWithLocale: locale];
 }
 
 - (void) descriptionWithLocale: (NSDictionary*)aLocale
 			indent: (unsigned)level
 			    to: (id<GNUDescriptionDestination>)output
 {
-  [output appendString: [(id)self descriptionWithLocale: aLocale indent: level]];
+  [output appendString:
+    [(id)self descriptionWithLocale: aLocale indent: level]];
+}
+
++ (void) descriptionWithLocale: (NSDictionary*)aLocale
+			indent: (unsigned)level
+			    to: (id<GNUDescriptionDestination>)output
+{
+  [output appendString:
+    [(id)self descriptionWithLocale: aLocale indent: level]];
 }
 
 + (void) poseAsClass: (Class)aClassObject
@@ -705,7 +896,7 @@ static BOOL double_release_check_enabled = NO;
 
 - (Class) classForPortCoder
 {
-    return [self classForCoder];
+  return [self classForCoder];
 }
 
 - (id) replacementObjectForArchiver: (NSArchiver*)anArchiver
@@ -720,22 +911,39 @@ static BOOL double_release_check_enabled = NO;
 
 - (id) replacementObjectForPortCoder: (NSPortCoder*)aCoder
 {
-    if ([aCoder isBycopy]) {
-	return self;
+  static Class	proxyClass = 0;
+  static IMP	proxyImp = 0;
+
+  if (proxyImp == 0)
+    {
+      proxyClass = [NSDistantObject class];
+      /*
+       * use get_imp() because NSDistantObject doesn't implement
+       * methodForSelector:
+       */
+      proxyImp = get_imp(fastClass((id)proxyClass),
+	@selector(proxyWithLocal:connection:));
     }
-    else if ([self isKindOfClass: [NSDistantObject class]]) {
-	return self;
+
+  if ([aCoder isBycopy])
+    {
+      return self;
     }
-    else {
-	return [NSDistantObject proxyWithLocal: self
-				    connection: [aCoder connection]];
+  else if ([self isKindOfClass: proxyClass])
+    {
+      return self;
+    }
+  else
+    {
+      return (*proxyImp)(proxyClass, @selector(proxyWithLocal:connection:),
+	self, [aCoder connection]);
     }
 }
 
 
 /* NSObject protocol */
 
-- autorelease
+- (id) autorelease
 {
 #if	GS_WITH_GC == 0
   if (double_release_check_enabled)
@@ -755,7 +963,7 @@ static BOOL double_release_check_enabled = NO;
   return self;
 }
 
-+ autorelease
++ (id) autorelease
 {
   return self;
 }
@@ -775,7 +983,7 @@ static BOOL double_release_check_enabled = NO;
   return (unsigned)self;
 }
 
-- (BOOL) isEqual: anObject
+- (BOOL) isEqual: (id)anObject
 {
   return (self == anObject);
 }
@@ -816,7 +1024,7 @@ static BOOL double_release_check_enabled = NO;
   return NO;
 }
 
-- performSelector: (SEL)aSelector
+- (id) performSelector: (SEL)aSelector
 {
   IMP msg;
 
@@ -837,7 +1045,7 @@ static BOOL double_release_check_enabled = NO;
   return (*msg)(self, aSelector);
 }
 
-- performSelector: (SEL)aSelector withObject: anObject
+- (id) performSelector: (SEL)aSelector withObject: (id) anObject
 {
   IMP msg;
 
@@ -859,7 +1067,9 @@ static BOOL double_release_check_enabled = NO;
   return (*msg)(self, aSelector, anObject);
 }
 
-- performSelector: (SEL)aSelector withObject: object1 withObject: object2
+- (id) performSelector: (SEL)aSelector
+	    withObject: (id) object1
+	    withObject: (id) object2
 {
   IMP msg;
 
@@ -895,7 +1105,12 @@ static BOOL double_release_check_enabled = NO;
     }
 
   if (NSDecrementExtraRefCountWasZero(self))
-    [self dealloc];
+    {
+      if (deallocNotifications == NO || [self _dealloc] == YES)
+	{
+	  [self dealloc];
+	}
+    }
 #endif
 }
 
@@ -916,7 +1131,7 @@ static BOOL double_release_check_enabled = NO;
 #endif
 }
 
-- retain
+- (id) retain
 {
 #if	GS_WITH_GC == 0
   NSIncrementExtraRefCount(self);
@@ -924,7 +1139,7 @@ static BOOL double_release_check_enabled = NO;
   return self;
 }
 
-+ retain
++ (id) retain
 {
   return self;
 }
@@ -943,14 +1158,14 @@ static BOOL double_release_check_enabled = NO;
   return UINT_MAX;
 }
 
-- self
+- (id) self
 {
   return self;
 }
 
-- (NSZone *)zone
+- (NSZone*) zone
 {
-    return fastZone(self);
+  return fastZone(self);
 }
 
 - (void) encodeWithCoder: (NSCoder*)aCoder
@@ -958,17 +1173,17 @@ static BOOL double_release_check_enabled = NO;
   return;
 }
 
-- initWithCoder: (NSCoder*)aDecoder
+- (id) initWithCoder: (NSCoder*)aDecoder
 {
   return self;
 }
 
-+ (int)version
++ (int) version
 {
   return class_get_version(self);
 }
 
-+ setVersion:(int)aVersion
++ (id) setVersion: (int)aVersion
 {
   if (aVersion < 0)
     [NSException raise: NSInvalidArgumentException
@@ -985,7 +1200,7 @@ static BOOL double_release_check_enabled = NO;
 
 /* NEXTSTEP Object class compatibility */
 
-- error:(const char *)aString, ...
+- error: (const char *)aString, ...
 {
 #define FMT "error: %s (%s)\n%s\n"
   char fmt[(strlen((char*)FMT)+strlen((char*)object_get_class_name(self))
@@ -1008,66 +1223,66 @@ static BOOL double_release_check_enabled = NO;
   return object_get_class_name(self);
 }
 
-- (BOOL)isKindOf:(Class)aClassObject
+- (BOOL) isKindOf: (Class)aClassObject
 {
-  return [self isKindOfClass:aClassObject];
+  return [self isKindOfClass: aClassObject];
 }
 
-- (BOOL)isMemberOf:(Class)aClassObject
+- (BOOL) isMemberOf: (Class)aClassObject
 {
-  return [self isMemberOfClass:aClassObject];
+  return [self isMemberOfClass: aClassObject];
 }
 
-+ (BOOL)instancesRespondTo:(SEL)aSel
++ (BOOL) instancesRespondTo: (SEL)aSel
 {
-  return [self instancesRespondToSelector:aSel];
+  return [self instancesRespondToSelector: aSel];
 }
 
-- (BOOL)respondsTo:(SEL)aSel
+- (BOOL) respondsTo: (SEL)aSel
 {
-  return [self respondsToSelector:aSel];
+  return [self respondsToSelector: aSel];
 }
 
 + (BOOL) conformsTo: (Protocol*)aProtocol
 {
-  return [self conformsToProtocol:aProtocol];
+  return [self conformsToProtocol: aProtocol];
 }
 
 - (BOOL) conformsTo: (Protocol*)aProtocol
 {
-  return [self conformsToProtocol:aProtocol];
+  return [self conformsToProtocol: aProtocol];
 }
 
-- (retval_t)performv:(SEL)aSel :(arglist_t)argFrame
+- (retval_t) performv: (SEL)aSel :(arglist_t)argFrame
 {
   return objc_msg_sendv(self, aSel, argFrame);
 }
 
-+ (IMP) instanceMethodFor:(SEL)aSel
++ (IMP) instanceMethodFor: (SEL)aSel
 {
   return [self instanceMethodForSelector:aSel];
 }
 
-+ (NSMethodSignature*)instanceMethodSignatureForSelector:(SEL)aSelector
++ (NSMethodSignature*) instanceMethodSignatureForSelector: (SEL)aSelector
 {
-    struct objc_method* mth = class_get_instance_method(self, aSelector);
+  struct objc_method* mth = class_get_instance_method(self, aSelector);
 
-    return mth ? [NSMethodSignature signatureWithObjCTypes:mth->method_types]
-		: nil;
+  return mth ? [NSMethodSignature signatureWithObjCTypes:mth->method_types]
+    : nil;
 }
 
-- (IMP) methodFor:(SEL)aSel
+- (IMP) methodFor: (SEL)aSel
 {
-  return [self methodForSelector:aSel];
+  return [self methodForSelector: aSel];
 }
 
-+ poseAs:(Class)aClassObject
++ (id) poseAs: (Class)aClassObject
 {
-  [self poseAsClass:aClassObject];
+  [self poseAsClass: aClassObject];
   return self;
 }
 
-- notImplemented:(SEL)aSel
+- (id) notImplemented: (SEL)aSel
 {
   [NSException
     raise: NSGenericException
@@ -1075,7 +1290,7 @@ static BOOL double_release_check_enabled = NO;
   return nil;
 }
 
-- doesNotRecognize:(SEL)aSel
+- (id) doesNotRecognize: (SEL)aSel
 {
   [NSException raise: NSGenericException
 	       format: @"%s does not recognize %s",
@@ -1083,12 +1298,12 @@ static BOOL double_release_check_enabled = NO;
   return nil;
 }
 
-- perform: (SEL)sel with: anObject
+- (id) perform: (SEL)sel with: (id)anObject
 {
   return [self performSelector:sel withObject:anObject];
 }
 
-- perform: (SEL)sel with: anObject with: anotherObject
+- (id) perform: (SEL)sel with: (id)anObject with: (id)anotherObject
 {
   return [self performSelector:sel withObject:anObject 
 	       withObject:anotherObject];
@@ -1117,46 +1332,61 @@ static BOOL double_release_check_enabled = NO;
   double_release_check_enabled = enable;
 }
 
-- (int)compare:anotherObject;
+- (int) compare: (id)anotherObject
 {
-  if ([self isEqual:anotherObject])
-    return 0;
+  if (anotherObject == self)
+    {
+      return NSOrderedSame;
+    }
+  if (anotherObject == nil)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"nil argument for compare:"];
+    }
+  if ([self isEqual: anotherObject])
+    {
+      return NSOrderedSame;
+    }
   // Ordering objects by their address is pretty useless, 
   // so subclasses should override this is some useful way.
-  else if (self > anotherObject)
-    return 1;
+  if (self > anotherObject)
+    {
+      return NSOrderedDescending;
+    }
   else 
-    return -1;
+    {
+      return NSOrderedAscending;
+    }
 }
 
-- (BOOL)isMetaClass
+- (BOOL) isMetaClass
 {
   return NO;
 }
 
-- (BOOL)isClass
+- (BOOL) isClass
 {
   return object_is_class(self);
 }
 
-- (BOOL)isInstance
+- (BOOL) isInstance
 {
   return object_is_instance(self);
 }
 
-- (BOOL)isMemberOfClassNamed:(const char *)aClassName
+- (BOOL) isMemberOfClassNamed: (const char*)aClassName
 {
   return ((aClassName!=NULL)
           &&!strcmp(class_get_class_name(self->isa), aClassName));
 }
 
-+ (struct objc_method_description *)descriptionForInstanceMethod:(SEL)aSel
++ (struct objc_method_description *) descriptionForInstanceMethod: (SEL)aSel
 {
   return ((struct objc_method_description *)
            class_get_instance_method(self, aSel));
 }
 
-- (struct objc_method_description *)descriptionForMethod:(SEL)aSel
+- (struct objc_method_description *) descriptionForMethod: (SEL)aSel
 {
   return ((struct objc_method_description *)
            (object_is_instance(self)
@@ -1164,7 +1394,7 @@ static BOOL double_release_check_enabled = NO;
             :class_get_class_method(self->isa, aSel)));
 }
 
-- (Class)transmuteClassTo:(Class)aClassObject
+- (Class) transmuteClassTo: (Class)aClassObject
 {
   if (object_is_instance(self))
     if (class_is_class(aClassObject))
@@ -1178,7 +1408,7 @@ static BOOL double_release_check_enabled = NO;
   return nil;
 }
 
-- subclassResponsibility:(SEL)aSel
+- (id) subclassResponsibility: (SEL)aSel
 {
   [NSException
     raise: NSGenericException
@@ -1186,7 +1416,7 @@ static BOOL double_release_check_enabled = NO;
   return nil;
 }
 
-- shouldNotImplement:(SEL)aSel
+- (id) shouldNotImplement: (SEL)aSel
 {
   [NSException
     raise: NSGenericException
@@ -1195,7 +1425,7 @@ static BOOL double_release_check_enabled = NO;
   return nil;
 }
 
-+ (int)streamVersion: (TypedStream*)aStream
++ (int) streamVersion: (TypedStream*)aStream
 {
   if (aStream->mode == OBJC_READONLY)
     return objc_get_stream_class_version (aStream, self);
@@ -1209,19 +1439,19 @@ static BOOL double_release_check_enabled = NO;
 // before doing their own archiving.  These methods are private, in
 // the sense that they should only be called from subclasses.
 
-- read: (TypedStream*)aStream
+- (id) read: (TypedStream*)aStream
 {
   // [super read: aStream];  
   return self;
 }
 
-- write: (TypedStream*)aStream
+- (id) write: (TypedStream*)aStream
 {
   // [super write: aStream];
   return self;
 }
 
-- awake
+- (id) awake
 {
   // [super awake];
   return self;
@@ -1253,3 +1483,33 @@ _fastMallocBuffer(unsigned size)
   return ((void*)&o[1])+fastMallocOffset;
 }
 
+
+/*
+ * Stuff for compatibility with 'Object' derived classes.
+ */
+@interface	Object (NSObjectCompat)
++ (void) release;
++ (id) retain;
+- (void) release;
+- (id) retain;
+@end
+
+@implementation	Object (NSObjectCompat)
++ (void) release
+{
+  return;
+}
++ (id) retain
+{
+  return self;
+}
+- (void) release
+{
+  return;
+}
+- (id) retain
+{
+  return self;
+}
+@end
+
