@@ -53,6 +53,7 @@
 
 #include <config.h>
 #include <base/preface.h>
+#include <unistd.h>
 
 #ifdef HAVE_STRERROR 
 #include <errno.h>
@@ -83,11 +84,18 @@ Perhaps your program failed to #include <Foundation/NSObject.h> or\n\
 <Foundation/Foundation.h>?\n\
 If that is not the problem, Please report the error to bug-gnustep@gnu.org.\n\n"
 #else
+#ifdef GS_PASS_ARGUMENTS
+#define _GNU_MISSING_MAIN_FUNCTION_CALL @"\nGNUSTEP Error:\n\
+A call to NSProcessInfo +initializeWithArguments:... must be made\n\
+as the first ObjC statment in main. This function is used to \n\
+establish the argv and environment variables.\n"
+#else
 #define _GNU_MISSING_MAIN_FUNCTION_CALL @"\nGNUSTEP Internal Error:\n\
 The private GNUstep function to establish the argv and environment\n\
 variables was not called.\n\
 Please report the error to bug-gnustep@gnu.org.\n\n"
 #endif
+#endif
 
 /*************************************************************************
  *** _NSConcreteProcessInfo
@@ -183,7 +191,7 @@ _gnu_process_args(int argc, char *argv[], char *env[])
     i = 0;
     while (env[i]) 
       {
-#if defined(__WIN32__)
+#if defined(__MINGW__)
 	char	buf[1024];
 	char	*cp;
 	DWORD	len;
@@ -269,6 +277,19 @@ static char	**_gnu_noobjc_env;
 #endif /* HAVE_PROGRAM_INVOCATION_NAME */
   
   // Read environment 
+
+  /* NB: This should *never* happen if your compiler tools are
+     sane.  But, if you are playing with them, you could break
+     them to the point you get here. :-) */
+  if (environ == NULL)
+    {
+      /* TODO: Try reading environment from /proc before aborting. */
+      fprintf(stderr, "Error: for some reason, environ == NULL " 
+	      "during GNUstep base initialization\n"
+	      "Please check the linking process\n");
+      abort();
+    }
+
   c = 0;
   while (environ[c] != NULL)
     c++;
@@ -456,8 +477,8 @@ _gnu_noobjc_free_vars(void)
 }
 #else /* !HAVE_PROC_FS || !HAVE_LOAD_METHOD */
 
-#ifdef __MINGW32__
-/* For Windows32API Library, we know the global variables */
+#ifdef __MINGW__
+/* For WindowsAPI Library, we know the global variables */
 extern int __argc;
 extern char** __argv;
 extern char** _environ;
@@ -469,10 +490,11 @@ extern char** _environ;
 }
 
 #else
+#ifndef GS_PASS_ARGUMENTS
 #undef main
 int main(int argc, char *argv[], char *env[])
 {
-#if	defined(__WIN32__) && !defined(__CYGWIN__)
+#if defined(__MINGW__)
   WSADATA lpWSAData;
 
   // Initialize Windows Sockets
@@ -481,21 +503,35 @@ int main(int argc, char *argv[], char *env[])
       printf("Could not startup Windows Sockets\n");
       exit(1);
     }
-#endif /* __WIN32__ */
+#endif /* __MINGW__ */
 
-#ifdef __MS_WIN32__
+#ifdef __MS_WIN__
   _MB_init_runtime();
-#endif /* __MS_WIN32__ */
+#endif /* __MS_WIN__ */
 
   _gnu_process_args(argc, argv, env);
 
   /* Call the user defined main function */
   return gnustep_base_user_main(argc, argv, env);
 }
-#endif /* __MINGW32__ */
+#endif /* !GS_PASS_ARGUMENTS */
+#endif /* __MINGW__ */
 
 #endif /* HAS_LOAD_METHOD && HAS_PROC_FS */ 
 
+/* Fallback method. User must call this method if none of the other
+   above hacks is being used
+*/
++ (void)initializeWithArguments:(char**)argv
+                          count:(int)argc
+                    environment:(char**)env;
+{
+  if (!_gnu_processName && !_gnu_arguments && !_gnu_environment)
+    {
+      _gnu_process_args(argc, argv, env);
+    }
+}
+
 /*************************************************************************
  *** Getting an NSProcessInfo Object
  *************************************************************************/
@@ -552,7 +588,7 @@ int main(int argc, char *argv[], char *env[])
 {
   int	pid;
 
-#if defined(__WIN32__)
+#if defined(__MINGW__)
   pid = (int)GetCurrentProcessId();
 #else
   pid = (int)getpid();
