@@ -25,6 +25,7 @@
 
 #include <config.h>
 #include <base/preface.h>
+#include <base/fast.x>
 #include <Foundation/NSMapTable.h>
 #include <Foundation/NSDate.h>
 #include <Foundation/NSValue.h>
@@ -35,12 +36,15 @@
 #include <Foundation/NSRunLoop.h>
 #include <Foundation/NSThread.h>
 #include <Foundation/NSDebug.h>
+#include <Foundation/NSDebug.h>
 
+#if HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#if	!defined(__WIN32__) || defined(__CYGWIN__)
-#include <time.h>
+#endif
+#if HAVE_SYS_TIME_H
 #include <sys/time.h>
-#endif /* !__WIN32__ */
+#endif
+#include <time.h>
 #include <limits.h>
 #include <string.h>		/* for memset() */
 
@@ -50,13 +54,13 @@ static NSDate	*theFuture = nil;
 
 
 /*
- *	The 'RunLoopWatcher' class was written to permit the (relatively)
+ *	The 'GSRunLoopWatcher' class was written to permit the (relatively)
  *	easy addition of new events to be watched for in the runloop.
  *
  *	To add a new type of event, the 'RunLoopEventType' enumeration must be
  *	extended, and the methods must be modified to handle the new type.
  *
- *	The internal variables if the RunLoopWatcher are used as follows -
+ *	The internal variables if the GSRunLoopWatcher are used as follows -
  *
  *	The '_date' variable contains a date after which the event is useless
  *	and the watcher can be removed from the runloop.
@@ -75,7 +79,7 @@ static NSDate	*theFuture = nil;
  *	NSRunLoops [-acceptInputForMode: beforeDate: ] method MUST contain
  *	code to watch for events of each type.
  *
- *	To set this variable, the method adding the RunLoopWatcher to the
+ *	To set this variable, the method adding the GSRunLoopWatcher to the
  *	runloop must ask the 'receiver' (or its delegate) to supply a date
  *	using the '[-limitDateForMode: ]' message.
  *
@@ -84,7 +88,7 @@ static NSDate	*theFuture = nil;
  
 static SEL	eventSel = @selector(receivedEvent:type:extra:forMode:);
 
-@interface RunLoopWatcher: NSObject
+@interface GSRunLoopWatcher: NSObject
 {
 @public
   NSDate		*_date;		/* First to match layout of NSTimer */
@@ -95,12 +99,12 @@ static SEL	eventSel = @selector(receivedEvent:type:extra:forMode:);
   RunLoopEventType	type;
   unsigned 		count;
 }
-- initWithType: (RunLoopEventType)type
-      receiver: (id)anObj
-          data: (void*)data;
+- (id) initWithType: (RunLoopEventType)type
+	   receiver: (id)anObj
+	       data: (void*)data;
 @end
 
-@implementation	RunLoopWatcher
+@implementation	GSRunLoopWatcher
 
 - (void) dealloc
 {
@@ -116,6 +120,7 @@ static SEL	eventSel = @selector(receivedEvent:type:extra:forMode:);
 
   switch (aType)
     {
+      case ET_EDESC: 	type = aType;	break;
       case ET_RDESC: 	type = aType;	break;
       case ET_WDESC: 	type = aType;	break;
       case ET_RPORT: 	type = aType;	break;
@@ -127,7 +132,8 @@ static SEL	eventSel = @selector(receivedEvent:type:extra:forMode:);
   if ([receiver respondsToSelector: eventSel] == YES) 
     handleEvent = [receiver methodForSelector: eventSel];
   else
-    handleEvent = 0;
+    [NSException raise: NSInvalidArgumentException
+		format: @"RunLoop listener has no event handling method"];
   data = item;
   return self;
 }
@@ -136,17 +142,17 @@ static SEL	eventSel = @selector(receivedEvent:type:extra:forMode:);
 
 /*
  *	Two optimisation functions that depend on a hack that the layout of
- *	the NSTimer class is known to be the same as RunLoopWatcher for the
+ *	the NSTimer class is known to be the same as GSRunLoopWatcher for the
  *	first two elements.
  */
 static inline NSDate* timerDate(NSTimer* timer)
 {
-  return ((RunLoopWatcher*)timer)->_date;
+  return ((GSRunLoopWatcher*)timer)->_date;
 }
 
 static inline BOOL timerInvalidated(NSTimer* timer)
 {
-  return ((RunLoopWatcher*)timer)->_invalidated;
+  return ((GSRunLoopWatcher*)timer)->_invalidated;
 }
 
 
@@ -162,19 +168,67 @@ static inline BOOL timerInvalidated(NSTimer* timer)
 #define GSI_ARRAY_RETAIN(X)	[(X).obj retain]
 #else
 #define GSI_ARRAY_RELEASE(X)	
-#define GSI_ARRAY_RETAIN(X)	(X).obj
+#define GSI_ARRAY_RETAIN(X)	
 #endif
 
 #include <base/GSIArray.h>
 
 static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
 {
-  return [((RunLoopWatcher *)(i0.obj))->_date 
-           compare: ((RunLoopWatcher *)(i1.obj))->_date];
+  return [((GSRunLoopWatcher *)(i0.obj))->_date 
+    compare: ((GSRunLoopWatcher *)(i1.obj))->_date];
 }
 
 
 
+/*
+ *	The GSRunLoopPerformer class is used to hold information about
+ *	messages which are due to be sent to objects once each runloop
+ *	iteration has passed.
+ */
+@interface GSRunLoopPerformer: NSObject
+{
+@public
+  SEL		selector;
+  id		target;
+  id		argument;
+  unsigned	order;
+}
+
+- (void) fire;
+- (id) initWithSelector: (SEL)aSelector
+		 target: (id)target
+	       argument: (id)argument
+		  order: (unsigned int)order;
+@end
+
+@implementation GSRunLoopPerformer
+
+- (void) fire
+{
+  [target performSelector: selector withObject: argument];
+}
+
+- (id) initWithSelector: (SEL)aSelector
+		 target: (id)aTarget
+	       argument: (id)anArgument
+		  order: (unsigned int)theOrder
+{
+  self = [super init];
+  if (self)
+    {
+      selector = aSelector;
+      target = aTarget;
+      argument = anArgument;
+      order = theOrder;
+    }
+  return self;
+}
+
+@end
+
+
+
 @interface NSRunLoop (TimedPerformers)
 - (NSMutableArray*) _timedPerformers;
 @end
@@ -187,67 +241,53 @@ static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
 @end
 
 /*
- *	The RunLoopPerformer class is used to hold information about
- *	messages which are due to be sent to objects once a particular
- *	runloop iteration has passed.
+ *	The GSTimedPerformer class is used to hold information about
+ *	messages which are due to be sent to objects at a particular time.
  */
-@interface RunLoopPerformer: NSObject <GCFinalization>
+@interface GSTimedPerformer: NSObject <GCFinalization>
 {
+@public
   SEL		selector;
   id		target;
   id		argument;
-  unsigned	order;
-  NSArray	*modes;
   NSTimer	*timer;
 }
 
 - (void) fire;
-- initWithSelector: (SEL)aSelector
-	    target: (id)target
-          argument: (id)argument
-             order: (unsigned int)order
-             modes: (NSArray*)modes;
-- (BOOL) matchesSelector: (SEL)aSelector
-		  target: (id)aTarget
-		argument: (id)anArgument;
-- (NSArray*) modes;
-- (unsigned int) order;
-- (void) setTimer: (NSTimer*)timer;
+- (id) initWithSelector: (SEL)aSelector
+		 target: (id)target
+	       argument: (id)argument
+		  delay: (NSTimeInterval)delay;
 @end
 
-@implementation RunLoopPerformer
+@implementation GSTimedPerformer
 
 - (void) dealloc
 {
   [self gcFinalize];
   RELEASE(target);
   RELEASE(argument);
-  RELEASE(modes);
   [super dealloc];
 }
 
 - (void) fire
 {
-  if (timer != nil)
-    {
-      timer = nil;
-      AUTORELEASE(RETAIN(self));
-      [[[NSRunLoop currentInstance] _timedPerformers]
-		removeObjectIdenticalTo: self];
-    }
+  timer = nil;
   [target performSelector: selector withObject: argument];
+  [[[NSRunLoop currentInstance] _timedPerformers]
+	    removeObjectIdenticalTo: self];
 }
 
 - (void) gcFinalize
 {
-  [timer invalidate];
+  if (timer != nil)
+    [timer invalidate];
 }
 
-- initWithSelector: (SEL)aSelector
-	    target: (id)aTarget
-          argument: (id)anArgument
-             order: (unsigned int)theOrder
-             modes: (NSArray*)theModes
+- (id) initWithSelector: (SEL)aSelector
+		 target: (id)aTarget
+	       argument: (id)anArgument
+		  delay: (NSTimeInterval)delay
 {
   self = [super init];
   if (self)
@@ -255,43 +295,14 @@ static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
       selector = aSelector;
       target = RETAIN(aTarget);
       argument = RETAIN(anArgument);
-      order = theOrder;
-      modes = [theModes copy];
+      timer = [NSTimer timerWithTimeInterval: delay
+				      target: self
+				    selector: @selector(fire)
+				    userInfo: nil
+				     repeats: NO];
     }
   return self;
 }
-
-- (BOOL) matchesSelector: (SEL)aSelector
-		  target: (id)aTarget
-		argument: (id)anArgument
-{
-  if (selector == aSelector)
-    {
-      if (target == aTarget)
-	{
-	  if ([argument isEqual: anArgument])
-	    {
-	      return YES;
-	    }
-	}
-    }
-  return NO;
-}
-
-- (NSArray*) modes
-{
-  return modes;
-}
-
-- (unsigned int) order
-{
-  return order;
-}
-
-- (void) setTimer: (NSTimer*)t
-{
-  timer = t;
-}
 @end
 
 @implementation NSObject (TimedPerformers)
@@ -300,45 +311,45 @@ static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
 					selector: (SEL)aSelector
 					  object: (id)arg
 {
-  NSMutableArray	*array;
-  int			i;
+  NSMutableArray	*perf = [[NSRunLoop currentInstance] _timedPerformers];
+  unsigned		count = [perf count];
 
-  RETAIN(target);
-  RETAIN(arg);
-  array = [[NSRunLoop currentInstance] _timedPerformers];
-  for (i = [array count]; i > 0; i--)
+  if (count > 0)
     {
-      if ([[array objectAtIndex: i-1] matchesSelector: aSelector
-					       target: target
-					     argument: arg])
+      GSTimedPerformer	*array[count];
+
+      IF_NO_GC(RETAIN(target));
+      IF_NO_GC(RETAIN(arg));
+      [perf getObjects: array];
+      while (count-- > 0)
 	{
-	  [array removeObjectAtIndex: i-1];
+	  GSTimedPerformer	*p = array[count];
+
+	  if (p->target == target && sel_eq(p->selector, aSelector)
+	    && [p->argument isEqual: arg])
+	    {
+	      [perf removeObjectAtIndex: count];
+	    }
 	}
+      RELEASE(arg);
+      RELEASE(target);
     }
-  RELEASE(arg);
-  RELEASE(target);
 }
 
 - (void) performSelector: (SEL)aSelector
 	      withObject: (id)argument
 	      afterDelay: (NSTimeInterval)seconds
 {
-  NSMutableArray	*array;
-  RunLoopPerformer	*item;
+  NSRunLoop		*loop = [NSRunLoop currentInstance];
+  GSTimedPerformer	*item;
 
-  array = [[NSRunLoop currentInstance] _timedPerformers];
-  item = [[RunLoopPerformer alloc] initWithSelector: aSelector
+  item = [[GSTimedPerformer alloc] initWithSelector: aSelector
 					     target: self
 					   argument: argument
-					      order: 0
-					      modes: nil];
-  [array addObject: item];
-  [item setTimer: [NSTimer scheduledTimerWithTimeInterval: seconds
-						   target: item
-						 selector: @selector(fire)
-						 userInfo: nil
-						  repeats: NO]];
+					      delay: seconds];
+  [[loop _timedPerformers] addObject: item];
   RELEASE(item);
+  [loop addTimer: item->timer forMode: NSDefaultRunLoopMode];
 }
 
 - (void) performSelector: (SEL)aSelector
@@ -346,48 +357,41 @@ static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
 	      afterDelay: (NSTimeInterval)seconds
 		 inModes: (NSArray*)modes
 {
-  NSRunLoop		*loop;
-  NSMutableArray	*array;
-  RunLoopPerformer	*item;
-  NSTimer		*timer;
-  int			i;
+  unsigned	count = [modes count];
 
-  if (modes == nil || [modes count] == 0)
+  if (count > 0)
     {
-      return;
-    }
-  loop = [NSRunLoop currentInstance];
-  array = [loop _timedPerformers];
-  item = [[RunLoopPerformer alloc] initWithSelector: aSelector
-					     target: self
-					   argument: argument
-					      order: 0
-					      modes: nil];
-  [array addObject: item];
-  timer = [NSTimer timerWithTimeInterval: seconds
-				  target: item
-				selector: @selector(fire)
-				userInfo: nil
-				 repeats: NO];
-  [item setTimer: timer];
-  RELEASE(item);
-  for (i = 0; i < [modes count]; i++)
-    {
-      [loop addTimer: timer forMode: [modes objectAtIndex: i]];
+      NSRunLoop		*loop = [NSRunLoop currentInstance];
+      NSString		*marray[count];
+      GSTimedPerformer	*item;
+      unsigned		i;
+
+      item = [[GSTimedPerformer alloc] initWithSelector: aSelector
+						 target: self
+					       argument: argument
+						  delay: seconds];
+      [[loop _timedPerformers] addObject: item];
+      RELEASE(item);
+      [modes getObjects: marray];
+      for (i = 0; i < count; i++)
+	{
+	  [loop addTimer: item->timer forMode: marray[i]];
+	}
     }
 }
 
 @end
+
 
 
 @interface NSRunLoop (Private)
 
-- (void) _addWatcher: (RunLoopWatcher*)item
+- (void) _addWatcher: (GSRunLoopWatcher*)item
 	     forMode: (NSString*)mode;
 - (void) _checkPerformers;
-- (RunLoopWatcher*) _getWatcher: (void*)data
-			   type: (RunLoopEventType)type
-		        forMode: (NSString*)mode;
+- (GSRunLoopWatcher*) _getWatcher: (void*)data
+			     type: (RunLoopEventType)type
+			  forMode: (NSString*)mode;
 - (NSMutableArray*) _performers;
 - (void) _removeWatcher: (void*)data
 		   type: (RunLoopEventType)type
@@ -399,7 +403,7 @@ static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
 
 /* Add a watcher to the list for the specified mode.  Keep the list in
    limit-date order. */
-- (void) _addWatcher: (RunLoopWatcher*) item forMode: (NSString*)mode
+- (void) _addWatcher: (GSRunLoopWatcher*) item forMode: (NSString*)mode
 {
   GSIArray	watchers;
   id		obj;
@@ -445,24 +449,27 @@ static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
 
 - (void) _checkPerformers
 {
-  RunLoopPerformer	*item;
-  NSArray		*array = [NSArray arrayWithArray: _performers];
-  int			count = [array count];
-  unsigned		pos;
-  int			i;
+  GSIArray	performers = NSMapGet(_mode_2_performers, _current_mode);
+  unsigned	count;
 
-  for (i = 0; i < count; i++)
+  if (performers != 0 && (count = GSIArrayCount(performers)) > 0)
     {
-      item = (RunLoopPerformer*)[array objectAtIndex: i];
+      GSRunLoopPerformer	*array[count];
+      unsigned			i;
 
-      pos = [_performers indexOfObjectIdenticalTo: item];
-      if (pos != NSNotFound)
+      /*
+       * Copy the array - so we don't get messed up by any changes caused
+       * by 'fire'ing the performers.
+       */
+      for (i = 0; i < count; i++)
 	{
-	  if ([[item modes] containsObject: _current_mode])
-	    {
-	      [_performers removeObjectAtIndex: pos];
-	      [item fire];
-	    }
+	  array[i] = RETAIN(GSIArrayItemAtIndex(performers, i).obj);
+	}
+
+      for (i = 0; i < count; i++)
+	{
+	  [array[i] fire];
+	  RELEASE(array[i]);
 	}
     }
 }
@@ -470,39 +477,39 @@ static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
 
 @implementation NSRunLoop(GNUstepExtensions)
 
-+ currentInstance
++ (id) currentInstance
 {
   return [self currentRunLoop];
 }
 
 + (NSString*) currentMode
 {
-  return [[NSRunLoop currentRunLoop] currentMode];
+  return [[self currentRunLoop] currentMode];
 }
 
 + (void) run
 {
-  [[NSRunLoop currentRunLoop] run];
+  [[self currentRunLoop] run];
 }
 
 + (void) runUntilDate: date
 {
-  [[NSRunLoop currentRunLoop] runUntilDate: date];
+  [[self currentRunLoop] runUntilDate: date];
 }
 
 + (void) runUntilDate: date forMode: (NSString*)mode
 {
-  [[NSRunLoop currentRunLoop] runUntilDate: date forMode: mode];
+  [[self currentRunLoop] runUntilDate: date forMode: mode];
 }
 
 + (BOOL) runOnceBeforeDate: date 
 {
-  return [[NSRunLoop currentRunLoop] runOnceBeforeDate: date];
+  return [[self currentRunLoop] runOnceBeforeDate: date];
 }
 
 + (BOOL) runOnceBeforeDate: date forMode: (NSString*)mode
 {
-  return [[NSRunLoop currentRunLoop] runOnceBeforeDate: date forMode: mode];
+  return [[self currentRunLoop] runOnceBeforeDate: date forMode: mode];
 }
 
 - (void) addEvent: (void*)data
@@ -510,7 +517,7 @@ static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
           watcher: (id<RunLoopEvents>)watcher
           forMode: (NSString*)mode
 {
-  RunLoopWatcher	*info;
+  GSRunLoopWatcher	*info;
 
   if (mode == nil)
     {
@@ -530,36 +537,15 @@ static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
       [self _removeWatcher: data type: type forMode: mode];
 
       /* Create new object to hold information. */
-      info = [[RunLoopWatcher alloc] initWithType: type
-					 receiver: watcher
-					     data: data];
+      info = [[GSRunLoopWatcher alloc] initWithType: type
+					   receiver: watcher
+					       data: data];
       /* Add the object to the array for the mode. */
       [self _addWatcher: info forMode: mode];
       RELEASE(info);		/* Now held in array.	*/
     }
 }
 
-- (void) addReadDescriptor: (int)fd
-		    object: (id <FdListening>)listener
-		   forMode: (NSString*)mode
-{
-  return [self addEvent: (void*)fd
-		   type: ET_RDESC
-		watcher: (id<RunLoopEvents>)listener
-		forMode: mode];
-}
-
-  /* Add our new handler information to the array. */
-- (void) addWriteDescriptor: (int)fd
-		     object: (id <FdSpeaking>)speaker
-		    forMode: (NSString*)mode
-{
-  return [self addEvent: (void*)fd
-		   type: ET_WDESC
-		watcher: (id<RunLoopEvents>)speaker
-		forMode: mode];
-}
-
 - (void) removeEvent: (void*)data
                 type: (RunLoopEventType)type
              forMode: (NSString*)mode
@@ -575,7 +561,7 @@ static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
     }
   else
     {
-      RunLoopWatcher	*info;
+      GSRunLoopWatcher	*info;
 
       info = [self _getWatcher: data type: type forMode: mode];
   
@@ -593,18 +579,6 @@ static NSComparisonResult aSort(GSIArrayItem i0, GSIArrayItem i1)
     }
 }
 
-- (void) removeReadDescriptor: (int)fd 
-		      forMode: (NSString*)mode
-{
-  return [self removeEvent: (void*)fd type: ET_RDESC forMode: mode all: NO];
-}
-
-- (void) removeWriteDescriptor: (int)fd
-		       forMode: (NSString*)mode
-{
-  return [self removeEvent: (void*)fd type: ET_WDESC forMode: mode all: NO];
-}
-
 - (BOOL) runOnceBeforeDate: date
 {
   return [self runOnceBeforeDate: date forMode: _current_mode];
@@ -694,8 +668,8 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
       [self currentRunLoop];
       theFuture = RETAIN([NSDate distantFuture]);
 #if	GS_WITH_GC == 0
-      wRelImp = [[RunLoopWatcher class] instanceMethodForSelector: wRelSel];
-      wRetImp = [[RunLoopWatcher class] instanceMethodForSelector: wRetSel];
+      wRelImp = [[GSRunLoopWatcher class] instanceMethodForSelector: wRelSel];
+      wRetImp = [[GSRunLoopWatcher class] instanceMethodForSelector: wRetSel];
 #endif
     }
 }
@@ -704,7 +678,7 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
 {
   static NSString	*key = @"NSRunLoopThreadKey";
   NSMutableDictionary	*d;
-  NSRunLoop*	r;
+  NSRunLoop		*r;
 
   d = GSCurrentThreadDictionary();
   r = [d objectForKey: key];
@@ -726,8 +700,11 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
 				     ArrayMapValueCallBacks, 0);
   _mode_2_watchers = NSCreateMapTable (NSObjectMapKeyCallBacks,
 					   ArrayMapValueCallBacks, 0);
-  _performers = [[NSMutableArray alloc] initWithCapacity: 8];
+  _mode_2_performers = NSCreateMapTable (NSObjectMapKeyCallBacks,
+					   ArrayMapValueCallBacks, 0);
   _timedPerformers = [[NSMutableArray alloc] initWithCapacity: 8];
+  _efdMap = NSCreateMapTable (NSIntMapKeyCallBacks,
+				  WatcherMapValueCallBacks, 0);
   _rfdMap = NSCreateMapTable (NSIntMapKeyCallBacks,
 				  WatcherMapValueCallBacks, 0);
   _wfdMap = NSCreateMapTable (NSIntMapKeyCallBacks,
@@ -738,7 +715,6 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
 - (void) dealloc
 {
   [self gcFinalize];
-  RELEASE(_performers);
   RELEASE(_timedPerformers);
   [super dealloc];
 }
@@ -747,6 +723,8 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
 {
   NSFreeMapTable(_mode_2_timers);
   NSFreeMapTable(_mode_2_watchers);
+  NSFreeMapTable(_mode_2_performers);
+  NSFreeMapTable(_efdMap);
   NSFreeMapTable(_rfdMap);
   NSFreeMapTable(_wfdMap);
 }
@@ -782,13 +760,13 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
 
 - (NSDate*) limitDateForMode: (NSString*)mode
 {
-  CREATE_AUTORELEASE_POOL(arp);
   id			saved_mode;
   NSDate		*when;
   GSIArray		timers;
   GSIArray		watchers;
   NSTimer		*min_timer = nil;
-  RunLoopWatcher	*min_watcher = nil;
+  GSRunLoopWatcher	*min_watcher = nil;
+  CREATE_AUTORELEASE_POOL(arp);
 
   saved_mode = _current_mode;
   _current_mode = mode;
@@ -938,9 +916,9 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
   return when;
 }
 
-- (RunLoopWatcher*) _getWatcher: (void*)data
-			   type: (RunLoopEventType)type
-		        forMode: (NSString*)mode
+- (GSRunLoopWatcher*) _getWatcher: (void*)data
+			     type: (RunLoopEventType)type
+			  forMode: (NSString*)mode
 {
   GSIArray		watchers;
 
@@ -956,7 +934,7 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
 
       while (i-- > 0)
 	{
-	  RunLoopWatcher	*info;
+	  GSRunLoopWatcher	*info;
 
 	  info = GSIArrayItemAtIndex(watchers, i).obj;
 	  if (info->type == type && info->data == data)
@@ -986,7 +964,7 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
 
       while (i-- > 0)
 	{
-	  RunLoopWatcher	*info;
+	  GSRunLoopWatcher	*info;
 
 	  info = GSIArrayItemAtIndex(watchers, i).obj;
 	  if (info->type == type && info->data == data)
@@ -1007,18 +985,20 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
 - (void) acceptInputForMode: (NSString*)mode 
 		 beforeDate: limit_date
 {
-  CREATE_AUTORELEASE_POOL(arp);
+  extern BOOL	GSCheckTasks();
   NSTimeInterval ti;
   struct timeval timeout;
   void *select_timeout;
-  fd_set fds;			/* The file descriptors we will listen to. */
   fd_set read_fds;		/* Copy for listening to read-ready fds. */
   fd_set exception_fds;		/* Copy for listening to exception fds. */
   fd_set write_fds;		/* Copy for listening for write-ready fds. */
   int select_return;
-  int fd_index;
+  int fdIndex;
+  int fdEnd;
   id saved_mode;
-  int num_inputs = 0;
+  int num_inputs = 0;		/* Number of descriptors being monitored. */
+  int end_inputs = 0;		/* Highest numbered descriptor plus one. */
+  CREATE_AUTORELEASE_POOL(arp);
 
   NSAssert(mode, NSInvalidArgumentException);
   saved_mode = _current_mode;
@@ -1049,6 +1029,7 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
       /* The LIMIT_DATE has already past; return immediately without
 	 polling any inputs. */
       [self _checkPerformers];
+      GSNotifyASAP();
       if (debug_run_loop)
 	printf ("\tNSRunLoop limit date past, returning\n");
       _current_mode = saved_mode;
@@ -1071,86 +1052,118 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
    *	The maps may not have been emptied if a previous call to this
    *	method was terminated by an exception.
    */
-  memset(&fds, '\0', sizeof(fds));
+  memset(&exception_fds, '\0', sizeof(exception_fds));
+  memset(&read_fds, '\0', sizeof(read_fds));
   memset(&write_fds, '\0', sizeof(write_fds));
+  NSResetMapTable(_efdMap);
   NSResetMapTable(_rfdMap);
   NSResetMapTable(_wfdMap);
 
-  /* Do the pre-listening set-up for the file descriptors of this mode. */
+  /*
+   * Do the pre-listening set-up for the file descriptors of this mode.
+   */
   {
-      GSIArray	watchers;
+    GSIArray	watchers;
 
-      watchers = NSMapGet(_mode_2_watchers, mode);
-      if (watchers) {
-	  int	i;
+    watchers = NSMapGet(_mode_2_watchers, mode);
+    if (watchers)
+      {
+	unsigned	i = GSIArrayCount(watchers);
 
-	  for (i = GSIArrayCount(watchers); i > 0; i--) {
-	      RunLoopWatcher	*info;
-	      int		fd;
+	while (i-- > 0)
+	  {
+	    GSRunLoopWatcher	*info;
+	    int			fd;
 
-	      info = GSIArrayItemAtIndex(watchers, i-1).obj;
-	      if (info->_invalidated == YES) {
-		GSIArrayRemoveItemAtIndex(watchers, i-1);
+	    info = GSIArrayItemAtIndex(watchers, i).obj;
+	    if (info->_invalidated == YES)
+	      {
+		GSIArrayRemoveItemAtIndex(watchers, i);
 		continue;
-              }
-	      switch (info->type) {
-		case ET_WDESC: 
-	          fd = (int)info->data;
-	          FD_SET (fd, &write_fds);
-	          NSMapInsert(_wfdMap, (void*)fd, info);
-	          num_inputs++;
+	      }
+	    switch (info->type)
+	      {
+		case ET_EDESC: 
+		  fd = (int)info->data;
+		  if (fd > end_inputs)
+		    end_inputs = fd;
+		  FD_SET (fd, &exception_fds);
+		  NSMapInsert(_efdMap, (void*)fd, info);
+		  num_inputs++;
 		  break;
 
 		case ET_RDESC: 
-	          fd = (int)info->data;
-	          FD_SET (fd, &fds);
-	          NSMapInsert(_rfdMap, (void*)fd, info);
-	          num_inputs++;
+		  fd = (int)info->data;
+		  if (fd > end_inputs)
+		    end_inputs = fd;
+		  FD_SET (fd, &read_fds);
+		  NSMapInsert(_rfdMap, (void*)fd, info);
+		  num_inputs++;
+		  break;
+
+		case ET_WDESC: 
+		  fd = (int)info->data;
+		  if (fd > end_inputs)
+		    end_inputs = fd;
+		  FD_SET (fd, &write_fds);
+		  NSMapInsert(_wfdMap, (void*)fd, info);
+		  num_inputs++;
 		  break;
 
 		case ET_RPORT: 
-		  {
-		    id	port = info->receiver;
-		    int port_fd_count = 128; // xxx #define this constant
-		    int port_fd_array[port_fd_count];
+		  if ([info->receiver isValid] == NO)
+		    {
+		      /*
+		       * We must remove an invalidated port.
+		       */
+		      info->_invalidated = YES;
+		      GSIArrayRemoveItemAtIndex(watchers, i);
+		    }
+		  else
+		    {
+		      id port = info->receiver;
+		      int port_fd_count = 128; // xxx #define this constant
+		      int port_fd_array[port_fd_count];
 
-		    if ([port respondsTo: @selector(getFds:count:)])
-		      [port getFds: port_fd_array count: &port_fd_count];
-		    if (debug_run_loop)
-		      printf("\tNSRunLoop listening to %d sockets\n",
-			    port_fd_count);
+		      if ([port respondsTo: @selector(getFds:count:)])
+			[port getFds: port_fd_array count: &port_fd_count];
+		      if (debug_run_loop)
+			printf("\tNSRunLoop listening to %d sockets\n",
+			      port_fd_count);
 
-		    while (port_fd_count--)
-		      {
-			FD_SET (port_fd_array[port_fd_count], &fds);
-			NSMapInsert(_rfdMap, 
-				     (void*)port_fd_array[port_fd_count],
-				    info);
-			num_inputs++;
-		      }
-		  }
+		      while (port_fd_count--)
+			{
+			  fd = port_fd_array[port_fd_count];
+			  FD_SET (port_fd_array[port_fd_count], &read_fds);
+			  if (fd > end_inputs)
+			    end_inputs = fd;
+			  NSMapInsert(_rfdMap, 
+			    (void*)port_fd_array[port_fd_count], info);
+			  num_inputs++;
+			}
+		    }
 		  break;
 	      }
 	  }
       }
   }
+  end_inputs++;
 
-  /* Wait for incoming data, listening to the file descriptors in _FDS. */
-  read_fds = fds;
-  exception_fds = fds;
-
-  /* Detect if the NSRunLoop is idle, and if necessary - dispatch the
-     notifications from NSNotificationQueue's idle queue? */
-  if (num_inputs == 0 && GSNotifyMore())
+  /*
+   * If there are notifications in the 'idle' queue, we try an instantaneous
+   * select so that, if there is no input pending, we can service the queue.
+   * Similarly, if a task has completed, we need to deliver it's notifications.
+   */
+  if (GSCheckTasks() || GSNotifyMore())
     {
       timeout.tv_sec = 0;
       timeout.tv_usec = 0;
       select_timeout = &timeout;
-      select_return = select (FD_SETSIZE, &read_fds, &write_fds, &exception_fds,
+      select_return = select (end_inputs, &read_fds, &write_fds, &exception_fds,
 			  select_timeout);
     }
   else
-    select_return = select (FD_SETSIZE, &read_fds, &write_fds, &exception_fds,
+    select_return = select (end_inputs, &read_fds, &write_fds, &exception_fds,
 			  select_timeout);
 
   if (debug_run_loop)
@@ -1160,6 +1173,7 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
     {
       if (errno == EINTR)
 	{
+	  GSCheckTasks();
 	  select_return = 0;
 	}
       else
@@ -1173,6 +1187,7 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
     }
   if (select_return == 0)
     {
+      NSResetMapTable(_efdMap);
       NSResetMapTable(_rfdMap);
       NSResetMapTable(_wfdMap);
       GSNotifyIdle();
@@ -1183,72 +1198,97 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
     }
   
   /*
-   *	Look at all the file descriptors select() says are ready for reading;
+   *	Look at all the file descriptors select() says are ready for action;
    *	notify the corresponding object for each of the ready fd's.
    *	NB. It is possible for a watcher to be missing from the map - if
    *	the event handler of a previous watcher has 'run' the loop again
    *	before returning.
+   *	NB. Each time this roop is entered, the starting position (_fdStart)
+   *	is incremented - this is to ensure a fair distribtion over all
+   *	inputs where multiple inputs are in use.  Note - _fdStart can be
+   *	modified while we are in the loop (by recursive calls).
    */
-  for (fd_index = 0; fd_index < FD_SETSIZE; fd_index++)
+  if (_fdStart >= end_inputs)
     {
-      if (FD_ISSET (fd_index, &write_fds))
-        {
-	  RunLoopWatcher	*watcher;
-
-	  watcher = NSMapGet(_wfdMap, (void*)fd_index);
-	  if (watcher != nil && watcher->_invalidated == NO)
-	    {
-	      /*
-	       * The watcher is still valid - so call it's receivers
-	       * event handling method.
-	       */
-	      if (watcher->handleEvent != 0)
-		{
-		  (*watcher->handleEvent)(watcher->receiver,
-			eventSel, watcher->data, watcher->type,
-			(void*)(gsaddr)fd_index, _current_mode);
-		}
-	      else if (watcher->type == ET_WDESC)
-		{
-		  [watcher->receiver readyForWritingOnFileDescriptor:
-				(int)(gsaddr)fd_index];
-		}
-	    }
-	  GSNotifyASAP();
-	  if (--select_return == 0)
-	    break;
-        }
-      if (FD_ISSET (fd_index, &read_fds))
-        {
-	  RunLoopWatcher	*watcher;
-
-	  watcher = (RunLoopWatcher*)NSMapGet(_rfdMap, (void*)fd_index);
-	  if (watcher != nil && watcher->_invalidated == NO)
-	    {
-	      /*
-	       * The watcher is still valid - so call it's receivers
-	       * event handling method.
-	       */
-	      if (watcher->handleEvent != 0)
-		{
-		  (*watcher->handleEvent)(watcher->receiver,
-			eventSel, watcher->data, watcher->type,
-			(void*)(gsaddr)fd_index, _current_mode);
-		}
-	      else if (watcher->type == ET_RDESC || watcher->type == ET_RPORT)
-		{
-		  [watcher->receiver readyForReadingOnFileDescriptor:
-				(int)(gsaddr)fd_index];
-		}
-	    }
-	  GSNotifyASAP();
-	  if (--select_return == 0)
-	    break;
-        }
+      _fdStart = 0;
+      fdIndex = 0;
+      fdEnd = 0;
     }
+  else
+    {
+      _fdStart++;
+      fdIndex = _fdStart;
+      fdEnd = _fdStart;
+    }
+  do
+    {
+      if (FD_ISSET (fdIndex, &exception_fds))
+        {
+	  GSRunLoopWatcher	*watcher;
+
+	  watcher = (GSRunLoopWatcher*)NSMapGet(_efdMap, (void*)fdIndex);
+	  if (watcher != nil && watcher->_invalidated == NO)
+	    {
+	      /*
+	       * The watcher is still valid - so call it's receivers
+	       * event handling method.
+	       */
+	      (*watcher->handleEvent)(watcher->receiver,
+		eventSel, watcher->data, watcher->type,
+		(void*)(gsaddr)fdIndex, _current_mode);
+	    }
+	  GSNotifyASAP();
+	  if (--select_return == 0)
+	    break;
+        }
+      if (FD_ISSET (fdIndex, &write_fds))
+        {
+	  GSRunLoopWatcher	*watcher;
+
+	  watcher = NSMapGet(_wfdMap, (void*)fdIndex);
+	  if (watcher != nil && watcher->_invalidated == NO)
+	    {
+	      /*
+	       * The watcher is still valid - so call it's receivers
+	       * event handling method.
+	       */
+	      (*watcher->handleEvent)(watcher->receiver,
+		eventSel, watcher->data, watcher->type,
+		(void*)(gsaddr)fdIndex, _current_mode);
+	    }
+	  GSNotifyASAP();
+	  if (--select_return == 0)
+	    break;
+        }
+      if (FD_ISSET (fdIndex, &read_fds))
+        {
+	  GSRunLoopWatcher	*watcher;
+
+	  watcher = (GSRunLoopWatcher*)NSMapGet(_rfdMap, (void*)fdIndex);
+	  if (watcher != nil && watcher->_invalidated == NO)
+	    {
+	      /*
+	       * The watcher is still valid - so call it's receivers
+	       * event handling method.
+	       */
+	      (*watcher->handleEvent)(watcher->receiver,
+		    eventSel, watcher->data, watcher->type,
+		    (void*)(gsaddr)fdIndex, _current_mode);
+	    }
+	  GSNotifyASAP();
+	  if (--select_return == 0)
+	    break;
+        }
+      if (++fdIndex >= end_inputs)
+	{
+	  fdIndex = 0;
+	}
+    }
+  while (fdIndex != fdEnd);
 
 
   /* Clean up before returning. */
+  NSResetMapTable(_efdMap);
   NSResetMapTable(_rfdMap);
   NSResetMapTable(_wfdMap);
 
@@ -1290,7 +1330,7 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
    * releases it.
    */
   d = [d earlierDate: date];
-  RETAIN(d);
+  IF_NO_GC(RETAIN(d));
 
   /* Wait, listening to our input sources. */
   [self acceptInputForMode: mode beforeDate: d];
@@ -1310,11 +1350,6 @@ const NSMapTableValueCallBacks ArrayMapValueCallBacks =
   [self runUntilDate: date forMode: _current_mode];
 }
 
-
-/* NSRunLoop mode strings. */
-
-id NSDefaultRunLoopMode = @"NSDefaultRunLoopMode";
-
 @end
 
 
@@ -1331,26 +1366,31 @@ id NSDefaultRunLoopMode = @"NSDefaultRunLoopMode";
 }
 
 - (void) cancelPerformSelector: (SEL)aSelector
-			target: target
-		      argument: argument
+			target: (id) target
+		      argument: (id) argument
 {
-  RunLoopPerformer	*item;
-  int			count = [_performers count];
-  int			i;
+  NSMapEnumerator	enumerator;
+  GSIArray		performers;
+  void			*mode;
 
-  RETAIN(target);
-  RETAIN(argument);
-  for (i = count; i > 0; i--)
+  enumerator = NSEnumerateMapTable(_mode_2_performers);
+
+  while (NSNextMapEnumeratorPair(&enumerator, &mode, (void**)&performers))
     {
-      item = (RunLoopPerformer*)[_performers objectAtIndex: (i-1)];
+      unsigned	count = GSIArrayCount(performers);
 
-      if ([item matchesSelector: aSelector target: target argument: argument])
+      while (count--)
 	{
-	  [_performers removeObjectAtIndex: (i-1)];
+	  GSRunLoopPerformer	*p;
+
+	  p = GSIArrayItemAtIndex(performers, count).obj;
+	  if (p->target == target && sel_eq(p->selector, aSelector)
+	    && p->argument == argument)
+	    {
+	      GSIArrayRemoveItemAtIndex(performers, count);
+	    }
 	}
     }
-  RELEASE(argument);
-  RELEASE(target);
 }
 
 - (void) configureAsServer
@@ -1359,42 +1399,59 @@ id NSDefaultRunLoopMode = @"NSDefaultRunLoopMode";
 }
 
 - (void) performSelector: (SEL)aSelector
-		  target: target
-		argument: argument
+		  target: (id)target
+		argument: (id)argument
 		   order: (unsigned int)order
 		   modes: (NSArray*)modes
 {
-  RunLoopPerformer	*item;
-  int			count = [_performers count];
+  unsigned		count = [modes count];
 
-  item = [[RunLoopPerformer alloc] initWithSelector: aSelector
-					     target: target
-					   argument: argument
-					      order: order
-					      modes: modes];
-  /* Add new item to list - reverse ordering */
-  if (count == 0)
+  if (count > 0)
     {
-      [_performers addObject: item];
-    }
-  else
-    {
-      int	i;
+      NSString			*array[count];
+      GSRunLoopPerformer	*item;
 
-      for (i = 0; i < count; i++)
+      item = [[GSRunLoopPerformer alloc] initWithSelector: aSelector
+						   target: target
+						 argument: argument
+						    order: order];
+
+      [modes getObjects: array];
+      while (count-- > 0)
 	{
-	  if ([[_performers objectAtIndex: i] order] <= order)
+	  NSString	*mode = array[count];
+	  GSIArray	performers = NSMapGet(_mode_2_performers, mode);
+	  unsigned	end;
+	  unsigned	i;
+
+	  if (performers == 0)
 	    {
-	      [_performers insertObject: item atIndex: i];
-	      break;
+	      NSZone	*z = [self zone];
+
+	      performers = NSZoneMalloc(z, sizeof(GSIArray_t));
+	      GSIArrayInitWithZoneAndCapacity(performers, z, 8);
+	      NSMapInsert(_mode_2_performers, mode, performers);
+	    }
+
+	  end = GSIArrayCount(performers);
+	  for (i = 0; i < end; i++)
+	    {
+	      GSRunLoopPerformer	*p;
+
+	      p = GSIArrayItemAtIndex(performers, i).obj;
+	      if (p->order <= order)
+		{
+		  GSIArrayInsertItem(performers, (GSIArrayItem)item, i);
+		  break;
+		}
+	    }
+	  if (i == end)
+	    {
+	      GSIArrayInsertItem(performers, (GSIArrayItem)item, i);
 	    }
 	}
-      if (i == count)
-	{
-	  [_performers addObject: item];
-	}
+      RELEASE(item);
     }
-  RELEASE(item);
 }
 
 - (void) removePort: (NSPort*)port
