@@ -22,6 +22,7 @@
    */
 
 #include <config.h>
+#include <string.h>
 #include <objc/objc-api.h>
 #include <Foundation/NSZone.h>
 #include <Foundation/NSException.h>
@@ -64,10 +65,8 @@ typeToName1(char type)
       case _C_UINT:	return "unsigned int";
       case _C_LNG:	return "long";
       case _C_ULNG:	return "unsigned long";
-#ifdef	_C_LNG_LNG
       case _C_LNG_LNG:	return "long long";
       case _C_ULNG_LNG:	return "unsigned long long";
-#endif
       case _C_FLT:	return "float";
       case _C_DBL:	return "double";
       case _C_PTR:	return "pointer";
@@ -99,6 +98,7 @@ typeToName2(char type)
 {
   switch (type & _GSC_MASK)
     {
+      case _GSC_CID:	return "class (encoded as id)";
       case _GSC_CLASS:	return "class";
       case _GSC_ID:	return "object";
       case _GSC_SEL:	return "selector";
@@ -110,10 +110,8 @@ typeToName2(char type)
       case _GSC_UINT:	return "unsigned int";
       case _GSC_LNG:	return "long";
       case _GSC_ULNG:	return "unsigned long";
-#ifdef	_GSC_LNG_LNG
       case _GSC_LNG_LNG:	return "long long";
       case _GSC_ULNG_LNG:	return "unsigned long long";
-#endif
       case _GSC_FLT:	return "float";
       case _GSC_DBL:	return "double";
       case _GSC_PTR:	return "pointer";
@@ -458,9 +456,12 @@ mapClassName(NSUnarchiverObjectInfo *info)
       (*tagImp)(src, tagSel, &ainfo, 0, &cursor);
       if (info != (ainfo & _GSC_MASK))
         {
-          [NSException raise: NSInternalInconsistencyException
-                      format: @"expected %s and got %s",
-                        typeToName2(info), typeToName2(ainfo)];
+	  if (info != _GSC_ID || (ainfo & _GSC_MASK) != _GSC_CID)
+	    {
+	      [NSException raise: NSInternalInconsistencyException
+			  format: @"expected %s and got %s",
+			    typeToName2(info), typeToName2(ainfo)];
+	    }
         }
 
       for (i = 0; i < count; i++)
@@ -518,7 +519,7 @@ mapClassName(NSUnarchiverObjectInfo *info)
 		   *	order to give the appearance that it's actually a
 		   *	new object.
 		   */
-		  RETAIN(obj);
+		  IF_NO_GC(RETAIN(obj));
 		}
 	      else
 		{
@@ -561,7 +562,10 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	  NSUnarchiverObjectInfo	*classInfo;
 	  Class		dummy;
 
-	  typeCheck(*type, _GSC_CLASS);
+	  if (*type != _C_ID)
+	    {
+	      typeCheck(*type, _GSC_CLASS);
+	    }
 	  /*
 	   *	Special case - a zero crossref value size is a nil pointer.
 	   */
@@ -884,7 +888,7 @@ mapClassName(NSUnarchiverObjectInfo *info)
 #if	GS_HAVE_I64
 	  bigval = val;
 #else
-	  val = GSSwapBigI64ToHost(val);
+	  bigval = GSSwapBigI64ToHost(val);
 #endif
 	  break;
 	}
@@ -959,102 +963,32 @@ mapClassName(NSUnarchiverObjectInfo *info)
 	{
 	  void		*b;
 	  NSData	*d;
+	  NSZone	*z;
 
-	  b = NSZoneMalloc(zone, l);
-	  d = [[NSData allocWithZone: zone] initWithBytesNoCopy: b
-							 length: l
-						       fromZone: zone];
-	  AUTORELEASE(d);
+#if	GS_WITH_GC
+	  z = GSAtomicMallocZone();
+#else
+	  z = zone;
+#endif
+	  b = NSZoneMalloc(z, l);
 	  [self decodeArrayOfObjCType: @encode(unsigned char)
 				count: l
 				   at: b];
+	  d = [[NSData allocWithZone: zone] initWithBytesNoCopy: b
+							 length: l
+						       fromZone: z];
+	  IF_NO_GC(AUTORELEASE(d));
 	  return d;
 	}
       else
-	[NSException raise: NSInternalInconsistencyException
-		    format: @"Decoding data object with unknown type"];
+	{
+	  [NSException raise: NSInternalInconsistencyException
+		      format: @"Decoding data object with unknown type"];
+	}
     }
   return [NSData data];
 }
 
-/*
- *	The [-decodeObject] method is implemented purely for performance -
- *	It duplicates the code for handling objects in the
- *	[-decodeValueOfObjCType:at:] method above, but differs in that the
- *	resulting object is autoreleased when it comes from this method.
- */
-- (id) decodeObject
-{
-  unsigned char	info;
-  unsigned	xref;
-  id		obj;
-
-  (*tagImp)(src, tagSel, &info, &xref, &cursor);
-  if ((info & _GSC_MASK) != _GSC_ID)
-    {
-      [NSException raise: NSInternalInconsistencyException
-		  format: @"expected object and got %s", typeToName2(info)];
-    }
-
-  /*
-   *	Special case - a zero crossref value is a nil pointer.
-   */
-  if ((info & _GSC_SIZE) == 0)
-    {
-      return nil;
-    }
-
-  if (info & _GSC_XREF)
-    {
-      if (xref >= GSIArrayCount(objMap))
-	{
-	  [NSException raise: NSInternalInconsistencyException
-		      format: @"object crossref missing - %d",
-			    xref];
-	}
-      obj = GSIArrayItemAtIndex(objMap, xref).obj;
-      /*
-       *	If it's a cross-reference, we don't need to autorelease it
-       *	since we didn't create it.
-       */
-      return obj;
-    }
-  else
-    {
-      Class	c;
-      id	rep;
-
-      if (xref != GSIArrayCount(objMap))
-	{
-	  [NSException raise: NSInternalInconsistencyException
-		      format: @"extra object crossref - %d",
-			    xref];
-	}
-      (*dValImp)(self, dValSel, @encode(Class), &c);
-
-      obj = [c allocWithZone: zone];
-      GSIArrayAddItem(objMap, (GSIArrayItem)obj);
-
-      rep = [obj initWithCoder: self];
-      if (rep != obj)
-	{
-	  obj = rep;
-	  GSIArraySetItemAtIndex(objMap, (GSIArrayItem)obj, xref);
-	}
-
-      rep = [obj awakeAfterUsingCoder: self];
-      if (rep != obj)
-	{
-	  obj = rep;
-	  GSIArraySetItemAtIndex(objMap, (GSIArrayItem)obj, xref);
-	}
-      /*
-       *	A newly allocated object needs to be autoreleased.
-       */
-      return AUTORELEASE(obj);
-    }
-}
-
 - (BOOL) isAtEnd
 {
   return (cursor >= [data length]);
@@ -1292,70 +1226,5 @@ mapClassName(NSUnarchiverObjectInfo *info)
   return YES;
 }
 
-/* libObjects compatibility */
-
-- (void) decodeArrayOfObjCType: (const char*) type
-		         count: (unsigned)count
-			    at: (void*)buf
-		      withName: (id*)name
-{
-  if (name)
-    {
-      *name = [self decodeObject];
-    }
-  else
-    {
-      (void)[self decodeObject];
-    }
-  [self decodeArrayOfObjCType: type count: count at: buf];
-}
-
-- (void) decodeIndent
-{
-}
-
-- (void) decodeValueOfCType: (const char*) type
-			 at: (void*)buf
-		   withName: (id*)name
-{
-  if (name)
-    {
-      *name = [self decodeObject];
-    }
-  else
-    {
-      (void)[self decodeObject];
-    }
-  (*dValImp)(self, dValSel, type, buf);
-}
-
-- (void) decodeValueOfObjCType: (const char*) type
-			    at: (void*)buf
-		      withName: (id*)name
-{
-  if (name)
-    {
-      *name = [self decodeObject];
-    }
-  else
-    {
-      (void)[self decodeObject];
-    }
-  (*dValImp)(self, dValSel, type, buf);
-}
-
-- (void) decodeObjectAt: (id*)anObject
-	       withName: (id*)name
-{
-  if (name)
-    {
-      *name = [self decodeObject];
-    }
-  else
-    {
-      (void)[self decodeObject];
-    }
-  (*dValImp)(self, dValSel, @encode(id), (void*)anObject);
-}
 @end
 
