@@ -24,15 +24,6 @@
 
 #include <config.h>
 #include <base/preface.h>
-#if 0 
-/* My Linux doesn't have <libc.h>.  Why is this necessary? 
-   What is a work-around that will work for all?  -mccallum*/
-#include <libc.h>
-/* If POSIX then:  #include <unistd.h> */
-#endif /* 0 */
-#if	!defined(__WIN32__)
-#include <pwd.h>
-#endif
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <errno.h>
@@ -311,7 +302,7 @@ static NSString	*pathForUser(NSString *user)
   path = [home stringByAppendingPathComponent: GNU_UserDefaultsPrefix];
   if ([mgr fileExistsAtPath: path isDirectory: &isDir] == NO)
     {
-      NSLog(@"Directory '%'@ does not exist - creating it", path);
+      NSLog(@"Directory '%@' does not exist - creating it", path);
       if ([mgr createDirectoryAtPath: path attributes: nil] == NO)
 	{
 	  NSLog(@"Unable to create user GNUstep directory '%@'", path);
@@ -405,26 +396,26 @@ static NSString	*pathForUser(NSString *user)
   // Check and if not existent add the Application and the Global domains
   if (![_persDomains objectForKey: processName])
     {
-      [_persDomains setObject: 
-		     [NSMutableDictionaryClass 
-		       dictionaryWithCapacity: 10] forKey: processName];
+      [_persDomains
+	setObject: [NSMutableDictionaryClass dictionaryWithCapacity: 10]
+	forKey: processName];
       [self __changePersistentDomain: processName];
     }
   if (![_persDomains objectForKey: NSGlobalDomain])
     {
-      [_persDomains setObject: 
-		   [NSMutableDictionaryClass 
-		     dictionaryWithCapacity: 10] forKey: NSGlobalDomain];
+      [_persDomains
+	setObject: [NSMutableDictionaryClass dictionaryWithCapacity: 10]
+	forKey: NSGlobalDomain];
       [self __changePersistentDomain: NSGlobalDomain];
     }
 	
   // Create volatile defaults and add the Argument and the Registration domains
   _tempDomains = [[NSMutableDictionaryClass alloc] initWithCapacity: 10];
   [_tempDomains setObject: [self __createArgumentDictionary] 
-	       forKey: NSArgumentDomain];
-  [_tempDomains setObject: 
-		 [NSMutableDictionaryClass
-		   dictionaryWithCapacity: 10] forKey: NSRegistrationDomain];
+		   forKey: NSArgumentDomain];
+  [_tempDomains
+    setObject: [NSMutableDictionaryClass dictionaryWithCapacity: 10]
+    forKey: NSRegistrationDomain];
 	
   return self;
 }
@@ -444,12 +435,12 @@ static NSString	*pathForUser(NSString *user)
 
 - (NSString*) description
 {
-  NSMutableString *desc =
-    [NSMutableString stringWithFormat: @"%@",[super description]];
+  NSMutableString *desc;
 
-  // $$$ Not Implemented
-  // It's good idea to put all useful info here -- so I can test it later
-  [self notImplemented: _cmd];
+  desc = [NSMutableString stringWithFormat: @"%@", [super description]];
+  [desc appendFormat: @" SearchList: %@", _searchList];
+  [desc appendFormat: @" Persistant: %@", _persDomains];
+  [desc appendFormat: @" Temporary: %@", _tempDomains];
   return desc;
 }
 
