@@ -36,8 +36,8 @@
 #include <Foundation/NSHost.h>
 #include <Foundation/NSByteOrder.h>
 
-#if	defined(__WIN32__) && !defined(__CYGWIN__)
-#include <Windows32/Sockets.h>
+#if defined(__MINGW__)
+#include <winsock.h>
 #else
 #include <time.h>
 #include <sys/time.h>
@@ -46,7 +46,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <signal.h>
-#endif /* __WIN32__ */
+#endif /* __MINGW__ */
 
 #include <sys/file.h>
 #include <sys/stat.h>
@@ -77,6 +77,10 @@
 #endif
 #endif
 
+#ifndef	INADDR_NONE
+#define	INADDR_NONE	-1
+#endif
+
 // Maximum data in single I/O operation
 #define	NETBUF_SIZE	4096
 
@@ -119,6 +123,8 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 
 #ifndef	HAVE_INET_ATON
       sin->sin_addr.s_addr = inet_addr([name cString]);
+      if (sin->sin_addr.s_addr == INADDR_NONE)
+	return NO;
 #else
       if (inet_aton([name cString], &sin->sin_addr) == 0)
 	return NO;
@@ -675,32 +681,9 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
     }
   else
     {
-      int	count;
-
-      /*
-       *	Determine number of bytes readable on descriptor.
-       */
-      if (ioctl(descriptor, FIONREAD, (char*)&count) < 0)
+      if ((len = read(descriptor, buf, sizeof(buf))) > 0)
 	{
-	  [NSException raise: NSFileHandleOperationException
-		      format: @"unable to use FIONREAD on descriptor - %s",
-		      strerror(errno)];
-	}
-
-      if (count == 0)
-	{
-	  len = 0;	/* End-of-file	*/
-	}
-      else
-	{
-	  if (count > sizeof(buf))
-	    {
-	      count = sizeof(buf);
-	    }
-	  if ((len = read(descriptor, buf, count)) > 0)
-	    {
-	      [d appendBytes: buf length: len];
-	    }
+	  [d appendBytes: buf length: len];
 	}
     }
   if (len < 0)
@@ -1241,7 +1224,8 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 	      struct sockaddr_in	sin;
 	      int			size = sizeof(sin);
 
-	      h = [[UnixFileHandle alloc] initWithFileDescriptor: desc];
+	      h = [[UnixFileHandle alloc] initWithFileDescriptor: desc
+						  closeOnDealloc: YES];
 	      getpeername(desc, (struct sockaddr*)&sin, &size);
 	      [h setAddr: &sin];
 	      [readInfo setObject: h
