@@ -18,6 +18,9 @@
 
    */
 
+#include "config.h"
+#include	<string.h>
+#include	<base/preface.h>
 #include	<Foundation/NSArray.h>
 #include	<Foundation/NSDictionary.h>
 #include	<Foundation/NSString.h>
@@ -29,9 +32,9 @@
 
 
 int
-main(int argc, char** argv)
+main(int argc, char** argv, char **env)
 {
-  NSAutoreleasePool	*pool = [NSAutoreleasePool new];
+  NSAutoreleasePool	*pool;
   NSUserDefaults	*defs;
   NSProcessInfo		*proc;
   NSArray		*args;
@@ -44,6 +47,10 @@ main(int argc, char** argv)
   BOOL			found = NO;
   int			i;
 
+#ifdef GS_PASS_ARGUMENTS
+  [NSProcessInfo initializeWithArguments:argv count:argc environment:env];
+#endif
+  pool = [NSAutoreleasePool new];
   proc = [NSProcessInfo processInfo];
   if (proc == nil)
     {
@@ -213,6 +220,8 @@ property list which is set as the value of a default.\n\n");
   if ([[args objectAtIndex: i] isEqual: @"read"] ||
       [[args objectAtIndex: i] isEqual: @"readkey"])
     {
+      NSDictionary	*locale = [defs dictionaryRepresentation];
+
       if ([[args objectAtIndex: i] isEqual: @"read"])
 	{
 	  if ([args count] == ++i)
@@ -266,7 +275,8 @@ property list which is set as the value of a default.\n\n");
 
 			  printf("%s %s '",
 				[domainName cString], [key cString]);
-			  ptr = [[obj description] cString];
+			  ptr = [[obj descriptionWithLocale: locale indent: 0]
+			    cString];
 			  while (*ptr)
 			    {
 			      if (*ptr == '\'')
@@ -289,7 +299,8 @@ property list which is set as the value of a default.\n\n");
 
 			  printf("%s %s '",
 				[domainName cString], [name cString]);
-			  ptr = [[obj description] cString];
+			  ptr = [[obj descriptionWithLocale: locale indent: 0]
+			    cString];
 			  while (*ptr)
 			    {
 			      if (*ptr == '\'')
