@@ -23,18 +23,20 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifndef __WIN32__
 #include <unistd.h>		/* for gethostname() */
+#ifndef __MINGW32__
 #include <sys/param.h>		/* for MAXHOSTNAMELEN */
 #include <sys/types.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>		/* for inet_ntoa() */
-#endif /* !__WIN32__ */
+#endif /* !__MINGW32__ */
 #include <errno.h>
 #include <limits.h>
 #include <string.h>		/* for strchr() */
 #include <ctype.h>		/* for strchr() */
-#ifndef __WIN32__
+#ifdef __MINGW32__
+#include <winsock.h>
+#else
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <netdb.h>
@@ -53,7 +55,7 @@
 
 #include <netinet/in.h>
 #include <net/if.h>
-#ifndef	SIOCGIFCONF
+#if	!defined(SIOCGIFCONF) || defined(__CYGWIN__)
 #include <sys/ioctl.h>
 #ifndef	SIOCGIFCONF
 #include <sys/sockio.h>
@@ -63,7 +65,11 @@
 #if	defined(__svr4__)
 #include <sys/stropts.h>
 #endif
-#endif /* !__WIN32__ */
+#endif /* !__MINGW32__ */
+
+#if HAVE_GETOPT_H
+#include <getopt.h>
+#endif
 
 #include	"gdomap.h"
 /*
@@ -95,9 +101,11 @@
 
 typedef	unsigned char	*uptr;
 int	debug = 0;		/* Extra debug logging.			*/
+int	nobcst = 0;		/* turn off broadcast probing.		*/
 int	nofork = 0;		/* turn off fork() for debugging.	*/
 int	noprobe = 0;		/* turn off probe for unknown servers.	*/
 int	interval = 600;		/* Minimum time (sec) between probes.	*/
+char	*pidfile = NULL;	/* file to write PID to			*/
 
 int	udp_sent = 0;
 int	tcp_sent = 0;
@@ -183,6 +191,8 @@ static plentry	*plist = 0;
  */
 int interfaces = 0;		/* Number of interfaces.	*/
 struct in_addr	*addr;		/* Address of each interface.	*/
+unsigned char	*bcok;		/* Broadcast OK for interface?	*/
+struct in_addr	*bcst;		/* Broadcast for interface.	*/
 struct in_addr	*mask;		/* Netmask of each interface.	*/
 
 static int
@@ -207,7 +217,7 @@ is_local_net(struct in_addr a)
 
   for (i = 0; i < interfaces; i++)
     {
-      if ((mask[i].s_addr&&addr[i].s_addr) == (mask[i].s_addr&&a.s_addr))
+      if ((mask[i].s_addr && addr[i].s_addr) == (mask[i].s_addr && a.s_addr))
 	{
 	  return 1;
 	}
@@ -624,7 +634,7 @@ prb_tim(long when)
   when -= 1800;
   for (i = prb_used - 1; i >= 0; i--)
     {
-      if (prb[i]->when < when && prb[i]->when < last_probe)
+      if (noprobe == 0 && prb[i]->when < when && prb[i]->when < last_probe)
 	{
 	  prb_del(&prb[i]->sin);
 	}
@@ -750,63 +760,108 @@ init_iface()
       close(desc);
       exit(1);
     }
+  if (addr != 0) free(addr);
   addr = (struct in_addr*)malloc(num_iface*IASIZE);
+  if (bcok != 0) free(bcok);
+  bcok = (char*)malloc(num_iface*sizeof(char));
+  if (bcst != 0) free(bcst);
+  bcst = (struct in_addr*)malloc(num_iface*IASIZE);
+  if (mask != 0) free(mask);
   mask = (struct in_addr*)malloc(num_iface*IASIZE);
 
   final = (struct ifreq*)&ifc.ifc_buf[ifc.ifc_len];
   for (ifr = ifc.ifc_req; ifr < final; ifr++)
     {
-      if (ifr->ifr_addr.sa_family == AF_INET)
-	{	/* IP interface */
-	  ifreq = *ifr;
-	  if (ioctl(desc, SIOCGIFFLAGS, (char *)&ifreq) < 0)
+      ifreq = *ifr;
+      if (ioctl(desc, SIOCGIFFLAGS, (char *)&ifreq) < 0)
+        {
+          perror("SIOCGIFFLAGS");
+        }
+      else if (ifreq.ifr_flags & IFF_UP)
+        {  /* interface is up */
+	  int	broadcast = 0;
+	  int	pointopoint = 0;
+
+	  if (ifreq.ifr_flags & IFF_BROADCAST)
 	    {
-	      perror("SIOCGIFFLAGS");
+	      broadcast = 1;
 	    }
-	  else if (ifreq.ifr_flags & IFF_UP)
-	    {	/* active interface */
-	      if (ioctl(desc, SIOCGIFADDR, (char *)&ifreq) < 0)
-		{
-		  perror("SIOCGIFADDR");
-		}
-	      else
-		{
-		  if (interfaces >= MAX_IFACE)
-		    {
-		      fprintf(stderr,
+#ifdef IFF_POINTOPOINT
+	  if (ifreq.ifr_flags & IFF_POINTOPOINT)
+	    {
+	      pointopoint = 1;
+	    }
+#endif
+          if (ioctl(desc, SIOCGIFADDR, (char *)&ifreq) < 0)
+            {
+              perror("SIOCGIFADDR");
+            }
+          else if (ifreq.ifr_addr.sa_family == AF_INET)
+            {	/* IP interface */
+	      if (interfaces >= MAX_IFACE)
+	        {
+	          fprintf(stderr,
 "You have too many network interfaces on your machine (in which case you need\n"
 "to change the 'MAX_IFACE' constant in gdomap.c and rebuild it), or your\n"
 "system is buggy, and you need to use the '-a' command line flag for\n"
 "gdomap to manually set the interface addresses and masks to be used.\n");
-		      close(desc);
-		      exit(1);
-		    }
-		  addr[interfaces] =
-			((struct sockaddr_in *)&ifreq.ifr_addr)->sin_addr;
-		  if (ioctl(desc, SIOCGIFNETMASK, (char *)&ifreq) < 0)
+	          close(desc);
+	          exit(1);
+	        }
+	      addr[interfaces] =
+		((struct sockaddr_in *)&ifreq.ifr_addr)->sin_addr;
+	      bcok[interfaces] = (broadcast | pointopoint);
+#ifdef IFF_POINTOPOINT
+	      if (pointopoint)
+		{
+		  if (ioctl(desc, SIOCGIFDSTADDR, (char*)&ifreq) < 0)
 		    {
-		      perror("SIOCGIFNETMASK");
-		      /*
-		       *	If we can't get a netmask - assume a class-c
-		       *	network.
-		       */
-		      mask[interfaces] = class_c_mask;
+		      perror("SIOCGIFDSTADDR");
+		      bcok[interfaces] = 0;
 		    }
 		  else
 		    {
+		      bcst[interfaces]
+			= ((struct sockaddr_in *)&ifreq.ifr_dstaddr)->sin_addr;
+		    }
+		}
+	      else
+#endif
+		{
+		  if (ioctl(desc, SIOCGIFBRDADDR, (char*)&ifreq) < 0)
+		    {
+		      perror("SIOCGIFBRDADDR");
+		      bcok[interfaces] = 0;
+		    }
+		  else
+		    {
+		      bcst[interfaces]
+			= ((struct sockaddr_in*)&ifreq.ifr_broadaddr)->sin_addr;
+		    }
+		}
+	      if (ioctl(desc, SIOCGIFNETMASK, (char *)&ifreq) < 0)
+	        {
+	          perror("SIOCGIFNETMASK");
+		  /*
+		   *	If we can't get a netmask - assume a class-c
+		   *	network.
+		   */
+		   mask[interfaces] = class_c_mask;
+		}
+              else
+                {
 /*
  *	Some systems don't have ifr_netmask
  */
 #ifdef	ifr_netmask
-		      mask[interfaces] =
-		        ((struct sockaddr_in *)&ifreq.ifr_netmask)->sin_addr;
+                  mask[interfaces] =
+                    ((struct sockaddr_in *)&ifreq.ifr_netmask)->sin_addr;
 #else
-		      mask[interfaces] =
-		        ((struct sockaddr_in *)&ifreq.ifr_addr)->sin_addr;
+                  mask[interfaces] =
+                    ((struct sockaddr_in *)&ifreq.ifr_addr)->sin_addr;
 #endif
-		    }
-		  interfaces++;
-		}
+                }
+              interfaces++;
 	    }
 	}
     }
@@ -1037,6 +1092,15 @@ init_ports()
     {
       fprintf(stderr, "Warning - unable to set 're-use' on UDP socket\n");
     }
+  if (nobcst == 0)
+    {
+      r = 1;
+      if ((setsockopt(udp_desc,SOL_SOCKET,SO_BROADCAST,(char*)&r,sizeof(r)))<0)
+	{
+	  nobcst++;
+	  fprintf(stderr, "Warning - unable to use 'broadcast' for probes\n");
+	}
+    }
   if ((r = fcntl(udp_desc, F_GETFL, 0)) >= 0)
     {
       r |= NBLK_OPT;
@@ -1062,6 +1126,12 @@ init_ports()
   if (bind(udp_desc, (void*)&sa, sizeof(sa)) < 0)
     {
       fprintf(stderr, "Unable to bind address to UDP socket\n");
+      if (errno == EACCES)
+	{
+	  fprintf(stderr, "You probably need to run gdomap as root,\n");
+	  fprintf(stderr, "or run the nameserver on a non-standard\n");
+	  fprintf(stderr, "port that does not require root privilege.\n");
+	}
       exit(1);
     }
 
@@ -1099,6 +1169,12 @@ init_ports()
   if (bind(tcp_desc, (void*)&sa, sizeof(sa)) < 0)
     {
       fprintf(stderr, "Unable to bind address to TCP socket\n");
+      if (errno == EACCES)
+	{
+	  fprintf(stderr, "You probably need to run gdomap as root,\n");
+	  fprintf(stderr, "or run the nameserver on a non-standard\n");
+	  fprintf(stderr, "port that does not require root privilege.\n");
+	}
       exit(1);
     }
   if (listen(tcp_desc, QUEBACKLOG) < 0)
@@ -1182,6 +1258,10 @@ init_probe()
   int	iface;
   int	i;
 
+  if (noprobe > 0)
+    {
+      return;
+    }
   if (debug > 2)
     {
       fprintf(stderr, "Initiating probe requests.\n");
@@ -1214,13 +1294,15 @@ init_probe()
 
   for (i = 0; i < nlist_size; i++)
     {
+      int		broadcast = 0;
+      int		elen = 0;
       struct in_addr	*other;
-      int		elen;
       struct in_addr	sin;
       int		high;
       int		low;
       unsigned long	net;
       int		j;
+      struct in_addr	b;
 
       /*
        *	Build up a list of addresses that we serve on this network.
@@ -1229,31 +1311,42 @@ init_probe()
 	{
 	  if ((addr[iface].s_addr & mask[iface].s_addr) == nlist[i])
 	    {
-	      unsigned long ha;		/* full host address.	*/
-	      unsigned long hm;		/* full netmask.	*/
-
-	      ha = ntohl(addr[iface].s_addr);
-	      hm = ntohl(mask[iface].s_addr);
-
-	      /*
-	       *	Make sure that our netmasks are restricted
-	       *	to class-c networks and subnets of those
-	       *	networks - we don't want to be probing
-	       *	more than a couple of hundred hosts!
-	       */
-	      if ((mask[iface].s_addr | class_c_mask.s_addr)
-		    != mask[iface].s_addr)
-		{
-		  fprintf(stderr, "gdomap - warning - netmask %s will be " 
-			"treated as 255.255.255.0 for %s\n",
-			inet_ntoa(mask[iface]), inet_ntoa(addr[iface]));
-		  hm |= ~255;
-		}
 	      sin = addr[iface];
-	      net = ha & hm & ~255;		/* class-c net number.	*/
-	      low = ha & hm & 255;		/* low end of subnet.	*/
-	      high = low | (255 & ~hm);	/* high end of subnet.	*/
-	      elen = other_addresses_on_net(sin, &other);
+	      if (bcok[iface])
+		{
+		  /*
+		   * Simple broadcast for this address.
+		   */
+		  b.s_addr = bcst[iface].s_addr;
+		  broadcast = 1;
+		}
+	      else
+		{
+		  unsigned long ha;		/* full host address.	*/
+		  unsigned long hm;		/* full netmask.	*/
+
+		  ha = ntohl(addr[iface].s_addr);
+		  hm = ntohl(mask[iface].s_addr);
+
+		  /*
+		   *	Make sure that our netmasks are restricted
+		   *	to class-c networks and subnets of those
+		   *	networks - we don't want to be probing
+		   *	more than a couple of hundred hosts!
+		   */
+		  if ((mask[iface].s_addr | class_c_mask.s_addr)
+			!= mask[iface].s_addr)
+		    {
+		      fprintf(stderr, "gdomap - warning - netmask %s will be " 
+			    "treated as 255.255.255.0 for %s\n",
+			    inet_ntoa(mask[iface]), inet_ntoa(addr[iface]));
+		      hm |= ~255;
+		    }
+		  net = ha & hm & ~255;		/* class-c net number.	*/
+		  low = ha & hm & 255;		/* low end of subnet.	*/
+		  high = low | (255 & ~hm);	/* high end of subnet.	*/
+		  elen = other_addresses_on_net(sin, &other);
+		}
 	      break;
 	    }
 	}
@@ -1288,6 +1381,13 @@ init_probe()
 		}
 	    }
 	}
+      else if (broadcast)
+	{
+	  /*
+	   *	Now broadcast probe on this network.
+	   */
+	  queue_probe(&b, &sin, 0, 0, 0);
+	}
       else
 	{
 	  /*
@@ -1642,6 +1742,14 @@ handle_recv()
 	{
 	  fprintf(stderr, "recvfrom %s\n", inet_ntoa(addr->sin_addr));
 	}
+      if (is_local_host(addr->sin_addr) == 1)
+	{
+	  if (debug)
+	    {
+	      fprintf(stderr, "recvfrom packet from self discarded\n");
+	    }
+	  return;
+	}
       handle_request(udp_desc);
     }
   else
@@ -1684,7 +1792,14 @@ handle_request(int desc)
 	{
 	  fprintf(stderr, "request type '%c' from chan %d", type, desc);
 	}
-      fprintf(stderr, " - name: '%.*s' port: %ld\n", size, buf, port);
+      if (type == GDO_PROBE || type == GDO_PREPLY)
+	{
+	  fprintf(stderr, "\n");
+	}
+      else
+	{
+	  fprintf(stderr, " - name: '%.*s' port: %ld\n", size, buf, port);
+	}
     }
 
   if (ptype != GDO_TCP_GDO && ptype != GDO_TCP_FOREIGN &&
@@ -2033,6 +2148,11 @@ handle_request(int desc)
 
 	  mcopy(&raddr, rbuf, IASIZE);
 	  mcopy(&laddr, rbuf+IASIZE, IASIZE);
+	  if (debug > 2)
+	    {
+	      fprintf(stderr, "Probe sent remote '%s'\n", inet_ntoa(raddr));
+	      fprintf(stderr, "Probe sent local  '%s'\n", inet_ntoa(laddr));
+	    }
 	
 	  mcopy(wbuf+IASIZE, &raddr, IASIZE);
 	  /*
@@ -2040,10 +2160,15 @@ handle_request(int desc)
 	   *	probing, try to select one on the same network to send back.
 	   *	otherwise, respond with the address it was probing.
 	   */
-	  if (is_local_host(laddr) == 0)
+	  if (is_local_host(laddr) == 0
+	    || laddr.s_addr == loopback.s_addr)
 	    {
 	      for (i = 0; i < interfaces; i++)
 		{
+		  if (addr[i].s_addr == loopback.s_addr)
+		    {
+		      continue;
+		    }
 		  if ((mask[i].s_addr && addr[i].s_addr) ==
 			(mask[i].s_addr && r_info[desc].addr.sin_addr.s_addr))
 		    {
@@ -2850,7 +2975,7 @@ int
 main(int argc, char** argv)
 {
   extern char	*optarg;
-  char	*options = "CHL:M:P:R:T:U:a:c:dfi:p";
+  char	*options = "CHI:L:M:P:R:T:U:a:bc:dfi:p";
   int		c;
   int		ptype = GDO_TCP_GDO;
   int		port = 0;
@@ -2876,6 +3001,7 @@ main(int argc, char** argv)
 	    printf("GNU Distributed Objects name server\n");
 	    printf("-C		help about configuration\n");
 	    printf("-H		general help\n");
+	    printf("-I		pid file to write pid\n");
 	    printf("-L name		perform lookup for name then quit.\n");
 	    printf("-M name		machine name for L (default local)\n");
 	    printf("-P number	port number required for R option.\n");
@@ -2889,7 +3015,7 @@ main(int argc, char** argv)
 	    printf("-d		extra debug logging.\n");
 	    printf("-f		avoid fork() to make debugging easy\n");
 	    printf("-i seconds	re-probe at this interval (roughly), min 60\n");
-	    printf("-p		obsolete no-op\n");
+	    printf("-p		disable probing for other servers\n");
 	    printf("\n");
 	    exit(0);
 
@@ -2916,7 +3042,7 @@ main(int argc, char** argv)
 	    printf("\n");
 	    printf(
 "A configuration file consists of a list of IP addresses to be probed.\n"
-"The IP addresses shoudl be in standard 'dot' notation, one per line.\n"
+"The IP addresses should be in standard 'dot' notation, one per line.\n"
 "Empty lines are permitted in the configuration file.\n"
 "Anything on a line after a hash ('#') is ignored.\n"
 "You tell gdomap about the config file with the '-c' command line option.\n");
@@ -2995,6 +3121,10 @@ printf(
 	    load_iface(optarg);
 	    break;
 
+	  case 'b':
+	    nobcst++;
+	    break;
+
 	  case 'c':
 	    {
 	      FILE	*fptr = fopen(optarg, "r");
@@ -3093,7 +3223,11 @@ printf(
 	      fclose(fptr);
 	    }
 	    break;
-	      
+
+	  case 'I':
+	    pidfile = optarg;
+	    break;
+
 	  case 'd':
 	    debug++;
 	    break;
@@ -3152,20 +3286,33 @@ printf(
 	}
     }
 
+  if (pidfile) {
+    {
+      FILE	*fptr = fopen(pidfile, "a");
+
+      if (fptr == 0)
+	{
+	  fprintf(stderr, "Unable to open pid file - '%s'\n", pidfile);
+	  exit(1);
+	}
+      fprintf(fptr, "%d\n", (int) getpid());
+      fclose(fptr);
+    }
+  }
+
   /*
    *	Ensure we don't have any open file descriptors which may refer
    *	to sockets bound to ports we may try to use.
    *
-   *	Use '/dev/tty' to produce logging output and use '/dev/null'
-   *	for stdin and stdout.
+   *	Use '/dev/null' for stdin and stdout.  Assume stderr is ok.
    */
   for (c = 0; c < FD_SETSIZE; c++)
     {
-      (void)close(c);
+      if (c != 2)
+	(void)close(c);
     }
   (void)open("/dev/null", O_RDONLY);	/* Stdin.	*/
   (void)open("/dev/null", O_WRONLY);	/* Stdout.	*/
-  (void)open("/dev/tty", O_WRONLY);	/* Stderr.	*/
 
   init_my_port();	/* Determine port to listen on.		*/
   if (interfaces == 0)
@@ -3195,6 +3342,27 @@ printf(
 	}
     }
   init_ports();	/* Create ports to handle requests.	*/
+
+  /*
+   * Try to become a 'safe' user now that we have
+   * done everything that needs root priv.
+   */ 
+  if (getuid () != 0)
+    {
+      setuid (getuid ());
+    }
+  else
+    {
+      setuid (-1);
+    }
+#if	!defined(__svr4__)
+  /*
+   * As another level of paranoia - restrict this process to /tmp
+   */
+  chdir("/tmp");
+  chroot("/tmp");
+#endif
+
   init_probe();	/* Probe other name servers on net.	*/
 
   if (debug)
@@ -3221,6 +3389,16 @@ queue_probe(struct in_addr* to, struct in_addr* from, int l, struct in_addr* e,
     {
       fprintf(stderr, "Probing for server on '%s'", inet_ntoa(*to));
       fprintf(stderr, " from '%s'\n", inet_ntoa(*from));
+      if (l > 0)
+	{
+	  int	i;
+
+	  fprintf(stderr, " %d additional local addresses sent -\n", l);
+	  for (i = 0; i < l; i++)
+	    {
+	      fprintf(stderr, " '%s'\n", inet_ntoa(e[i]));
+	    }
+	}
     }
   mzero(&sin, sizeof(sin));
   sin.sin_family = AF_INET;
