@@ -928,6 +928,41 @@ static inline void
 DPSdocedited(GSCTXT *ctxt, int edited, int window)
 __attribute__((unused));
 
+/* ----------------------------------------------------------------------- */
+/* GNUstep Event and other I/O extensions */
+/* ----------------------------------------------------------------------- */
+static inline NSEvent*
+DPSGetEvent(GSCTXT *ctxt, unsigned mask, NSDate* limit, NSString *mode)
+__attribute__((unused));
+
+static inline NSEvent*
+DPSPeekEvent(GSCTXT *ctxt, unsigned mask, NSDate* limit, NSString *mode)
+__attribute__((unused));
+
+static inline void
+DPSDiscardEvents(GSCTXT *ctxt, unsigned mask, NSEvent* limit)
+__attribute__((unused));
+
+static inline void
+DPSPostEvent(GSCTXT *ctxt, NSEvent* anEvent, BOOL atStart)
+__attribute__((unused));
+
+static inline void
+DPSmouselocation(GSCTXT *ctxt, float *x, float *y)
+__attribute__((unused));
+
+static inline void
+DPSsetinputstate(GSCTXT *ctxt, int window, int state)
+__attribute__((unused));
+
+static inline void
+DPScurrentserverdevice(GSCTXT *ctxt, void **serverptr)
+__attribute__((unused));
+
+static inline void
+DPScurrentwindowdevice(GSCTXT *ctxt, int win, void **windowptr)
+__attribute__((unused));
+
 /* ----------------------------------------------------------------------- */
 /* Color operations */
 /* ----------------------------------------------------------------------- */
@@ -2516,4 +2551,25 @@ DPSmouselocation(GSCTXT *ctxt, float *x, float *y)
     (ctxt, @selector(DPSmouselocation::), x, y);
 }
 
+static inline void
+DPSsetinputstate(GSCTXT *ctxt, int window, int state)
+{
+  (ctxt->methods->DPSsetinputstate__)
+    (ctxt, @selector(DPSsetinputstate::), window, state);
+}
+
+static inline void
+DPScurrentserverdevice(GSCTXT *ctxt, void **serverptr)
+{
+  (ctxt->methods->DPScurrentserverdevice_)
+    (ctxt, @selector(DPScurrentserverdevice:), serverptr);
+}
+
+static inline void
+DPScurrentwindowdevice(GSCTXT *ctxt, int win, void **windowptr)
+{
+  (ctxt->methods->DPScurrentwindowdevice__)
+    (ctxt, @selector(DPScurrentwindowdevice::), win, windowptr);
+}
+
 #endif	
