@@ -514,6 +514,13 @@ typedef struct {
 	(NSGraphicsContext*, SEL, NSEvent*, BOOL);
   void (*DPSmouselocation__)
 	(NSGraphicsContext*, SEL, float*, float*);
+  void (*DPSsetinputstate__)
+	(NSGraphicsContext*, SEL, int, int);
+
+  void (*DPScurrentserverdevice_)
+	(NSGraphicsContext*, SEL, void **);
+  void (*DPScurrentwindowdevice__)
+	(NSGraphicsContext*, SEL, int, void **);
 
 } gsMethodTable;
 
