@@ -47,7 +47,9 @@ void __dummy_GMAppKit_functionForLinking() {}
   for (i = 0; i < count; i++) {
     NSWindow* window = [windows1 objectAtIndex:i];
 
-    if (![window isKindOfClass:[NSMenu class]])
+    if (([window isKindOfClass:[NSMenu class]] == NO)
+	&& ([NSStringFromClass ([window class]) 
+	    isEqualToString: @"NSMenuPanel"] == NO))
       [windows2 addObject:window];
   }
   [archiver encodeObject:windows2 withName:@"windows"];
@@ -64,6 +66,8 @@ void __dummy_GMAppKit_functionForLinking() {}
 - (id)initWithModelUnarchiver:(GMUnarchiver*)unarchiver
 {
   NSArray* windows;
+  NSEnumerator *enumerator;
+  NSWindow *win;
   NSWindow* keyWindow;
   NSWindow* mainWindow;
   NSMenu* mainMenu;
@@ -76,6 +80,14 @@ void __dummy_GMAppKit_functionForLinking() {}
 #endif
 
   windows = [unarchiver decodeObjectWithName:@"windows"];
+  enumerator = [windows objectEnumerator];
+  while ((win = [enumerator nextObject]) != nil)
+    {
+      /* If we did not retain the windows here, they would all get 
+	 released at the end of the event loop. */
+      RETAIN (win);
+    }
+
   keyWindow = [unarchiver decodeObjectWithName:@"keyWindow"];
   mainWindow = [unarchiver decodeObjectWithName:@"mainWindow"];
 
@@ -145,6 +157,7 @@ void __dummy_GMAppKit_functionForLinking() {}
 @end /* NSBox (GMArchiverMethods) */
 
 
+#if 0
 @implementation NSButton (GMArchiverMethods)
 
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
@@ -201,6 +214,7 @@ void __dummy_GMAppKit_functionForLinking() {}
 }
 
 @end /* NSButton (GMArchiverMethods) */
+#endif
 
 
 @implementation NSCell (GMArchiverMethods)
@@ -227,18 +241,38 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 - (id)initWithModelUnarchiver:(GMUnarchiver*)unarchiver
 {
+    int cellType = [unarchiver decodeIntWithName:@"type"];
     NSFont* font = [unarchiver decodeObjectWithName:@"font"];
-    if (!font)
-        font = [NSFont userFontOfSize:0];
 
+    // this is a tricky object to decode, because a number of its methods
+    // have side-effects; [-setEntryType:] converts the cell to a text-type
+    // cell and sets its font to the system font, so it comes first
+    [self setEntryType:[unarchiver decodeIntWithName:@"entryType"]];
+
+    // now set the font
     [self setFont:font];
 
-    // if (model_version >= 2) {
-    [self setStringValue:[unarchiver decodeStringWithName:@"stringValue"]];
-    [self setEntryType:[unarchiver decodeIntWithName:@"entryType"]];
+    // both [-setImage:] and [-setStringValue:] convert the cell to an
+    // image or text cell (respectively), so they must be called in the
+    // correct order for the type of cell desired
+    switch (cellType)
+      {
+        case NSTextCellType:
+          [self setImage:[unarchiver decodeObjectWithName:@"image"]];
+          [self setStringValue:
+              [unarchiver decodeStringWithName:@"stringValue"]];
+          break;
+        case NSImageCellType:
+          [self setStringValue:
+              [unarchiver decodeStringWithName:@"stringValue"]];
+          [self setImage:[unarchiver decodeObjectWithName:@"image"]];
+          break;
+        case NSNullCellType:
+          [self setType: NSNullCellType];
+          break;
+      }
     [self setAlignment:[unarchiver decodeIntWithName:@"alignment"]];
     [self setWraps:[unarchiver decodeBOOLWithName:@"wraps"]];
-    [self setImage:[unarchiver decodeObjectWithName:@"image"]];
     [self setState:[unarchiver decodeIntWithName:@"state"]];
     [self setEnabled:[unarchiver decodeBOOLWithName:@"isEnabled"]];
     [self setBordered:[unarchiver decodeBOOLWithName:@"isBordered"]];
@@ -247,9 +281,8 @@ void __dummy_GMAppKit_functionForLinking() {}
     [self setSelectable:[unarchiver decodeBOOLWithName:@"isSelectable"]];
     [self setScrollable:[unarchiver decodeBOOLWithName:@"isScrollable"]];
     [self setContinuous:[unarchiver decodeBOOLWithName:@"isContinuous"]];
-    [self sendActionOn:[unarchiver decodeIntWithName:@"sendActionMask"]];
-    [self setType:[unarchiver decodeIntWithName:@"type"]];
-    // }
+    /* Temporary commented out so buttons keep on working - new code fixing this under testing */
+    //    [self sendActionOn:[unarchiver decodeIntWithName:@"sendActionMask"]];
 
     return self;
 }
@@ -403,8 +436,13 @@ void __dummy_GMAppKit_functionForLinking() {}
     nc = [unarchiver decodeIntWithName:@"numberOfColumns"];
     cell_array = [unarchiver decodeObjectWithName:@"cells"];
     [self renewRows:nr columns:nc];
-    for (i = 0; (i < [cell_array count]) && (i < nr*nc); i++) {
-        [self putCell:[cell_array objectAtIndex:i] atRow:i/nc column:i%nc];
+    for (i = 0; (i < [cell_array count]) && (i < nr*nc); i++)
+    {
+        id	cell = [cell_array objectAtIndex:i];
+
+        [self putCell:cell atRow:i/nc column:i%nc];
+        if ([cell state])
+            [self selectCellAtRow:i/nc column:i%nc];
     }
 
     decodedDelegate = [unarchiver decodeObjectWithName:@"delegate"];
@@ -456,8 +494,6 @@ void __dummy_GMAppKit_functionForLinking() {}
 
   [self setContentView: AUTORELEASE([NSClipView new])];
 
-  [self setBackgroundColor:
-    [unarchiver decodeObjectWithName: @"backgroundColor"]];
   [self setBorderType:
     [unarchiver decodeIntWithName: @"borderType"]];
   [self setHasHorizontalScroller:
@@ -466,6 +502,8 @@ void __dummy_GMAppKit_functionForLinking() {}
     [unarchiver decodeBOOLWithName: @"hasVerticalScroller"]];
   [self setDocumentView:
     [unarchiver decodeObjectWithName: @"documentView"]];
+  [self setBackgroundColor:
+    [unarchiver decodeObjectWithName: @"backgroundColor"]];
 
   return self;
 }
@@ -702,10 +740,17 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 + (id)createObjectForModelUnarchiver:(GMUnarchiver*)unarchiver
 {
-  id image = [NSImage imageNamed:[unarchiver decodeStringWithName:@"name"]];
+  id image = nil;
+  NSString *imageName;
+
+  imageName = [unarchiver decodeStringWithName:@"name"];
+
+  if (imageName)
+    image = [NSImage imageNamed: imageName];
+
+  if (image == nil)
+    image = [NSImage imageNamed:@"GNUstepMenuImage"];
 
-  if (!image)
-    image = [NSImage imageNamed:@"NSRadioButton"];
   return image;
 }
 
@@ -723,14 +768,18 @@ void __dummy_GMAppKit_functionForLinking() {}
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
 {
   [archiver encodeString:[self title] withName:@"title"];
-  if ([self respondsToSelector: @selector(image)])
-    [archiver encodeObject:[self image] withName:@"image"];
-  if ([self respondsToSelector: @selector(onStateImage)])
-    [archiver encodeObject:[self onStateImage] withName:@"onStateImage"];
-  if ([self respondsToSelector: @selector(offStateImage)])
-    [archiver encodeObject:[self offStateImage] withName:@"offStateImage"];
-  if ([self respondsToSelector: @selector(mixedStateImage)])
-    [archiver encodeObject:[self mixedStateImage] withName:@"mixedStateImage"];
+  if ([self hasSubmenu] == NO)
+    {
+      if ([self respondsToSelector: @selector(image)])
+        [archiver encodeObject:[self image] withName:@"image"];
+      if ([self respondsToSelector: @selector(onStateImage)])
+        [archiver encodeObject:[self onStateImage] withName:@"onStateImage"];
+      if ([self respondsToSelector: @selector(offStateImage)])
+        [archiver encodeObject:[self offStateImage] withName:@"offStateImage"];
+      if ([self respondsToSelector: @selector(mixedStateImage)])
+         [archiver encodeObject:[self mixedStateImage]
+                       withName:@"mixedStateImage"];
+    }
   [archiver encodeString:[self keyEquivalent] withName:@"keyEquivalent"];
   if ([self respondsToSelector: @selector(state)])
     [archiver encodeInt:[self state] withName:@"state"];
@@ -835,13 +884,20 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 @end /* NSMenu (GMArchiverMethods) */
 
+/* This class is special - to avoid having the cell encoded we encode 
+   it directly.  Perhaps it would be wiser to do differently. */
 @implementation NSPopUpButton (GMArchiverMethods)
 
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
 {
-  [archiver encodeBOOL:[self pullsDown] withName:@"pullsDown"];
+  [archiver encodeBOOL:     [self pullsDown] withName: @"pullsDown"];
+  [archiver encodeBOOL:     [self isEnabled] withName: @"isEnabled"];
+  [archiver encodeInt:      [self tag]       withName: @"tag"];
+  [archiver encodeObject:   [self target]    withName: @"target"];
+  [archiver encodeSelector: [self action]    withName: @"action"];
+  [archiver encodeInt:      [self autoresizingMask]
+             withName:      @"autoresizingMask"];
   
-#if 1
   /* OUCH! This code crashes the translator; probably we interfere somehow with
      the way NSPopUpButton is handled by the NeXT's NIB code. Sorry, the
      popup buttons cannot be handled by the convertor! */
@@ -849,54 +905,63 @@ void __dummy_GMAppKit_functionForLinking() {}
   //[archiver encodeString:[self titleOfSelectedItem] withName:@"selectedItem"];
   //[archiver encodeString:[self title] withName:@"selectedItem"];
   //[super encodeWithModelArchiver:archiver];
-#endif
-  [archiver encodeRect:[self frame] withName:@"frame"];
+
+  [archiver encodeRect: [self frame] withName:@"frame"];
 }
 
 + (id)createObjectForModelUnarchiver:(GMUnarchiver*)unarchiver
 {
-  NSRect rect = [unarchiver decodeRectWithName:@"frame"];
-  NSPopUpButton *popup = \
-    [[[NSPopUpButton allocWithZone:[unarchiver objectZone]]
-                    initWithFrame:rect
-                        pullsDown:[unarchiver decodeBOOLWithName:@"pullsDown"]]
-    autorelease];
-  if (!popup)
-    NSLog (@"cannot create the requested view!");
+  NSRect         rect;
+  NSPopUpButton *popup;
+  BOOL           pullsDown;
 
+  rect      = [unarchiver decodeRectWithName: @"frame"];
+  pullsDown = [unarchiver decodeBOOLWithName: @"pullsDown"];
+
+  popup = [NSPopUpButton allocWithZone: [unarchiver objectZone]];
+  popup = [popup initWithFrame: rect pullsDown: pullsDown];
+  AUTORELEASE (popup);
+  
   return popup;
 }
 
 - (id)initWithModelUnarchiver:(GMUnarchiver*)unarchiver
 {
-  /* Check the following: the program simply crashes if there's nothing in the
-     model file */
-
-  int i, count;
-  NSMutableArray* decodedItems
-      = [unarchiver decodeArrayWithName:@"itemArray"];
-
-  self = [super initWithModelUnarchiver:unarchiver];
-
-  if (decodedItems) {
-      for (i = 0, count = [decodedItems count]; i < count; i++) {
+  int      i, count;
+  NSString *string;
+  NSArray  *decodedItems;
+  
+  decodedItems = [unarchiver decodeArrayWithName: @"itemArray"];
+  
+  if (decodedItems) 
+    {
+      count = [decodedItems count]; 
+      for (i = 0; i < count; i++) 
+	{
 	  id item = [decodedItems objectAtIndex:i];
 	  id myItem;
-	  
-	  [self addItemWithTitle:[item title]];
+      
+	  [self addItemWithTitle: [item title]];
 	  myItem = [self itemAtIndex:i];
-	  [myItem setTarget:[item target]];
-	  [myItem setAction:[item action]];
-	  [myItem setEnabled:[item isEnabled]];
-	  [myItem setTag:[item tag]];
-	  [myItem setKeyEquivalent:[item keyEquivalent]];
-      }
-  }
-
-  [self selectItemWithTitle:[unarchiver decodeStringWithName:@"selectedItem"]];
-  [self synchronizeTitleAndSelectedItem];
+	  [myItem setTarget:        [item target]];
+	  [myItem setAction:        [item action]];
+	  [myItem setEnabled:       [item isEnabled]];
+	  [myItem setTag:           [item tag]];
+	  [myItem setKeyEquivalent: [item keyEquivalent]];
+	}
+    }
+  
+  string = [unarchiver decodeStringWithName: @"selectedItem"];
+  [self selectItemWithTitle: string];
 
+  [self setEnabled: [unarchiver decodeBOOLWithName:     @"isEnabled"]];
+  [self setTag:     [unarchiver decodeIntWithName:      @"tag"]];
+  [self setTarget:  [unarchiver decodeObjectWithName:   @"target"]];
+  [self setAction:  [unarchiver decodeSelectorWithName: @"action"]];
 
+  [self setAutoresizingMask:
+      [unarchiver decodeIntWithName: @"autoresizingMask"]];
+  
   return self;
 }
 
@@ -932,47 +997,19 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
 {
-  id theCell = [self cell];
-
   [super encodeWithModelArchiver:archiver];
 
-  [archiver encodeBOOL:[self isSelectable] withName:@"isSelectable"];
   [archiver encodeSelector:[self errorAction] withName:@"errorAction"];
-  [archiver encodeObject:[self textColor] withName:@"textColor"];
-  [archiver encodeObject:[self backgroundColor] withName:@"backgroundColor"];
-  [archiver encodeBOOL:[self drawsBackground] withName:@"drawsBackground"];
-  [archiver encodeBOOL:[self isBordered] withName:@"isBordered"];
-  [archiver encodeBOOL:[self isBezeled] withName:@"isBezeled"];
   [archiver encodeObject:[self delegate] withName:@"delegate"];
-  [archiver encodeString:[self stringValue] withName:@"stringValue"];
-  [archiver encodeBOOL:[self isEditable] withName:@"isEditable"];
-  [archiver encodeBOOL:[theCell isScrollable] withName:@"isScrollable"];
 }
 
 - (id)initWithModelUnarchiver:(GMUnarchiver*)unarchiver
 {
-  id theCell;
-
   self = [super initWithModelUnarchiver:unarchiver];
 
-  [self setSelectable:[unarchiver decodeBOOLWithName:@"isSelectable"]];
   [self setErrorAction:[unarchiver decodeSelectorWithName:@"errorAction"]];
-  [self setTextColor:[unarchiver decodeObjectWithName:@"textColor"]];
-  [self setBackgroundColor:
-	    [unarchiver decodeObjectWithName:@"backgroundColor"]];
-  [self setDrawsBackground:[unarchiver decodeBOOLWithName:@"drawsBackground"]];
-
-  [self setBordered:[unarchiver decodeBOOLWithName:@"isBordered"]];
-  [self setBezeled:[unarchiver decodeBOOLWithName:@"isBezeled"]];
-
   [self setDelegate:[unarchiver decodeObjectWithName:@"delegate"]];
 
-  theCell = [self cell];
-
-  [theCell setStringValue:[unarchiver decodeStringWithName:@"stringValue"]];
-  [self setEditable:[unarchiver decodeBOOLWithName:@"isEditable"]];
-  [theCell setScrollable:[unarchiver decodeBOOLWithName:@"isScrollable"]];
-
   return self;
 }
 
@@ -984,7 +1021,26 @@ void __dummy_GMAppKit_functionForLinking() {}
 // subclasses may not want to encode all subviews...
 - (NSArray *)subviewsForModel
 {
+#ifdef GNU_GUI_LIBRARY  
   return [self subviews];
+#else /* Do not encode Apple's private classes */
+  NSArray        *views;
+  NSMutableArray *viewsToEncode;
+  id             e, o;
+  NSString       *classString;
+  
+  views = [self subviews];
+  viewsToEncode = [NSMutableArray array];
+  e = [views objectEnumerator];
+  while ((o = [e nextObject]))
+    {
+      classString = NSStringFromClass ([o class]);
+      if ([classString isEqualToString: @"_NSKeyboardFocusClipView"] == NO)
+	[viewsToEncode addObject: o];
+    }
+  
+  return viewsToEncode;
+#endif
 }
 
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
@@ -1098,13 +1154,14 @@ void __dummy_GMAppKit_functionForLinking() {}
   unsigned styleMask = [unarchiver decodeUnsignedIntWithName:@"styleMask"];
   NSRect ctRect = [unarchiver decodeRectWithName:@"contentFrame"];
 
-  NSWindow* win = [[[NSWindow allocWithZone:[unarchiver objectZone]]
-			initWithContentRect:ctRect
-			styleMask:styleMask backing:backingType defer:YES]
-			autorelease];
+  NSWindow	*win = [[self allocWithZone: [unarchiver objectZone]]
+			initWithContentRect: ctRect
+				  styleMask: styleMask
+				    backing: backingType
+				      defer: YES];
   //  printf("content: %g, %g -- frame %g, %g\n", ctRect.size.width, ctRect.size.height, [win frame].size.width, [win frame].size.height);
 
-  return win;
+  return AUTORELEASE(win);
 }
 
 - (id)initWithModelUnarchiver:(GMUnarchiver*)unarchiver
@@ -1136,7 +1193,14 @@ void __dummy_GMAppKit_functionForLinking() {}
   [self setBackgroundColor:
 	  [unarchiver decodeObjectWithName:@"backgroundColor"]];
   if ([unarchiver decodeBOOLWithName:@"isVisible"])
-    [self orderFront:nil];
+    {
+      /* If we are the main gmodel of the application, this code is
+	 being executed before the app became active, so we have to
+	 force ordering front of the window.  TODO: Refine the code so
+	 that if we are not the main gmodel then a more appropriate 
+         [self orderFront:nil] is used.*/
+      [self orderFrontRegardless];
+    }
 
 #if GNU_GUI_LIBRARY
 	[[self contentView] setNeedsDisplay:YES];
@@ -1151,36 +1215,11 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
 {
-    NSPoint wnOrigin = [self frame].origin;
-    NSRect ctFrame = [[self contentView] frame];
-
-    ctFrame.origin = wnOrigin;
-
-    [archiver encodeRect:ctFrame withName:@"contentFrame"];
-    [archiver encodeSize:[self maxSize] withName:@"maxSize"];
-    [archiver encodeSize:[self minSize] withName:@"minSize"];
-    [archiver encodeString:[self frameAutosaveName]
-	      withName:@"frameAutosaveName"];
-    [archiver encodeInt:[self level] withName:@"level"];
-    [archiver encodeBOOL:[self isVisible] withName:@"isVisible"];
-    [archiver encodeBOOL:[self isAutodisplay] withName:@"isAutodisplay"];
-    [archiver encodeString:[self title] withName:@"title"];
-    [archiver encodeString:[self representedFilename]
-	    withName:@"representedFilename"];
-    [archiver encodeBOOL:[self isReleasedWhenClosed]
-	    withName:@"isReleasedWhenClosed"];
-    [archiver encodeObject:[self contentView] withName:@"contentView"];
-    [archiver encodeBOOL:[self hidesOnDeactivate]
-	    withName:@"hidesOnDeactivate"];
-    [archiver encodeObject:[self backgroundColor] withName:@"backgroundColor"];
-    [archiver encodeUnsignedInt:[self styleMask] withName:@"styleMask"];
-    [archiver encodeUnsignedInt:[self backingType] withName:@"backingType"];
-    [archiver encodeConditionalObject:[self initialFirstResponder] 
-	      withName:@"initialFirstResponder"];
-
+    [super encodeWithModelArchiver: archiver];
+    
     [archiver encodeBOOL:[self isFloatingPanel] withName:@"isFloatingPanel"];
     [archiver encodeBOOL:[self becomesKeyOnlyIfNeeded]
-            withName:@"becomesKeyOnlyIfNeeded"];
+	      withName:@"becomesKeyOnlyIfNeeded"];
     [archiver encodeBOOL:[self worksWhenModal] withName:@"worksWhenModal"];
 }
 
@@ -1190,55 +1229,26 @@ void __dummy_GMAppKit_functionForLinking() {}
 			   @"backingType"];
     unsigned styleMask = [unarchiver decodeUnsignedIntWithName:@"styleMask"];
     NSRect ctRect = [unarchiver decodeRectWithName:@"contentFrame"];
-    NSPanel* panel = [[[NSPanel allocWithZone:[unarchiver objectZone]]
+    NSPanel* panel = [[[[self class] allocWithZone:[unarchiver objectZone]]
 		     initWithContentRect:ctRect
 		     styleMask:styleMask backing:backingType defer:YES]
 		    autorelease];
+    // use [self class] here, so it works for subclasses
 
     return panel;
 }
 
 -(id)initWithModelUnarchiver :(GMUnarchiver *)unarchiver
 {
-    NSString* frameAutosaveName;
-    
-    [self setContentView:[unarchiver decodeObjectWithName:@"contentView"]];
-    [self setMaxSize:[unarchiver decodeSizeWithName:@"maxSize"]];
-    [self setMinSize:[unarchiver decodeSizeWithName:@"minSize"]];
-    
-    frameAutosaveName = [unarchiver decodeStringWithName:@"frameAutosaveName"];
-    if (frameAutosaveName)
-	[self setFrameAutosaveName:frameAutosaveName];
-    
-#ifdef GNU_GUI_LIBRARY
-    window_level = [unarchiver decodeIntWithName:@"level"];
-#endif
+    [super initWithModelUnarchiver: unarchiver];
 
-    [self setInitialFirstResponder: 
-	    [unarchiver decodeObjectWithName:@"initialFirstResponder"]];    
-    [self setAutodisplay:[unarchiver decodeBOOLWithName:@"isAutodisplay"]];
-    [self setTitle:[unarchiver decodeStringWithName:@"title"]];
-    [self setRepresentedFilename:
-        [unarchiver decodeStringWithName:@"representedFilename"]];
-    [self setReleasedWhenClosed:
-        [unarchiver decodeBOOLWithName:@"isReleasedWhenClosed"]];
-    [self setHidesOnDeactivate:
-        [unarchiver decodeBOOLWithName:@"hidesOnDeactivate"]];
-    [self setBackgroundColor:
-        [unarchiver decodeObjectWithName:@"backgroundColor"]];
-    if ([unarchiver decodeBOOLWithName:@"isVisible"])
-	[self orderFront:nil];
-
-    [self setFloatingPanel:
-        [unarchiver decodeBOOLWithName:@"isFloatingPanel"]];
+    [self setFloatingPanel: 
+	    [unarchiver decodeBOOLWithName: @"isFloatingPanel"]];
     [self setBecomesKeyOnlyIfNeeded:
-        [unarchiver decodeBOOLWithName:@"becomesKeyOnlyIfNeeded"]];
+	    [unarchiver decodeBOOLWithName: @"becomesKeyOnlyIfNeeded"]];
     [self setWorksWhenModal:
-        [unarchiver decodeBOOLWithName:@"setWorksWhenModal"]];
+	    [unarchiver decodeBOOLWithName: @"setWorksWhenModal"]];
 
-#if GNU_GUI_LIBRARY
-	[[self contentView] setNeedsDisplay:YES];
-#endif
     return self;
 }
 
@@ -1249,44 +1259,16 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
 {
-    //NSWindow specific
-    [archiver encodeRect:[self frame] withName:@"frame"];
-    [archiver encodeSize:[self maxSize] withName:@"maxSize"];
-    [archiver encodeSize:[self minSize] withName:@"minSize"];
-    [archiver encodeString:[self frameAutosaveName]
-	      withName:@"frameAutosaveName"];
-    [archiver encodeInt:[self level] withName:@"level"];
-    [archiver encodeBOOL:[self isVisible] withName:@"isVisible"];
-    [archiver encodeBOOL:[self isAutodisplay] withName:@"isAutodisplay"];
-    [archiver encodeString:[self title] withName:@"title"];
-    [archiver encodeString:[self representedFilename]
-	    withName:@"representedFilename"];
-    [archiver encodeBOOL:[self isReleasedWhenClosed]
-	    withName:@"isReleasedWhenClosed"];
-    [archiver encodeObject:[self contentView] withName:@"contentView"];
-    [archiver encodeBOOL:[self hidesOnDeactivate]
-	    withName:@"hidesOnDeactivate"];
-    [archiver encodeObject:[self backgroundColor] withName:@"backgroundColor"];
-    [archiver encodeUnsignedInt:[self styleMask] withName:@"styleMask"];
-    [archiver encodeUnsignedInt:[self backingType] withName:@"backingType"];
-    [archiver encodeConditionalObject:[self initialFirstResponder] 
-	      withName:@"initialFirstResponder"];
+  [super encodeWithModelArchiver: archiver];
 
-    // NSPanel specific
-    [archiver encodeBOOL:[self isFloatingPanel] withName:@"isFloatingPanel"];
-    [archiver encodeBOOL:[self becomesKeyOnlyIfNeeded]
-            withName:@"becomesKeyOnlyIfNeeded"];
-    [archiver encodeBOOL:[self worksWhenModal] withName:@"worksWhenModal"];
-
-    // NSSavePanel specific
-    [archiver encodeString:[self prompt] withName:@"prompt"];
-    [archiver encodeObject:[self accessoryView] withName:@"accessoryView"];
-    [archiver encodeString:[self requiredFileType]
-            withName:@"requiredFileType"];
-    [archiver encodeBOOL:[self treatsFilePackagesAsDirectories]
-            withName:@"treatsFilePackagesAsDirectories"];
-    [archiver encodeString:[self directory]
-            withName:@"directory"];
+  [archiver encodeString:[self prompt] withName:@"prompt"];
+  [archiver encodeObject:[self accessoryView] withName:@"accessoryView"];
+  [archiver encodeString:[self requiredFileType]
+            withName: @"requiredFileType"];
+  [archiver encodeBOOL:[self treatsFilePackagesAsDirectories]
+            withName: @"treatsFilePackagesAsDirectories"];
+  [archiver encodeString:[self directory]
+            withName: @"directory"];
 }
 
 + (id)createObjectForModelUnarchiver:(GMUnarchiver*)unarchiver
@@ -1310,43 +1292,7 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 -(id)initWithModelUnarchiver :(GMUnarchiver *)unarchiver
 {
-    NSString* frameAutosaveName;
-    
-    //NSWindow specifics
-    [self setContentView:[unarchiver decodeObjectWithName:@"contentView"]];
-    [self setMaxSize:[unarchiver decodeSizeWithName:@"maxSize"]];
-    [self setMinSize:[unarchiver decodeSizeWithName:@"minSize"]];
-    
-    frameAutosaveName = [unarchiver decodeStringWithName:@"frameAutosaveName"];
-    if (frameAutosaveName)
-	[self setFrameAutosaveName:frameAutosaveName];
-    
-#ifdef GNU_GUI_LIBRARY
-    window_level = [unarchiver decodeIntWithName:@"level"];
-#endif
-    
-    [self setInitialFirstResponder: 
-	    [unarchiver decodeObjectWithName:@"initialFirstResponder"]];
-    [self setAutodisplay:[unarchiver decodeBOOLWithName:@"isAutodisplay"]];
-    [self setTitle:[unarchiver decodeStringWithName:@"title"]];
-    [self setRepresentedFilename:
-        [unarchiver decodeStringWithName:@"representedFilename"]];
-    [self setReleasedWhenClosed:
-        [unarchiver decodeBOOLWithName:@"isReleasedWhenClosed"]];
-    [self setHidesOnDeactivate:
-        [unarchiver decodeBOOLWithName:@"hidesOnDeactivate"]];
-    [self setBackgroundColor:
-        [unarchiver decodeObjectWithName:@"backgroundColor"]];
-    if ([unarchiver decodeBOOLWithName:@"isVisible"])
-	[self orderFront:nil];
-
-    //NSPanel specifics
-    [self setFloatingPanel:
-        [unarchiver decodeBOOLWithName:@"isFloatingPanel"]];
-    [self setBecomesKeyOnlyIfNeeded:
-        [unarchiver decodeBOOLWithName:@"becomesKeyOnlyIfNeeded"]];
-    [self setWorksWhenModal:
-        [unarchiver decodeBOOLWithName:@"setWorksWhenModal"]];
+    [super initWithModelUnarchiver: unarchiver];
 
     //NSSavePanel specifics
     [self setPrompt:[unarchiver decodeStringWithName:@"prompt"]];
@@ -1491,6 +1437,7 @@ void __dummy_GMAppKit_functionForLinking() {}
   self = [super initWithModelUnarchiver:unarchiver];
 
   [self setColor:[unarchiver decodeObjectWithName:@"color"]];
+  [self setBordered:[unarchiver decodeBOOLWithName:@"isBordered"]];
 
   return self;
 }
@@ -1500,6 +1447,7 @@ void __dummy_GMAppKit_functionForLinking() {}
   [super encodeWithModelArchiver:archiver];
 
   [archiver encodeObject:[self color] withName:@"color"];
+  [archiver encodeBOOL:[self isBordered] withName:@"isBordered"];
 }
 
 @end /* NSColorWell (GMArchiverMethods) */
@@ -1532,6 +1480,38 @@ void __dummy_GMAppKit_functionForLinking() {}
 
 @end
 
+@implementation NSSliderCell (GMArchiverMethods)
+
+- (void)encodeWithModelArchiver:(GMArchiver*)archiver
+{
+  [super encodeWithModelArchiver:archiver];
+
+  [archiver encodeFloat:[self knobThickness] withName:@"knobThickness"];
+  [archiver encodeObject:[self image] withName:@"image"];
+  [archiver encodeDouble:[self maxValue] withName:@"maxValue"];
+  [archiver encodeDouble:[self minValue] withName:@"minValue"];
+
+  // title, color, and font info is encoded by the title cell
+  [archiver encodeObject:[self titleCell] withName:@"titleCell"];
+}
+
+- (id)initWithModelUnarchiver:(GMUnarchiver*)unarchiver
+{
+  [super initWithModelUnarchiver:unarchiver];
+
+  [self setKnobThickness:[unarchiver decodeFloatWithName:@"knobThickness"]];
+  [self setImage:[unarchiver decodeObjectWithName:@"image"]];
+  [self setMaxValue:[unarchiver decodeDoubleWithName:@"maxValue"]];
+  [self setMinValue:[unarchiver decodeDoubleWithName:@"minValue"]];
+
+  // title, color, and font info is encoded by the title cell
+  [self setTitleCell:[unarchiver decodeObjectWithName:@"titleCell"]];
+
+  return self;
+}
+
+@end /* NSSliderCell (GMArchiverMethods) */
+
 @implementation NSTextFieldCell (GMArchiverMethods)
 
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
@@ -1577,3 +1557,91 @@ void __dummy_GMAppKit_functionForLinking() {}
 }
 
 @end /* NSFormCell (GMArchiverMethods) */
+
+@implementation NSText (GMArchiverMethods)
+
+- (void)encodeWithModelArchiver:(GMArchiver*)archiver
+{
+  [super encodeWithModelArchiver:archiver];
+
+  [archiver encodeString: [self string] withName: @"string"];
+  [archiver encodeConditionalObject: [self backgroundColor] 
+	    withName: @"backgroundColor"];
+  [archiver encodeBOOL:[self drawsBackground] withName:@"drawsBackground"];
+  [archiver encodeObject:[self textColor] withName:@"textColor"];
+  [archiver encodeBOOL:[self isEditable] withName:@"isEditable"];
+  [archiver encodeBOOL:[self isSelectable] withName:@"isSelectable"];
+  [archiver encodeBOOL:[self isFieldEditor] withName:@"isFieldEditor"];
+  [archiver encodeBOOL:[self isRichText] withName:@"isRichText"];
+  [archiver encodeBOOL:[self importsGraphics] withName:@"importsGraphics"];
+  [archiver encodeBOOL:[self usesFontPanel] withName:@"usesFontPanel"];
+  [archiver encodeObject:[self font] withName:@"font"];
+  [archiver encodeInt:[self alignment] withName:@"alignment"];
+  [archiver encodeSize:[self maxSize] withName:@"maxSize"];
+  [archiver encodeSize:[self minSize] withName:@"minSize"];
+  [archiver encodeBOOL:[self isVerticallyResizable] withName:@"isVerticallyResizable"];
+  [archiver encodeBOOL:[self isHorizontallyResizable] withName:@"isHorizontallyResizable"];
+  [archiver encodeConditionalObject: [self delegate] withName: @"delegate"];
+}
+
+- (id)initWithModelUnarchiver:(GMUnarchiver*)unarchiver
+{
+  self = [super initWithModelUnarchiver: unarchiver];
+
+  [self setString: [unarchiver decodeStringWithName: @"string"]];
+  [self setBackgroundColor:
+	     [unarchiver decodeObjectWithName:@"backgroundColor"]];
+  [self setDrawsBackground:
+	     [unarchiver decodeBOOLWithName:@"drawsBackground"]];
+  [self setTextColor:[unarchiver decodeObjectWithName:@"textColor"]];
+  [self setEditable:[unarchiver decodeBOOLWithName:@"isEditable"]];
+  [self setSelectable:[unarchiver decodeBOOLWithName:@"isSelectable"]];
+  [self setFieldEditor:[unarchiver decodeBOOLWithName:@"isFieldEditor"]];
+  [self setRichText:[unarchiver decodeBOOLWithName:@"isRichText"]];
+  [self setImportsGraphics:[unarchiver decodeBOOLWithName:@"importsGraphics"]];
+  [self setUsesFontPanel:[unarchiver decodeBOOLWithName:@"usesFontPanel"]];
+  [self setFont:[unarchiver decodeObjectWithName:@"font"]];
+  [self setAlignment:[unarchiver decodeIntWithName:@"alignment"]];
+  [self setMaxSize:[unarchiver decodeSizeWithName:@"maxSize"]];
+  [self setMinSize:[unarchiver decodeSizeWithName:@"minSize"]];
+  [self setVerticallyResizable:
+	    [unarchiver decodeBOOLWithName:@"isVerticallyResizable"]];
+  [self setHorizontallyResizable:
+	    [unarchiver decodeBOOLWithName:@"isHorizontallyResizable"]];
+  [self setDelegate: [unarchiver decodeObjectWithName: @"delegate"]];
+
+  return self;
+}
+
+@end /* NSText (GMArchiverMethods) */
+
+@implementation NSTextView (GMArchiverMethods)
+
+- (void)encodeWithModelArchiver:(GMArchiver*)archiver
+{
+  [super encodeWithModelArchiver:archiver];
+
+  // Currently the text container is not encoded
+  [archiver encodeSize:[self textContainerInset] withName:@"textContainerInset"];
+  [archiver encodeBOOL:[self allowsUndo] withName:@"allowsUndo"];
+  [archiver encodeBOOL:[self usesRuler] withName:@"usesRuler"];
+  [archiver encodeBOOL:[self isRulerVisible] withName:@"isRulerVisible"];
+  [archiver encodeObject:[self insertionPointColor] withName:@"insertionPointColor"];
+}
+
+- (id)initWithModelUnarchiver:(GMUnarchiver*)unarchiver
+{
+  self = [super initWithModelUnarchiver: unarchiver];
+
+  // Currently the text container is not encoded
+  [self setTextContainerInset:[unarchiver decodeSizeWithName:@"textContainerInset"]];
+  [self setAllowsUndo:[unarchiver decodeBOOLWithName:@"allowsUndo"]];
+  [self setUsesRuler:[unarchiver decodeBOOLWithName:@"usesRuler"]];
+  [self setRulerVisible:[unarchiver decodeBOOLWithName:@"isRulerVisible"]];
+  [self setInsertionPointColor:
+	    [unarchiver decodeObjectWithName:@"insertionPointColor"]];
+
+  return self;
+}
+
+@end /* NSTextView (GMArchiverMethods) */
