@@ -44,6 +44,26 @@ void __dummy_IMLoading_functionForLinking()
   return [GMModel loadIMFile:path owner:owner bundle:self];
 }
 
++ (BOOL)loadGModelNamed:(NSString *)gmodelName owner:(id)owner
+{
+  NSBundle *bundle;
+  NSString *path;
+
+  /* Pull off the .gmodel extension (if any) for pathForResource:ofType: */
+  if ([[gmodelName pathExtension] isEqualToString:@"gmodel"])
+    gmodelName = [gmodelName stringByDeletingPathExtension];
+
+  /* Use owner's bundle (if any) just like +loadNibNamed:owner: does. */
+  bundle = [NSBundle bundleForClass: [owner class]];
+
+  if (bundle == nil)
+    bundle = [NSBundle mainBundle];
+
+  path = [bundle pathForResource:gmodelName ofType:@"gmodel"];
+
+  return [GMModel loadIMFile:path owner:owner bundle:bundle];
+}
+
 @end /* NSBundle(IMLoading) */
 
 
@@ -71,7 +91,7 @@ BOOL _fileOwnerDecoded = NO;
   id previousNibOwner = _nibOwner;
   GMModel* decoded;
 
-  if (![path hasSuffix:@".gmodel"])
+  if (![[path pathExtension] isEqualToString:@"gmodel"])
     path = [path stringByAppendingPathExtension:@"gmodel"];
 
   /* First check to see if path is an absolute path; if so try to load the
@@ -134,8 +154,19 @@ BOOL _fileOwnerDecoded = NO;
   for (i = 0, count = [objects count]; i < count; i++) {
     id object = [[objects objectAtIndex:i] nibInstantiate];
 
+    // If the object responds to the awakeFromModel
+    // method, send that message.  If it doesn't then
+    // check for the awakeFromNib selector and send that.
+    // This ensures compatibility w/ apps ported from OPENSTEP.
     if ([object respondsToSelector:@selector(awakeFromModel)])
-      [object awakeFromModel];
+      {
+	[object awakeFromModel];
+      }
+    else
+      if ([object respondsToSelector:@selector(awakeFromNib)])
+	{
+	  [object awakeFromNib];
+	}
   }
 }
 
