@@ -45,25 +45,28 @@
 /* Maximum number of planes */
 #define MAX_PLANES 5
 
-/* Backend protocol - methods that must be implemented by the backend to
-   complete the class */
-@protocol NXBitmapImageRepBackend
-- (BOOL) draw;
+/* Backend methods (optional) */
+@interface NSBitmapImageRep (Backend)
++ (NSArray *) _wrasterFileTypes;
+- _initFromWrasterFile: (NSString *)filename number: (int)imageNumber;
 @end
 
 @implementation NSBitmapImageRep 
 
 /* Given a TIFF image (from the libtiff library), load the image information
    into our data structure.  Reads the specified image. */
-- _initFromImage: (TIFF *)image number: (int)imageNumber
+- _initFromTIFFImage: (TIFF *)image number: (int)imageNumber
 {
   NSString* space;
   NSTiffInfo* info;
 
+  /* Seek to the correct image and get the dictionary information */
   info = NSTiffGetInfo(imageNumber, image);
   if (!info) 
     {
-      [NSException raise:NSTIFFException format: @"Read invalid TIFF info"];
+      RELEASE(self);
+      NSLog(@"Tiff read invalid TIFF info in directory %d", imageNumber);
+      return nil;
     }
 
   /* 8-bit RGB will be converted to 24-bit by the tiff routines, so account
@@ -87,21 +90,29 @@
 		pixelsHigh: info->height
 		bitsPerSample: info->bitsPerSample
 		samplesPerPixel: info->samplesPerPixel
-		hasAlpha: (info->samplesPerPixel > 3)
+		hasAlpha: (info->extraSamples > 0)
 		isPlanar: (info->planarConfig == PLANARCONFIG_SEPARATE)
 		colorSpaceName: space
 		bytesPerRow: 0
 		bitsPerPixel: 0];
   compression = info->compression;
+  comp_factor = 255 * (1 - ((float)info->quality)/100.0);
 
-  if (NSTiffRead(imageNumber, image, NULL, [self bitmapData]))
+  if (NSTiffRead(image, info, [self bitmapData]))
     {
-      [NSException raise:NSTIFFException format: @"Read invalid TIFF image"];
+      RELEASE(self);
+      NSLog(@"Tiff read invalid TIFF image data in directory %d", imageNumber);
+      return nil;
     }
 
   return self;
 }
 
+- _initFromWrasterFile: (NSString *)filename number: (int)imageNumber
+{
+  return nil;
+}
+
 + (id) imageRepWithData: (NSData *)tiffData
 {
   NSArray* array;
@@ -112,51 +123,86 @@
   return nil;
 }
 
-+ (NSArray *) imageRepsWithData: (NSData *)tiffData
++ (NSArray*) imageRepsWithData: (NSData *)tiffData
 {
-  int images;
-  TIFF*       image;
-  NSTiffInfo* info;
-  NSMutableArray* array;
+  int		 i, images;
+  TIFF		 *image;
+  NSMutableArray *array;
 
-  image = NSTiffOpenData((char *)[tiffData bytes], [tiffData length], 
-			 "r", NULL);
-  if (!image)
+  image = NSTiffOpenDataRead((char *)[tiffData bytes], [tiffData length]);
+  if (image == NULL)
     {
-      [NSException raise:NSTIFFException format: @"Read invalid TIFF data"];
+      NSLog(@"Tiff unable to open/parse TIFF data");
+      return nil;
     }
 
-  array = [NSMutableArray arrayWithCapacity:1];
-  images = 0;
-  while ((info = NSTiffGetInfo(images, image))) 
+  images = NSTiffGetImageCount(image);
+  NSDebugLLog(@"NSImage", @"Image contains %d directories", images);
+  array = [NSMutableArray arrayWithCapacity: images];
+  for (i = 0; i < images; i++)
     {
       NSBitmapImageRep* imageRep;
-
-      OBJC_FREE(info);
-      imageRep = [[[[self class] alloc]
-		   _initFromImage: image number: images] autorelease];
-      [array addObject: imageRep];
-      images++;
+      imageRep = [[[self class] alloc] _initFromTIFFImage: image number: i];
+      if (imageRep)
+	[array addObject: AUTORELEASE(imageRep)];
     }
   NSTiffClose(image);
 
   return array;
 }
 
+/* A special method used mostly when we have the wraster library in the
+   backend, which can read several more image formats */
++ (NSArray*) imageRepsWithFile: (NSString *)filename
+{
+  NSString *ext;
+  int	   images;
+  NSMutableArray *array;
+  NSBitmapImageRep* imageRep;
+
+  /* Don't use this for TIFF images, use the regular ...Data methods */
+  ext = [filename pathExtension];
+  if (!ext)
+    {
+      NSLog(@"Extension missing from filename - '%@'", filename);
+      return nil;
+    }
+  if ([[self imageUnfilteredFileTypes] indexOfObject: ext] != NSNotFound)
+    {
+      NSData* data = [NSData dataWithContentsOfFile: filename];
+      return [self imageRepsWithData: data];
+    }
+
+  array = [NSMutableArray arrayWithCapacity: 2];
+  images = 0;
+  do
+    {
+      imageRep = [[[self class] alloc] _initFromWrasterFile: filename 
+				                     number: images];
+      if (imageRep)
+	[array addObject: AUTORELEASE(imageRep)];
+      images++;
+    }
+  while (imageRep);
+
+  return array;
+}
+
 /* Loads only the default (first) image from the TIFF image contained in
    data. */
 - (id) initWithData: (NSData *)tiffData
 {
   TIFF 	*image;
 
-  image = NSTiffOpenData((char *)[tiffData bytes], [tiffData length], 
-			 "r", NULL);
-  if (!image)
+  image = NSTiffOpenDataRead((char *)[tiffData bytes], [tiffData length]);
+  if (image == 0)
     {
-      [NSException raise:NSTIFFException format: @"Read invalid TIFF data"];
+      RELEASE(self);
+      NSLog(@"Tiff read invalid TIFF info from data");
+      return nil;
     }
 
-  [self _initFromImage:image number: -1];
+  [self _initFromTIFFImage:image number: -1];
   NSTiffClose(image);
   return self;
 }
@@ -195,7 +241,7 @@
   numColors     = spp;
   hasAlpha   = alpha;  
   _isPlanar   = isPlanar;
-  _colorSpace = [colorSpaceName retain];
+  _colorSpace = RETAIN(colorSpaceName);
   if (!pixelBits)
     pixelBits = bps * ((_isPlanar) ? 1 : spp);
   bitsPerPixel            = pixelBits;
@@ -212,13 +258,39 @@
       for (i = 0; i < ((_isPlanar) ? numColors : 1); i++)
  	imagePlanes[i] = planes[i];
     }
+  if (alpha)
+    {
+      unsigned char	*bData = (unsigned char*)[self bitmapData];
+      BOOL		allOpaque = YES;
+      unsigned		offset = numColors - 1;
+      unsigned		limit = size.height * size.width;
+      unsigned		i;
+
+      for (i = 0; i < limit; i++)
+	{
+	  unsigned	a;
+
+	  bData += offset;
+	  a = *bData++;
+	  if (a != 255)
+	    {
+	      allOpaque = NO;
+	      break;
+	    }
+	}
+      [self setOpaque: allOpaque];
+    }
+  else
+    {
+      [self setOpaque: YES];
+    }
   return self;
 }
 
 - (void) dealloc
 {
   OBJC_FREE(imagePlanes);
-  [imageData release];
+  RELEASE(imageData);
   [super dealloc];
 }
 
@@ -232,16 +304,18 @@
   copy->numColors = numColors;
   copy->bitsPerPixel = bitsPerPixel;
   copy->compression = compression;
+  copy->comp_factor = comp_factor;
   copy->_isPlanar = _isPlanar;
   copy->imagePlanes = 0;
   copy->imageData = [imageData copy];
 
   return copy;
 }
+
 + (BOOL) canInitWithData: (NSData *)data
 {
-  TIFF *image = NULL;
-  image = NSTiffOpenData((char *)[data bytes], [data length], "r", NULL);
+  TIFF	*image = NULL;
+  image = NSTiffOpenDataRead((char *)[data bytes], [data length]);
   NSTiffClose(image);
 
   return (image) ? YES : NO;
@@ -249,13 +323,26 @@
 
 + (BOOL) canInitWithPasteboard: (NSPasteboard *)pasteboard
 {
-  [self notImplemented: _cmd];
-  return NO;
+  return [[pasteboard types] containsObject: NSTIFFPboardType]; 
+}
+
++ (NSArray *) _wrasterFileTypes
+{
+  return nil;
 }
 
 + (NSArray *) imageFileTypes
 {
-  return [self imageUnfilteredFileTypes];
+  NSArray *wtypes = [self _wrasterFileTypes];
+  if (wtypes)
+    {
+      wtypes = AUTORELEASE([wtypes mutableCopy]);
+      [(NSMutableArray *)wtypes addObjectsFromArray: 
+			   [self imageUnfilteredFileTypes]];
+    }
+  else
+    wtypes = [self imageUnfilteredFileTypes];
+  return wtypes;
 }
 
 + (NSArray *) imagePasteboardTypes
@@ -327,10 +414,11 @@
       
       length = (long)numColors * bytesPerRow * _pixelsHigh 
  	* sizeof(unsigned char);
-      imageData = [[NSMutableData dataWithLength: length] retain];
+      imageData = RETAIN([NSMutableData dataWithLength: length]);
       if (!imagePlanes)
  	OBJC_MALLOC(imagePlanes, unsigned char*, MAX_PLANES);
       bits = [imageData mutableBytes];
+      imagePlanes[0] = bits;
       if (_isPlanar) 
 	{
 	  for (i=1; i < numColors; i++) 
@@ -340,7 +428,6 @@
 	}
       else
 	{
-	  imagePlanes[0] = bits;
 	  for (i= 1; i < MAX_PLANES; i++) 
 	    imagePlanes[i] = NULL;
 	}
@@ -353,70 +440,168 @@
 
 - (BOOL) draw
 {
-  return NO;
+  NSRect irect = NSMakeRect(0, 0, size.width, size.height);
+  NSDrawBitmap(irect,
+	       _pixelsWide,
+	       _pixelsHigh,
+	       bitsPerSample,
+	       numColors,
+	       bitsPerPixel,
+	       bytesPerRow,
+	       _isPlanar,
+	       hasAlpha,
+	       _colorSpace,
+	       imagePlanes);
+  return YES;
 }
 
 //
 // Producing a TIFF Representation of the Image 
 //
-+ (NSData *) TIFFRepresentationOfImageRepsInArray: (NSArray *)anArray
++ (NSData*) TIFFRepresentationOfImageRepsInArray: (NSArray *)anArray
 {
   [self notImplemented: _cmd];
   return nil;
 }
 
-+ (NSData *) TIFFRepresentationOfImageRepsInArray: (NSArray *)anArray
-		usingCompression: (NSTIFFCompression)compressionType
-		factor: (float)factor
++ (NSData*) TIFFRepresentationOfImageRepsInArray: (NSArray *)anArray
+				usingCompression: (NSTIFFCompression)type
+					  factor: (float)factor
 {
   [self notImplemented: _cmd];
   return nil;
 }
 
-- (NSData *) TIFFRepresentation
+- (NSData*) TIFFRepresentation
 {
-  [self notImplemented: _cmd];
-  return nil;
+  NSTiffInfo	info;
+  TIFF		*image;
+  char		*bytes = 0;
+  long		length = 0;
+
+  info.imageNumber = 0;
+  info.subfileType = 255;
+  info.width = _pixelsWide;
+  info.height = _pixelsHigh;
+  info.bitsPerSample = bitsPerSample;
+  info.samplesPerPixel = numColors;
+
+  if (_isPlanar)
+    info.planarConfig = PLANARCONFIG_SEPARATE;
+  else
+    info.planarConfig = PLANARCONFIG_CONTIG;
+
+  if (_colorSpace == NSDeviceRGBColorSpace)
+    info.photoInterp = PHOTOMETRIC_RGB;
+  else if (_colorSpace == NSDeviceWhiteColorSpace)
+    info.photoInterp = PHOTOMETRIC_MINISBLACK;
+  else if (_colorSpace == NSDeviceBlackColorSpace)
+    info.photoInterp = PHOTOMETRIC_MINISWHITE;
+  else
+    info.photoInterp = PHOTOMETRIC_RGB;
+
+  info.compression = compression;
+  info.quality = (1 - ((float)comp_factor)/255.0) * 100;
+  info.numImages = 1;
+  info.error = 0;
+
+  image = NSTiffOpenDataWrite(&bytes, &length);
+  if (image == 0)
+    {
+      [NSException raise: NSTIFFException format: @"Write TIFF open failed"];
+    }
+  if (NSTiffWrite(image, &info, [self bitmapData]) != 0)
+    {
+      [NSException raise: NSTIFFException format: @"Write TIFF data failed"];
+    }
+  NSTiffClose(image);
+  return [NSData dataWithBytesNoCopy: bytes length: length];
 }
 
-- (NSData *) TIFFRepresentationUsingCompression: (NSTIFFCompression)compressionType
-		factor: (float)factor
+- (NSData*) TIFFRepresentationUsingCompression: (NSTIFFCompression)type
+					factor: (float)factor
 {
-  [self notImplemented: _cmd];
-  return nil;
+  NSData		*data;
+  NSTIFFCompression	oldType = compression;
+  float			oldFact = comp_factor;
+
+  [self setCompression: type factor: factor];
+  data = [self TIFFRepresentation];
+  [self setCompression: oldType factor: oldFact];
+  return data;
 }
 
 //
 // Setting and Checking Compression Types 
 //
 + (void) getTIFFCompressionTypes: (const NSTIFFCompression **)list
-		count: (int *)numTypes
+			   count: (int *)numTypes
 {
-  [self notImplemented: _cmd];
+  static NSTIFFCompression	types[] = {
+    NSTIFFCompressionNone,
+    NSTIFFCompressionCCITTFAX3,
+    NSTIFFCompressionCCITTFAX4,
+    NSTIFFCompressionLZW,
+    NSTIFFCompressionJPEG,
+    NSTIFFCompressionPackBits
+  };
+
+  *list = types;
+  *numTypes = sizeof(types)/sizeof(*types);
 }
 
-+ (NSString *) localizedNameForTIFFCompressionType: (NSTIFFCompression)compression
++ (NSString*) localizedNameForTIFFCompressionType: (NSTIFFCompression)type
 {
-  [self notImplemented: _cmd];
-  return nil;
+  switch (type)
+    {
+      case NSTIFFCompressionNone: return @"NSTIFFCompressionNone";
+      case NSTIFFCompressionCCITTFAX3: return @"NSTIFFCompressionCCITTFAX3";
+      case NSTIFFCompressionCCITTFAX4: return @"NSTIFFCompressionCCITTFAX4";
+      case NSTIFFCompressionLZW: return @"NSTIFFCompressionLZW";
+      case NSTIFFCompressionJPEG: return @"NSTIFFCompressionJPEG";
+      case NSTIFFCompressionNEXT: return @"NSTIFFCompressionNEXT";
+      case NSTIFFCompressionPackBits: return @"NSTIFFCompressionPackBits";
+      case NSTIFFCompressionOldJPEG: return @"NSTIFFCompressionOldJPEG";
+      default: return nil;
+    }
 }
 
-- (BOOL) canBeCompressedUsing: (NSTIFFCompression)compression
+- (BOOL) canBeCompressedUsing: (NSTIFFCompression)type
 {
-  [self notImplemented: _cmd];
-  return NO;
+  switch (type)
+    {
+      case NSTIFFCompressionCCITTFAX3:
+      case NSTIFFCompressionCCITTFAX4:
+	if (numColors == 1 && bitsPerSample == 1)
+	  return YES;
+	else
+	  return NO;
+
+      case NSTIFFCompressionNone:
+      case NSTIFFCompressionLZW: 
+      case NSTIFFCompressionJPEG:
+      case NSTIFFCompressionPackBits:
+	return YES;
+
+      case NSTIFFCompressionNEXT:
+      case NSTIFFCompressionOldJPEG:
+      default:
+	return NO;
+    }
 }
 
-- (void) getCompression: (NSTIFFCompression *)compression
-		factor: (float *)factor
+- (void) getCompression: (NSTIFFCompression*)type
+		 factor: (float*)factor
 {
-  [self notImplemented: _cmd];
+  *type = compression;
+  *factor = comp_factor;
 }
 
-- (void) setCompression: (NSTIFFCompression)compression
-		factor: (float)factor
+- (void) setCompression: (NSTIFFCompression)type
+		 factor: (float)factor
 {
-  [self notImplemented: _cmd];
+  compression = type;
+  comp_factor = factor;
 }
 
 //
@@ -424,13 +609,19 @@
 //
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  [self notImplemented: _cmd];
+  NSData	*data = [self TIFFRepresentation];
+
+  [super encodeWithCoder: aCoder];
+  [data encodeWithCoder: aCoder];
 }
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [self notImplemented: _cmd];
-  return nil;
+  NSData	*data;
+
+  self = [super initWithCoder: aDecoder];
+  data = [aDecoder decodeObject];
+  return [self initWithData: data];
 }
 
 @end
