@@ -48,22 +48,20 @@
 #include <AppKit/NSGraphics.h>
 #include <AppKit/PSOperators.h>
 
-
-
 @implementation NSButtonCell
 
-//
-// Class methods
-//
+/*
+ * Class methods
+ */
 + (void) initialize
 {
   if (self == [NSButtonCell class])
     [self setVersion: 1];
 }
 
-//
-// Instance methods
-//
+/*
+ * Instance methods
+ */
 - (id) _init
 {
   // Implicitly performed by allocation:
@@ -89,7 +87,7 @@
   return self;
 }
 
-- (id) initImageCell: (NSImage *)anImage
+- (id) initImageCell: (NSImage*)anImage
 {
   [super initImageCell: anImage];
 
@@ -98,7 +96,7 @@
   return [self _init];
 }
 
-- (id) initTextCell: (NSString *)aString
+- (id) initTextCell: (NSString*)aString
 {
   [super initTextCell: aString];
 
@@ -115,30 +113,30 @@
   [super dealloc];
 }
 
-//
-// Setting the Titles
-//
-- (NSString *) title								
+/*
+ * Setting the Titles
+ */
+- (NSString*) title								
 {
   return [self stringValue];
 }
 
-- (NSString *) alternateTitle					
+- (NSString*) alternateTitle					
 {
   return _altContents;
 }
 
-- (void) setFont: (NSFont *)fontObject		
+- (void) setFont: (NSFont*)fontObject		
 {
   [super setFont: fontObject];
 }
 
-- (void) setTitle: (NSString *)aString
+- (void) setTitle: (NSString*)aString
 {
   [self setStringValue: aString];
 }
 
-- (void) setAlternateTitle: (NSString *)aString
+- (void) setAlternateTitle: (NSString*)aString
 {
   NSString* string = [aString copy];
 
@@ -146,13 +144,13 @@
   [string release]; 
   if (_control_view)
     if ([_control_view isKindOfClass: [NSControl class]])
-      [(NSControl *)_control_view updateCell: self];
+      [(NSControl*)_control_view updateCell: self];
 }
 
-//
-// Setting the Images
-//
-- (NSImage *) alternateImage						
+/*
+ * Setting the Images
+ */
+- (NSImage*) alternateImage						
 {
   return _altImage;
 }
@@ -162,7 +160,7 @@
   return _cell.image_position;
 }
 
-- (void) setAlternateImage: (NSImage *)anImage
+- (void) setAlternateImage: (NSImage*)anImage
 {
   ASSIGN(_altImage, anImage);
 }
@@ -172,10 +170,10 @@
   _cell.image_position = aPosition;
 }
 
-//
-// Setting the Repeat Interval
-//
-- (void) getPeriodicDelay: (float *)delay interval: (float *)interval
+/*
+ * Setting the Repeat Interval
+ */
+- (void) getPeriodicDelay: (float*)delay interval: (float*)interval
 {
   *delay = _delayInterval;
   *interval = _repeatInterval;
@@ -185,12 +183,11 @@
 {
   _delayInterval = delay;
   _repeatInterval = interval;
-  [self setContinuous: YES];
 }
 
-//
-// Setting the Key Equivalent
-//
+/*
+ * Setting the Key Equivalent
+ */
 - (NSString*) keyEquivalent
 {
   return _keyEquivalent;
@@ -230,9 +227,9 @@
   ASSIGN(_keyEquivalentFont, [NSFont fontWithName: fontName size: fontSize]);
 }
 
-//
-// Modifying Graphic Attributes
-//
+/*
+ * Modifying Graphic Attributes
+ */
 - (BOOL) isTransparent					
 {
   return _buttoncell_is_transparent;
@@ -245,16 +242,18 @@
 
 - (BOOL) isOpaque
 {
-  // MacOS-X says we should return !transparent && [self isBordered], 
-  // but that's wrong in our case, since if there is no border, 
-  // we draw the interior of the cell to fill completely the bounds.  
-  // They are likely to draw differently.
+  /*
+   * MacOS-X says we should return !transparent && [self isBordered], 
+   * but that's wrong in our case, since if there is no border, 
+   * we draw the interior of the cell to fill completely the bounds.  
+   * They are likely to draw differently.
+   */
   return !_buttoncell_is_transparent;
 }
 
-//
-// Modifying Graphic Attributes
-//
+/*
+ * Modifying Graphic Attributes
+ */
 - (int) highlightsBy						
 {
   return _highlightsByMask;
@@ -354,10 +353,10 @@
   return _cell.state;
 }
 
-//
-// Displaying
-//
-- (NSColor *) textColor
+/*
+ * Displaying
+ */
+- (NSColor*) textColor
 {
   if (_cell.is_disabled == YES)
     return [NSColor disabledControlTextColor];
@@ -492,8 +491,11 @@
 
   if (titleToDisplay && (ipos == NSImageAbove || ipos == NSImageBelow))
     {
-      titleSize = NSMakeSize([_cell_font widthOfString: titleToDisplay], 
-			     [_cell_font boundingRectForFont].size.height);
+      NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:
+					   _cell_font, 
+					 NSFontAttributeName,
+					 nil];
+      titleSize = [titleToDisplay sizeWithAttributes: dict];
     }
 
   if (flippedView == YES)
@@ -525,11 +527,11 @@
 	imageRect.size.width = imageSize.width;
 	imageRect.size.height = cellFrame.size.height;
 	if (_cell.is_bordered || _cell.is_bezeled) 
-	    {
-	      imageRect.origin.x += 3;
-	      imageRect.size.height -= 2;
-	      imageRect.origin.y += 1;
-	    }
+	  {
+	    imageRect.origin.x += 3;
+	    imageRect.size.height -= 2;
+	    imageRect.origin.y += 1;
+	  }
 	titleRect = imageRect;
 	titleRect.origin.x += imageSize.width + xDist;
 	titleRect.size.width = cellFrame.size.width - imageSize.width - xDist;
@@ -545,11 +547,11 @@
 	imageRect.size.width = imageSize.width;
 	imageRect.size.height = cellFrame.size.height;
 	if (_cell.is_bordered || _cell.is_bezeled) 
-	    {
-	      imageRect.origin.x -= 3;
-	      imageRect.size.height -= 2;
-	      imageRect.origin.y += 1;
-	    }
+	  {
+	    imageRect.origin.x -= 3;
+	    imageRect.size.height -= 2;
+	    imageRect.origin.y += 1;
+	  }
 	titleRect.origin = cellFrame.origin;
 	titleRect.size.width = cellFrame.size.width - imageSize.width - xDist;
 	titleRect.size.height = cellFrame.size.height;
@@ -561,8 +563,11 @@
 	break;
 
       case NSImageAbove: 
-	/* In this case, imageRect is all the space we can allocate above the text. 
-	   The drawing code below will then center the image in imageRect. */
+	/*
+         * In this case, imageRect is all the space we can allocate
+	 * above the text. 
+	 * The drawing code below will then center the image in imageRect.
+	 */
 	titleRect.origin.x = cellFrame.origin.x;
 	titleRect.origin.y = cellFrame.origin.y;
 	titleRect.size.width = cellFrame.size.width;
@@ -588,8 +593,11 @@
 	break;
 
       case NSImageBelow: 
-	/* In this case, imageRect is all the space we can allocate below the text. 
-	   The drawing code below will then center the image in imageRect. */
+	/*
+	 * In this case, imageRect is all the space we can allocate
+	 * below the text. 
+	 * The drawing code below will then center the image in imageRect.
+	 */
 	titleRect.origin.x = cellFrame.origin.x;
 	titleRect.origin.y = cellFrame.origin.y + cellFrame.size.height;
 	titleRect.origin.y -= titleSize.height;
@@ -656,9 +664,9 @@
   NSSize	imageSize;
   NSSize	titleSize;
   
-  // 
-  // The following code must be kept in sync with -drawInteriorWithFrame
-  //
+  /* 
+   * The following code must be kept in sync with -drawInteriorWithFrame
+   */
   
   if (_cell.is_highlighted)
     mask = _highlightsByMask;
@@ -687,52 +695,60 @@
   else 
     imageSize = NSZeroSize;
   
-  if (titleToDisplay)
-    titleSize = NSMakeSize ([_cell_font widthOfString: titleToDisplay], 
-			    [_cell_font boundingRectForFont].size.height); 
-  else 
-    titleSize = NSZeroSize;
+  if (titleToDisplay != nil)
+    {
+      NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:
+					   _cell_font, 
+					 NSFontAttributeName,
+					 nil];
+      titleSize = [titleToDisplay sizeWithAttributes: dict];
+
+    }
+  else
+    {
+      titleSize = NSZeroSize;
+    }
   
   switch (_cell.image_position)
     {
-    case NSNoImage: 
-      s = titleSize;
-      break;
-      
-    case NSImageOnly: 
-      s = imageSize;
-      break;
-      
-    case NSImageLeft: 
-    case NSImageRight: 
-      s.width = imageSize.width + titleSize.width + xDist;
-      if (imageSize.height > titleSize.height)
-	s.height = imageSize.height;
-      else 
-	s.height = titleSize.height;
-      break;
-      
-    case NSImageBelow: 
-    case NSImageAbove: 
-      if (imageSize.width > titleSize.width)
-	s.width = imageSize.width;
-      else 
-	s.width = titleSize.width;
-      s.height = imageSize.height + titleSize.height; // + yDist ??
-      break;
-      
-    case NSImageOverlaps: 
-      if (imageSize.width > titleSize.width)
-	s.width = imageSize.width;
-      else
-	s.width = titleSize.width;
-      
-      if (imageSize.height > titleSize.height)
-	s.height = imageSize.height;
-      else
-	s.height = titleSize.height;
+      case NSNoImage: 
+	s = titleSize;
+	break;
+	
+      case NSImageOnly: 
+	s = imageSize;
+	break;
+	
+      case NSImageLeft: 
+      case NSImageRight: 
+	s.width = imageSize.width + titleSize.width + xDist;
+	if (imageSize.height > titleSize.height)
+	  s.height = imageSize.height;
+	else 
+	  s.height = titleSize.height;
+	break;
+	
+      case NSImageBelow: 
+      case NSImageAbove: 
+	if (imageSize.width > titleSize.width)
+	  s.width = imageSize.width;
+	else 
+	  s.width = titleSize.width;
+	s.height = imageSize.height + titleSize.height; // + yDist ??
+	break;
+	
+      case NSImageOverlaps: 
+	if (imageSize.width > titleSize.width)
+	  s.width = imageSize.width;
+	else
+	  s.width = titleSize.width;
+	
+	if (imageSize.height > titleSize.height)
+	  s.height = imageSize.height;
+	else
+	  s.height = titleSize.height;
 
-      break;
+	break;
     }
   
   // Get border size
@@ -743,7 +759,7 @@
     borderSize = NSZeroSize;
 
   if ((_cell.is_bordered || _cell.is_bezeled) 
-      && (_cell.image_position != NSImageOnly))
+    && (_cell.image_position != NSImageOnly))
     {
       borderSize.height += 1;
       borderSize.width  += 3;
@@ -760,9 +776,11 @@
 {
   if (_cell.is_bordered)
     {
-      // Special case:  Buttons have only three different paths for border.
-      // One white path at the top left corner, one black path at the
-      // bottom right and another in dark gray at the inner bottom right.
+      /*
+       * Special case:  Buttons have only three different paths for border.
+       * One white path at the top left corner, one black path at the
+       * bottom right and another in dark gray at the inner bottom right.
+       */
       float yDelta = [_control_view isFlipped] ? 1. : 2.;
       return NSMakeRect (theRect.origin.x + 1.,
 			 theRect.origin.y + yDelta,
@@ -773,9 +791,9 @@
     return theRect;
 }
 
-//
-// NSCopying protocol
-//
+/*
+ * NSCopying protocol
+ */
 - (id) copyWithZone: (NSZone*)zone
 {
   NSButtonCell	*c = [super copyWithZone: zone];
@@ -794,9 +812,9 @@
   return c;
 }
 
-//
-// NSCoding protocol
-//
+/*
+ * NSCoding protocol
+ */
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
   BOOL tmp;
