@@ -77,15 +77,15 @@ static Class cellClass;
 - (id) initWithFrame: (NSRect)frameRect
 {
   [super initWithFrame: frameRect];
-  [self setCell: AUTORELEASE([cellClass new])];
-  tag = 0;
+  [self setCell: AUTORELEASE([[[self class] cellClass] new])];
+  _tag = 0;
 
   return self;
 }
 
 - (void) dealloc
 {
-  RELEASE(cell);
+  RELEASE(_cell);
   [super dealloc];
 }
 
@@ -95,7 +95,7 @@ static Class cellClass;
 - (id) copyWithZone: (NSZone*)zone
 {
   id		c = NSCopyObject(self, 0, zone);
-  NSCell	*o = [cell copy];
+  NSCell	*o = [_cell copy];
 
   [c setCell: o];
   RELEASE(o);
@@ -107,7 +107,7 @@ static Class cellClass;
  */
 - (id) cell
 {
-  return cell;
+  return _cell;
 }
 
 - (void) setCell: (NSCell *)aCell
@@ -116,10 +116,7 @@ static Class cellClass;
     [NSException raise: NSInvalidArgumentException
 		format: @"attempt to set non-cell object for control cell"];
 
-  [cell setControlView: nil];
-  [aCell setControlView: self];
-
-  ASSIGN(cell, aCell);
+  ASSIGN(_cell, aCell);
 }
 
 /*
@@ -140,10 +137,7 @@ static Class cellClass;
  */
 - (id) selectedCell
 {
-  if ([cell state])
-    return cell;
-  else
-    return nil;
+  return _cell;
 }
 
 - (int) selectedTag
@@ -171,18 +165,24 @@ static Class cellClass;
 
 - (void) setDoubleValue: (double)aDouble
 {
+  [self abortEditing];
+
   [[self selectedCell] setDoubleValue: aDouble];
   [self setNeedsDisplay: YES];
 }
 
 - (void) setFloatValue: (float)aFloat
 {
+  [self abortEditing];
+
   [[self selectedCell] setFloatValue: aFloat];
   [self setNeedsDisplay: YES];
 }
 
 - (void) setIntValue: (int)anInt
 {
+  [self abortEditing];
+
   [[self selectedCell] setIntValue: anInt];
   [self setNeedsDisplay: YES];
 }
@@ -194,6 +194,8 @@ static Class cellClass;
 
 - (void) setStringValue: (NSString *)aString
 {
+  [self abortEditing];
+
   [[self selectedCell] setStringValue: aString];
   [self setNeedsDisplay: YES];
 }
@@ -235,33 +237,33 @@ static Class cellClass;
  */
 - (NSTextAlignment) alignment
 {
-  if (cell)
-    return [cell alignment];
+  if (_cell)
+    return [_cell alignment];
   else
     return NSLeftTextAlignment;
 }
 
 - (NSFont *) font
 {
-  if (cell)
-    return [cell font];
+  if (_cell)
+    return [_cell font];
   else
     return nil;
 }
 
 - (void) setAlignment: (NSTextAlignment)mode
 {
-  if (cell)
+  if (_cell)
     {
-      [cell setAlignment: mode];
+      [_cell setAlignment: mode];
       [self setNeedsDisplay: YES];
     }
 }
 
 - (void) setFont: (NSFont *)fontObject
 {
-  if (cell)
-    [cell setFont: fontObject];
+  if (_cell)
+    [_cell setFont: fontObject];
 }
 
 - (void) setFloatingPointFormat: (BOOL)autoRange
@@ -285,47 +287,54 @@ static Class cellClass;
 
 - (void) validateEditing
 {
-}					// FIX ME
+}			
 
 /*
  * Resizing the Control
  */
 - (void) calcSize
 {
-}					// FIX ME
+}
 
 - (void) sizeToFit
 {
+  [self setFrameSize: [_cell cellSize]];
 }
 
 /*
  * Displaying the Control and Cell
  */
+- (BOOL) isOpaque
+{
+  return [_cell isOpaque];
+}
+
 - (void) drawRect: (NSRect)aRect
 {
-  [self drawCell: cell];
+  [self drawCell: _cell];
 }
 
 - (void) drawCell: (NSCell *)aCell
 {
-  if (cell == aCell)
+  if (_cell == aCell)
     {
-      [cell drawWithFrame: bounds inView: self];
+      [_cell drawWithFrame: _bounds inView: self];
     }
 }
 
 - (void) drawCellInside: (NSCell *)aCell
 {
-  if (cell == aCell)
+  if (_cell == aCell)
     {
-      [cell drawInteriorWithFrame: bounds inView: self];
+      [_cell drawInteriorWithFrame: _bounds 
+	    inView: self];
     }
 }
 
 - (void) selectCell: (NSCell *)aCell
 {
-  if (cell == aCell)
-    [cell setState: 1];
+  if (_cell == aCell)
+    [_cell setState: 1];
 }
 
 - (void) updateCell: (NSCell *)aCell
@@ -337,18 +346,22 @@ static Class cellClass;
 {
   [self setNeedsDisplay: YES];
 }
+- (void) performClick: (id)sender
+{
+  [_cell performClick: sender];
+}
 
 /*
  * Target and Action
  */
 - (SEL) action
 {
-  return [cell action];
+  return [_cell action];
 }
 
 - (BOOL) isContinuous
 {
-  return [cell isContinuous];
+  return [_cell isContinuous];
 }
 
 - (BOOL) sendAction: (SEL)theAction to: (id)theTarget
@@ -363,27 +376,27 @@ static Class cellClass;
 
 - (int) sendActionOn: (int)mask
 {
-  return [cell sendActionOn: mask];
+  return [_cell sendActionOn: mask];
 }
 
 - (void) setAction: (SEL)aSelector
 {
-  [cell setAction: aSelector];
+  [_cell setAction: aSelector];
 }
 
 - (void) setContinuous: (BOOL)flag
 {
-  [cell setContinuous: flag];
+  [_cell setContinuous: flag];
 }
 
 - (void) setTarget: (id)anObject
 {
-  [cell setTarget: anObject];
+  [_cell setTarget: anObject];
 }
 
 - (id) target
 {
-  return [cell target];
+  return [_cell target];
 }
 
 /*
@@ -391,12 +404,12 @@ static Class cellClass;
  */
 - (void) setTag: (int)anInt
 {
-  tag = anInt;
+  _tag = anInt;
 }
 
 - (int) tag
 {
-  return tag;
+  return _tag;
 }
 
 /*
@@ -412,20 +425,20 @@ static Class cellClass;
   unsigned int event_mask = NSLeftMouseDownMask | NSLeftMouseUpMask |
 			    NSMouseMovedMask | NSLeftMouseDraggedMask |
 			    NSRightMouseDraggedMask;
-
   NSDebugLog(@"NSControl mouseDown\n");
 
   if (![self isEnabled])
     return;
 
-  if ([cell isContinuous])
-    oldActionMask = [cell sendActionOn: 0];
+  if (_ignoresMultiClick && ([theEvent clickCount] > 1))
+    [super mouseDown: theEvent];
+
+  if ([_cell isContinuous])
+    oldActionMask = [_cell sendActionOn: 0];
   else
-    oldActionMask = [cell sendActionOn: NSPeriodicMask];
+    oldActionMask = [_cell sendActionOn: NSPeriodicMask];
 
-  [window _captureMouse: self];
-
-  [self lockFocus];
+  [_window _captureMouse: self];
 
   e = theEvent;
   while (!done) 		// loop until mouse goes up
@@ -434,20 +447,20 @@ static Class cellClass;
       location = [self convertPoint: location fromView: nil];
       // ask the cell to track the mouse only
       // if the mouse is within the cell
-      if ((location.x >= 0) && (location.x < bounds.size.width) &&
-		      (location.y >= 0 && location.y < bounds.size.height))
+      if ((location.x >= 0) && (location.x < _bounds.size.width) &&
+		      (location.y >= 0 && location.y < _bounds.size.height))
 	{
-	  [cell highlight: YES withFrame: bounds inView: self];
-	  [window flushWindow];
-	  if ([cell trackMouse: e
-			inRect: bounds
-			ofView: self
-		  untilMouseUp: YES])
+	  [_cell highlight: YES withFrame: _bounds inView: self];
+	  [_window flushWindow];
+	  if ([_cell trackMouse: e
+		     inRect: _bounds
+		     ofView: self
+		     untilMouseUp: [[_cell class] prefersTrackingUntilMouseUp]])
 	    done = mouseUp = YES;
 	  else
 	    {
-	      [cell highlight: NO withFrame: bounds inView: self];
-	      [window flushWindow];
+	      [_cell highlight: NO withFrame: _bounds inView: self];
+	      [_window flushWindow];
 	    }
 	}
 
@@ -463,55 +476,34 @@ static Class cellClass;
 	done = YES;
     }
 
-  [window _releaseMouse: self];
+  [_window _releaseMouse: self];
 
   if (mouseUp)
     {
 //      	[cell setState: ![cell state]];
-      [cell highlight: NO withFrame: bounds inView: self];
-      [window flushWindow];
+      [_cell highlight: NO withFrame: _bounds inView: self];
+      [_window flushWindow];
     }
-  [self unlockFocus];
-  [cell sendActionOn: oldActionMask];
+
+  [_cell sendActionOn: oldActionMask];
 
   if (mouseUp)
     [self sendAction: [self action] to: [self target]];
 }
 
+- (void) resetCursorRects
+{
+  [_cell resetCursorRect: _bounds inView: self];
+}
+
 - (BOOL) ignoresMultiClick
 {
-  return NO;
+  return _ignoresMultiClick;
 }
 
 - (void) setIgnoresMultiClick: (BOOL)flag
 {
-}
-
-/*
- * Methods Implemented by the Delegate
- */
-- (BOOL) control: (NSControl *)control
-		textShouldBeginEditing: (NSText *)fieldEditor
-{
-  return NO;
-}
-
-- (BOOL) control: (NSControl *)control
-		textShouldEndEditing: (NSText *)fieldEditor
-{
-  return NO;
-}
-
-- (void) controlTextDidBeginEditing: (NSNotification *)aNotification
-{
-}
-
-- (void) controlTextDidEndEditing: (NSNotification *)aNotification
-{
-}
-
-- (void) controlTextDidChange: (NSNotification *)aNotification
-{
+  _ignoresMultiClick = flag;
 }
 
 /*
@@ -521,16 +513,16 @@ static Class cellClass;
 {
   [super encodeWithCoder: aCoder];
 
-  [aCoder encodeValueOfObjCType: @encode(int) at: &tag];
-  [aCoder encodeObject: cell];
+  [aCoder encodeValueOfObjCType: @encode(int) at: &_tag];
+  [aCoder encodeObject: _cell];
 }
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
   [super initWithCoder: aDecoder];
 
-  [aDecoder decodeValueOfObjCType: @encode(int) at: &tag];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &cell];
+  [aDecoder decodeValueOfObjCType: @encode(int) at: &_tag];
+  [aDecoder decodeValueOfObjCType: @encode(id) at: &_cell];
 
   return self;
 }
