@@ -129,11 +129,15 @@ static NSString	*NSMenuLocationsKey = @"NSMenuLocations";
 
 - (void)_setOwnedByPopUp: (BOOL)flag
 {
-  menu_is_beholdenToPopUpButton = flag;
-
-  if (flag == YES)
+  if (menu_is_beholdenToPopUpButton != flag)
     {
-      [titleView removeFromSuperviewWithoutNeedingDisplay];
+      menu_is_beholdenToPopUpButton = flag;
+      if (flag == YES)
+	{
+	  [titleView removeFromSuperviewWithoutNeedingDisplay];
+	  [aWindow setLevel: NSPopUpMenuWindowLevel];
+	  [bWindow setLevel: NSPopUpMenuWindowLevel];
+	}
     }
 }
 
@@ -415,7 +419,14 @@ static NSString	*NSMenuLocationsKey = @"NSMenuLocations";
  */
 - (int) indexOfItem: (id <NSMenuItem>)anObject
 {
-  return [menu_items indexOfObjectIdenticalTo: anObject];
+  int index;
+
+  index = [menu_items indexOfObjectIdenticalTo: anObject];
+
+  if (index == NSNotFound)
+    return -1;
+  else
+    return index;
 }
 
 - (int) indexOfItemWithTitle: (NSString*)aTitle
@@ -1265,9 +1276,9 @@ static NSString	*NSMenuLocationsKey = @"NSMenuLocations";
     }
 }
 
-- (void) initDefaults
+- (void) _initDefaults
 {
-  [super initDefaults];
+  [super _initDefaults];
   window_level = NSSubmenuWindowLevel;
 }
 
@@ -1327,17 +1338,16 @@ static NSString	*NSMenuLocationsKey = @"NSMenuLocations";
 - (void) drawRect: (NSRect)rect
 {
   NSGraphicsContext *ctxt = GSCurrentContext();
-  NSRect             workRect = rect;
-
+  NSRect             workRect = [self bounds];
 
   // Draw the dark gray upper left lines.
   DPSgsave(ctxt);
-    DPSsetlinewidth(ctxt, 1);
-    DPSsetgray(ctxt, 0.333);
-    DPSmoveto(ctxt, workRect.origin.x, workRect.origin.y);
-    DPSrlineto(ctxt, 0, workRect.size.height);
-    DPSrlineto(ctxt, workRect.size.width, 0);
-    DPSstroke(ctxt);
+  DPSsetlinewidth(ctxt, 1);
+  DPSsetgray(ctxt, 0.333);
+  DPSmoveto(ctxt, workRect.origin.x, workRect.origin.y);
+  DPSrlineto(ctxt, 0, workRect.size.height);
+  DPSrlineto(ctxt, workRect.size.width, 0);
+  DPSstroke(ctxt);
   DPSgrestore(ctxt);
 
   // Draw the title box's button.
