@@ -31,6 +31,7 @@
 #include <AppKit/NSPasteboard.h>
 #include <AppKit/NSApplication.h>
 #include <AppKit/NSWorkspace.h>
+#include <AppKit/NSFileWrapper.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSData.h>
 #include <Foundation/NSDictionary.h>
@@ -541,11 +542,44 @@ static  NSMapTable              *mimeMap = NULL;
 
 - (BOOL) writeFileContents: (NSString*)filename
 {
+  NSFileWrapper *wrapper;
   NSData	*data;
   NSString	*type;
   BOOL		ok = NO;
 
-  data = [NSData dataWithContentsOfFile: filename];
+  wrapper = [[NSFileWrapper alloc] initWithPath: filename];
+  data = [wrapper serializedRepresentation];
+  RELEASE(wrapper);
+  type = NSCreateFileContentsPboardType([filename pathExtension]);
+  NS_DURING
+    {
+      ok = [target setData: data
+		   forType: type
+		    isFile: YES
+		  oldCount: changeCount];
+    }
+  NS_HANDLER
+    {
+      ok = NO;
+      [NSException raise: NSPasteboardCommunicationException
+		  format: @"%@", [localException reason]];
+    }
+  NS_ENDHANDLER
+  return ok;
+}
+
+- (BOOL)writeFileWrapper:(NSFileWrapper *)wrapper
+{
+  NSString *filename = [wrapper preferredFilename];
+  NSData *data;
+  NSString *type;
+  BOOL ok = NO;
+
+  if (filename == nil)
+    [NSException raise: NSInvalidArgumentException
+		 format: @"Cannot put file on pastboard with no preferred filename"];
+
+  data = [wrapper serializedRepresentation];
   type = NSCreateFileContentsPboardType([filename pathExtension]);
   NS_DURING
     {
@@ -666,19 +700,40 @@ static  NSMapTable              *mimeMap = NULL;
 			    toFile: (NSString*)filename
 {
   NSData	*d;
+  NSFileWrapper *wrapper;
 
   if (type == nil)
     {
       type = NSCreateFileContentsPboardType([filename pathExtension]);
     }
   d = [self dataForType: type];
-  if ([d writeToFile: filename atomically: NO] == NO)
+  if (d == nil)
     {
+      d = [self dataForType: NSFileContentsPboardType];
+      if (d == nil)
+	return nil;
+    }
+
+  wrapper = [[NSFileWrapper alloc] initWithSerializedRepresentation: d];
+  if ([wrapper writeToFile: filename atomically: NO updateFilenames: NO] == NO)
+    {
+      RELEASE(wrapper);
       return nil;
     }
+  RELEASE(wrapper);
   return filename;
 }
 
+- (NSFileWrapper *)readFileWrapper
+{
+  NSData *d = [self dataForType: NSFileContentsPboardType];
+
+  if (d == nil)
+    return nil;
+
+  return AUTORELEASE([[NSFileWrapper alloc] initWithSerializedRepresentation: d]);
+}
+
 - (NSString*) stringForType: (NSString*)dataType
 {
   return [self propertyListForType: dataType];
