@@ -61,99 +61,100 @@ static Class cellClass;
 
 - (id) initWithFrame: (NSRect)frameRect
 {
-  NSSliderCell* theCell = [[[isa cellClass] new] autorelease];
+  NSSliderCell	*theCell = AUTORELEASE([[isa cellClass] new]);
 
   [super initWithFrame: frameRect];
 
   // set our cell
   [self setCell: theCell];
   [theCell setState: 1];
+  [theCell setContinuous: YES];
   return self;
 }
 
 - (NSImage *) image
 {
-  return [[self cell] image];
+  return [_cell image];
 }
 
 - (int) isVertical
 {
-  return [[self cell] isVertical];
+  return [_cell isVertical];
 }
 
 - (float) knobThickness
 {
-  return [[self cell] knobThickness];
+  return [_cell knobThickness];
 }
 
 - (void) setImage: (NSImage *)backgroundImage
 {
-  [[self cell] setImage: backgroundImage];
+  [_cell setImage: backgroundImage];
 }
 
 - (void) setKnobThickness: (float)aFloat
 {
-  [[self cell] setKnobThickness: aFloat];
+  [_cell setKnobThickness: aFloat];
 }
 
 - (void) setTitle: (NSString *)aString
 {
-  [[self cell] setTitle: aString];
+  [_cell setTitle: aString];
 }
 
 - (void) setTitleCell: (NSCell *)aCell
 {
-  [[self cell] setTitleCell: aCell];
+  [_cell setTitleCell: aCell];
 }
 
 - (void) setTitleColor: (NSColor *)aColor
 {
-  [[self cell] setTitleColor: aColor];
+  [_cell setTitleColor: aColor];
 }
 
 - (void) setTitleFont: (NSFont *)fontObject
 {
-  [[self cell] setTitleFont: fontObject];
+  [_cell setTitleFont: fontObject];
 }
 
 - (NSString *) title
 {
-  return [[self cell] title];
+  return [_cell title];
 }
 
 - (id) titleCell
 {
-  return [[self cell] titleCell];
+  return [_cell titleCell];
 }
 
 - (NSColor *) titleColor
 {
-  return [[self cell] titleColor];
+  return [_cell titleColor];
 }
 
 - (NSFont *) titleFont
 {
-  return [[self cell] titleFont];
+  return [_cell titleFont];
 }
 
 - (double) maxValue
 {
-  return [[self cell] maxValue];
+  return [_cell maxValue];
 }
 
 - (double) minValue
 {
-  return [[self cell] minValue];
+  return [_cell minValue];
 }
 
 - (void) setMaxValue: (double)aDouble
 {
-  [[self cell] setMaxValue: aDouble];
+  [_cell setMaxValue: aDouble];
 }
 
 - (void) setMinValue: (double)aDouble
 {
-  [[self cell] setMinValue: aDouble];
+  [_cell setMinValue: aDouble];
 }
 
 - (BOOL) acceptsFirstMouse: (NSEvent *)theEvent
@@ -163,12 +164,17 @@ static Class cellClass;
 
 - (void) drawRect: (NSRect)rect
 {
-  [[self cell] drawWithFrame: rect inView: self];
+  /*
+   * Give the cell our bounds to draw in, not the rect - if we give it a rect
+   * that represents an exposed part of this view, it will try to draw the
+   * slider knob positioned in that rectangle ... which is wrong.
+   */
+  [_cell drawWithFrame: _bounds inView: self];
 }
 
 - (float)_floatValueForMousePoint: (NSPoint)point knobRect: (NSRect)knobRect
 {
-  NSSliderCell* theCell = [self cell];
+  NSSliderCell* theCell = _cell;
   NSRect slotRect = [theCell trackRect];
   BOOL isVertical = [theCell isVertical];
   float minValue = [theCell minValue];
@@ -229,8 +235,8 @@ static Class cellClass;
 			  | NSPeriodicMask;
   NSPoint point = [self convertPoint: [theEvent locationInWindow] fromView: nil];
   NSEventType eventType = [theEvent type];
-  BOOL isContinuous = [self isContinuous];
-  NSSliderCell* theCell = [self cell];
+  BOOL isContinuous = [_cell isContinuous];
+  NSSliderCell* theCell = _cell;
   float oldFloatValue = [theCell floatValue];
   id target = [theCell target];
   SEL action = [theCell action];
@@ -259,8 +265,8 @@ static Class cellClass;
 	      if (floatValue != oldFloatValue)
 		{
 		  [theCell setFloatValue: floatValue];
-		  [theCell drawWithFrame: [self bounds] inView: self];
-		  [window flushWindow];
+		  [theCell drawWithFrame: _bounds inView: self];
+		  [_window flushWindow];
 		  if (isContinuous)
 		    [target performSelector: action withObject: self];
 		  oldFloatValue = floatValue;
@@ -278,11 +284,9 @@ static Class cellClass;
 - (void) mouseDown: (NSEvent *)theEvent
 {
   NSPoint location = [self convertPoint: [theEvent locationInWindow]fromView: nil];
-  NSSliderCell* theCell = [self cell];
+  NSSliderCell* theCell = _cell;
   NSRect rect;
 
-  [self lockFocus];
-
   rect = [theCell knobRectFlipped: [self isFlipped]];
   if (![self mouse: location inRect: rect])
     {
@@ -291,16 +295,14 @@ static Class cellClass;
 					       knobRect: rect];
 
       [theCell setFloatValue: floatValue];
-      if ([self isContinuous])
+      if ([theCell isContinuous])
 	[[theCell target] performSelector: [theCell action]
 			       withObject: self];
-      [theCell drawWithFrame: [self bounds] inView: self];
-      [window flushWindow];
+      [theCell drawWithFrame: _bounds inView: self];
+      [_window flushWindow];
     }
 
   [self trackKnob: theEvent knobRect: rect];
-
-  [self unlockFocus];
 }
 
 @end
