@@ -1,10 +1,12 @@
 /*
    NSSliderCell.m
 
-   Copyright (C) 1996 Free Software Foundation, Inc.
+   Copyright (C) 1996,1999 Free Software Foundation, Inc.
 
    Author: Ovidiu Predescu <ovidiu@net-community.com>
    Date: September 1997
+   Rewrite: Richard Frith-Macdonald <richard@brainstorm.co.uk>
+   Date: 1999
   
    This file is part of the GNUstep GUI Library.
 
@@ -39,14 +41,13 @@
 
 - (id) init
 {
-  [self initImageCell: nil];
+  self = [self initImageCell: nil];
   _altIncrementValue = -1;
   _isVertical = -1;
   _minValue = 0;
   _maxValue = 1;
-  _floatValue = 0;
-  [self setBordered: YES];
-  [self setBezeled: YES];
+  _cell.is_bordered = YES;
+  _cell.is_bezeled = YES;
 
   _knobCell = [NSCell new];
 
@@ -55,26 +56,16 @@
 
 - (void) dealloc
 {
-  [_titleCell release];
-  [_knobCell release];
+  RELEASE(_titleCell);
+  RELEASE(_knobCell);
   [super dealloc];
 }
 
-- (BOOL) isFliped
+- (BOOL) isFlipped
 {
   return YES;
 }
 
-- (void) setFloatValue: (float)aFloat
-{
-  if (aFloat < _minValue)
-    _floatValue = _minValue;
-  else if (aFloat > _maxValue)
-    _floatValue = _maxValue;
-  else
-    _floatValue = aFloat;
-}
-
 - (void) drawBarInside: (NSRect)rect flipped: (BOOL)flipped
 {
   [[NSColor scrollBarColor] set];					
@@ -83,27 +74,29 @@
 
 - (NSRect) knobRectFlipped: (BOOL)flipped
 {
-  NSImage* image = [_knobCell image];
-  NSSize size;
-  NSPoint origin;
-  float floatValue;
+  NSImage	*image = [_knobCell image];
+  NSSize	size;
+  NSPoint	origin;
+  float		floatValue = [self floatValue];
 
   if (_isVertical && flipped)
-    _floatValue = _maxValue + _minValue - _floatValue;
+    {
+      floatValue = _maxValue + _minValue - floatValue;
+    }
 
-  floatValue = (_floatValue - _minValue) / (_maxValue - _minValue);
+  floatValue = (floatValue - _minValue) / (_maxValue - _minValue);
 
   size = [image size];
 
-  if (_isVertical)
+  if (_isVertical == YES)
     {
-      origin.x = 0;
-      origin.y = (_trackRect.size.height - size.height) * floatValue;
+      origin = _trackRect.origin;
+      origin.y += (_trackRect.size.height - size.height) * floatValue;
     }
   else
     {
-      origin.x = (_trackRect.size.width - size.width) * floatValue;
-      origin.y = 0;
+      origin = _trackRect.origin;
+      origin.x += (_trackRect.size.width - size.width) * floatValue;
     }
 
   return NSMakeRect (origin.x, origin.y, size.width, size.height); 
@@ -121,49 +114,62 @@
 
 - (void) drawInteriorWithFrame: (NSRect)cellFrame inView: (NSView*)controlView
 {
-  BOOL vertical = (cellFrame.size.height > cellFrame.size.width);
-  NSImage* image;
-  NSSize size;
+  BOOL		vertical = (cellFrame.size.height > cellFrame.size.width);
+  NSImage	*image;
+  NSSize	size;
 
-  if (vertical != _isVertical) {
-    if (vertical) {
-      image = [NSImage imageNamed: @"common_SliderVert"];
-      size = [image size];
-      [_knobCell setImage: image];
-      [image setSize: NSMakeSize (cellFrame.size.width, size.height)];
+  cellFrame = [self drawingRectForBounds: cellFrame];
+
+  [controlView lockFocus];
+  if (vertical != _isVertical)
+    {
+      if (vertical == YES)
+	{
+	  image = [NSImage imageNamed: @"common_SliderVert"];
+	  size = [image size];
+	  [_knobCell setImage: image];
+	  [image setSize: NSMakeSize (cellFrame.size.width, size.height)];
+	}
+      else
+	{
+	  image = [NSImage imageNamed: @"common_SliderHoriz"];
+	  size = [image size];
+	  [_knobCell setImage: image];
+	  [image setSize: NSMakeSize (size.width, cellFrame.size.height)];
+	}
     }
-    else {
-      image = [NSImage imageNamed: @"common_SliderHoriz"];
-      size = [image size];
-      [_knobCell setImage: image];
-      [image setSize: NSMakeSize (size.width, cellFrame.size.height)];
-    }
-  }
   _isVertical = vertical;
 
   _trackRect = cellFrame;
 
-  if (_titleCell)
-    [_titleCell drawInteriorWithFrame: cellFrame inView: controlView];
-
   [self drawBarInside: cellFrame flipped: [controlView isFlipped]];
+  if (_titleCell != nil)
+    {
+      [_titleCell drawInteriorWithFrame: cellFrame inView: controlView];
+    }
   [self drawKnob];
+  [controlView unlockFocus];
+}
+
+- (BOOL) isOpaque
+{
+  return YES;
 }
 
 - (float) knobThickness
 {
-  NSImage* image = [_knobCell image];
-  NSSize size = [image size];
+  NSImage	*image = [_knobCell image];
+  NSSize	size = [image size];
 
   return _isVertical ? size.height : size.width;
 }
 
 - (void) setKnobThickness: (float)thickness
 {
-  NSImage* image = [_knobCell image];
-  NSSize size = [image size];
+  NSImage	*image = [_knobCell image];
+  NSSize	size = [image size];
 
-  if (_isVertical)
+  if (_isVertical == YES)
     size.height = thickness;
   else
     size.width = thickness;
@@ -179,46 +185,111 @@
 - (void) setMinValue: (double)aDouble
 {
   _minValue = aDouble;
-  if (_floatValue < _minValue)
-    _floatValue = _minValue;
 }
 
 - (void) setMaxValue: (double)aDouble
 {
   _maxValue = aDouble;
-  if (_floatValue > _maxValue)
-    _floatValue = _maxValue;
 }
 
-- (id) titleCell				{ return _titleCell; }
-- (NSColor*) titleColor			{ return [_titleCell textColor]; }
-- (NSFont*) titleFont			{ return [_titleCell font]; }
-- (void) setTitle: (NSString*)title	{ [_titleCell setStringValue: title]; }
-- (NSString*) title			{ return [_titleCell stringValue]; }
-- (void) setTitleCell: (NSCell*)aCell	{ ASSIGN(_titleCell, aCell); }
-- (void) setTitleColor: (NSColor*)color	{ [_titleCell setTextColor: color]; }
-- (void) setTitleFont: (NSFont*)font	{ [_titleCell setFont: font]; }
-- (int) isVertical			{ return _isVertical; }
-- (double) altIncrementValue		{ return _altIncrementValue; }
-+ (BOOL) prefersTrackingUntilMouseUp	{ return YES; }
-- (NSRect) trackRect			{ return _trackRect; }
-- (double) minValue			{ return _minValue; }
-- (double) maxValue			{ return _maxValue; }
-- (float) floatValue			{ return _floatValue; }
+- (id) titleCell
+{
+  return _titleCell;
+}
+
+- (NSColor*) titleColor
+{
+  return [_titleCell textColor];
+}
+
+- (NSFont*) titleFont
+{
+  return [_titleCell font];
+}
+
+- (void) setTitle: (NSString*)title
+{
+  [_titleCell setStringValue: title];
+}
+
+- (NSString*) title
+{
+  return [_titleCell stringValue];
+}
+
+- (void) setTitleCell: (NSCell*)aCell
+{
+  ASSIGN(_titleCell, aCell);
+}
+
+- (void) setTitleColor: (NSColor*)color
+{
+  [_titleCell setTextColor: color];
+}
+
+- (void) setTitleFont: (NSFont*)font
+{
+  [_titleCell setFont: font];
+}
+
+- (int) isVertical
+{
+  return _isVertical;
+}
+
+- (double) altIncrementValue
+{
+  return _altIncrementValue;
+}
+
++ (BOOL) prefersTrackingUntilMouseUp
+{
+  return YES;
+}
+
+- (NSRect) trackRect
+{
+  return _trackRect;
+}
+
+- (double) minValue
+{
+  return _minValue;
+}
+
+- (double) maxValue
+{
+  return _maxValue;
+}
+
+- (float) floatValue
+{
+  float	aFloat = [super floatValue];
+
+  if (aFloat < _minValue)
+    return _minValue;
+  else if (aFloat > _maxValue)
+    return _maxValue;
+  return aFloat;
+}
 
 - (id) initWithCoder: (NSCoder*)decoder
 {
   self = [super initWithCoder: decoder];
-  [decoder decodeValuesOfObjCTypes: "ffff",
-	      &_minValue, &_maxValue, &_floatValue, &_altIncrementValue];
+  [decoder decodeValuesOfObjCTypes: "fffi",
+    &_minValue, &_maxValue, &_altIncrementValue, &_isVertical];
+  [decoder decodeValueOfObjCType: @encode(id) at: &_titleCell];
+  [decoder decodeValueOfObjCType: @encode(id) at: &_knobCell];
   return self;
 }
 
 - (void) encodeWithCoder: (NSCoder*)coder
 {
   [super encodeWithCoder: coder];
-  [coder encodeValuesOfObjCTypes: "ffff",
-	      &_minValue, &_maxValue, &_floatValue, &_altIncrementValue];
+  [coder encodeValuesOfObjCTypes: "fffi",
+    &_minValue, &_maxValue, &_altIncrementValue, &_isVertical];
+  [coder encodeValueOfObjCType: @encode(id) at: &_titleCell];
+  [coder encodeValueOfObjCType: @encode(id) at: &_knobCell];
 }
 
 @end
