@@ -484,13 +484,12 @@ drawRun(GSTextRun *run, NSPoint origin, GSDrawInfo *draw)
    */
   if (draw->flip)
     {
-      origin.y += (run->height - run->baseline);
+      origin.y -= run->base;
     }
   else
     {
-      origin.y -= (run->height - run->baseline);
+      origin.y += run->base;
     }
-
   /*
    * Set current font and color if necessary.
    */
@@ -602,14 +601,18 @@ setupRun(GSTextRun *run, unsigned length, unichar *chars, unsigned pos,
   // Get superscript
   num = (NSNumber*)[attr objectForKey: NSSuperscriptAttributeName]; 
   if (num == nil)
-    run->superscript = 0;
+    run->base = 0.0;
+    //run->superscript = 0;
   else
-    run->superscript = [num intValue];
+    // interprete as a baseline change without font change
+    run->base = 3.0 * [num intValue];
+    //run->superscript = [num intValue];
 
   // Get baseline offset
   num = (NSNumber*)[attr objectForKey: NSBaselineOffsetAttributeName]; 
   if (num == nil)
-    run->base = 0.0;
+      ; // Use value from superscript!
+    //run->base = 0.0;
   else
     run->base = [num floatValue];
 
@@ -716,6 +719,14 @@ drawChunk(GSTextChunk *chunk, NSPoint origin, GSDrawInfo *draw)
   GSTextRun	*run = &chunk->run0;
 
   origin.x += chunk->xpos;
+  if (draw->flip)
+    {
+      origin.y += (chunk->height - chunk->baseline);
+    }
+  else
+    {
+      origin.y -= (chunk->height - chunk->baseline);
+    }
   while (run != 0)
     {
       drawRun(run, origin, draw);
@@ -1153,6 +1164,7 @@ setupLine(GSTextLine *line, NSAttributedString *str, NSRange range,
   NSString		*allText = [self string];
   unsigned		length = [allText length];
   unsigned		paraPos = 0;
+  NSRange		styleRange;
   NSParagraphStyle	*style = nil;
   GSTextLine		current;
 
@@ -1167,7 +1179,6 @@ setupLine(GSTextLine *line, NSAttributedString *str, NSRange range,
       NSRange	line;		// Range of current line.
       NSRange	eol;		// Range of newline character.
       unsigned	position;	// Position in NSString.
-      BOOL	firstLine = YES;
 
       /*
        * Determine the range of the next paragraph of text (in 'para') and set
@@ -1187,12 +1198,12 @@ setupLine(GSTextLine *line, NSAttributedString *str, NSRange range,
 
       do
 	{
-	  if (firstLine == YES)
+	  if (style == nil || NSMaxRange(styleRange) > position)
 	    {
 	      style = (NSParagraphStyle*)[self
 				    attribute: NSParagraphStyleAttributeName
 				      atIndex: position
-			       effectiveRange: 0];
+			       effectiveRange: &styleRange];
 	      if (style == nil)
 		style = defStyle;
 	    }
@@ -1323,5 +1334,79 @@ setupLine(GSTextLine *line, NSAttributedString *str, NSRange range,
   return size;
 }
 
-@end
 
+#if 1
+- (NSSize) sizeRange: (NSRange) lineRange
+{
+  return [[self attributedSubstringFromRange: lineRange] size];
+}
+
+- (void) drawRange: (NSRange) lineRange atPoint: (NSPoint) aPoint
+{
+  [[self attributedSubstringFromRange: lineRange] drawAtPoint: aPoint];
+}
+
+- (void) drawRange: (NSRange) lineRange inRect: (NSRect)aRect
+{
+  [[self attributedSubstringFromRange: lineRange] drawInRect: aRect];
+}
+
+#else
+- (NSSize) sizeRange: (NSRange) lineRange
+{
+  NSRect retRect = NSZeroRect;
+  NSRange currRange = NSMakeRange (lineRange.location, 0);
+  NSPoint currPoint = NSMakePoint (0, 0);
+  NSString *string = [self string];
+  
+
+  for (; NSMaxRange (currRange) < NSMaxRange (lineRange);) // draw all "runs"
+    {
+      NSDictionary *attributes = [self attributesAtIndex: NSMaxRange(currRange) 
+				       longestEffectiveRange: &currRange 
+				       inRange: lineRange];
+      NSString *substring = [string substringWithRange: currRange];
+      NSRect sizeRect = NSMakeRect (currPoint.x, 0, 0, 0);
+      
+      sizeRect.size = [substring sizeWithAttributes: attributes];
+      retRect = NSUnionRect (retRect, sizeRect);
+      currPoint.x += sizeRect.size.width;
+      //<!> size attachments
+    } 
+  return retRect.size;
+}
+
+- (void) drawRange: (NSRange) lineRange atPoint: (NSPoint) aPoint
+{
+  NSRange currRange = NSMakeRange (lineRange.location, 0);
+  NSPoint currPoint;
+  NSString *string = [self string];
+  
+  for (currPoint = aPoint; NSMaxRange (currRange) < NSMaxRange (lineRange);)
+    {
+      // draw all "runs"
+      NSDictionary *attributes = [self attributesAtIndex: NSMaxRange(currRange) 
+				       longestEffectiveRange: &currRange 
+				       inRange: lineRange];
+      NSString *substring = [string substringWithRange: currRange];
+      [substring drawAtPoint: currPoint withAttributes: attributes];
+      currPoint.x += [substring sizeWithAttributes: attributes].width;
+      //<!> draw attachments
+    }
+}
+
+- (void) drawRange: (NSRange)aRange inRect: (NSRect)aRect
+{
+  NSString *substring = [[self string] substringWithRange: aRange];
+  
+  [substring drawInRect: aRect
+	     withAttributes: [NSDictionary dictionaryWithObjectsAndKeys: 
+					     [NSFont systemFontOfSize: 12.0], 
+					     NSFontAttributeName,
+					     [NSColor blueColor], 
+					     NSForegroundColorAttributeName,
+					     nil]];
+}
+#endif
+
+@end
