@@ -29,7 +29,6 @@
 #include <Foundation/NSProcessInfo.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSNotification.h>
-#include <AppKit/NSMatrix.h>
 #include <AppKit/NSApplication.h>
 #include <AppKit/NSWindow.h>
 #include <AppKit/NSEvent.h>
@@ -39,10 +38,10 @@
 #include <AppKit/NSRulerView.h>
 #include <AppKit/NSPasteboard.h>
 #include <AppKit/NSSpellChecker.h>
-#include <AppKit/NSFontPanel.h>
 #include <AppKit/NSControl.h>
 #include <AppKit/NSLayoutManager.h>
 #include <AppKit/NSTextStorage.h>
+#include <AppKit/NSColorPanel.h>
 
 @implementation NSTextView
 
@@ -50,8 +49,6 @@
 
 + (void) initialize
 {
-  [super initialize];
-
   if ([self class] == [NSTextView class])
     {
       [self setVersion: 1];
@@ -61,17 +58,12 @@
 
 + (void) registerForServices
 {
-  NSArray	*r;
-  NSArray	*s;
+  NSArray *types;
       
-  /*
-   * FIXME - should register for all types of data we support, not just string.
-   */
-  r  = [NSArray arrayWithObjects: NSStringPboardType, nil];
-  s  = [NSArray arrayWithObjects: NSStringPboardType, nil];
+  types  = [NSArray arrayWithObjects: NSStringPboardType, NSRTFPboardType, NSRTFDPboardType, nil];
  
-  [[NSApplication sharedApplication] registerServicesMenuSendTypes: s
-						       returnTypes: r];
+  [[NSApplication sharedApplication] registerServicesMenuSendTypes: types
+						       returnTypes: types];
 }
 
 /* Initializing Methods */
@@ -83,109 +75,121 @@
 
   [self setTextContainer: aTextContainer];
   [self setEditable: YES];
+  [self setUsesFontPanel: YES];
+  [self setUsesRuler: YES];
 
   return self;
 }
 
 - (id) initWithFrame: (NSRect)frameRect
 {
-  textStorage = [[NSTextStorage alloc] init];
+  NSTextContainer *aTextContainer = 
+      [[NSTextContainer alloc] initWithContainerSize: frameRect.size];
+  NSLayoutManager *layoutManager = [[NSLayoutManager alloc] init];
 
-  layoutManager = [[NSLayoutManager alloc] init];
+  [layoutManager addTextContainer: aTextContainer];
+  RELEASE(aTextContainer);
 
-  [textStorage addLayoutManager: layoutManager];
+  _textStorage = [[NSTextStorage alloc] init];
+  [_textStorage addLayoutManager: layoutManager];
   RELEASE(layoutManager);
 
-  textContainer
-    = [[NSTextContainer alloc] initWithContainerSize: frameRect.size];
-  [layoutManager addTextContainer: textContainer];
-  RELEASE(textContainer);
-
-  return [self initWithFrame: frameRect textContainer: textContainer];
+  return [self initWithFrame: frameRect textContainer: aTextContainer];
 }
 
 - (void) setTextContainer: (NSTextContainer*)aTextContainer
 {
-  ASSIGN(textContainer, aTextContainer);
+  ASSIGN(_textContainer, aTextContainer);
+  // FIXME: Could also get a reference to the layout manager
+  ASSIGN(_textStorage, [[aTextContainer layoutManager] textStorage]);
 }
 
 - (NSTextContainer*) textContainer
 {
-  return textContainer;
+  return _textContainer;
 }
 
 - (void) replaceTextContainer: (NSTextContainer*)aTextContainer
 {
   // Notify layoutManager of change?
 
-  ASSIGN(textContainer, aTextContainer);
+  ASSIGN(_textContainer, aTextContainer);
 }
 
 - (void) setTextContainerInset: (NSSize)inset
 {
-  textContainerInset = inset;
+  _textContainerInset = inset;
+  [self invalidateTextContainerOrigin];
 }
 
 - (NSSize) textContainerInset
 {
-  return textContainerInset;
+  return _textContainerInset;
 }
 
 - (NSPoint) textContainerOrigin
 {
-  // use bounds, inset, and used rect.
-  NSRect bRect = [self bounds];
-
-  return NSZeroPoint;
+  return _textContainerOrigin;
 }
 
 - (void) invalidateTextContainerOrigin
 {
-  tv_resetTextContainerOrigin = YES;
+  // recompute the textContainerOrigin
+  // use bounds, inset, and used rect.
+  /*
+  NSRect bRect = [self bounds];
+  NSRect uRect = [[self layoutManager] usedRectForTextContainer: _textContainer];
+
+  if ([self isFlipped])
+    _textContainerOrigin = ;
+  else
+    _textContainerOrigin = ;
+  */
 }
 
 - (NSLayoutManager*) layoutManager
 {
-  return [textContainer layoutManager];
+  return [_textContainer layoutManager];
 }
 
 - (NSTextStorage*) textStorage
 {
-  return textStorage;
+  return _textStorage;
 }
 
 - (void) setBackgroundColor: (NSColor*)aColor
 {
-  ASSIGN(tv_backGroundColor, aColor);
+  ASSIGN(_background_color, aColor);
 }
 
 - (NSColor*) backgroundColor
 {
-  return tv_backGroundColor;
+  return _background_color;
+}
+
+- (void) setAllowsUndo: (BOOL)flag
+{
+  _tf.allows_undo = flag;
+}
+
+- (BOOL) allowsUndo
+{
+  return _tf.allows_undo;
 }
 
 - (void) setDrawsBackground: (BOOL)flag
 {
-  tv_drawsBackground = flag;
+  _tf.draws_background = flag;
 }
 
 - (BOOL) drawsBackground
 {
-  return tv_drawsBackground;
+  return _tf.draws_background;
 }
 
 - (void) setNeedsDisplayInRect: (NSRect)aRect
 	 avoidAdditionalLayout: (BOOL)flag
 {
-/*
-  NSRange	glyphsToDraw;
-
-  glyphsToDraw = [layoutManager glyphRangeForTextContainer: textContainer];
-  [self lockFocus];
-  [layoutManager drawGlyphsForGlyphRange: glyphsToDraw
-				 atPoint: [self frame].origin];
-  [self unlockFocus];
-*/
 }
 
 /* We override NSView's setNeedsDisplayInRect: */
@@ -197,33 +201,16 @@
 
 - (BOOL) shouldDrawInsertionPoint
 {
-  return tv_shouldDrawInsertionPoint;
+  return [super shouldDrawInsertionPoint];
 }
 
 - (void) drawInsertionPointInRect: (NSRect)aRect
 			    color: (NSColor*)aColor
 			 turnedOn: (BOOL)flag
 {
-  [self lockFocus];
-
-  NSDebugLLog(@"NSText", 
-    @"drawInsertionPointInRect: (%f, %f)", aRect.size.width, aRect.size.height);
-
-  aRect.size.width = 1;
-
-  if (flag)
-    {
-      [aColor set];
-      NSRectFill(aRect);
-    }
-  else
-    {
-      [[self backgroundColor] set];
-      NSRectFill(aRect);
-    }
-
-  [self unlockFocus];
-  [_window flushWindow];
+  [super drawInsertionPointInRect: aRect
+	 color: aColor
+	 turnedOn: flag];
 }
 
 - (void) setConstrainedFrameSize: (NSSize)desiredSize
@@ -237,93 +224,102 @@
   // release drag information
 }
 
+- (unsigned int) dragOperationForDraggingInfo: (id <NSDraggingInfo>)dragInfo 
+					 type: (NSString *)type
+{
+  //FIXME
+  return NSDragOperationNone;
+}
+
 - (void) setEditable: (BOOL)flag
 {
   if (flag)
-    tv_selectable = flag;
+    _tf.is_selectable = flag;
 
-  tv_editable = flag;
+  _tf.is_editable = flag;
 }
 
 - (BOOL) isEditable
 {
-  return tv_editable;
+  return _tf.is_editable;
 }
 
 - (void) setSelectable: (BOOL)flag
 {
-  tv_selectable = flag;
+  _tf.is_selectable = flag;
 }
 
 - (BOOL) isSelectable
 {
-  return tv_selectable;
+  return _tf.is_selectable;
 }
 
 - (void) setFieldEditor: (BOOL)flag
 {
-  tv_fieldEditor = flag;
+  _tf.is_field_editor = flag;
 }
 
 - (BOOL) isFieldEditor
 {
-  return tv_fieldEditor;
+  return _tf.is_field_editor;
 }
 
 - (void) setRichText: (BOOL)flag
 {
   if (!flag)
-    tv_acceptDraggedFiles = flag;
+    _tf.imports_graphics = flag;
 
-  tv_richText = flag;
+  _tf.is_rich_text = flag;
+  [self updateDragTypeRegistration];
 }
 
 - (BOOL) isRichText
 {
-  return tv_richText;
+  return _tf.is_rich_text;
 }
 
 - (void) setImportsGraphics: (BOOL)flag
 {
   if (flag)
-    tv_richText = flag;
+    _tf.is_rich_text = flag;
 
-  tv_acceptDraggedFiles = flag;
+  _tf.imports_graphics = flag;
+  [self updateDragTypeRegistration];
 }
 
 - (BOOL) importsGraphics
 {
-  return tv_acceptDraggedFiles;
+  return _tf.imports_graphics;
 }
 
 - (void) setUsesFontPanel: (BOOL)flag
 {
-  tv_usesFontPanel = flag;
+  _tf.uses_font_panel = flag;
 }
 
 - (BOOL) usesFontPanel
 {
-  return tv_usesFontPanel;
+  return _tf.uses_font_panel;
 }
 
 - (void) setUsesRuler: (BOOL)flag
 {
-  tv_usesRuler = flag;
+  _tf.uses_ruler = flag;
 }
 
 - (BOOL) usesRuler
 {
-  return tv_usesRuler;
+  return _tf.uses_ruler;
 }
 
 - (void) setRulerVisible: (BOOL)flag
 {
-  tv_rulerVisible = flag;
+  [super setRulerVisible: flag];
 }
 
 - (BOOL) isRulerVisible
 {
-  return tv_rulerVisible;
+  return _tf.is_ruler_visible;
 }
 
 - (void) setSelectedRange: (NSRange)charRange
@@ -334,7 +330,7 @@
     postNotificationName: NSTextViewDidChangeSelectionNotification
     object: self];
 */
-  tv_selectedRange = charRange;
+  _selected_range = charRange;
   [self setSelectionGranularity: NSSelectByCharacter];
 
   // Also removes the marking from
@@ -343,61 +339,64 @@
 
 - (NSRange) selectedRange
 {
-  return tv_selectedRange;
+  return _selected_range;
 }
 
 - (void) setSelectedRange: (NSRange)charRange
 		 affinity: (NSSelectionAffinity)affinity
 	   stillSelecting: (BOOL)flag
 {
-  NSDebugLLog(@"NSText", @"setSelectedRange stillSelecting.");
+  // Use affinity to determine the insertion point
 
-  tv_selectedRange = charRange;
-  [self setSelectionGranularity: NSSelectByCharacter];
-
-  // FIXME, more.
+  if (flag)
+    {
+      _selected_range = charRange;
+      [self setSelectionGranularity: NSSelectByCharacter];
+    }
+  else
+      [self setSelectedRange: charRange];
 }
 
 - (NSSelectionAffinity) selectionAffinity
 {
-  return tv_selectionAffinity;
+  return _selectionAffinity;
 }
 
 - (void) setSelectionGranularity: (NSSelectionGranularity)granularity
 {
-  tv_selectionGranularity = granularity;
+  _selectionGranularity = granularity;
 }
 
 - (NSSelectionGranularity) selectionGranularity
 {
-  return tv_selectionGranularity;
+  return _selectionGranularity;
 }
 
 - (void) setInsertionPointColor: (NSColor*)aColor
 {
-  ASSIGN(tv_caretColor, aColor);
+  ASSIGN(_caret_color, aColor);
 }
 
 - (NSColor*) insertionPointColor
 {
-  return tv_caretColor;
+  return _caret_color;
 }
 
 - (void) updateInsertionPointStateAndRestartTimer: (BOOL)flag
 {
-  // tv_caretLocation =
+  // _caretLocation =
 
   // restart blinking timer.
 }
 
 - (void) setSelectedTextAttributes: (NSDictionary*)attributes
 {
-  ASSIGN(tv_selectedTextAttributes, attributes);
+  ASSIGN(_selectedTextAttributes, attributes);
 }
 
 - (NSDictionary*) selectedTextAttributes
 {
-  return tv_selectedTextAttributes;
+  return _selectedTextAttributes;
 }
 
 - (NSRange) markedRange
@@ -409,18 +408,19 @@
 
 - (void) setMarkedTextAttributes: (NSDictionary*)attributes
 {
-  ASSIGN(tv_markedTextAttributes, attributes);
+  ASSIGN(_markedTextAttributes, attributes);
 }
 
 - (NSDictionary*) markedTextAttributes
 {
-  return tv_markedTextAttributes;
+  return _markedTextAttributes;
 }
 
 - (NSString*) preferredPasteboardTypeFromArray: (NSArray*)availableTypes
 		    restrictedToTypesFromArray: (NSArray*)allowedTypes
 {
-  // No idea.
+  return [super preferredPasteboardTypeFromArray: availableTypes
+		restrictedToTypesFromArray: allowedTypes];
 }
 
 - (BOOL) readSelectionFromPasteboard: (NSPasteboard*)pboard
@@ -434,8 +434,7 @@ preferred type of data and then reads the data using the
 readSelectionFromPasteboard: type: method. Returns YES if the
 data was successfully read.
 */
-
-  return NO;
+  return [super readSelectionFromPasteboard: pboard];
 }
 
 - (BOOL) readSelectionFromPasteboard: (NSPasteboard*)pboard
@@ -451,17 +450,20 @@ default types. Use the rangeForUserTextChange method to obtain the range
 of characters (if any) to be replaced by the new data.
 */
 
-  return NO;
+  return [super readSelectionFromPasteboard: pboard
+		type: type];
 }
 
 - (NSArray*) readablePasteboardTypes
 {
   // get default types, what are they?
+  return [super readablePasteboardTypes];
 }
 
 - (NSArray*) writablePasteboardTypes
 {
   // the selected text can be written to the pasteboard with which types.
+  return [super writablePasteboardTypes];
 }
 
 - (BOOL) writeSelectionToPasteboard: (NSPasteboard*)pboard
@@ -475,13 +477,13 @@ super's implementation of the method to handle any types of data your
 overridden version does not.
 */
 
-  return NO;
+  return [super writeSelectionToPasteboard: pboard
+		type: type];
 }
 
 - (BOOL) writeSelectionToPasteboard: (NSPasteboard*)pboard
 			      types: (NSArray*)types
 {
-
 /* Writes the current selection to pboard under each type in the types
 array. Returns YES if the data for any single type was written
 successfully.
@@ -489,122 +491,232 @@ successfully.
 You should not need to override this method. You might need to invoke this
 method if you are implementing a new type of pasteboard to handle services
 other than copy/paste or dragging. */
-
-  return NO;
+  return [super writeSelectionToPasteboard: pboard
+		types: types];
 }
 
 - (void) alignJustified: (id)sender
 {
-/*
-  if (!tv_richText)
-    // if plain all text is jsutified.
-  else
-    // selected range is fully justified.
-*/
+  [self setAlignment: NSJustifiedTextAlignment
+	range: [self rangeForUserParagraphAttributeChange]];   
 }
 
 - (void) changeColor: (id)sender
 {
-//  NSColor *aColor = [sender color];
+  NSColor *aColor = (NSColor*)[sender color];
+  NSRange aRange = [self rangeForUserCharacterAttributeChange];
+
+  if (aRange.location == NSNotFound)
+    return;
 
   // sets the color for the selected range.
+  [self setTextColor: aColor
+	range: aRange];
 }
 
 - (void) setAlignment: (NSTextAlignment)alignment
 		range: (NSRange)aRange
 { 
-/*
-Sets the alignment of the paragraphs containing characters in aRange to
-alignment. alignment is one of: 
-
-      NSLeftTextAlignment
-      NSRightTextAlignment 
-      NSCenterTextAlignment 
-      NSJustifiedTextAlignment 
-      NSNaturalTextAlignment
-*/
+  [super setAlignment: alignment
+	 range: aRange];
 }
 
 - (void) setTypingAttributes: (NSDictionary*)attributes
 {
-  // more?
-
-  ASSIGN(tv_typingAttributes, attributes);
+  [super setTypingAttributes: attributes];
 }
 
 - (NSDictionary*) typingAttributes
 {
-  return tv_typingAttributes;
+  return [super typingAttributes];
 }
 
 - (void) useStandardKerning: (id)sender
 {
   // rekern for selected range if rich text, else rekern entire document.
+  NSRange aRange = [self rangeForUserCharacterAttributeChange];
+
+  if (aRange.location == NSNotFound)
+    return;
+  
+  if (![self shouldChangeTextInRange: aRange
+	    replacementString: nil])
+    return;
+  [_textStorage beginEditing];
+  [_textStorage removeAttribute: NSKernAttributeName
+		range: aRange];
+  [_textStorage endEditing];
+  [self didChangeText];
 }
 
 - (void) lowerBaseline: (id)sender
 {
-/*
-  if (tv_richText)
-    // lower baseline by one point for selected text
+  id value;
+  float sValue;
+  NSRange effRange;
+  NSRange aRange = [self rangeForUserCharacterAttributeChange];
+
+  if (aRange.location == NSNotFound)
+    return;
+
+  if (![self shouldChangeTextInRange: aRange
+	    replacementString: nil])
+    return;
+  [_textStorage beginEditing];
+  // We take the value form the first character and use it for the whole range
+  value = [_textStorage attribute: NSBaselineOffsetAttributeName
+			atIndex: aRange.location
+			effectiveRange: &effRange];
+
+  if (value != nil)
+    sValue = [value floatValue] + 1.0;
   else
-    // lower baseline for entire document.
-*/
+    sValue = 1.0;
+
+  [_textStorage addAttribute: NSBaselineOffsetAttributeName
+		value: [NSNumber numberWithFloat: sValue]
+		range: aRange];
 }
 
 - (void) raiseBaseline: (id)sender
 {
-/*
-  if (tv_richText)
-    // raise baseline by one point for selected text
+  id value;
+  float sValue;
+  NSRange effRange;
+  NSRange aRange = [self rangeForUserCharacterAttributeChange];
+
+  if (aRange.location == NSNotFound)
+    return;
+
+  if (![self shouldChangeTextInRange: aRange
+	    replacementString: nil])
+    return;
+  [_textStorage beginEditing];
+  // We take the value form the first character and use it for the whole range
+  value = [_textStorage attribute: NSBaselineOffsetAttributeName
+			atIndex: aRange.location
+			effectiveRange: &effRange];
+
+  if (value != nil)
+    sValue = [value floatValue] - 1.0;
   else
-    // raise baseline for entire document.
-*/
+    sValue = -1.0;
+
+  [_textStorage addAttribute: NSBaselineOffsetAttributeName
+		value: [NSNumber numberWithFloat: sValue]
+		range: aRange];
+  [_textStorage endEditing];
+  [self didChangeText];
 }
 
 - (void) turnOffKerning: (id)sender
 {
-/*
-  if (tv_richText)
-    // turn off kerning in selection.
-  else
-    // turn off kerning document wide.
-*/
+  NSRange aRange = [self rangeForUserCharacterAttributeChange];
+
+  if (aRange.location == NSNotFound)
+    return;
+  
+  if (![self shouldChangeTextInRange: aRange
+	    replacementString: nil])
+    return;
+  [_textStorage beginEditing];
+  [_textStorage addAttribute: NSKernAttributeName
+		value: [NSNumber numberWithFloat: 0.0]
+		range: aRange];
+  [_textStorage endEditing];
+  [self didChangeText];
 }
 
 - (void) loosenKerning: (id)sender
 {
-/*
-  if (tv_richText)
-    // loosen kerning in selection.
-  else
-    // loosen kerning document wide.
-*/
+  NSRange aRange = [self rangeForUserCharacterAttributeChange];
+
+  if (aRange.location == NSNotFound)
+    return;
+
+  if (![self shouldChangeTextInRange: aRange
+	    replacementString: nil])
+    return;
+  [_textStorage beginEditing];
+  // FIXME: Should use the current kerning and work relative to point size
+  [_textStorage addAttribute: NSKernAttributeName
+		value: [NSNumber numberWithFloat: 1.0]
+		range: aRange];
+  [_textStorage endEditing];
+  [self didChangeText];
 }
 
 - (void) tightenKerning: (id)sender
 {
-/*
-  if (tv_richText)
-    // tighten kerning in selection.
-  else
-    // tighten kerning document wide.
-*/
+  NSRange aRange = [self rangeForUserCharacterAttributeChange];
+
+  if (aRange.location == NSNotFound)
+    return;
+
+  if (![self shouldChangeTextInRange: aRange
+	    replacementString: nil])
+    return;
+  [_textStorage beginEditing];
+  // FIXME: Should use the current kerning and work relative to point size
+  [_textStorage addAttribute: NSKernAttributeName
+		value: [NSNumber numberWithFloat: -1.0]
+		range: aRange];
+  [_textStorage endEditing];
+  [self didChangeText];
 }
 
 - (void) useStandardLigatures: (id)sender
 {
-  // well.
+  NSRange aRange = [self rangeForUserCharacterAttributeChange];
+
+  if (aRange.location == NSNotFound)
+    return;
+
+  if (![self shouldChangeTextInRange: aRange
+	    replacementString: nil])
+    return;
+  [_textStorage beginEditing];
+  [_textStorage addAttribute: NSLigatureAttributeName
+		value: [NSNumber numberWithInt: 1]
+		range: aRange];
+  [_textStorage endEditing];
+  [self didChangeText];
 }
 
 - (void) turnOffLigatures: (id)sender
 {
-  // sure.
+  NSRange aRange = [self rangeForUserCharacterAttributeChange];
+
+  if (aRange.location == NSNotFound)
+    return;
+
+  if (![self shouldChangeTextInRange: aRange
+	    replacementString: nil])
+    return;
+  [_textStorage beginEditing];
+  [_textStorage addAttribute: NSLigatureAttributeName
+		value: [NSNumber numberWithInt: 0]
+		range: aRange];
+  [_textStorage endEditing];
+  [self didChangeText];
 }
 
 - (void) useAllLigatures: (id)sender
 {
-  // as you say.
+  NSRange aRange = [self rangeForUserCharacterAttributeChange];
+
+  if (aRange.location == NSNotFound)
+    return;
+
+  if (![self shouldChangeTextInRange: aRange
+	    replacementString: nil])
+    return;
+  [_textStorage beginEditing];
+  [_textStorage addAttribute: NSLigatureAttributeName
+		value: [NSNumber numberWithInt: 2]
+		range: aRange];
+  [_textStorage endEditing];
+  [self didChangeText];
 }
 
 - (void) clickedOnLink: (id)link
@@ -613,11 +725,10 @@ alignment. alignment is one of:
 
 /* Notifies the delegate that the user clicked in a link at the specified
 charIndex. The delegate may take any appropriate actions to handle the
-click in its textView: clickedOnLink: atIndex: method.Notifies the delegate
-that the user clicked in a link at the specified charIndex. The delegate
-may take any appropriate actions to handle the click in its
-textView: clickedOnLink: atIndex: method. */
- 
+click in its textView: clickedOnLink: atIndex: method. */
+  if (_delegate != nil && 
+      [_delegate respondsToSelector: @selector(textView:clickedOnLink:atIndex:)])
+      [_delegate textView: self clickedOnLink: link atIndex: charIndex];
 }
 
 /*
@@ -627,17 +738,19 @@ replacing the selection.
 
 - (void) pasteAsPlainText: (id)sender
 {
-  [self insertText: [sender string]];
+  [self readSelectionFromPasteboard: [NSPasteboard generalPasteboard]
+				type: NSStringPboardType];
 }
 
 - (void) pasteAsRichText: (id)sender
 {
-  [self insertText: [sender string]];
+  [self readSelectionFromPasteboard: [NSPasteboard generalPasteboard]
+				type: NSRTFPboardType];
 }
 
 - (void) updateFontPanel
 {
-  // [fontPanel setFont: [self fontFromRange]];
+  [super updateFontPanel];
 }
 
 - (void) updateRuler
@@ -647,61 +760,67 @@ replacing the selection.
 
 - (NSArray*) acceptableDragTypes
 {
+  return [self readablePasteboardTypes];
 }
 
 - (void) updateDragTypeRegistration
 {
+  // FIXME: Should change registration for all our text views
+  if (_tf.is_editable && _tf.is_rich_text)
+    [self registerForDraggedTypes: [self acceptableDragTypes]];
+  else
+    [self unregisterDraggedTypes];
 }
 
 - (NSRange) selectionRangeForProposedRange: (NSRange)proposedSelRange
 			       granularity: (NSSelectionGranularity)granularity
 {
-  NSRange retRange;
-
-  switch (granularity)
-    {
-      case NSSelectByParagraph: 
-        // we need to: 1, find how far to end of paragraph; 2, increase
-        // range.
-      case NSSelectByWord: 
-        // we need to: 1, find how far to end of word; 2, increase range.
-      case NSSelectByCharacter: 
-      default: 
-        retRange = proposedSelRange;
-    }
-
-  return retRange;
+  return [super selectionRangeForProposedRange: proposedSelRange
+		granularity: granularity];
 }
 
 - (NSRange) rangeForUserCharacterAttributeChange
 {
-  if (!tv_editable || !tv_usesFontPanel)
-    return NSMakeRange(NSNotFound, 0);
-
-  if (tv_richText)
-    return tv_selectedRange;
-  else
-    return NSMakeRange(NSNotFound, 0); // should be entire contents.
+  return [super rangeForUserCharacterAttributeChange];
 }
 
 - (NSRange) rangeForUserParagraphAttributeChange
 {
-  if (!tv_editable)
-    return NSMakeRange(NSNotFound, 0);
-
-  if (tv_richText)
-    return [self selectionRangeForProposedRange: tv_selectedRange
-		granularity: NSSelectByParagraph];
-  else
-    return NSMakeRange(NSNotFound, 0); // should be entire contents.
+  return [super rangeForUserParagraphAttributeChange];
 }
 
 - (NSRange) rangeForUserTextChange
 {
-  if (!tv_editable || !tv_usesRuler)
-    return NSMakeRange(NSNotFound, 0);
+  return [super rangeForUserTextChange];
+}
 
-  return tv_selectedRange;
+
+- (id) validRequestorForSendType: (NSString*)sendType
+		      returnType: (NSString*)returnType
+{
+/*
+Returns self if sendType specifies a type of data the text view can put on
+the pasteboard and returnType contains a type of data the text view can
+read from the pasteboard; otherwise returns nil.
+*/
+
+ return [super validRequestorForSendType: sendType
+		returnType: returnType];
+}
+
+- (int) spellCheckerDocumentTag
+{
+  return [super spellCheckerDocumentTag];
+}
+
+- (void) insertText: (NSString*)aString
+{
+  [super insertText: aString];
+}
+
+- (void) sizeToFit
+{
+  [super sizeToFit];
 }
 
 - (BOOL) shouldChangeTextInRange: (NSRange)affectedCharRange
@@ -722,7 +841,7 @@ Text in the class description for more information. If you can't determine
 the affected range or replacement string before beginning changes, pass
 (NSNotFound, 0) and nil for these values. */
 
-  return NO;
+  return YES;
 }
 
 - (void) didChangeText
@@ -733,24 +852,38 @@ the affected range or replacement string before beginning changes, pass
 
 - (void) setSmartInsertDeleteEnabled: (BOOL)flag
 {
-  tv_smartInsertDelete = flag;
+  _tf.smart_insert_delete = flag;
 }
 
 - (BOOL) smartInsertDeleteEnabled
 {
-  return tv_smartInsertDelete;
+  return _tf.smart_insert_delete;
 }
 
 - (NSRange) smartDeleteRangeForProposedRange: (NSRange)proposedCharRange
 {
-// FIXME.
+  // FIXME.
   return proposedCharRange;
 }
 
+- (NSString *)smartInsertAfterStringForString: (NSString *)aString 
+			       replacingRange: (NSRange)charRange
+{
+  // FIXME.
+  return nil;
+}
+
+- (NSString *)smartInsertBeforeStringForString: (NSString *)aString 
+				replacingRange: (NSRange)charRange
+{
+  // FIXME.
+  return nil;
+}
+
 - (void) smartInsertForString: (NSString*)aString
 	       replacingRange: (NSRange)charRange
-		 beforeString: (NSString*)beforeString 
-		  afterString: (NSString*)afterString
+		 beforeString: (NSString**)beforeString 
+		  afterString: (NSString**)afterString
 {
 
 /* Determines whether whitespace needs to be added around aString to
@@ -769,7 +902,13 @@ NSTextView uses this method as necessary. You can also use it in
 implementing your own methods that insert text. To do so, invoke this
 method with the proper arguments, then insert beforeString, aString, and
 afterString in order over charRange. */
+  if (beforeString)
+    *beforeString = [self smartInsertBeforeStringForString: aString 
+			  replacingRange: charRange];
 
+  if (afterString)
+    *afterString = [self smartInsertAfterStringForString: aString 
+			 replacingRange: charRange];
 }
 
 - (BOOL) resignFirstResponder
@@ -805,27 +944,6 @@ afterString in order over charRange. */
   return YES;
 }
 
-- (id) validRequestorForSendType: (NSString*)sendType
-		      returnType: (NSString*)returnType
-{
-/*
-Returns self if sendType specifies a type of data the text view can put on
-the pasteboard and returnType contains a type of data the text view can
-read from the pasteboard; otherwise returns nil.
-*/
-
- return nil;
-}
-
-- (int) spellCheckerDocumentTag
-{
-/*
-  if (!tv_spellTag)
-    tv_spellTag = [[NSSpellingServer sharedServer] uniqueSpellDocumentTag];
-*/
-  return tv_spellTag;
-}
-
 - (void) rulerView: (NSRulerView*)aRulerView
      didMoveMarker: (NSRulerMarker*)aMarker
 {
@@ -851,6 +969,11 @@ and notifications, and only invokes this method if permission is granted.
 */
 }
 
+- (void)rulerView:(NSRulerView *)ruler 
+     didAddMarker:(NSRulerMarker *)marker
+{
+}
+
 - (void) rulerView: (NSRulerView*)aRulerView
    handleMouseDown: (NSEvent*)theEvent
 {
@@ -934,8 +1057,8 @@ container, returning the modified location. */
   [super setDelegate: anObject];
  
 #define SET_DELEGATE_NOTIFICATION(notif_name) \
-  if ([delegate respondsToSelector: @selector(textView##notif_name: )]) \
-    [nc addObserver: delegate \
+  if ([_delegate respondsToSelector: @selector(textView##notif_name: )]) \
+    [nc addObserver: _delegate \
            selector: @selector(textView##notif_name: ) \
                name: NSTextView##notif_name##Notification \
              object: self]
@@ -944,79 +1067,60 @@ container, returning the modified location. */
   SET_DELEGATE_NOTIFICATION(WillChangeNotifyingTextView);
 }
 
-- (void) setString: (NSString*)string
-{
-  NSAttributedString	*aString;
-
-  aString = [NSAttributedString alloc];
-  aString = [aString initWithString: string
-			 attributes: [self typingAttributes]];
-  AUTORELEASE(aString);
-
-//  [textStorage replaceRange: NSMakeRange(0, [string length])
-//	withString: aString];
-
-  [textStorage setAttributedString: aString];
-
-//replaceCharactersInRange: NSMakeRange(0, [string length])
-//                   withAttributedString: aString];
-
-//  [textStorage insertAttributedString: aString atIndex: 0];
-}
-
-- (void) setText: (NSString*)string
-{
-  [self setString: string];
-}
-
-- (void) insertText: (NSString*)aString
-{
-  NSDebugLLog(@"NSText", @"%@", aString);
-
-  if (![aString isKindOfClass: [NSAttributedString class]])
-    aString = [[NSAttributedString alloc] initWithString: aString
-		attributes: [self typingAttributes]];
-
-  [textStorage replaceCharactersInRange: [self selectedRange]
-       withAttributedString: (NSAttributedString*)aString];
-
-  [self sizeToFit];                       // ScrollView interaction
-
-  [self setSelectedRange: NSMakeRange([self 
-    selectedRange].location+[aString length],0)];
-
-  [self display];
-  [_window update]; 
-
-  NSLog(@"%@", [textStorage string]);
-  /*
-   * broadcast notification
-   */
-  [[NSNotificationCenter defaultCenter]
-    postNotificationName: NSTextDidChangeNotification
-    object: self];
-}
-
-- (void) sizeToFit
-{
-  NSLog(@"sizeToFit called.\n");
-}
-
 - (void) drawRect: (NSRect)aRect
 {
+  // currently use super implementation, which is working, 
+  // although the code here is correcter
+  [super drawRect: aRect];
+/*
   NSRange	glyphRange;
+  NSLayoutManager *layoutManager = [self layoutManager];
 
-  if (tv_backGroundColor)
+  if (_background_color != nil)
     {
-      [tv_backGroundColor set];
+      [_background_color set];
       NSRectFill(aRect);
     }
-  glyphRange = [layoutManager glyphRangeForTextContainer: textContainer];
+  glyphRange = [layoutManager glyphRangeForTextContainer: _textContainer];
   if (glyphRange.length > 0)
     {
       [layoutManager drawGlyphsForGlyphRange: glyphRange
 				     atPoint: [self frame].origin];
     }
+*/
 }
 
 @end
+
+@implementation NSTextView(NSTextInput)
+// This are all the NSTextInput methods that are not implemented on NSTextView
+// or one of its super classes.
+
+- (void)setMarkedText:(NSString *)aString selectedRange:(NSRange)selRange
+{
+}
+
+- (BOOL)hasMarkedText
+{
+  return NO;
+}
+
+- (void)unmarkText
+{
+}
+
+- (NSArray*)validAttributesForMarkedText
+{
+  return nil;
+}
+
+- (long)conversationIdentifier
+{
+  return 0;
+}
+
+- (NSRect)firstRectForCharacterRange:(NSRange)theRange
+{
+  return NSZeroRect;
+}
+@end
