@@ -265,6 +265,10 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 
   RETAIN(aView);
   [aView removeFromSuperview];
+  if (aView->_coordinates_valid)
+    {
+      (*invalidateImp)(aView, invalidateSel);
+    }
   [aView viewWillMoveToWindow: _window];
   [aView viewWillMoveToSuperview: self];
   [aView setNextResponder: self];
@@ -301,6 +305,10 @@ GSSetDragTypes(NSView* obj, NSArray *types)
     }
   RETAIN(aView);
   [aView removeFromSuperview];
+  if (aView->_coordinates_valid)
+    {
+      (*invalidateImp)(aView, invalidateSel);
+    }
   [aView viewWillMoveToWindow: _window];
   [aView viewWillMoveToSuperview: self];
   [aView setNextResponder: self];
@@ -372,78 +380,50 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 
 - (void) removeFromSuperviewWithoutNeedingDisplay
 {
-  /* This must be first because it invokes -resignFirstResponder:, 
-     which assumes the view is still in the view hierarchy */
-  if ([_window firstResponder] == self)
-    [_window makeFirstResponder: _window];
-  /*
-   * We MUST make sure that coordinates are invalidated even if we have
-   * no superview - cos they may have been rebuilt since we lost the
-   * superview and the fact that this method has been invoked probably
-   * means we are about to be placed in a new view where the coordinate
-   * system will be different.
-   */
-  if (_coordinates_valid)
+  if (_super_view != nil)
     {
-      (*invalidateImp)(self, invalidateSel);
+      [_super_view removeSubview: self];
     }
-  if (!_super_view)
-    {
-      return;
-    }
-  RETAIN(self);
-  [_super_view->_sub_views removeObjectIdenticalTo: self];
-  if ([_super_view->_sub_views count] == 0)
-    {
-      _super_view->_rFlags.has_subviews = 0;
-    }
-  _super_view = nil;
-  [self viewWillMoveToWindow: nil];
-  RELEASE(self);
 }
 
 - (void) removeFromSuperview
 {
-  /* This must be first because it invokes -resignFirstResponder:, 
-     which assumes the view is still in the view hierarchy */
-  if ([_window firstResponder] == self)
-    [_window makeFirstResponder: _window];
+  if (_super_view != nil)
+    {
+      [_super_view setNeedsDisplayInRect: _frame];
+      [_super_view removeSubview: self];
+    }
+}
+
+- (void) removeSubview: (NSView*)aSubview
+{
   /*
-   * We MUST make sure that coordinates are invalidated even if we have
-   * no superview - cos they may have been rebuilt since we lost the
-   * superview and the fact that this method has been invoked probably
-   * means we are about to be placed in a new view where the coordinate
-   * system will be different.
+   * This must be first because it invokes -resignFirstResponder:, 
+   * which assumes the view is still in the view hierarchy
    */
-  if (_coordinates_valid)
+  if ([_window firstResponder] == aSubview)
     {
-      (*invalidateImp)(self, invalidateSel);
+      [_window makeFirstResponder: _window];
     }
-  if (!_super_view)
+  aSubview->_super_view = nil;
+  [aSubview viewWillMoveToWindow: nil];
+  [_sub_views removeObjectIdenticalTo: aSubview];
+  if ([_sub_views count] == 0)
     {
-      return;
+      _rFlags.has_subviews = 0;
     }
-  [_super_view setNeedsDisplayInRect: _frame];
-  RETAIN(self);
-  [_super_view->_sub_views removeObjectIdenticalTo: self];
-  if ([_super_view->_sub_views count] == 0)
-    {
-      _super_view->_rFlags.has_subviews = 0;
-    }
-  _super_view = nil;
-  [self viewWillMoveToWindow: nil];
-  RELEASE(self);
 }
 
 - (void) replaceSubview: (NSView*)oldView with: (NSView*)newView
 {
   if (newView == oldView)
-    return;
-
+    {
+      return;
+    }
   /*
    * NB. we implement the replacement in full rather than calling addSubview:
    * since classes like NSBox override these methods but expect to be able to
-   * call [super replaceSubview: with: ] safely.
+   * call [super replaceSubview:with:] safely.
    */
   if (oldView == nil)
     {
@@ -454,6 +434,10 @@ GSSetDragTypes(NSView* obj, NSArray *types)
        */
       RETAIN(newView);
       [newView removeFromSuperview];
+      if (newView->_coordinates_valid)
+	{
+	  (*invalidateImp)(newView, invalidateSel);
+	}
       [newView viewWillMoveToWindow: _window];
       [newView viewWillMoveToSuperview: self];
       [newView setNextResponder: self];
@@ -486,6 +470,10 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 	   */
 	  RETAIN(newView);
 	  [newView removeFromSuperview];
+	  if (newView->_coordinates_valid)
+	    {
+	      (*invalidateImp)(newView, invalidateSel);
+	    }
 	  index = [_sub_views indexOfObjectIdenticalTo: oldView];
 	  [oldView removeFromSuperview];
 	  [newView viewWillMoveToWindow: _window];
@@ -522,6 +510,14 @@ GSSetDragTypes(NSView* obj, NSArray *types)
     {
       return;
     }
+  if (_coordinates_valid)
+    {
+      (*invalidateImp)(self, invalidateSel);
+    }
+  if (_rFlags.has_currects != 0)
+    {
+      [self discardCursorRects];
+    }
   if (_rFlags.has_draginfo)
     {
       NSGraphicsContext	*ctxt = GSCurrentContext();
@@ -536,10 +532,6 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 	  [ctxt _addDragTypes: t toWindow: [newWindow windowNumber]];
 	}
     }
-  if (newWindow == nil && [_window firstResponder] == self)
-    {
-      [_window makeFirstResponder: _window];
-    }
 
   _window = newWindow;
 
@@ -564,18 +556,58 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 - (void) rotateByAngle: (float)angle
 {
   if (_coordinates_valid)
-    (*invalidateImp)(self, invalidateSel);
+    {
+      (*invalidateImp)(self, invalidateSel);
+    }
   [_boundsMatrix rotateByAngle: angle];
   _is_rotated_from_base = _is_rotated_or_scaled_from_base = YES;
 
   if (_post_bounds_changes)
-    [[NSNotificationCenter defaultCenter]
-		  postNotificationName: NSViewBoundsDidChangeNotification
-				object: self];
+    {
+      [[NSNotificationCenter defaultCenter]
+		    postNotificationName: NSViewBoundsDidChangeNotification
+				  object: self];
+    }
+}
+
+- (void) _updateBoundsMatrix
+{
+  float sx;
+  float sy;
+  
+  if (_bounds.size.width == 0)
+    {
+      if (_frame.size.width == 0)
+	sx = 1;
+      else
+	sx = FLT_MAX;
+    }
+  else
+    {
+      sx = _frame.size.width / _bounds.size.width;
+    }
+  
+  if (_bounds.size.height == 0)
+    {
+      if (_frame.size.height == 0)
+	sy = 1;
+      else
+	sy = FLT_MAX;
+    }
+  else
+    {
+      sy = _frame.size.height / _bounds.size.height;
+    }
+  
+  [_boundsMatrix scaleTo: sx : sy];
+  if (sx != 1 || sy != 1)
+    _is_rotated_or_scaled_from_base = YES;
 }
 
 - (void) setFrame: (NSRect)frameRect
 {
+  BOOL	changedOrigin = NO;
+  BOOL	changedSize = NO;
   NSSize old_size = _frame.size;
 
   if (frameRect.size.width < 0)
@@ -588,23 +620,44 @@ GSSetDragTypes(NSView* obj, NSArray *types)
       NSWarnMLog(@"given negative height", 0);
       frameRect.size.height = 0;
     }
-  if (_coordinates_valid)
-    (*invalidateImp)(self, invalidateSel);
+
+  if (NSMinX(_frame) != NSMinX(frameRect) 
+      || NSMinY(_frame) != NSMinY(frameRect))
+    changedOrigin = YES;
+  if (NSWidth(_frame) != NSWidth(frameRect) 
+      || NSHeight(_frame) != NSHeight(frameRect))
+    changedSize = YES;
+  
   _frame = frameRect;
   _bounds.size = _frame.size;
-  [_frameMatrix setFrameOrigin: _frame.origin];
+  if (changedOrigin)
+    [_frameMatrix setFrameOrigin: _frame.origin];
 
-  [self resizeSubviewsWithOldSize: old_size];
-  if (_post_frame_changes)
-    [[NSNotificationCenter defaultCenter]
-		    postNotificationName: NSViewFrameDidChangeNotification
-				  object: self];
+  if (changedSize && _is_rotated_or_scaled_from_base)
+    {
+      [self _updateBoundsMatrix];
+    }
+
+  if (changedSize || changedOrigin)
+    {
+      if (_coordinates_valid)
+	{
+	  (*invalidateImp)(self, invalidateSel);
+	}
+      [self resizeSubviewsWithOldSize: old_size];
+      if (_post_frame_changes)
+	[[NSNotificationCenter defaultCenter]
+	  postNotificationName: NSViewFrameDidChangeNotification
+	                object: self];
+    }
 }
 
 - (void) setFrameOrigin: (NSPoint)newOrigin
 {
   if (_coordinates_valid)
-    (*invalidateImp)(self, invalidateSel);
+    {
+      (*invalidateImp)(self, invalidateSel);
+    }
   _frame.origin = newOrigin;
   [_frameMatrix setFrameOrigin: _frame.origin];
 
@@ -629,20 +682,26 @@ GSSetDragTypes(NSView* obj, NSArray *types)
       newSize.height = 0;
     }
   if (_coordinates_valid)
-    (*invalidateImp)(self, invalidateSel);
+    {
+      (*invalidateImp)(self, invalidateSel);
+    }
   _frame.size = _bounds.size = newSize;
 
   [self resizeSubviewsWithOldSize: old_size];
   if (_post_frame_changes)
-    [[NSNotificationCenter defaultCenter]
-		    postNotificationName: NSViewFrameDidChangeNotification
-				  object: self];
+    {
+      [[NSNotificationCenter defaultCenter]
+		      postNotificationName: NSViewFrameDidChangeNotification
+				    object: self];
+    }
 }
 
 - (void) setFrameRotation: (float)angle
 {
   if (_coordinates_valid)
-    (*invalidateImp)(self, invalidateSel);
+    {
+      (*invalidateImp)(self, invalidateSel);
+    }
   [_frameMatrix setFrameRotation: angle];
   _is_rotated_from_base = _is_rotated_or_scaled_from_base = YES;
 
@@ -674,9 +733,6 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 
 - (void) scaleUnitSquareToSize: (NSSize)newSize
 {
-  float sx;
-  float sy;
-
   if (newSize.width < 0)
     {
       NSWarnMLog(@"given negative width", 0);
@@ -688,38 +744,15 @@ GSSetDragTypes(NSView* obj, NSArray *types)
       newSize.height = 0;
     }
   if (_coordinates_valid)
-    (*invalidateImp)(self, invalidateSel);
-
+    {
+      (*invalidateImp)(self, invalidateSel);
+    }
   _bounds.size.width = _frame.size.width / newSize.width;
   _bounds.size.height = _frame.size.height / newSize.height;
 
   _is_rotated_or_scaled_from_base = YES;
-
-  if (_bounds.size.width == 0)
-    {
-      if (_frame.size.width == 0)
-	sx = 1;
-      else
-	sx = FLT_MAX;
-    }
-  else
-    {
-      sx = _frame.size.width / _bounds.size.width;
-    }
-
-  if (_bounds.size.height == 0)
-    {
-      if (_frame.size.height == 0)
-	sy = 1;
-      else
-	sy = FLT_MAX;
-    }
-  else
-    {
-      sy = _frame.size.height / _bounds.size.height;
-    }
-
-  [_boundsMatrix scaleBy: sx : sy];
+  
+  [self _updateBoundsMatrix];
 
   if (_post_bounds_changes)
     [[NSNotificationCenter defaultCenter]
@@ -729,8 +762,6 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 
 - (void) setBounds: (NSRect)aRect
 {
-  float sx, sy;
-
   if (aRect.size.width < 0)
     {
       NSWarnMLog(@"given negative width", 0);
@@ -742,39 +773,13 @@ GSSetDragTypes(NSView* obj, NSArray *types)
       aRect.size.height = 0;
     }
   if (_coordinates_valid)
-    (*invalidateImp)(self, invalidateSel);
-
+    {
+      (*invalidateImp)(self, invalidateSel);
+    }
   _bounds = aRect;
-  [_boundsMatrix setFrameOrigin: NSMakePoint(-_bounds.origin.x,-_bounds.origin.y)];
-
-  if (_bounds.size.width == 0)
-    {
-      if (_frame.size.width == 0)
-	sx = 1;
-      else
-	sx = FLT_MAX;
-    }
-  else
-    {
-      sx = _frame.size.width / _bounds.size.width;
-    }
-
-  if (_bounds.size.height == 0)
-    {
-      if (_frame.size.height == 0)
-	sy = 1;
-      else
-	sy = FLT_MAX;
-    }
-  else
-    {
-      sy = _frame.size.height / _bounds.size.height;
-    }
-
-  [_boundsMatrix scaleTo: sx : sy];
-
-  if (sx != 1 || sy != 1)
-    _is_rotated_or_scaled_from_base = YES;
+  [_boundsMatrix
+    setFrameOrigin: NSMakePoint(-_bounds.origin.x,-_bounds.origin.y)];
+  [self _updateBoundsMatrix];
 
   if (_post_bounds_changes)
     [[NSNotificationCenter defaultCenter]
@@ -787,19 +792,21 @@ GSSetDragTypes(NSView* obj, NSArray *types)
   _bounds.origin = newOrigin;
 
   if (_coordinates_valid)
-    (*invalidateImp)(self, invalidateSel);
+    {
+      (*invalidateImp)(self, invalidateSel);
+    }
   [_boundsMatrix setFrameOrigin: NSMakePoint(-newOrigin.x, -newOrigin.y)];
 
   if (_post_bounds_changes)
-    [[NSNotificationCenter defaultCenter]
-		    postNotificationName: NSViewBoundsDidChangeNotification
+    {
+      [[NSNotificationCenter defaultCenter]
+		      postNotificationName: NSViewBoundsDidChangeNotification
 				  object: self];
+    }
 }
 
 - (void) setBoundsSize: (NSSize)newSize
 {
-  float sx, sy;
-
   if (newSize.width < 0)
     {
       NSWarnMLog(@"given negative width", 0);
@@ -816,35 +823,7 @@ GSSetDragTypes(NSView* obj, NSArray *types)
     }
 
   _bounds.size = newSize;
-
-  if (_bounds.size.width == 0)
-    {
-      if (_frame.size.width == 0)
-	sx = 1;
-      else
-	sx = FLT_MAX;
-    }
-  else
-    {
-      sx = _frame.size.width / _bounds.size.width;
-    }
-
-  if (_bounds.size.height == 0)
-    {
-      if (_frame.size.height == 0)
-	sy = 1;
-      else
-	sy = FLT_MAX;
-    }
-  else
-    {
-      sy = _frame.size.height / _bounds.size.height;
-    }
-
-  [_boundsMatrix scaleTo: sx : sy];
-
-  if (sx != 1 || sy != 1)
-    _is_rotated_or_scaled_from_base = YES;
+  [self _updateBoundsMatrix];
 
   if (_post_bounds_changes)
     [[NSNotificationCenter defaultCenter]
@@ -855,20 +834,26 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 - (void) setBoundsRotation: (float)angle
 {
   if (_coordinates_valid)
-    (*invalidateImp)(self, invalidateSel);
+    {
+      (*invalidateImp)(self, invalidateSel);
+    }
   [_boundsMatrix setFrameRotation: angle];
   _is_rotated_from_base = _is_rotated_or_scaled_from_base = YES;
 
   if (_post_bounds_changes)
-    [[NSNotificationCenter defaultCenter]
-		    postNotificationName: NSViewBoundsDidChangeNotification
-				  object: self];
+    {
+      [[NSNotificationCenter defaultCenter]
+		      postNotificationName: NSViewBoundsDidChangeNotification
+				    object: self];
+    }
 }
 
 - (void) translateOriginToPoint: (NSPoint)point
 {
   if (_coordinates_valid)
-    (*invalidateImp)(self, invalidateSel);
+    {
+      (*invalidateImp)(self, invalidateSel);
+    }
   [_boundsMatrix translateToPoint: point];
 
   if (_post_bounds_changes)
@@ -917,9 +902,13 @@ GSSetDragTypes(NSView* obj, NSArray *types)
   new = [matrix pointInMatrixSpace: aPoint];
 
   if (_coordinates_valid)
-    matrix = _matrixFromWindow;
+    {
+      matrix = _matrixFromWindow;
+    }
   else
-    matrix = [self _matrixFromWindow];
+    {
+      matrix = [self _matrixFromWindow];
+    }
   new = [matrix pointInMatrixSpace: new];
 
   return new;
@@ -930,16 +919,24 @@ GSSetDragTypes(NSView* obj, NSArray *types)
   NSPoint	new;
   NSAffineTransform	*matrix;
 
-  if (!aView)
-    aView = [[_window contentView] superview];
+  if (aView == nil)
+    {
+      aView = [[_window contentView] superview];
+    }
   if (aView == self || aView == nil)
-    return aPoint;
+    {
+      return aPoint;
+    }
   NSAssert(_window == [aView window], NSInvalidArgumentException);
 
   if (_coordinates_valid)
-    matrix = _matrixToWindow;
+    {
+      matrix = _matrixToWindow;
+    }
   else
-    matrix = [self _matrixToWindow];
+    {
+      matrix = [self _matrixToWindow];
+    }
   new = [matrix pointInMatrixSpace: aPoint];
 
   matrix = [aView _matrixFromWindow];
@@ -953,10 +950,14 @@ GSSetDragTypes(NSView* obj, NSArray *types)
   NSAffineTransform	*matrix;
   NSRect	r;
 
-  if (!aView)
-    aView = [[_window contentView] superview];
+  if (aView == nil)
+    {
+      aView = [[_window contentView] superview];
+    }
   if (aView == self || aView == nil)
-    return aRect;
+    {
+      return aRect;
+    }
   NSAssert(_window == [aView window], NSInvalidArgumentException);
 
   matrix = [aView _matrixToWindow];
@@ -964,15 +965,20 @@ GSSetDragTypes(NSView* obj, NSArray *types)
   r.size = [matrix sizeInMatrixSpace: aRect.size];
 
   if (_coordinates_valid)
-    matrix = _matrixFromWindow;
+    {
+      matrix = _matrixFromWindow;
+    }
   else
-    matrix = [self _matrixFromWindow];
+    {
+      matrix = [self _matrixFromWindow];
+    }
   r.origin = [matrix pointInMatrixSpace: r.origin];
   r.size = [matrix sizeInMatrixSpace: r.size];
 
   if (aView->_rFlags.flipped_view  != _rFlags.flipped_view)
-    r.origin.y -= r.size.height;
-
+    {
+      r.origin.y -= r.size.height;
+    }
   return r;
 }
 
@@ -981,16 +987,23 @@ GSSetDragTypes(NSView* obj, NSArray *types)
   NSAffineTransform	*matrix;
   NSRect	r;
 
-  if (!aView)
-    aView = [[_window contentView] superview];
+  if (aView == nil)
+    {
+      aView = [[_window contentView] superview];
+    }
   if (aView == self || aView == nil)
-    return aRect;
+    {
+      return aRect;
+    }
   NSAssert(_window == [aView window], NSInvalidArgumentException);
-
   if (_coordinates_valid)
-    matrix = _matrixToWindow;
+    {
+      matrix = _matrixToWindow;
+    }
   else
-    matrix = [self _matrixToWindow];
+    {
+      matrix = [self _matrixToWindow];
+    }
   r.origin = [matrix pointInMatrixSpace: aRect.origin];
   r.size = [matrix sizeInMatrixSpace: aRect.size];
 
@@ -999,29 +1012,37 @@ GSSetDragTypes(NSView* obj, NSArray *types)
   r.size = [matrix sizeInMatrixSpace: r.size];
 
   if (aView->_rFlags.flipped_view  != _rFlags.flipped_view)
-    r.origin.y -= r.size.height;
-
+    {
+      r.origin.y -= r.size.height;
+    }
   return r;
 }
 
 - (NSSize) convertSize: (NSSize)aSize fromView: (NSView*)aView
 {
-  NSSize	new;
+  NSSize		new;
   NSAffineTransform	*matrix;
 
-  if (!aView)
-    aView = [[_window contentView] superview];
+  if (aView == nil)
+    {
+      aView = [[_window contentView] superview];
+    }
   if (aView == self || aView == nil)
-    return aSize;
+    {
+      return aSize;
+    }
   NSAssert(_window == [aView window], NSInvalidArgumentException);
-
   matrix = [aView _matrixToWindow];
   new = [matrix sizeInMatrixSpace: aSize];
 
   if (_coordinates_valid)
-    matrix = _matrixFromWindow;
+    {
+      matrix = _matrixFromWindow;
+    }
   else
-    matrix = [self _matrixFromWindow];
+    {
+      matrix = [self _matrixFromWindow];
+    }
   new = [matrix sizeInMatrixSpace: new];
 
   return new;
@@ -1029,19 +1050,26 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 
 - (NSSize) convertSize: (NSSize)aSize toView: (NSView*)aView
 {
-  NSSize	new;
+  NSSize		new;
   NSAffineTransform	*matrix;
 
-  if (!aView)
-    aView = [[_window contentView] superview];
+  if (aView == nil)
+    {
+      aView = [[_window contentView] superview];
+    }
   if (aView == self || aView == nil)
-    return aSize;
+    {
+      return aSize;
+    }
   NSAssert(_window == [aView window], NSInvalidArgumentException);
-
   if (_coordinates_valid)
-    matrix = _matrixToWindow;
+    {
+      matrix = _matrixToWindow;
+    }
   else
-    matrix = [self _matrixToWindow];
+    {
+      matrix = [self _matrixToWindow];
+    }
   new = [matrix sizeInMatrixSpace: aSize];
 
   matrix = [aView _matrixFromWindow];
@@ -1087,7 +1115,7 @@ GSSetDragTypes(NSView* obj, NSArray *types)
   float		change;
   float		changePerOption;
   int		options = 0;
-  NSSize	old_size = _frame.size;
+  NSRect        newFrame, newBounds;
   NSSize	superViewFrameSize = [_super_view frame].size;
   BOOL		changedOrigin = NO;
   BOOL		changedSize = NO;
@@ -1095,6 +1123,8 @@ GSSetDragTypes(NSView* obj, NSArray *types)
   if (_autoresizingMask == NSViewNotSizable)
     return;
 
+  newFrame = _frame;
+  newBounds = _bounds;
   /*
    * determine if and how the X axis can be resized
    */
@@ -1115,18 +1145,18 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 
       if (_autoresizingMask & NSViewWidthSizable)
 	{
-	  float oldFrameWidth = _frame.size.width;
+	  float oldFrameWidth = newFrame.size.width;
 
-	  _frame.size.width += changePerOption;
+	  newFrame.size.width += changePerOption;
 	  if (_is_rotated_or_scaled_from_base)
-	    _bounds.size.width *= _frame.size.width/oldFrameWidth;
+	    newBounds.size.width *= newFrame.size.width/oldFrameWidth;
 	  else
-	    _bounds.size.width += changePerOption;
+	    newBounds.size.width += changePerOption;
 	  changedSize = YES;
 	}
       if (_autoresizingMask & NSViewMinXMargin)
 	{
-	  _frame.origin.x += changePerOption;
+	  newFrame.origin.x += changePerOption;
 	  changedOrigin = YES;
 	}
     }
@@ -1152,13 +1182,13 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 
       if (_autoresizingMask & NSViewHeightSizable)
 	{
-	  float oldFrameHeight = _frame.size.height;
+	  float oldFrameHeight = newFrame.size.height;
 
-	  _frame.size.height += changePerOption;
+	  newFrame.size.height += changePerOption;
 	  if (_is_rotated_or_scaled_from_base)
-	    _bounds.size.height *= _frame.size.height/oldFrameHeight;
+	    newBounds.size.height *= newFrame.size.height/oldFrameHeight;
 	  else
-	    _bounds.size.height += changePerOption;
+	    newBounds.size.height += changePerOption;
 	  changedSize = YES;
 	}
       if (_autoresizingMask & (NSViewMaxYMargin | NSViewMinYMargin))
@@ -1167,7 +1197,7 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 	    {
 	      if (_autoresizingMask & NSViewMaxYMargin)
 		{
-		  _frame.origin.y += changePerOption;
+		  newFrame.origin.y += changePerOption;
 		  changedOrigin = YES;
 		}
 	    }
@@ -1175,54 +1205,17 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 	    {
 	      if (_autoresizingMask & NSViewMinYMargin)
 		{
-		  _frame.origin.y += changePerOption;
+		  newFrame.origin.y += changePerOption;
 		  changedOrigin = YES;
 		}
 	    }
 	}
     }
-
-  if (changedOrigin)
-    [_frameMatrix setFrameOrigin: _frame.origin];
-
+  [self setFrame: newFrame];
+  /* Since setFrame sets the bounds itself, reset it correctly */
+  _bounds = newBounds;
   if (changedSize && _is_rotated_or_scaled_from_base)
-    {
-      float sx;
-      float sy;
-
-      if (_bounds.size.width == 0)
-	{
-	  if (_frame.size.width == 0)
-	    sx = 1;
-	  else
-	    sx = FLT_MAX;
-	}
-      else
-	{
-	  sx = _frame.size.width / _bounds.size.width;
-	}
-
-      if (_bounds.size.height == 0)
-	{
-	  if (_frame.size.height == 0)
-	    sy = 1;
-	  else
-	    sy = FLT_MAX;
-	}
-      else
-	{
-	  sy = _frame.size.height / _bounds.size.height;
-	}
-
-      [_boundsMatrix scaleTo: sx : sy];
-    }
-
-  if (changedSize || changedOrigin)
-    {
-      if (_coordinates_valid)
-	(*invalidateImp)(self, invalidateSel);
-      [self resizeSubviewsWithOldSize: old_size];
-    }
+    [self _updateBoundsMatrix];
 }
 
 - (void) allocateGState
@@ -1264,6 +1257,8 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 
   [ctxt lockFocusView: self inRect: rect];
   wrect = [self convertRect: rect toView: nil];
+  NSDebugLLog(@"NSView", @"Displaying rect \n\t%@\n\t window %p", 
+	      NSStringFromRect(wrect), _window);
   window_t = (struct NSWindow_struct *)_window;
   [window_t->rectsBeingDrawn addObject: [NSValue valueWithRect: wrect]];
 
@@ -1303,6 +1298,9 @@ GSSetDragTypes(NSView* obj, NSArray *types)
       h = NSHeight(rect);
       DPSrectviewclip(ctxt, x, y, w, h);
 
+      /* Add any scaling due to the bounds matrix */
+      //[_boundsMatrix concat];
+
       /* Allow subclases to make other modifications */
       [self setUpGState];
       _renew_gstate = 0;
@@ -1561,14 +1559,20 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 
       /*
        * If the rect we displayed contains the _invalidRect or _visibleRect
-       * then we can empty _invalidRect.	 If all subviews have been
-       * fully displayed, so this view no longer needs to be displayed.
+       * then we can empty _invalidRect.
+       * If all subviews have been fully displayed, we can also turn off the
+       * 'needs_display' flag.
        */
       if (NSEqualRects(aRect, NSUnionRect(neededRect, aRect)) == YES)
 	{
 	  _invalidRect = NSZeroRect;
 	  _rFlags.needs_display = subviewNeedsDisplay;
 	}
+      if (_rFlags.needs_display == YES
+	&& NSEqualRects(aRect, NSUnionRect(_visibleRect, aRect)) == YES)
+	{
+	  _rFlags.needs_display = NO;
+	}
       [_window flushWindow];
     }
 }
@@ -1674,13 +1678,18 @@ GSSetDragTypes(NSView* obj, NSArray *types)
   /*
    * If the rect we displayed contains the _invalidRect or _visibleRect
    * then we can empty _invalidRect.  If all subviews have been
-   * fully displayed, so this view no longer needs to be displayed.
+   * fully displayed, we can also turn off the 'needs_display' flag.
    */
   if (NSEqualRects(aRect, NSUnionRect(neededRect, aRect)) == YES)
     {
       _invalidRect = NSZeroRect;
       _rFlags.needs_display = subviewNeedsDisplay;
     }
+  if (_rFlags.needs_display == YES
+    && NSEqualRects(aRect, NSUnionRect(_visibleRect, aRect)) == YES)
+    {
+      _rFlags.needs_display = NO;
+    }
   [_window flushWindow];
 }
 
@@ -1782,13 +1791,7 @@ GSSetDragTypes(NSView* obj, NSArray *types)
   aPoint = [self convertPoint: aPoint toView: s];
   if (NSEqualPoints(aPoint, [s bounds].origin) == NO)
     {
-      id	cSuper = [s superview];
-
-      aPoint = [s constrainScrollPoint: aPoint];
       [s scrollToPoint: aPoint];
-
-      if ([cSuper respondsToSelector:@selector(reflectScrolledClipView:)])
-	[(NSScrollView *)cSuper reflectScrolledClipView: s];
     }
 }
 
@@ -1844,14 +1847,8 @@ GSSetDragTypes(NSView* obj, NSArray *types)
 
       if (shouldScroll)
 	{
-	  id		cSuper = [s superview];
-
 	  aPoint = [self convertPoint: aPoint toView: s];
-	  aPoint = [s constrainScrollPoint: aPoint];
 	  [s scrollToPoint: aPoint];
-	  if ([cSuper respondsToSelector:@selector(reflectScrolledClipView:)])
-	    [(NSScrollView *)cSuper reflectScrolledClipView: s];
-
 	  return YES;
 	}
     }
@@ -2255,14 +2252,16 @@ static NSView* findByTag(NSView *view, int aTag, unsigned *level)
 	    source: (id)sourceObject
 	 slideBack: (BOOL)slideFlag
 {
- NSView *dragView = (NSView *)[GSCurrentContext() _dragInfo];
- [dragView dragImage: anImage
-                  at: viewLocation
-              offset: initialOffset
-               event: event
-          pasteboard: pboard
-              source: sourceObject
-           slideBack: slideFlag];
+  NSView *dragView = (NSView*)[GSCurrentContext() _dragInfo];
+
+  
+  [dragView dragImage: anImage
+		   at: viewLocation
+	       offset: initialOffset
+		event: event
+	   pasteboard: pboard
+	       source: sourceObject
+	    slideBack: slideFlag];
 }
 
 - (void) registerForDraggedTypes: (NSArray*)types
@@ -2674,7 +2673,9 @@ static NSView* findByTag(NSView *view, int aTag, unsigned *level)
 		  NSView	*sub = array[i];
 
 		  if (sub->_coordinates_valid == YES)
-		    (*invalidateImp)(sub, invalidateSel);
+		    {
+		      (*invalidateImp)(sub, invalidateSel);
+		    }
 		}
 	    }
 	}
@@ -2691,7 +2692,9 @@ static NSView* findByTag(NSView *view, int aTag, unsigned *level)
 - (NSAffineTransform*) _matrixFromWindow
 {
   if (_coordinates_valid == NO)
-    [self _rebuildCoordinates];
+    {
+      [self _rebuildCoordinates];
+    }
   return _matrixFromWindow;
 }
 
@@ -2705,7 +2708,9 @@ static NSView* findByTag(NSView *view, int aTag, unsigned *level)
 - (NSAffineTransform*) _matrixToWindow
 {
   if (_coordinates_valid == NO)
-    [self _rebuildCoordinates];
+    {
+      [self _rebuildCoordinates];
+    }
   return _matrixToWindow;
 }
 
@@ -2763,6 +2768,5 @@ static NSView* findByTag(NSView *view, int aTag, unsigned *level)
 	}
     }
 }
-
 @end
 
