@@ -50,6 +50,8 @@
 #define stringify_it(X) #X
 #define	mkpath(X) stringify_it(X) "/Tools"
 
+#define PosixExecutePermission	(0111)
+
 static NSString	*GSWorkspaceNotification = @"GSWorkspaceNotification";
 
 @interface	_GSWorkspaceCenter: NSNotificationCenter
@@ -74,10 +76,19 @@ static NSString	*GSWorkspaceNotification = @"GSWorkspaceNotification";
   if (self != nil)
     {
       remote = RETAIN([NSDistributedNotificationCenter defaultCenter]);
-      [remote addObserver: self
-		 selector: @selector(_handleRemoteNotification:)
-		     name: nil
-		   object: GSWorkspaceNotification];
+      NS_DURING
+        [remote addObserver: self
+		   selector: @selector(_handleRemoteNotification:)
+		       name: nil
+		     object: GSWorkspaceNotification];
+      NS_HANDLER
+        NSUserDefaults *defs = [NSUserDefaults standardUserDefaults];
+        if ([defs boolForKey: @"GSLogWorkspaceTimeout"])
+          NSLog(@"NSWorkspace caught exception %@: %@", 
+	        [localException name], [localException reason]);
+        else
+          [localException raise];
+      NS_ENDHANDLER
     }
   return self;
 }
@@ -94,7 +105,16 @@ static NSString	*GSWorkspaceNotification = @"GSWorkspaceNotification";
   rem = [NSNotification notificationWithName: [aNotification name]
 				      object: GSWorkspaceNotification
 				    userInfo: [aNotification userInfo]];
-  [remote postNotification: rem];
+  NS_DURING
+    [remote postNotification: rem];
+  NS_HANDLER
+    NSUserDefaults *defs = [NSUserDefaults standardUserDefaults];
+    if ([defs boolForKey: @"GSLogWorkspaceTimeout"])
+      NSLog(@"NSWorkspace caught exception %@: %@", 
+	    [localException name], [localException reason]);
+    else
+      [localException raise];
+  NS_ENDHANDLER
 }
 
 - (void) postNotificationName: (NSString*)name 
@@ -179,7 +199,7 @@ static NSString			*_rootPath = @"/";
       env = [[NSProcessInfo processInfo] environment];
       if (!env || !(home = [env objectForKey: @"GNUSTEP_USER_ROOT"]))
 	{
-	  home = [NSString stringWithFormat: @"%@/GNUstep", NSHomeDirectory()];
+	  home = [NSHomeDirectory() stringByAppendingPathComponent: @"GNUstep"];
 	}
 
       /*
@@ -446,14 +466,14 @@ extIconForApp(NSWorkspace *ws, NSString *appName, NSDictionary *typeInfo)
   if (role == nil || [role isEqualToString: @"Editor"])
     {
       appName = [prefs objectForKey: @"Editor"];
-      if (appName)
+      if (appName != nil)
 	{
 	  info = [apps objectForKey: appName];
-	  if (info)
+	  if (info != nil)
 	    {
-	      if (app)
+	      if (app != 0)
 		*app = appName;
-	      if (inf)
+	      if (inf != 0)
 		*inf = info;
 	      return YES;
 	    }
@@ -462,14 +482,14 @@ extIconForApp(NSWorkspace *ws, NSString *appName, NSDictionary *typeInfo)
   if (role == nil || [role isEqualToString: @"Viewer"])
     {
       appName = [prefs objectForKey: @"Viewer"];
-      if (appName)
+      if (appName != nil)
 	{
 	  info = [apps objectForKey: appName];
-	  if (info)
+	  if (info != nil)
 	    {
-	      if (app)
+	      if (app != 0)
 		*app = appName;
-	      if (inf)
+	      if (inf != 0)
 		*inf = info;
 	      return YES;
 	    }
@@ -501,17 +521,17 @@ extIconForApp(NSWorkspace *ws, NSString *appName, NSDictionary *typeInfo)
 	  str = [info objectForKey: @"NSRole"];
 	  if (str == nil || [str isEqualToString: @"Editor"])
 	    {
-	      if (app)
+	      if (app != 0)
 		*app = appName;
-	      if (inf)
+	      if (inf != 0)
 		*inf = info;
 	      return YES;
 	    }
 	  else if ([str isEqualToString: @"Viewer"])
 	    {
-	      if (app)
+	      if (app != 0)
 		*app = appName;
-	      if (inf)
+	      if (inf != 0)
 		*inf = info;
 	      found = YES;
 	    }
@@ -529,9 +549,9 @@ extIconForApp(NSWorkspace *ws, NSString *appName, NSDictionary *typeInfo)
 	  if ((str == nil && [role isEqualToString: @"Editor"])
 	    || [str isEqualToString: role])
 	    {
-	      if (app)
+	      if (app != 0)
 		*app = appName;
-	      if (inf)
+	      if (inf != 0)
 		*inf = info;
 	      return YES;
 	    }
@@ -736,7 +756,69 @@ inFileViewerRootedAtPath: (NSString *)rootFullpath
 	    application: (NSString **)appName
 		   type: (NSString **)type
 {
-  return NO;
+  NSFileManager *fm = [NSFileManager defaultManager];
+  NSDictionary *attributes;
+  NSString *fileType;
+  NSString *extension = [fullPath pathExtension];
+
+  attributes = [fm fileAttributesAtPath:fullPath traverseLink:YES];
+  if (attributes)
+    {
+      fileType = [attributes fileType];
+      if ([fileType isEqualToString: NSFileTypeRegular])
+        {
+          if ([attributes filePosixPermissions] & PosixExecutePermission)
+            {
+              *type = NSShellCommandFileType;
+              *appName = nil;
+            }
+          else
+            {
+              *type = NSPlainFileType;
+              *appName = [self getBestAppInRole:nil forExtension:extension];
+            }
+        }
+      else if([fileType isEqualToString: NSFileTypeDirectory])
+        {
+          if ([extension isEqualToString: @"app"]
+              || [extension isEqualToString: @"debug"]
+              || [extension isEqualToString: @"profile"])
+            {
+              *type = NSApplicationFileType;
+              *appName = nil;
+            }
+          else if ([extension isEqualToString: @"bundle"])
+            {
+              *type = NSPlainFileType;
+              *appName = nil;
+            }
+          // the idea here is that if the parent directory's fileSystemNumber
+          // differs, this must be a filesystem mount point
+          else if ([[fm fileAttributesAtPath:
+                       [fullPath stringByDeletingLastPathComponent]
+                                traverseLink:YES] fileSystemNumber]
+                     != [attributes fileSystemNumber])
+            {
+              *type = NSFilesystemFileType;
+              *appName = nil;
+            }
+          else
+            {
+              *type = NSDirectoryFileType;
+              *appName = nil;
+            }
+        }
+      else
+        {
+          // this catches sockets, character special, block special, and
+          // unknown file types
+          *type = NSPlainFileType;
+          *appName = nil;
+        }
+      return YES;
+    }
+  else
+    return NO;
 }
 
 - (NSImage *) iconForFile: (NSString *)aPath
@@ -1216,7 +1298,7 @@ inFileViewerRootedAtPath: (NSString *)rootFullpath
   NSData		*data;
 
   ext = [ext lowercaseString];
-  if (extPreferences)
+  if (extPreferences != nil)
     map = [extPreferences mutableCopy];
   else
     map = [NSMutableDictionary new];
@@ -1252,7 +1334,7 @@ inFileViewerRootedAtPath: (NSString *)rootFullpath
   [map setObject: inf forKey: ext];
   RELEASE(inf);
   RELEASE(extPreferences);
-  extPreferences = inf;
+  extPreferences = map;
   data = [NSSerializer serializePropertyList: extPreferences];
   [data writeToFile: extPrefPath atomically: YES];
 }
@@ -1264,7 +1346,7 @@ inFileViewerRootedAtPath: (NSString *)rootFullpath
   NSData		*data;
 
   ext = [ext lowercaseString];
-  if (extPreferences)
+  if (extPreferences != nil)
     map = [extPreferences mutableCopy];
   else
     map = [NSMutableDictionary new];
@@ -1279,7 +1361,7 @@ inFileViewerRootedAtPath: (NSString *)rootFullpath
   [map setObject: inf forKey: ext];
   RELEASE(inf);
   RELEASE(extPreferences);
-  extPreferences = inf;
+  extPreferences = map;
   data = [NSSerializer serializePropertyList: extPreferences];
   [data writeToFile: extPrefPath atomically: YES];
 }
