@@ -27,9 +27,22 @@
 #ifndef _rtfConsumer_h_INCLUDE
 #define _rtfConsumer_h_INCLUDE
 
-#include	"Parsers/rtfScanner.h"
+@interface RTFConsumer: NSObject
+{
+@public
+  NSMutableDictionary *documentAttributes;
+  NSMutableDictionary *fonts;
+  NSMutableArray *colours;
+  NSMutableArray *attrs;
+  NSMutableAttributedString *result;
+  int ignore;
+}
 
-/*	external symbols from the grammer	*/
-int	GSRTFparse(void *ctxt, RTFscannerCtxt *lctxt);
++ (NSAttributedString*) parseRTF: (NSData *)rtfData 
+	      documentAttributes: (NSDictionary **)dict;
++ (NSAttributedString*) parseRTFD: (NSFileWrapper *)rtfFile 
+	       documentAttributes: (NSDictionary **)dict;
+
+@end
 
 #endif
