@@ -227,11 +227,16 @@ init(int argc, char** argv)
 }
 
 int
-main(int argc, char** argv)
+main(int argc, char** argv, char **env)
 {
-  NSAutoreleasePool *pool = [NSAutoreleasePool new];
-  ExampleServices *server = [ExampleServices new];
+  NSAutoreleasePool *pool;
+  ExampleServices *server;
 
+#ifdef GS_PASS_ARGUMENTS
+  [NSProcessInfo initializeWithArguments:argv count:argc environment:env];
+#endif
+  pool = [NSAutoreleasePool new];
+  server = [ExampleServices new];
   init(argc, argv);
 
   // [NSObject enableDoubleReleaseCheck: YES];
