@@ -3,27 +3,23 @@
 
    GNUstep pasteboard server
 
-   Copyright (C) 1997 Free Software Foundation, Inc.
+   Copyright (C) 1997,1999 Free Software Foundation, Inc.
 
    Author:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
    Date: August 1997
    
-   This file is part of the GNUstep GUI Library.
+   This file is part of the GNUstep Project
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-   
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+    
+   You should have received a copy of the GNU General Public  
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
 */ 
 
 #include <gnustep/gui/config.h>
@@ -45,7 +41,7 @@
 #include <Foundation/NSObjCRuntime.h>
 #include <AppKit/NSPasteboard.h>
 
-#include "PasteboardServer.h"
+#include <gnustep/gui/GSPasteboardServer.h>
 
 #include <signal.h>
 
@@ -56,10 +52,10 @@ int verbose = 0;
 
 #define MAXHIST 100
 
-PasteboardServer    *server = nil;
-NSConnection	    *conn = nil;
-NSLock		*dictionary_lock = nil;
-NSMutableDictionary*	pasteboards = nil;
+PasteboardServer	*server = nil;
+NSConnection		*conn = nil;
+NSLock			*dictionary_lock = nil;
+NSMutableDictionary	*pasteboards = nil;
 
 @interface PasteboardData: NSObject
 {
@@ -77,7 +73,7 @@ NSMutableDictionary*	pasteboards = nil;
 	      wantsChangedOwner: (BOOL)wants
 	      hasStdDataForType: (BOOL)StdData
 	      hasGNUDataForType: (BOOL)GNUData;
-- (void) checkConnection: (NSConnection*)c;
+- (BOOL) checkConnection: (NSConnection*)c;
 - (NSData*) data;
 - (NSData*) newDataWithVersion: (int)version;
 - (id) owner;
@@ -110,9 +106,10 @@ NSMutableDictionary*	pasteboards = nil;
   return d;
 }
 
-- (void) checkConnection: (NSConnection*)c
+- (BOOL) checkConnection: (NSConnection*)c
 {
-  id o;
+  BOOL	ourConnection = NO;
+  id	o;
 
   if (owner && [owner isProxy] && [owner connectionForProxy] == c)
     {
@@ -122,6 +119,7 @@ NSMutableDictionary*	pasteboards = nil;
       o = pboard;
       pboard = nil;
       [o release];
+      ourConnection = YES;
     }
   if (pboard && [pboard isProxy] && [pboard connectionForProxy] == c)
     {
@@ -131,7 +129,9 @@ NSMutableDictionary*	pasteboards = nil;
       o = pboard;
       pboard = nil;
       [o release];
+      ourConnection = YES;
     }
+  return ourConnection;
 }
 
 - (void) dealloc
@@ -143,11 +143,16 @@ NSMutableDictionary*	pasteboards = nil;
   [super dealloc];
 }
 
+- (NSString*) description
+{
+  return [NSString stringWithFormat: @"PasteboardData for type '%@'", type];
+}
+
 - (NSData*) data
 {
   if (verbose)
     {
-      printf("get data for %x\n", (unsigned)self);
+      NSLog(@"get data for %x\n", (unsigned)self);
     }
   return data;
 }
@@ -185,7 +190,7 @@ NSMutableDictionary*	pasteboards = nil;
 {
   if (verbose)
     {
-      printf("set data for %x\n", (unsigned)self);
+      NSLog(@"set data for %x\n", (unsigned)self);
     }
   [d retain];
   [data release];
@@ -222,7 +227,7 @@ NSMutableDictionary*	pasteboards = nil;
 			   pboard: (id)aPboard
 			      ref: (int)count;
 - (void) addTypes: (NSArray*)types owner: (id)owner pasteboard: (id)pb;
-- (void) checkConnection: (NSConnection*)c;
+- (BOOL) checkConnection: (NSConnection*)c;
 - (BOOL) hasBeenFiltered;
 - (PasteboardData*) itemForType: (NSString*)type;
 - (void) lostOwnership;
@@ -262,11 +267,11 @@ NSMutableDictionary*	pasteboards = nil;
 	  e->hasGNUDataForType = YES;
 	}
 
-      e->items = [[NSMutableArray alloc] initWithCapacity:[someTypes count]];
+      e->items = [[NSMutableArray alloc] initWithCapacity: [someTypes count]];
       for (i = 0; i < [someTypes count]; i++)
 	{
-	  NSString*	type = [someTypes objectAtIndex:i];
-	  PasteboardData*   d;
+	  NSString		*type = [someTypes objectAtIndex: i];
+	  PasteboardData	*d;
 
 	  d = [PasteboardData newWithType: type
 				    owner: anOwner
@@ -274,10 +279,14 @@ NSMutableDictionary*	pasteboards = nil;
 			wantsChangedOwner: e->wantsChangedOwner
 			hasStdDataForType: e->hasStdDataForType
 			hasGNUDataForType: e->hasGNUDataForType];
-	  [e->items addObject:d];
+	  [e->items addObject: d];
 	  [d release];
 	}
       e->refNum = count;
+      if (verbose > 1)
+	{
+	  NSLog(@"New PasteboardEntry %d with items - %@", count, e->items);
+	}
     }
   return e;
 }
@@ -307,9 +316,9 @@ NSMutableDictionary*	pasteboards = nil;
 
   for (i = 0; i < [newTypes count]; i++)
     {
-      NSString*	  type = (NSString*)[newTypes objectAtIndex:i];
+      NSString	*type = (NSString*)[newTypes objectAtIndex: i];
 
-      if ([self itemForType:type] == nil)
+      if ([self itemForType: type] == nil)
 	{
 	  PasteboardData*   d;
 
@@ -319,16 +328,21 @@ NSMutableDictionary*	pasteboards = nil;
 			wantsChangedOwner: wants
 			hasStdDataForType: StdData
 			hasGNUDataForType: GNUData];
-	  [items addObject:d];
+	  [items addObject: d];
 	  [d release];
 	}
     }
+  if (verbose > 1)
+    {
+      NSLog(@"Modified PasteboardEntry %d with items - %@", refNum, items);
+    }
 }
 
-- (void) checkConnection: (NSConnection*)c
+- (BOOL) checkConnection: (NSConnection*)c
 {
-  unsigned int i;
-  id o;
+  BOOL		ourConnection = NO;
+  unsigned	i;
+  id		o;
 
   if (owner && [owner isProxy] && [owner connectionForProxy] == c)
     {
@@ -338,7 +352,9 @@ NSMutableDictionary*	pasteboards = nil;
       o = pboard;
       pboard = nil;
       [o release];
+      ourConnection = YES;
     }
+
   if (pboard && [pboard isProxy] && [pboard connectionForProxy] == c)
     {
       o = owner;
@@ -347,18 +363,23 @@ NSMutableDictionary*	pasteboards = nil;
       o = pboard;
       pboard = nil;
       [o release];
+      ourConnection = YES;
     }
 
   for (i = [items count]; i > 0; i--)
     {
-      PasteboardData* d = [items objectAtIndex: i-1];
+      PasteboardData	*d = [items objectAtIndex: i-1];
 
-      [d checkConnection: c];
-      if ([d data] == nil && [d owner] == nil)
+      if ([d checkConnection: c] == YES && [d data] == nil && [d owner] == nil)
 	{
-	  [items removeObjectAtIndex:i-1];
+	  if (verbose > 1)
+	    {
+	      NSLog(@"Removing item from PasteboardEntry %d\n", refNum);
+	    }
+	  [items removeObjectAtIndex: i-1];
 	}
     }
+  return ourConnection;
 }
 
 - (void) dealloc
@@ -376,11 +397,12 @@ NSMutableDictionary*	pasteboards = nil;
 
 - (PasteboardData*) itemForType: (NSString*)type
 {
-  unsigned i;
+  unsigned i, count;
 
-  for (i = 0; i < [items count]; i++)
+  count = [items count];
+  for (i = 0; i < count; i++)
     {
-      PasteboardData* d = [items objectAtIndex:i];
+      PasteboardData	*d = [items objectAtIndex: i];
 
       if ([[d type] isEqual: type])
 	{
@@ -460,7 +482,7 @@ NSMutableDictionary*	pasteboards = nil;
 
 
 
-@interface PasteboardObject: NSObject <PasteboardObject>
+@interface PasteboardObject: NSObject <GSPasteboardObj>
 {
   NSString	*name;
   int		nextCount;
@@ -478,13 +500,13 @@ NSMutableDictionary*	pasteboards = nil;
 - (NSString*) availableTypeFromArray: (NSArray*)types
 	     changeCount: (int*)count;
 - (int) changeCount;
-- (void) checkConnection: (NSConnection*)c;
+- (BOOL) checkConnection: (NSConnection*)c;
 - (NSData*) dataForType: (NSString*)type
 	   oldCount: (int)count
       mustBeCurrent: (BOOL)flag;
 - (int) declareTypes: (NSArray*)types
 	       owner: (id)owner
-      pasteboard: (id)pboard;
+	  pasteboard: (id)pboard;
 - (PasteboardEntry*) entryByCount: (int)count;
 - (NSString*) name;
 - (void) releaseGlobally;
@@ -595,14 +617,19 @@ NSMutableDictionary*	pasteboards = nil;
   return 0;
 }
 
-- (void) checkConnection: (NSConnection*)c
+- (BOOL) checkConnection: (NSConnection*)c
 {
-  unsigned int i;
+  unsigned	i;
+  BOOL		found = NO;
 
   for (i = 0; i < [history count]; i++)
     {
-      [[history objectAtIndex: i] checkConnection: c];
+      if ([[history objectAtIndex: i] checkConnection: c] == YES)
+	{
+	  found = YES;
+	}
     }
+  return found;
 }
 
 - (NSData*) dataForType: (NSString*)type
@@ -621,12 +648,11 @@ NSMutableDictionary*	pasteboards = nil;
     }
   if (verbose)
     {
-      printf("get data for type '%s' version %d\n",
-	[type cString], e ? [e refNum] : -1);
+      NSLog(@"get data for type '%@' version %d\n", type, e ? [e refNum] : -1);
     }
   if (e)
     {
-      PasteboardData* d = [e itemForType: type];
+      PasteboardData	*d = [e itemForType: type];
 
       if (d)
 	{
@@ -647,17 +673,17 @@ NSMutableDictionary*	pasteboards = nil;
 	       owner: (id)owner
 	  pasteboard: (NSPasteboard*)pb
 {
-  PasteboardEntry*  old = [current retain];
+  PasteboardEntry	*old = [current retain];
 
-  current = [PasteboardEntry newWithTypes:types
-				    owner:owner
-				   pboard:pb
-				      ref:nextCount++];
-  [history addObject:current];
+  current = [PasteboardEntry newWithTypes: types
+				    owner: owner
+				   pboard: pb
+				      ref: nextCount++];
+  [history addObject: current];
   [current release];
   if ([history count] > histLength)
     {
-      [history removeObjectAtIndex:0];
+      [history removeObjectAtIndex: 0];
     }
   [old lostOwnership];
   [old release];
@@ -709,17 +735,16 @@ NSMutableDictionary*	pasteboards = nil;
 	  isFile: (BOOL)flag
 	oldCount: (int)count
 {
-  PasteboardEntry*  e;
+  PasteboardEntry	*e;
 
   if (verbose)
     {
-      printf("set data for type '%s' version %d\n",
-	[type cString], count);
+      NSLog(@"set data for type '%@' version %d\n", type, count);
     }
   e = [self entryByCount: count];
   if (e)
     {
-      PasteboardData* d;
+      PasteboardData	*d;
 
       if (flag)
 	{
@@ -811,35 +836,38 @@ NSMutableDictionary*	pasteboards = nil;
 
 
 
-@interface PasteboardServer : NSObject <PasteboardServer>
+@interface PasteboardServer : NSObject <GSPasteboardSvr>
 {
   NSMutableArray*   permenant;
 }
-- (NSConnection*) connection: ancestor didConnect: newConn;
-- connectionBecameInvalid: notification;
+- (BOOL) connection: (NSConnection*)ancestor
+  shouldMakeNewConnection: (NSConnection*)newConn;
+- (id) connectionBecameInvalid: (NSNotification*)notification;
 
-- (id<PasteboardObject>) pasteboardByFilteringData: (NSData*)data
-					    ofType: (NSString*)type
-					    isFile: (BOOL)flag;
-- (id<PasteboardObject>) pasteboardByFilteringTypesInPasteboard: pb;
-- (id<PasteboardObject>) pasteboardWithName: (NSString*)name;
-- (id<PasteboardObject>) pasteboardWithUniqueName;
+- (id<GSPasteboardObj>) pasteboardByFilteringData: (NSData*)data
+					   ofType: (NSString*)type
+					   isFile: (BOOL)flag;
+- (id<GSPasteboardObj>) pasteboardByFilteringTypesInPasteboard: pb;
+- (id<GSPasteboardObj>) pasteboardWithName: (NSString*)name;
+- (id<GSPasteboardObj>) pasteboardWithUniqueName;
 - (NSArray*) typesFilterableTo: (NSString*)type;
 @end
 
 @implementation PasteboardServer
 
-- (NSConnection*) connection: ancestor didConnect: newConn
+- (BOOL) connection: (NSConnection*)ancestor
+  shouldMakeNewConnection: (NSConnection*)newConn
 {
-  [NotificationDispatcher addObserver: self
-			     selector: @selector(connectionBecameInvalid:)
-				 name: NSConnectionDidDieNotification
-			       object: newConn];
+  [[NSNotificationCenter defaultCenter]
+    addObserver: self
+       selector: @selector(connectionBecameInvalid:)
+	   name: NSConnectionDidDieNotification
+	 object: newConn];
   [newConn setDelegate: self];
-  return newConn;
+  return YES;
 }
 
-- connectionBecameInvalid: notification
+- (id) connectionBecameInvalid: (NSNotification*)notification
 {
   id connection = [notification object];
 
@@ -861,13 +889,13 @@ NSMutableDictionary*	pasteboards = nil;
   return self;
 }
 
-- (void)dealloc
+- (void) dealloc
 {
   [permenant release];
   [super dealloc];
 }
 
-- init
+- (id) init
 {
   self = [super init];
   if (self)
@@ -886,26 +914,26 @@ NSMutableDictionary*	pasteboards = nil;
   return self;
 }
 
-- (id<PasteboardObject>) pasteboardByFilteringData: (NSData*)data
-					    ofType: (NSString*)type
-					    isFile: (BOOL)flag
+- (id<GSPasteboardObj>) pasteboardByFilteringData: (NSData*)data
+					   ofType: (NSString*)type
+					   isFile: (BOOL)flag
 {
   [self notImplemented: _cmd];
   return nil;
 }
 
-- (id<PasteboardObject>) pasteboardByFilteringTypesInPasteboard: pb
+- (id<GSPasteboardObj>) pasteboardByFilteringTypesInPasteboard: pb
 {
   [self notImplemented: _cmd];
   return nil;
 }
 
-- (id<PasteboardObject>) pasteboardWithName: (NSString*)name
+- (id<GSPasteboardObj>) pasteboardWithName: (NSString*)name
 {
   return [PasteboardObject pasteboardWithName: name];
 }
 
-- (id<PasteboardObject>) pasteboardWithUniqueName
+- (id<GSPasteboardObj>) pasteboardWithUniqueName
 {
   return [PasteboardObject pasteboardWithName: nil];
 }
@@ -923,6 +951,7 @@ NSMutableDictionary*	pasteboards = nil;
 static int
 ihandler(int sig)
 {
+  signal(sig, SIG_DFL);
   abort();
 }
 
@@ -1003,10 +1032,14 @@ init(int argc, char** argv)
 
 
 int
-main(int argc, char** argv)
+main(int argc, char** argv, char **env)
 {
-  NSAutoreleasePool	*pool = [NSAutoreleasePool new];
+  NSAutoreleasePool	*pool;
 
+#ifdef GS_PASS_ARGUMENTS
+  [NSProcessInfo initializeWithArguments:argv count:argc environment:env];
+#endif
+  pool = [NSAutoreleasePool new];
   init(argc, argv);
 
   // [NSObject enableDoubleReleaseCheck: YES];
@@ -1020,20 +1053,20 @@ main(int argc, char** argv)
     }
 
   /* Register a connection that provides the server object to the network */
-  conn = [NSConnection newRegisteringAtName:PBSNAME
-		 withRootObject:server];
-  
-  if (conn == nil)
+  conn = [NSConnection defaultConnection];
+  [conn setRootObject: server];
+  if ([conn registerName: PBSNAME] == NO)
     {
       NSLog(@"Unable to register with name server.\n");
       exit(1);
     }
 
-  [conn setDelegate:server];
-  [NotificationDispatcher addObserver: server
-		 selector: @selector(connectionBecameInvalid:)
-		 name: NSConnectionDidDieNotification
-		   object: conn];
+  [conn setDelegate: server];
+  [[NSNotificationCenter defaultCenter]
+    addObserver: server
+       selector: @selector(connectionBecameInvalid:)
+	   name: NSConnectionDidDieNotification
+	 object: conn];
 
   if (verbose)
     {
