@@ -30,7 +30,7 @@
 
 
 int
-main(int argc, char** argv)
+main(int argc, char** argv, char **env)
 {
   NSAutoreleasePool	*pool;
   NSProcessInfo		*proc;
@@ -38,6 +38,9 @@ main(int argc, char** argv)
   unsigned		index;
 
   // [NSObject enableDoubleReleaseCheck: YES];
+#ifdef GS_PASS_ARGUMENTS
+  [NSProcessInfo initializeWithArguments:argv count:argc environment:env];
+#endif
 
   pool = [NSAutoreleasePool new];
 
