@@ -1,3 +1,4 @@
+#!/bin/sh
 #
 #   clean_os.sh
 #
@@ -20,4 +21,14 @@
 #   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 # Nothing to clean right now
-echo $1
+case "$1" in
+    # Remove version number for FreeBSD
+    freebsdelf*)
+	echo freebsdelf
+	exit 0
+	;;
+    *)
+	echo $1
+        exit 0
+	;;
+esac
