@@ -32,6 +32,8 @@
 #include <fcntl.h>
 #include "config.h"
 
+#include <DPS/PSres.h>
+
 #include <Foundation/NSDictionary.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSFileManager.h>
@@ -40,7 +42,9 @@
 #include <Foundation/NSDate.h>
 
 #include <AppKit/NSFont.h>
+#include <gnustep/xdps/NSDPSContext.h>
 #include "AFMFileFontInfo.h"
+#include "general.h"
 #include "fonts.h"
 
 /* This dictionary keeps global font info objects indexed by AFM file names.
@@ -50,6 +54,8 @@
    scaled to the font size. */
 static NSMutableDictionary* globalFontInfoDictionary = nil;
 
+static NSMutableDictionary* globalFileDictionary = nil;
+
 #ifndef LIB_FOUNDATION_LIBRARY
 /* Define callbacks for a map table that has cStrings as keys */
 
@@ -98,6 +104,14 @@ static const NSMapTableKeyCallBacks NSNonOwnedCStringMapKeyCallBacks = {
 }; 
 #endif /* LIB_FOUNDATION_LIBRARY */
 
+static int
+afmEnumerator (char* resourceType, char* resourceName, char* resourceFile,
+             char* fontsList)
+{
+  NSString* afmFileName = [NSString stringWithCString: resourceFile];
+  [(NSMutableArray*)fontsList addObject: afmFileName];
+  return 0;
+}
 
 @implementation AFMFileFontInfo
 
@@ -108,79 +122,107 @@ static const NSMapTableKeyCallBacks NSNonOwnedCStringMapKeyCallBacks = {
   if (!initialized) {
     initialized = YES;
     globalFontInfoDictionary = [NSMutableDictionary new];
+    globalFileDictionary = [NSMutableDictionary new];
   }
 }
 
-+ (AFMFileFontInfo*)fontInfoForFont:(NSFont*)font
-  afmFilename:(NSString*)afmFileName
+- (NSString *) _getFontResource
 {
-  AFMFileFontInfo* fontInfo;
-  const float* matrix = [font matrix];
-
-  /* Check to see if the font was previously requested. */
-  if ((fontInfo = [globalFontInfoDictionary objectForKey:afmFileName]))
-    return [fontInfo newTransformedFontInfoForMatrix:matrix];
-
-  /* The font was not previously requested. Parse the AFM file and create a
-     new font info for it. */
-  fontInfo = [self createFontInfoFromAFMFileName:afmFileName];
-
-  if (!fontInfo)
-    return nil;
-
-  /* Record into the global font info dictionary that we've parsed the AFM
-     file. */
-  [globalFontInfoDictionary setObject:fontInfo forKey:afmFileName];
-
-  return [fontInfo newTransformedFontInfoForMatrix:matrix];
+  NSString* fontFile;
+  DPSContext ctxt;
+  char font_file[FILENAME_MAX + 1];
+  
+  ctxt = [(NSDPSContext *)GSCurrentContext() xDPSContext];
+  
+  
+  fontFile = nil;
+  PSWGetFontFile(ctxt, [fontName cString], font_file);
+  if (strncmp(font_file, "/Resource", 9) == 0)
+    {
+      char completePath[FILENAME_MAX + 1];
+      int found;
+      /* We're using GS which returns files differently */
+      PSWGSFontFile(ctxt, [fontName cString], font_file);
+      PSWCompleteFilename (font_file, &found, completePath);
+      if (found) 
+	{
+	  fontFile = [NSString stringWithCString: completePath];
+	  fontFile = [[fontFile stringByDeletingPathExtension]
+		       stringByAppendingPathExtension:@"afm"];
+	}
+    }
+  else if (font_file[0] != '/')
+    {
+      /* Can't handle this yet, try the Adobe way... */
+      NSMutableArray *fontsList = [[NSMutableArray new] autorelease];
+      NSDebugLLog(@"Fonts", @"Font file resource returned relative-path %s", 
+		  font_file);
+      EnumeratePSResourceFiles (NULL, NULL, PSResFontAFM, [fontName cString],
+				afmEnumerator, (char*)fontsList);
+      if (![fontsList count])
+	NSLog (@"WARNING: Font file not found! Check if the PSRESOURCEPATH "
+	       @"environment variable points to the right directory. If so then "
+	       @"probably you did not build the PS resource database using the "
+	       @"`makepsres' utility.");
+      else
+	fontFile = [fontsList objectAtIndex: 0];
+    }
+     
+  if (fontFile)
+    NSDebugLLog(@"Fonts", @"Found AFM file %@ for font %@", fontFile, fontName);
+  return fontFile;
 }
 
-+ (AFMFileFontInfo*)createFontInfoFromAFMFileName:(NSString*)fileName
+- _setFontInfo
 {
-  AFMFileFontInfo* fontInfo = [[self new] autorelease];
-  NSMutableDictionary* afmDict = fontInfo->afmDictionary;
-  NSAutoreleasePool* pool = nil;
+  NSAutoreleasePool* pool;
   AFMFontInfo* cFontInfo;
   AFMGlobalFontInfo* gfi;
-  const char* fileNameCString;
   int fontType;
   FILE* file;
   int i, code;
 
-/* Do this until gnustep-base will contain the NSFileManager class. Not very
-   accurate since NeXT Foundation library does have NSFileManager. On NT or
-   Win95 probably the 'else' code should be modified such that the cstring
-   passed to access() contains \ instead of / (in case of NSFileManager this
-   is hidden inside its implementation). */
-#if defined(LIB_FOUNDATION_LIBRARY) || defined(NeXT_PDO)
-  if (![[NSFileManager defaultManager] isReadableFileAtPath:fileName])
-    return nil;
-#else
-  if (access([fileName cString], R_OK) < 0)
-    return nil;
-#endif
+  afmFileName = [globalFileDictionary objectForKey: fontName];
+  if (afmFileName == nil)
+    {
+      afmFileName = [self _getFontResource];
+      if (afmFileName == nil)
+	{
+	  NSLog (@"No AFM file for font %@. Can't get font info", fontName);
+	  return nil;
+	}
+      [globalFileDictionary setObject: afmFileName forKey: fontName];
+    }
+  RETAIN(afmFileName);
 
-  fileNameCString = [fileName cString];
-  file = fopen (fileNameCString, "r");
-  if (!file) {
-    NSLog (@"cannot open AFM file %@", fileName);
-    abort ();
-  }
+  if (![[NSFileManager defaultManager] isReadableFileAtPath: afmFileName])
+    {
+      NSLog (@"Cannot read AFM file %@ for font %@", afmFileName, fontName);
+      return nil;
+    }
+
+  file = fopen ([afmFileName cString], "r");
+  if (!file) 
+    {
+      NSLog (@"Cannot open AFM file %@ for font %@", afmFileName, fontName);
+      return nil;
+    }
 
   code = AFMParseFile (file, &cFontInfo, AFM_G | AFM_M | AFM_P);
   if (code != afm_ok) {
     switch (code) {
       case afm_parseError:
-	NSLog (@"parse error in AFM file %@", fileName);
+	NSLog (@"parse error in AFM file %@", afmFileName);
 	break;
       case afm_earlyEOF:
-	NSLog (@"unexpected EOF in AFM file %@", fileName);
+	NSLog (@"unexpected EOF in AFM file %@", afmFileName);
 	break;
       case afm_storageProblem:
 	NSLog (@"memory allocation problem while parsing the AFM file %@",
-		fileName);
+		afmFileName);
 	break;
     }
+    fclose (file);
     return nil;
   }
   fclose (file);
@@ -193,97 +235,96 @@ static const NSMapTableKeyCallBacks NSNonOwnedCStringMapKeyCallBacks = {
    * Set the font information in instance variables and in the AFM dictionary.
    */
 
-  fontInfo->fontName = [[NSString stringWithCString:gfi->fontName] retain];
-  [afmDict setObject:fontInfo->fontName forKey:NSAFMFontName];
+  /* The font may actually be a GS alias, so do not set the fontName
+   * field. */
+  //  fontName = [[NSString stringWithCString:gfi->fontName] retain];
+  [fontDictionary setObject: fontName forKey: NSAFMFontName];
 
-  fontInfo->familyName = [[NSString stringWithCString:gfi->familyName] retain];
-  [afmDict setObject:fontInfo->familyName forKey:NSAFMFamilyName];
+  familyName = [[NSString stringWithCString:gfi->familyName] retain];
+  [fontDictionary setObject: familyName forKey: NSAFMFamilyName];
 
-  fontInfo->italicAngle = gfi->italicAngle;
-  [afmDict setObject:[[NSNumber numberWithFloat:gfi->italicAngle] stringValue]
-	   forKey:NSAFMItalicAngle];
+  italicAngle = gfi->italicAngle;
+  [fontDictionary setObject: [NSNumber numberWithFloat:gfi->italicAngle]
+	      forKey: NSAFMItalicAngle];
 
-  fontInfo->weight = [[NSString stringWithCString:gfi->weight] retain];
-  [afmDict setObject:fontInfo->weight forKey:NSAFMWeight];
+  weight = [[NSString stringWithCString:gfi->weight] retain];
+  [fontDictionary setObject: weight forKey: NSAFMWeight];
 
-  fontInfo->underlinePosition = ((float)gfi->underlinePosition) / 1000;
-  [afmDict setObject:[[NSNumber numberWithFloat:fontInfo->underlinePosition]
-				stringValue]
-	   forKey:NSAFMUnderlinePosition];
+  underlinePosition = ((float)gfi->underlinePosition) / 1000;
+  [fontDictionary setObject: [NSNumber numberWithFloat:underlinePosition]
+	      forKey: NSAFMUnderlinePosition];
 
-  fontInfo->underlineThickness = ((float)gfi->underlineThickness) / 1000;
-  [afmDict setObject:[[NSNumber numberWithFloat:fontInfo->underlineThickness]
-				stringValue]
-	   forKey:NSAFMUnderlineThickness];
+  underlineThickness = ((float)gfi->underlineThickness) / 1000;
+  [fontDictionary setObject: [NSNumber numberWithFloat:underlineThickness]
+	      forKey: NSAFMUnderlineThickness];
 
-  fontInfo->capHeight = ((float)gfi->capHeight) / 1000;
-  [afmDict setObject:[[NSNumber numberWithFloat:fontInfo->capHeight]
-				stringValue]
-	   forKey:NSAFMCapHeight];
+  capHeight = ((float)gfi->capHeight) / 1000;
+  [fontDictionary setObject: [NSNumber numberWithFloat:capHeight]
+	      forKey: NSAFMCapHeight];
 
-  fontInfo->xHeight = ((float)gfi->xHeight) / 1000;
-  [afmDict setObject:[[NSNumber numberWithFloat:fontInfo->xHeight] stringValue]
-	   forKey:NSAFMXHeight];
+  xHeight = ((float)gfi->xHeight) / 1000;
+  [fontDictionary setObject: [NSNumber numberWithFloat:xHeight]
+	      forKey: NSAFMXHeight];
 
-  fontInfo->descender = ((float)gfi->descender) / 1000;
-  [afmDict setObject:[[NSNumber numberWithFloat:fontInfo->descender]
-				stringValue]
-	   forKey:NSAFMDescender];
+  descender = ((float)gfi->descender) / 1000;
+  [fontDictionary setObject: [NSNumber numberWithFloat:descender]
+	      forKey: NSAFMDescender];
 
-  fontInfo->ascender = ((float)gfi->ascender) / 1000;
-  [afmDict setObject:[[NSNumber numberWithFloat:fontInfo->ascender]
-				stringValue]
-	   forKey:NSAFMAscender];
+  ascender = ((float)gfi->ascender) / 1000;
+  [fontDictionary setObject: [NSNumber numberWithFloat:ascender]
+	      forKey: NSAFMAscender];
 
-  fontInfo->encodingScheme = [[NSString stringWithCString:gfi->encodingScheme]
+  encodingScheme = [[NSString stringWithCString:gfi->encodingScheme]
 					retain];
-  [afmDict setObject:fontInfo->encodingScheme forKey:NSAFMEncodingScheme];
+  [fontDictionary setObject: encodingScheme forKey: NSAFMEncodingScheme];
 
-  [afmDict setObject:[NSString stringWithCString:gfi->afmVersion]
-	   forKey:NSAFMFormatVersion];
-  [afmDict setObject:[NSString stringWithCString:gfi->notice]
-	   forKey:NSAFMNotice];
-  [afmDict setObject:[NSString stringWithCString:gfi->version]
-	   forKey:NSAFMVersion];
+  [fontDictionary setObject: [NSString stringWithCString:gfi->afmVersion]
+	      forKey: NSAFMFormatVersion];
+  [fontDictionary setObject:[NSString stringWithCString:gfi->notice]
+	      forKey: NSAFMNotice];
+  [fontDictionary setObject:[NSString stringWithCString:gfi->version]
+	      forKey: NSAFMVersion];
 
   /* Setup bbox as expected by NSFont */
-  fontInfo->fontBBox.origin.x = ((float)gfi->fontBBox.llx) / 1000;
-  fontInfo->fontBBox.origin.y = ((float)gfi->fontBBox.lly) / 1000;
-  fontInfo->fontBBox.size.width
+  fontBBox.origin.x = ((float)gfi->fontBBox.llx) / 1000;
+  fontBBox.origin.y = ((float)gfi->fontBBox.lly) / 1000;
+  fontBBox.size.width
       = ((float)(gfi->fontBBox.urx - gfi->fontBBox.llx)) / 1000;
-  fontInfo->fontBBox.size.height
+  fontBBox.size.height
       = ((float)(gfi->fontBBox.ury - gfi->fontBBox.lly)) / 1000;
 
-  fontInfo->isFixedPitch = gfi->isFixedPitch;
+  isFixedPitch = gfi->isFixedPitch;
 
   /* Get the font type from the DGS server */
-  PSWGetFontType ([fontInfo->fontName cString], &fontType);
-  fontInfo->isBaseFont = (fontType == 1);
+  PSWGetFontType ([fontName cString], &fontType);
+  isBaseFont = (fontType != 0);
 
-  fontInfo->maximumAdvancement.width = FLT_MIN;
-  fontInfo->maximumAdvancement.height = FLT_MIN;
-  fontInfo->minimumAdvancement.width = FLT_MAX;
-  fontInfo->minimumAdvancement.height = FLT_MAX;
+  maximumAdvancement.width = FLT_MIN;
+  maximumAdvancement.height = FLT_MIN;
+  minimumAdvancement.width = FLT_MAX;
+  minimumAdvancement.height = FLT_MAX;
 
   /* Fill in the widths and glyphs arrays. */
+  glyphsByName = NSCreateMapTable (NSNonOwnedCStringMapKeyCallBacks,
+				   NSObjectMapValueCallBacks, 256);
   for (i = 0; i < cFontInfo->numOfChars; i++) {
     AFMCharMetricInfo charMetricInfo = cFontInfo->cmi[i];
     AFMGlyphInfo* glyph
 	= [AFMGlyphInfo glyphFromAFMCharMetricInfo:&charMetricInfo];
     NSSize glyphAdvancement = [glyph advancement];
 
-    NSMapInsert (fontInfo->glyphsByName, [[glyph name] cString], glyph);
-    fontInfo->maximumAdvancement.width
-	= MAX (fontInfo->maximumAdvancement.width, glyphAdvancement.width);
-    fontInfo->maximumAdvancement.height
-	= MAX (fontInfo->maximumAdvancement.height, glyphAdvancement.height);
-    fontInfo->minimumAdvancement.width
-	= MIN (fontInfo->minimumAdvancement.width, glyphAdvancement.width);
-    fontInfo->minimumAdvancement.height
-	= MIN (fontInfo->minimumAdvancement.height, glyphAdvancement.height);
+    NSMapInsert (glyphsByName, [[glyph name] cString], glyph);
+    maximumAdvancement.width
+	= MAX (maximumAdvancement.width, glyphAdvancement.width);
+    maximumAdvancement.height
+	= MAX (maximumAdvancement.height, glyphAdvancement.height);
+    minimumAdvancement.width
+	= MIN (minimumAdvancement.width, glyphAdvancement.width);
+    minimumAdvancement.height
+	= MIN (minimumAdvancement.height, glyphAdvancement.height);
     if (charMetricInfo.code > 0) {
-      fontInfo->widths[charMetricInfo.code] = charMetricInfo.wx;
-      fontInfo->glyphs[charMetricInfo.code] = [glyph retain];
+      widths[charMetricInfo.code] = charMetricInfo.wx;
+      glyphs[charMetricInfo.code] = [glyph retain];
     }
   }
 
@@ -305,7 +346,7 @@ static const NSMapTableKeyCallBacks NSNonOwnedCStringMapKeyCallBacks = {
          grouped on the first glyph. */
       if (strcmp (pkd.name1, previousGlyphName)) {
 	previousGlyphName = pkd.name1;
-	glyph1 = NSMapGet (fontInfo->glyphsByName, pkd.name1);
+	glyph1 = NSMapGet (glyphsByName, pkd.name1);
       }
       [glyph1 incrementNumberOfKernPairs];
     }
@@ -316,17 +357,17 @@ static const NSMapTableKeyCallBacks NSNonOwnedCStringMapKeyCallBacks = {
 
       if (strcmp (pkd.name1, previousGlyphName)) {
 	previousGlyphName = pkd.name1;
-	glyph1 = NSMapGet (fontInfo->glyphsByName, pkd.name1);
+	glyph1 = NSMapGet (glyphsByName, pkd.name1);
       }
 
-      glyph2 = NSMapGet (fontInfo->glyphsByName, pkd.name2);
+      glyph2 = NSMapGet (glyphsByName, pkd.name2);
 
       if (!glyph1) {
-	NSLog (@"unknown glyph name %s in AFM file %@", pkd.name1, fileName);
+	NSLog (@"unknown glyph name %s in AFM file %@", pkd.name1, afmFileName);
 	continue;
       }
       if (!glyph2) {
-	NSLog (@"unknown glyph name %s in AFM file %@", pkd.name2, fileName);
+	NSLog (@"unknown glyph name %s in AFM file %@", pkd.name2, afmFileName);
 	continue;
       }
 
@@ -357,65 +398,26 @@ static const NSMapTableKeyCallBacks NSNonOwnedCStringMapKeyCallBacks = {
 
   [pool release];
 
-  return fontInfo;
-}
-
-- init
-{
-  int i;
-
-  self = [super init];
-
-  afmDictionary = [[NSMutableDictionary dictionaryWithCapacity:25] retain];
-  glyphsByName = NSCreateMapTable (NSNonOwnedCStringMapKeyCallBacks,
-				   NSObjectMapValueCallBacks, 256);
-
-  for (i = 0; i < 256; i++)
-    widths[i] = 0.0;
-
   return self;
 }
 
-- (void)dealloc
+- (void) transformUsingMatrix: (const float*)fmatrix
 {
-  int i;
-
-  /* Don't free the glyphsByName map table. It is owned by the first font info
-     object that is never freed. */
-  [afmDictionary release];
-  [fontName release];
-  [familyName release];
-  [weight release];
-  [encodingScheme release];
-
-  for (i = 0; i < 256; i++)
-    [glyphs[i] release];
-
-  [super dealloc];
-}
-
-- (AFMFileFontInfo*)newTransformedFontInfoForMatrix:(const float*)matrix
-{
-  AFMFileFontInfo* new = [[self copy] autorelease];
-
-  [new transformUsingMatrix:matrix];
-  return new;
-}
-
-- (void)transformUsingMatrix:(const float*)matrix
-{
-  float a = matrix[0];
-  float b = matrix[1];
-  float c = matrix[2];
-  float d = matrix[3];
-  float tx = matrix[4];
-  float ty = matrix[5];
+  float a = fmatrix[0];
+  float b = fmatrix[1];
+  float c = fmatrix[2];
+  float d = fmatrix[3];
+  float tx = fmatrix[4];
+  float ty = fmatrix[5];
   float x1, y1, width1, height1;
   int i;
 
+  memcpy(matrix, fmatrix, sizeof(matrix));
+
   /* The doc says the result of -widths is not affected by font's matrix. */
-  for (i = 0; i < 256; i++)
-    [glyphs[i] transformUsingMatrix:matrix];
+  for (i = 0; i < 256; i++) {
+    [glyphs[i] transformUsingMatrix: fmatrix];
+  }
 
   x1 = fontBBox.origin.x;
   y1 = fontBBox.origin.y;
@@ -445,64 +447,138 @@ static const NSMapTableKeyCallBacks NSNonOwnedCStringMapKeyCallBacks = {
   xHeight = a * xHeight + tx;
 }
 
-- copy
+- (AFMFileFontInfo *) transformedFontInfoForMatrix: (const float *)fmatrix
+{
+  AFMFileFontInfo *new = [self mutableCopy];
+  [new transformUsingMatrix: fmatrix];
+  return AUTORELEASE(new);
+}
+
+- initUnscaledWithFontName: (NSString*)name
+{
+  AFMFileFontInfo *fontInfo;
+
+  /* Check whether the font info is cached */
+  fontInfo = [globalFontInfoDictionary objectForKey: name];
+  if(fontInfo != nil)
+    {
+      RELEASE(self);
+      // retain to act like we were alloc'd
+      return RETAIN(fontInfo);
+    }
+  /* Tough.  Parse the AFM file and create a new font info for
+     the unscaled font. */
+  [super init];
+  fontName = RETAIN(name);
+  if ([self _setFontInfo] == nil)
+    {
+      RELEASE(self);
+      return nil;
+    }
+  /* Cache the font info for later use */
+  [globalFontInfoDictionary setObject: self forKey: fontName];
+
+  return self;
+}
+
+- initWithFontName: (NSString*)name matrix: (const float *)fmatrix
+{
+  AFMFileFontInfo *fontInfo, *baseFontInfo;
+  NSString *nameWithMatrix;
+
+  nameWithMatrix = [NSString stringWithFormat:
+                             @"%@ %.3f %.3f %.3f %.3f %.3f %.3f",
+                             name,
+                             fmatrix[0], fmatrix[1], fmatrix[2], fmatrix[3],
+                             fmatrix[4], fmatrix[5]];
+
+  /* Check whether the scaled font info is cached */
+  fontInfo = [globalFontInfoDictionary objectForKey: nameWithMatrix];
+  if(fontInfo != nil)
+    {
+      RELEASE(self);
+      // retain to act like we were alloc'd
+      return RETAIN(fontInfo);
+    }
+  /* No such luck.  Grab an unscaled font info and create a new scaled
+     one. */
+  baseFontInfo = [[AFMFileFontInfo alloc] initUnscaledWithFontName: name];
+  if(baseFontInfo == nil) 
+    {
+      RELEASE(self);
+      return nil;
+    }
+  RELEASE(self);                // Yuck.
+  AUTORELEASE(baseFontInfo);
+  fontInfo = [baseFontInfo mutableCopy];
+  memcpy(fontInfo->matrix, fmatrix, sizeof(fontInfo->matrix));
+  [fontInfo transformUsingMatrix: fmatrix];
+
+  /* We're done, cache the scaled info. */
+  [globalFontInfoDictionary setObject: fontInfo forKey: nameWithMatrix];
+
+  return fontInfo;
+}
+
+- (void) dealloc
 {
-  AFMFileFontInfo* new = [isa alloc];
   int i;
 
-  new->afmDictionary = [afmDictionary retain];
-  new->glyphsByName = glyphsByName;
-  new->maximumAdvancement.width = maximumAdvancement.width;
-  new->maximumAdvancement.height = maximumAdvancement.height;
-  new->minimumAdvancement.width = minimumAdvancement.width;
-  new->minimumAdvancement.height = minimumAdvancement.height;
+  /* Don't free the glyphsByName map table. It is owned by the first font info
+     object that is never freed. */
+  for (i = 0; i < 256; i++)
+    TEST_RELEASE(glyphs[i]);
 
-  for (i = 0; i < 256; i++) {
-    new->widths[i] = widths[i];
-    new->glyphs[i] = [glyphs[i] copy];
-  }
+  TEST_RELEASE(afmFileName);
+  [super dealloc];
+}
 
-  new->fontName = [fontName retain];
-  new->familyName = [familyName retain];
-  new->italicAngle = italicAngle;
-  new->weight = [weight retain];
-  new->underlinePosition = underlinePosition;
-  new->underlineThickness = underlineThickness;
-  new->capHeight = capHeight;
-  new->xHeight = xHeight;
-  new->descender = descender;
-  new->ascender = ascender;
-  new->encodingScheme = [encodingScheme retain];
+- (void)set
+{
+  if ([[GSCurrentContext() focusView] isFlipped])
+    {
+      float invmatrix[6];
+      memcpy(invmatrix, matrix, sizeof(invmatrix));
+      invmatrix[3] = -invmatrix[3];
+      PSWSetFont ([fontName cString], invmatrix);
+    }
+  else
+    PSWSetFont ([fontName cString], matrix);
+}
 
-  new->fontBBox.origin.x = fontBBox.origin.x;
-  new->fontBBox.origin.y = fontBBox.origin.y;
-  new->fontBBox.size.width = fontBBox.size.width;
-  new->fontBBox.size.height = fontBBox.size.height;
-
-  new->isFixedPitch = isFixedPitch;
-  new->isBaseFont = isBaseFont;
+- (NSString*)afmFileContents
+{
+  return [NSString stringWithContentsOfFile: afmFileName];
+}
 
+- copyWithZone: (NSZone *)zone
+{
+  AFMFileFontInfo* new;
+  if (NSShouldRetainWithZone(self, zone))
+    new = RETAIN(self);
+  else
+    {
+      int i;
+      new = [super copyWithZone: zone];
+      for (i = 0; i < 256; i++) {
+        new->glyphs[i] = [glyphs[i] copyWithZone: zone];
+      }
+      new->afmFileName = [afmFileName copyWithZone: zone];
+    }
   return new;
 }
 
-- (NSString*)encodingScheme			{ return encodingScheme; }
-- (BOOL)isBaseFont				{ return isBaseFont; }
-- (BOOL)isFixedPitch				{ return isFixedPitch; }
-- (NSDictionary*)afmDictionary			{ return afmDictionary; }
-- (NSRect)boundingRectForFont			{ return fontBBox; }
-- (NSString*)displayName			{ return familyName; }
-- (NSString*)familyName				{ return familyName; }
-- (NSString*)fontName				{ return fontName; }
-- (float)ascender				{ return ascender; }
-- (float)descender				{ return descender; }
-- (float)capHeight				{ return capHeight; }
-- (float)italicAngle				{ return italicAngle; }
-- (NSSize)maximumAdvancement			{ return maximumAdvancement; }
-- (NSSize)minimumAdvancement			{ return minimumAdvancement; }
-- (float)underlinePosition			{ return underlinePosition; }
-- (float)underlineThickness			{ return underlineThickness; }
-- (float)xHeight				{ return xHeight; }
-- (float*)widths				{ return widths; }
+- mutableCopyWithZone: (NSZone *)zone
+{
+  int i;
+  AFMFileFontInfo* new = [super mutableCopyWithZone: zone];
+  for (i = 0; i < 256; i++) {
+    new->glyphs[i] = [glyphs[i] mutableCopyWithZone: zone];
+  }
+  new->afmFileName = [afmFileName copyWithZone: zone];
+
+  return new;
+}
 
 - (NSSize)advancementForGlyph:(NSGlyph)glyph
 {
@@ -577,8 +653,9 @@ static const NSMapTableKeyCallBacks NSNonOwnedCStringMapKeyCallBacks = {
   const char* cString = [string cString];
   float width = 0;
 
-  for (i = 0; i < length; i++)
+  for (i = 0; i < length; i++) {
     width += [glyphs[(int)cString[i]] advancement].width;
+  }
 
   return width;
 }
@@ -705,30 +782,45 @@ static const NSMapTableKeyCallBacks NSNonOwnedCStringMapKeyCallBacks = {
   return advancement;
 }
 
-- copy
+- copyWithZone: (NSZone *)zone
 {
-  AFMGlyphInfo* new = [isa alloc];
+  AFMGlyphInfo *copy;
+  if (NSShouldRetainWithZone(self, zone))
+    copy = RETAIN(self);
+  else
+    {
+      copy = (AFMGlyphInfo*) NSCopyObject (self, 0, zone);
+      copy->name = [name copyWithZone: zone];
+      if (kerning) 
+	{
+	  copy->kerning = malloc (numOfPairs * sizeof (tPairKerningInfo));
+	  memcpy (copy->kerning, kerning, 
+		  numOfPairs * sizeof (tPairKerningInfo));
+	}
+    }
+  return copy;
+}
 
-  new->name = [name retain];
-  new->code = code;
-  new->bbox = bbox;
-  new->advancement = advancement;
-
-  if (kerning) {
-    new->numOfPairs = numOfPairs;
-    new->kerning = malloc (numOfPairs * sizeof (tPairKerningInfo));
-    memcpy (new->kerning, kerning, numOfPairs * sizeof (tPairKerningInfo));
-  }
-
-  return new;
+- mutableCopyWithZone: (NSZone *)zone
+{
+  AFMGlyphInfo *copy;
+  copy = (AFMGlyphInfo*) NSCopyObject (self, 0, zone);
+  copy->name = [name copyWithZone: zone];
+  if (kerning) 
+    {
+      copy->kerning = malloc (numOfPairs * sizeof (tPairKerningInfo));
+      memcpy (copy->kerning, kerning, 
+              numOfPairs * sizeof (tPairKerningInfo));
+    }
+  return copy;
 }
 
 - (void)dealloc
 {
-  [name release];
-  [super dealloc];
+  RELEASE(name);
   if (kerning)
     free (kerning);
+  [super dealloc];
 }
 
 - (NSString*)name	{ return name; }
