@@ -24,19 +24,19 @@
    */
 
 #include <math.h>
-#include <gnustep/xdps/NSDPSContext.h>
+#include <gnustep/xdps/NSDPSContextWindow.h>
 #include <DPS/psops.h>
+#include "SharedX/xrtools.h"
+#include "extensions.h"
+
+#define XDPY (((RContext *)context)->dpy)
+#define XSCR (((RContext *)context)->screen_number)
 
 @implementation NSDPSContext (Ops)
 
 /* ----------------------------------------------------------------------- */
 /* Color operations */
 /* ----------------------------------------------------------------------- */
-- (void)DPScolorimage
-{
-  PScolorimage();
-}
-
 - (void)DPScurrentblackgeneration
 {
   PScurrentblackgeneration();
@@ -546,7 +546,7 @@
   PSexecuserobject(index);
 }
 
-- (void)DPSusefineuserobject: (int)index
+- (void)DPSundefineuserobject: (int)index
 {
   PSundefineuserobject(index);
 }
@@ -664,6 +664,19 @@
   PSimagemask();
 }
 
+- (void)DPScolorimage
+{
+  PScolorimage();
+}
+
+- (void)DPSalphaimage
+{
+  if (ext_flags & ALPHAIMAGE_EXT)
+    PSWalphaimage();
+  else
+    NSDebugLLog(@"NSDPSContext", @"DPS does not support alphaimage op\n");
+}
+
 - (void)DPSkshow: (const char *)s 
 {
   PSkshow(s);
@@ -957,38 +970,45 @@
   PSsetXrgbactual(r, g, b, success);
 }
 
+- (void)DPSsetgcdrawable: (void *)gc : (void *)draw : (int)x : (int)y 
+{
+  PSsetXgcdrawable(XGContextFromGC(gc), (int)draw, x, y);
+}
+
+- (void)DPSsetgcdrawablecolor: (void *)gc : (void *)draw : (int)x : (int)y 
+				  : (const int *)colorInfo
+{
+  PSsetXgcdrawablecolor(XGContextFromGC(gc), (int)draw, x, y, colorInfo);
+}
+
 /*-------------------------------------------------------------------------*/
 /* Graphics Extension Ops */
 /*-------------------------------------------------------------------------*/
 - (void) DPScomposite: (float)x : (float)y : (float)w : (float)h : (int)gstateNum : (float)dx : (float)dy : (int)op
 {
-#ifdef HAVE_FUNC_DPSCOMPOSITE
-  PScomposite(x, y, w, h, gstateNum, dx, dy, op);
-#else
-  /* FIXME: Use for version which actually implements this */
-  if ([dps_version intValue] > 2017)
-    DPSPrintf(dps_context, "%f %f %f %f %d %f %f %d composite\n",
-	      x, y, w, h, gstateNum, dx, dy, op);
+  if (ext_flags & COMPOSITE_EXT)
+    PSWcomposite(x, y, w, h, gstateNum, dx, dy, op);
   else
-    NSDebugLLog(@"NSDPSContext", @"DPS does not support composite op\n");
-#endif
+    {
+      NSRect s = NSMakeRect(x, y, w, h);
+      NSPoint d = NSMakePoint(dx, dy);
+      NSDebugLLog(@"NSDPSContext", @"DPS does not support composite op\n");
+      [self wait];
+      NSCopyBits(gstateNum, s, d);
+    }
 }
 
 - (void) DPScompositerect: (float)x : (float)y : (float)w : (float)h : (int)op
 {
-#ifdef HAVE_FUNC_DPSCOMPOSITE
-  PScompositerect(x, y, w, h, op);
-#else
-  /* FIXME: Use for version which actually implements this */
-  if ([dps_version intValue] > 2017)
-    DPSPrintf(dps_context, "%f %f %f %f %d compositerect\n",
-	      x, y, w, h, op);
+  if (ext_flags & COMPOSITERECT_EXT)
+    PSWcompositerect(x, y, w, h, op);
   else
     {
       /* Try to emulate this */
       float gray;
       XGCValues gcv;
-      PXKWindow *window;
+      NSWindow *window;
+      gswindow_device_t *windev;
       
       PScurrentgray(&gray);
       if (fabs(gray - 0.667) < .002)
@@ -1016,16 +1036,16 @@
 	case   NSCompositeSourceAtop:
 	  gcv.function = GXcopy;
 	  break;
-	case   NSCompositeDataOver:
+	case   NSCompositeDestinationOver:
 	  gcv.function = GXcopy;
 	  break;
-	case   NSCompositeDataIn:
+	case   NSCompositeDestinationIn:
 	  gcv.function = GXcopy;
 	  break;
-	case   NSCompositeDataOut:
+	case   NSCompositeDestinationOut:
 	  gcv.function = GXcopy;
 	  break;
-	case   NSCompositeDataAtop:
+	case   NSCompositeDestinationAtop:
 	  gcv.function = GXcopy;
 	  break;
 	case   NSCompositeXOR:
@@ -1045,96 +1065,55 @@
 	  break;
 	}
       
-      window = (PXKWindow *)[[self focusView] window];
-      XChangeGC(context->dpy, [window xGC], GCFunction, &gcv);
+      window = [[self focusView] window];
+      windev = [NSDPSContext _windowWithTag: [window windowNumber]];
+      [self wait];
+      XChangeGC(XDPY, windev->gc, GCFunction, &gcv);
       PSrectfill(x, y, w, h);
       [window flushWindow];
       gcv.function = GXcopy;
-      XChangeGC(context->dpy, [window xGC], GCFunction, &gcv);
+      XChangeGC(XDPY, windev->gc, GCFunction, &gcv);
       PSsetgray(gray);
     }
-#endif
 }
 
 - (void) DPSdissolve: (float)x : (float)y : (float)w : (float)h : (int)gstateNum
  : (float)dx : (float)dy : (float)delta
 {
-#ifdef HAVE_FUNC_DPSCOMPOSITE
-  PSdissolve(x, y, w, h, gstateNum, dx, dy, delta);
-#else
-  /* FIXME: Use for version which actually implements this */
-  if ([dps_version intValue] > 2017)
-    DPSPrintf(dps_context, "%f %f %f %f %d %f %f %f dissolve\n",
-	      x, y, w, h, gstateNum, dx, dy, delta);
+  if (ext_flags & DISSOLVE_EXT)
+    PSWdissolve(x, y, w, h, gstateNum, dx, dy, delta);
   else
     NSDebugLLog(@"NSDPSContext", @"DPS does not support dissolve op\n");
-#endif
 }
 
 - (void) DPSreadimage
 {
-#ifdef HAVE_FUNC_DPSREADIMAGE
-  PSreadimage();
-#else
-  /* FIXME: Use for version which actually implements this */
-  if ([dps_version intValue] > 2017)
-    DPSPrintf(dps_context, "readimage\n");
+  if (ext_flags & READIMAGE_EXT)
+    PSWreadimage();
   else
     NSDebugLLog(@"NSDPSContext", @"DPS does not support readimage op\n");
-#endif
 }
 
 - (void) DPSsetalpha: (float)a
 {
-#ifdef HAVE_FUNC_DPSREADIMAGE
-  PSsetalpha(a);
-#else
-  /* FIXME: Use for version which actually implements this */
-  if ([dps_version intValue] > 2017)
-    DPSPrintf(dps_context, "%f setalpha\n", a);
+  if (ext_flags & SETALPHA_EXT)
+    PSWsetalpha(a);
   else
     NSDebugLLog(@"NSDPSContext", @"DPS does not support setalpha op\n");
-#endif
 }
 
 - (void) DPScurrentalpha: (float *)alpha
 {
-#ifdef HAVE_FUNC_DPSREADIMAGE
-  PScurrentalpha(alpha);
-#else
-  NSDebugLLog(@"NSDPSContext", @"DPS does not support currentalpha op\n");
-#endif
+  if (ext_flags & SETALPHA_EXT)
+    PSWcurrentalpha(alpha);
+  else
+    NSDebugLLog(@"NSDPSContext", @"DPS does not support currentalpha op\n");
 }
 
-/*
- * Return mouse location in base coords ignoring the event loop
- */
-- (void) DPSmouselocation: (float*)x :(float*)y
+- (void) DPSflushpage
 {
-  NSPoint	mouseLocation;
-  Window	rootWin;
-  Window	childWin;
-  int		currentX;
-  int		currentY;
-  int		winX;
-  int		winY;
-  unsigned	mask;
-  BOOL		ok;
-
-  ok = XQueryPointer (context->dpy, [self xDisplayRootWindow],
-    &rootWin, &childWin, &currentX, &currentY, &winX, &winY, &mask);
-  if (ok)
-    {
-      int	xScreen = [(PXKScreen *)[NSScreen mainScreen] xScreen];
-
-      *x = currentX;
-      *y = DisplayHeight(context->dpy, xScreen) - currentY;
-    }
-  else
-    {
-      *x = 0;
-      *y = 0;
-    }
+  if (ext_flags & FLUSHPAGE_EXT)
+    PSWflushpage();
 }
 
 @end
