@@ -29,9 +29,10 @@
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#include "SharedX/xrtools.h"
 
-int _sendXString(Display* display, Window window, const char* string);
-Window _findXWindow(Display* display, Window topwindow, Window window_to_check,
-		    int x, int y);
+void
+GSWinitcontext(GSCTXT *ctxt, int window_number, GC xgc, Drawable drawable, 
+	int xoff, int yoff);
 
 #endif
