@@ -30,19 +30,9 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
-#include <SharedX/wraster.h>
 #include <Foundation/NSGeometry.h>
 #include "gnustep/xgps/XGContext.h"
-#include "SharedX/wraster.h"
-
-void XRDrawString(NSString *str, NSRect rect);
-void XRDrawXImage(RXImage *xImage, NSRect rect);
-void XRDrawLine(NSPoint a, NSPoint b, NSColor *color);
-void XRSetCurrentFont(XGFont *aFont);
-
-int _sendXString(Display* display, Window window, const char* string);
-Window _findXWindow(Display* display, Window topwindow, Window window_to_check,
-		    int x, int y);
+#include "SharedX/xrtools.h"
 
 void GSWinitcontext(GSCTXT *ctxt, int window_num, GC xgc, Drawable drawable, 
 		    int xoff, int yoff);
@@ -56,8 +46,8 @@ extern int _pixmap_combine_alpha(RContext *context, RXImage *source_im,
 		     XGDrawMechanism drawMechanism);
 extern int _bitmap_combine_alpha(RContext *context, 
 		      const unsigned char *image_data,
-		      int numColors, BOOL usesAlpha, RXImage *dest_im,
-		      RXImage *dest_alpha, float origin_x, float origin_y,
+		      float width, float height, int numColors,
+		      BOOL usesAlpha, RXImage *dest_im, RXImage *dest_alpha,
 		      NSRect drect, NSCompositingOperation op,
 		      XGDrawMechanism drawMechanism);
 
