@@ -28,56 +28,43 @@
 #include "XGDrawingEngine.h"
 
 #include "gnustep/xgps/XGContext.h"
+#include <gnustep/xgps/XGContextPrivate.h>
 #include "gnustep/xgps/XGGState.h"
 #include <AppKit/NSAffineTransform.h>
 #include <AppKit/PSOperators.h>
+#include <AppKit/GSFontInfo.h>
 #include <Foundation/NSDebug.h>
 #include <Foundation/NSUserDefaults.h>
 
 #define	CUR_CONTEXT	((XGContext*)[XGContext currentContext])
 
-void XGInitializeDrawingEngine(void);
-
 /* Initialize AppKit backend */
 BOOL 
 initialize_gnustep_backend(void)
 {                           
-  id obj;
-  NSDebugLog(@"Using the GNUstep GUI X/GPS backend.\n");
+  NSDebugLog(@"Initializing GNUstep GUI X/GPS backend.\n");
 
   [NSGraphicsContext setDefaultContextClass: [XGContext class]];
-  [XGFontManager poseAsClass:[NSFontManager class]];
-  [XGFont poseAsClass:[NSFont class]];  
-  obj = [[NSUserDefaults standardUserDefaults] stringForKey: @"ImageCompositing"];
-  if (obj && [obj boolValue] == NO)
-    [XGBitmapImageRep poseAsClass: [NSBitmapImageRep class]];
-  else
-    NSDebugLog(@"Using ImageCompositing functions");
+  [GSFontEnumerator setDefaultClass: [XGFontEnumerator class]];
+  [GSFontInfo setDefaultClass: [XGFontInfo class]];
 
-  NSDebugLog(@"Initializing XGPS drawing engine\n");
-  XGInitializeDrawingEngine();
-  [NSFontManager sharedFontManager];
+  /* Creating the context automatically makes it the default one */
+  [XGContext defaultContextWithInfo: nil];
 
   if (GSDebugSet(@"XSynchronize") == YES)
     XSynchronize([XGContext currentXDisplay], True);
 
+  [NSFontManager sharedFontManager];
   return YES;
 }
 
-/* Initialize drawing engine */
-void 
-XGInitializeDrawingEngine(void)
-{
-  XGContext *context;
-  context = (XGContext *)[XGContext defaultContextWithInfo: nil];
-  [XGContext setCurrentContext: context];
-}
-
 /* Common graphics functions */
-void NSHighlightRect(NSRect aRect)       
+void
+NSHighlightRect(NSRect aRect)       
 {
-  XGContext *ctxt = CUR_CONTEXT;
-  float x, y, w, h;
+  XGContext	*ctxt = CUR_CONTEXT;
+  float		x, y, w, h;
+
   x = NSMinX(aRect);
   y = NSMinY(aRect);
   w = NSWidth(aRect);
@@ -87,9 +74,11 @@ void NSHighlightRect(NSRect aRect)
   [[[ctxt focusView] window] flushWindow];
 }
 
-void NSRectFill(NSRect aRect)
+void
+NSRectFill(NSRect aRect)
 {
   float x, y, w, h;
+
   x = NSMinX(aRect);
   y = NSMinY(aRect);
   w = NSWidth(aRect);
@@ -98,19 +87,22 @@ void NSRectFill(NSRect aRect)
   DPSrectfill(CUR_CONTEXT, x, y, w, h);
 }
 
-void NSRectFillList(const NSRect *rects, int count)
+void
+NSRectFillList(const NSRect *rects, int count)
 {
   int i;
   
-  for (i=0; i<count; i++)
+  for (i = 0; i < count; i++)
     NSRectFill(rects[i]);
 }
 
-void NSRectFillListWithGrays(const NSRect *rects,const float *grays,int count) 
+void
+NSRectFillListWithGrays(const NSRect *rects,const float *grays,int count) 
 {
 }
 
-void NSDrawButton(const NSRect aRect, const NSRect clipRect)
+void
+NSDrawButton(const NSRect aRect, const NSRect clipRect)
 {
   float x, y, w, h, v;
   XGContext	*ctxt;
@@ -167,9 +159,10 @@ void NSDrawButton(const NSRect aRect, const NSRect clipRect)
   DPSstroke(ctxt);
 }
 
-void NSDrawGrayBezel(const NSRect aRect, const NSRect clipRect)
+void
+NSDrawGrayBezel(const NSRect aRect, const NSRect clipRect)
 {
-  float x, y, w, h, v;
+  float		x, y, w, h, v;
   XGContext	*ctxt;
   XGGState	*current;
 
@@ -231,9 +224,10 @@ void NSDrawGrayBezel(const NSRect aRect, const NSRect clipRect)
   DPSstroke(ctxt);
 }
 
-void NSDrawGroove(const NSRect aRect, const NSRect clipRect)
+void
+NSDrawGroove(const NSRect aRect, const NSRect clipRect)
 {
-  float x, y, w, h, v;
+  float		x, y, w, h, v;
   XGContext	*ctxt;
   XGGState	*current;
 
@@ -284,7 +278,8 @@ void NSDrawGroove(const NSRect aRect, const NSRect clipRect)
   DPSrectstroke(ctxt, x+1, y-v, w, h);
 }
 
-void NSFrameRect(const NSRect aRect)  
+void
+NSFrameRect(const NSRect aRect)  
 {
   float x, y, w, h;
   XGContext	*ctxt;
@@ -302,7 +297,8 @@ void NSFrameRect(const NSRect aRect)
   DPSrectstroke(ctxt, x, y, w, h);
 }
 
-void NSFrameRectWithWidth(const NSRect aRect, float frameWidth)
+void
+NSFrameRectWithWidth(const NSRect aRect, float frameWidth)
 {
   float x, y, w, h;
   XGContext	*ctxt;
@@ -320,52 +316,8 @@ void NSFrameRectWithWidth(const NSRect aRect, float frameWidth)
   DPSrectstroke(ctxt, x, y, w, h);
 }
 
-void XRDrawString(NSString *str, NSRect rect)      
-{
-  XGContext *ctxt = CUR_CONTEXT;
-  XGGState *current = [ctxt xrCurrentGState];
-
-  if (current->viewIsFlipped == YES)
-    {
-      rect.origin.y += rect.size.height;
-    }
-  DPSmoveto(ctxt, NSMinX(rect), NSMinY(rect));
-  DPSshow(ctxt, [str cString]);
-}
-
-void XRDrawXImage(RXImage *xImage, NSRect rect)      
-{
-  XGContext *ctxt = CUR_CONTEXT;
-  XGGState *current = [ctxt xrCurrentGState];
-
-  if (current->viewIsFlipped == YES)
-    {
-      rect.origin.y -= rect.size.height;
-    }
-  DPSmoveto(ctxt, NSMinX(rect), NSMinY(rect));
-  [current copyImage: xImage ofSize: rect.size];
-}
-
 void
-XRDrawLine(NSPoint a, NSPoint b, NSColor *color)
-{
-  XGContext *ctxt = CUR_CONTEXT;
-  DPSmoveto(ctxt, a.x, a.y);
-  DPSlineto(ctxt, b.x, b.y);
-  DPSstroke(ctxt);
-}
-
-void
-XRSetCurrentFont(XGFont *aFont)
-{
-  XFontStruct *xfont;
-  XGContext *ctxt = CUR_CONTEXT;
-  XGGState *current = [ctxt xrCurrentGState];
-  xfont = [aFont xFontStruct];
-  [current setFontStruct: xfont];
-}
-
-void NSRectClip(NSRect aRect)      
+NSRectClip(NSRect aRect)      
 {
   XGContext *ctxt = CUR_CONTEXT;
   float x, y, w, h;
@@ -379,14 +331,16 @@ void NSRectClip(NSRect aRect)
   DPSnewpath(ctxt);
 }
 
-void NSRectClipList(const NSRect *rects, int count)
+void
+NSRectClipList(const NSRect *rects, int count)
 {
   int i;
   for (i=0; i<count; i++)
     NSRectClip(rects[i]);
 }
 
-void XRRemoveClipPath(void)
+void
+XRRemoveClipPath(void)
 {
 }
 
@@ -394,7 +348,8 @@ void XRRemoveClipPath(void)
 //
 // Optimize Drawing
 //
-void NSEraseRect(NSRect aRect)
+void
+NSEraseRect(NSRect aRect)
 {
   XGContext	*ctxt;
   ctxt = CUR_CONTEXT;
@@ -404,7 +359,8 @@ void NSEraseRect(NSRect aRect)
   DPSgrestore(ctxt);
 }
 
-void NSDrawWhiteBezel(const NSRect aRect, const NSRect clipRect)
+void
+NSDrawWhiteBezel(const NSRect aRect, const NSRect clipRect)
 {
   float x, y, w, h, v;
   XGContext	*ctxt;
@@ -467,7 +423,8 @@ void NSDrawWhiteBezel(const NSRect aRect, const NSRect clipRect)
   DPSstroke(ctxt);
 }
 
-void NSDrawBezel(NSRect aRect, NSRect clipRect)
+void
+NSDrawBezel(NSRect aRect, NSRect clipRect)
 {
   NSDrawGrayBezel(aRect, clipRect);
 }
@@ -483,59 +440,19 @@ void NSDrawBezel(NSRect aRect, NSRect clipRect)
 //
 //*****************************************************************************
 
-NSRect NSDrawTiledRects(NSRect boundsRect, NSRect clipRect,
-			const NSRectEdge *sides, const float *grays, int count)
+NSRect
+NSDrawTiledRects(NSRect boundsRect, NSRect clipRect,
+  const NSRectEdge *sides, const float *grays, int count)
 {
   return NSZeroRect;
 }
 
-//
-// Color Functions
-//
-//
-// Get Information About Color Space and Window Depth
-//
-const NSWindowDepth *NSAvailableWindowDepths(void)
-{
-  return NULL;
-}
-
-NSWindowDepth NSBestDepth(NSString *colorSpace,
-			  int bitsPerSample, int bitsPerPixel,
-			  BOOL planar, BOOL *exactMatch)
-{
-  return 0;
-}
-
-int NSBitsPerPixelFromDepth(NSWindowDepth depth)
-{
-  return 0;
-}
-
-int NSBitsPerSampleFromDepth(NSWindowDepth depth)
-{
-  return 0;
-}
-
-NSString *NSColorSpaceFromDepth(NSWindowDepth depth)
-{
-  return nil;
-}
-
-int NSNumberOfColorComponents(NSString *colorSpaceName)
-{
-  return 0;
-}
-
-BOOL NSPlanarFromDepth(NSWindowDepth depth)
-{
-  return NO;
-}
 
 //
 // Read the Color at a Screen Position
 //
-NSColor *NSReadPixel(NSPoint location)
+NSColor*
+NSReadPixel(NSPoint location)
 {
   return nil;
 }
@@ -546,14 +463,16 @@ NSColor *NSReadPixel(NSPoint location)
 //
 // Filter Characters Entered into a Text Object
 //
-unsigned short NSEditorFilter(unsigned short theChar,
-			      int flags, NSStringEncoding theEncoding)
+unsigned short
+NSEditorFilter(unsigned short theChar,
+  int flags, NSStringEncoding theEncoding)
 {
   return 0;
 }
 
-unsigned short NSFieldFilter(unsigned short theChar,
-			   int flags, NSStringEncoding theEncoding)
+unsigned short
+NSFieldFilter(unsigned short theChar,
+ int flags, NSStringEncoding theEncoding)
 {
   return 0;
 }
@@ -561,12 +480,14 @@ unsigned short NSFieldFilter(unsigned short theChar,
 //
 // Calculate or Draw a Line of Text (in Text Object)
 //
-int NSDrawALine(id self, NSLayInfo *layInfo)
+int
+NSDrawALine(id self, NSLayInfo *layInfo)
 {
   return 0;
 }
 
-int NSScanALine(id self, NSLayInfo *layInfo)
+int
+NSScanALine(id self, NSLayInfo *layInfo)
 {
   return 0;
 }
@@ -574,27 +495,28 @@ int NSScanALine(id self, NSLayInfo *layInfo)
 //
 // Calculate Font Ascender, Descender, and Line Height (in Text Object)
 //
-void NSTextFontInfo(id fid,
-		    float *ascender, float *descender,
-		    float *lineHeight)
+void
+NSTextFontInfo(id fid, float *ascender, float *descender, float *lineHeight)
 {}
 
 //
 // Access Text Object's Word Tables
 //
-NSData * NSDataWithWordTable(const unsigned char *smartLeft,
-			     const unsigned char *smartRight,
-			     const unsigned char *charClasses,
-			     const NSFSM *wrapBreaks,
-			     int wrapBreaksCount,
-			     const NSFSM *clickBreaks,
-			     int clickBreaksCount,
-			     BOOL charWrap)
+NSData*
+NSDataWithWordTable(const unsigned char *smartLeft,
+  const unsigned char *smartRight,
+  const unsigned char *charClasses,
+  const NSFSM *wrapBreaks,
+  int wrapBreaksCount,
+  const NSFSM *clickBreaks,
+  int clickBreaksCount,
+  BOOL charWrap)
 {
   return nil;
 }
 
-void NSReadWordTable(NSZone *zone,
+void
+NSReadWordTable(NSZone *zone,
 		     NSData *data,
 		     unsigned char **smartLeft,
 		     unsigned char **smartRight,
@@ -609,44 +531,52 @@ void NSReadWordTable(NSZone *zone,
 //
 // Array Allocation Functions for Use by the NSText Class
 //
-NSTextChunk *NSChunkCopy(NSTextChunk *pc, NSTextChunk *dpc)
+NSTextChunk*
+NSChunkCopy(NSTextChunk *pc, NSTextChunk *dpc)
 {
   return NULL;
 }
 
-NSTextChunk *NSChunkGrow(NSTextChunk *pc, int newUsed)
+NSTextChunk*
+NSChunkGrow(NSTextChunk *pc, int newUsed)
 {
   return NULL;
 }
 
-NSTextChunk *NSChunkMalloc(int growBy, int initUsed)
+NSTextChunk*
+NSChunkMalloc(int growBy, int initUsed)
 {
   return NULL;
 }
 
-NSTextChunk *NSChunkRealloc(NSTextChunk *pc)
+NSTextChunk*
+NSChunkRealloc(NSTextChunk *pc)
 {
   return NULL;
 }
 
-NSTextChunk *NSChunkZoneCopy(NSTextChunk *pc,
+NSTextChunk*
+NSChunkZoneCopy(NSTextChunk *pc,
                              NSTextChunk *dpc,
                              NSZone *zone)
 {
   return NULL;
 }
 
-NSTextChunk *NSChunkZoneGrow(NSTextChunk *pc, int newUsed, NSZone *zone)
+NSTextChunk*
+NSChunkZoneGrow(NSTextChunk *pc, int newUsed, NSZone *zone)
 {
   return NULL;
 }
 
-NSTextChunk *NSChunkZoneMalloc(int growBy, int initUsed, NSZone *zone)
+NSTextChunk*
+NSChunkZoneMalloc(int growBy, int initUsed, NSZone *zone)
 {
   return NULL;
 }
 
-NSTextChunk *NSChunkZoneRealloc(NSTextChunk *pc, NSZone *zone)
+NSTextChunk*
+NSChunkZoneRealloc(NSTextChunk *pc, NSZone *zone)
 {
   return NULL;
 }
@@ -658,11 +588,13 @@ NSTextChunk *NSChunkZoneRealloc(NSTextChunk *pc, NSZone *zone)
 //
 // Copy an image
 //
-void NSCopyBitmapFromGState(int srcGstate, NSRect srcRect, NSRect destRect)
+void
+NSCopyBitmapFromGState(int srcGstate, NSRect srcRect, NSRect destRect)
 {
 }
 
-void NSCopyBits(int srcGstate, NSRect srcRect, NSPoint destPoint)
+void
+NSCopyBits(int srcGstate, NSRect srcRect, NSPoint destPoint)
 {
   float x, y, w, h;
 
@@ -678,7 +610,8 @@ void NSCopyBits(int srcGstate, NSRect srcRect, NSPoint destPoint)
 //
 // Play the System Beep
 //
-void NSBeep(void)
+void
+NSBeep(void)
 {
   XBell([XGContext currentXDisplay], 50);
 }
@@ -686,11 +619,13 @@ void NSBeep(void)
 //
 // Draw a Distinctive Outline around Linked Data
 //
-void NSFrameLinkRect(NSRect aRect, BOOL isDestination)
+void
+NSFrameLinkRect(NSRect aRect, BOOL isDestination)
 {
 }
 
-float NSLinkFrameThickness(void)
+float
+NSLinkFrameThickness(void)
 {
   return 0;
 }
@@ -807,7 +742,7 @@ _sendKeysym(Display* display, Window window, KeySym keysym)
 }
 
 int
- _sendXString(Display* display, Window window, const char* string)
+_sendXString(Display* display, Window window, const char* string)
 {
   int	i;
   int	status = 0;
@@ -832,50 +767,3 @@ int
   return status;
 }
 
-Window _findXWindow(Display* display,
-		    Window topwindow,
-		    Window window_to_check,
-		    int x, int y)
-{
-  int		newx, newy;
-  Window	window;
-
-  // find the lowest X window in the heirarchy that the global x,y
-  // coordinates are in.
-  if (topwindow == (Window) None) 
-    {
-      fprintf(stderr,"_findXWindow() error: topwindow == (Window) None \n");
-
-      return (Window) None;
-    }
-
-  if (window_to_check == (Window) None)
-    {
-      fprintf(stderr,
-	"_findXWindow() error: window_to_check == (Window) None \n");
-
-      return topwindow;
-    }
-
-  // find lowest X window by iterating thru the X window's heirarchy while
-  // XTranslateCoordinates returns an X Window via the window ptr
-
-  window = window_to_check;
-  while ((XTranslateCoordinates(display, topwindow, window_to_check,
-	    x, y, &newx, &newy, &window) != 0)
-	    && (window != (Window) None))
-    {
-      if (window != (Window) None)
-	{
-	  topwindow = window_to_check;
-	  window_to_check = window;
-	  x = newx;
-	  y = newy;
-	}
-    }
-
-  if (window == (Window) None)
-    window = window_to_check;
-
-  return window;
-}
