@@ -25,13 +25,16 @@
 #include "gnustep/xgps/XGContext.h"
 #include "gnustep/xgps/XGGState.h"
 #include "gnustep/xgps/XGContextPrivate.h"
+#include "gnustep/xgps/XGContextWindow.h"
 #include "gnustep/xgps/XGDrawObject.h"
-#include "wraster/xrtools.h"
+#include "XGDrawingEngine.h"
+#include "SharedX/xrtools.h"
 #include <base/fast.x>
 
 #define XDPY (context->dpy)
 #define XSCR (context->screen_number)
 
+static BOOL shouldDrawAlpha = NO;
 
 typedef struct _DrawObject_t {
   ctxt_object_t type;
@@ -106,6 +109,14 @@ static void *RetainDraw(DrawObject_t *o)
   if (_point.x < 0) \
     NSLog(@"DPS (xgps): No current point\n");
 
+#define GS_NEW_ALPHA_BUFFER(abuffer, draw, xgc, width, height, depth)	\
+  do {									\
+    abuffer = XCreatePixmap(XDPY, draw, width, height, depth);		\
+    XFillRectangle(XDPY, abuffer, xgc, 0, 0, width, height);		\
+  } while (0)
+
+#define AINDEX 5
+
 @interface XGGState (Private)
 - (void) _paintPath;
 - (void) setClipMask;
@@ -134,11 +145,11 @@ static inline XRectangle XGWindowRectToX(XGGState *s, NSRect r)
 {
   XRectangle newRect;
 
-  newRect.width = floor(r.size.width);
-  newRect.height = floor(r.size.height);
+  newRect.x = floor(r.origin.x + s->offset.x);
+  newRect.width = floor(r.origin.x + r.size.width + s->offset.x) - newRect.x;
 
-  newRect.x = floor(r.origin.x) + s->offset.x;
   newRect.y = floor(s->offset.y - r.origin.y - r.size.height);
+  newRect.height = floor(s->offset.y - r.origin.y) - newRect.y;
 
   return newRect;
 }
@@ -166,6 +177,10 @@ static	Region	emptyRegion;
   if (beenHere == NO)
     {
       XPoint	pts[5];
+      id obj = [[NSUserDefaults standardUserDefaults]
+		 stringForKey: @"GraphicCompositing"];
+      if (obj)
+	shouldDrawAlpha = [obj boolValue];
 
       beenHere = YES;
       pts[0].x = 0; pts[0].y = 0;
@@ -183,13 +198,17 @@ static	Region	emptyRegion;
 {
   [super init];
 
+  xgcontext = drawContext;
   context = [drawContext xrContext];
   ctm = [[NSAffineTransform allocWithZone: fastZone(self)] init];
   path = NULL;
   point.x = -1;
   offset = NSMakePoint(0, 0);
   draw = 0;
+  alpha_buffer = 0;
   xfont = NULL;
+  color.field[AINDEX] = 1.0;
+  xgcntxt = None;
   return self;
 }
 
@@ -247,22 +266,49 @@ static	Region	emptyRegion;
   return [(XGGState *)NSCopyObject(self, 0, fastZone(self)) deepen];
 }
 
+- (void) setWindow: (int)number;
+{
+  window = number;
+}
+
 - (void) setDrawable: (Drawable)theDrawable;
 {
+  gswindow_device_t *gs_win;
   draw = theDrawable;
+  gs_win = [XGContext _windowWithTag: window];
+  drawingAlpha = NO;
+  if (gs_win != NULL && gs_win->alpha_buffer != 0)
+    {
+      alpha_buffer = gs_win->alpha_buffer;
+      if (shouldDrawAlpha)
+	drawingAlpha = YES;
+    }
 }
 
 - (void) setGraphicContext: (GC)xGraphicContext
 {
+  GC source;
+  unsigned long	mask;
+  BOOL old_shared;
+
+  source = xgcntxt;
+  old_shared = sharedGC;
   if (xGraphicContext == None)
     return;
   if (xGraphicContext == xgcntxt)
     return;
-  if (xgcntxt && sharedGC == NO)
-    XFreeGC(XDPY, xgcntxt);
   
   xgcntxt = xGraphicContext;
   sharedGC = YES;		/* Not sure if we really own the GC */
+  /* Update the GC to reflect our settings */
+  if (source == None)
+    return;
+  mask = GCForeground | GCFont | GCFunction | GCFillRule
+    | GCCapStyle | GCJoinStyle | GCLineWidth;
+  XCopyGC(XDPY, source, mask, xgcntxt); 
+
+  if (source != None && old_shared == NO)
+    XFreeGC(XDPY, source);
 }
 
 /* Set various characteristics of the graphic context */
@@ -280,30 +326,32 @@ static	Region	emptyRegion;
   Region region;
 
   COPY_GC_ON_CHANGE;
-  if (!clipregion && !context->viewclip)
+  if (!clipregion && !xgcontext->viewclip)
     {
       XSetClipMask(XDPY, xgcntxt, None);
       return;
     }
 
-  if (clipregion && context->viewclip)
+  if (clipregion && xgcontext->viewclip)
     {
       region = XCreateRegion();
-      XIntersectRegion(clipregion, context->viewclip, region);
+      XIntersectRegion(clipregion, xgcontext->viewclip, region);
     }
-  else if (!context->viewclip)
+  else if (!xgcontext->viewclip)
     region = clipregion;
   else 
-    region = context->viewclip;
+    region = xgcontext->viewclip;
 
   XSetRegion(XDPY, xgcntxt, region);
-  if (clipregion && context->viewclip)
+  if (clipregion && xgcontext->viewclip)
     XDestroyRegion(region);
 }
 
 - (void) setColor: (xr_device_color_t)acolor;
 {
+  float alpha = color.field[AINDEX];
   color = acolor;
+  color.field[AINDEX] = alpha;
   gcv.foreground = xrColorToPixel(context, color);
   [self setGCValues: gcv withMask: GCForeground];
 }
@@ -385,6 +433,11 @@ static	Region	emptyRegion;
   return (draw ? YES : NO);
 }
 
+- (int) window
+{
+  return window;
+}
+
 - (Drawable) drawable
 {
   return draw;
@@ -395,17 +448,13 @@ static	Region	emptyRegion;
   return xgcntxt;
 }
 
-- copyDrawable: (Drawable)source fromRect: (NSRect)aRect
-{
-  return [self copyDrawable:source fromRect:aRect toPoint: NSMakePoint(0,0)];
-}
-    
-- (id) copyDrawable: (Drawable)source fromRect: (NSRect)aRect 
-				       toPoint: (NSPoint)aPoint
+- (void) copyBits: (XGGState*)source fromRect: (NSRect)aRect 
+				      toPoint: (NSPoint)aPoint
 {
   XRectangle	dst;
   XRectangle    src;
   NSRect	flushRect;
+  Drawable	from;
 
   flushRect.size = aRect.size;
   flushRect.origin = aPoint;
@@ -413,18 +462,248 @@ static	Region	emptyRegion;
   CHECK_GC;
   if (draw == 0)
     DPS_ERROR(DPSinvalidid, @"No Drawable defined");
-  if (source == 0)
-    source = draw;
+  from = source->draw;
+  if (from == 0)
+    DPS_ERROR(DPSinvalidid, @"No source Drawable defined");
 
-  src = XGViewRectToX(self, aRect);
+  src = XGViewRectToX(source, aRect);
   dst = XGViewRectToX(self, flushRect);
-  XCopyArea(XDPY, source, draw, xgcntxt,
+  XCopyArea(XDPY, from, draw, xgcntxt,
                 src.x, src.y, src.width, src.height, dst.x, dst.y);
-
-  return self;
 }
 
-- copyImage: (RXImage *)image ofSize: (NSSize)size
+- (void) _compositeGState: (XGGState *)source 
+                fromRect: (NSRect)aRect
+                 toPoint: (NSPoint)aPoint
+                      op: (NSCompositingOperation)op
+{
+  XRectangle sr, dr;
+  NSRect flushRect;
+  NSRect drect, srect, bounds;
+  RXImage *source_im, *dest_im, *source_alpha, *dest_alpha;
+  gswindow_device_t *source_win, *dest_win;
+
+  if (!source)
+    source = self;
+
+  /* Get our drawable info */
+  dest_win = [XGContext _windowWithTag: window];
+  if (!dest_win)
+    DPS_ERROR(DPSinvalidid, @"Invalid composite gstate");
+
+  if (dest_win->buffer == 0 && dest_win->map_state != IsViewable)
+    {
+      /* Why bother drawing? */
+      return;
+    }
+
+  /*
+   * Find the current view clipregion and see if the image is inside it.
+   */
+  flushRect.size = aRect.size;
+  flushRect.origin = aPoint;
+  dr = XGViewRectToX(self, flushRect);
+  aPoint.x = dr.x;
+  aPoint.y = dr.y;
+
+  /* Intersect the rects to find what part of the image we need to get */
+  drect = NSMakeRect(dr.x, dr.y, dr.width, dr.height);
+  bounds = NSMakeRect(0, 0, NSWidth(dest_win->xframe), 
+		      NSHeight(dest_win->xframe));
+  drect = NSIntersectionRect(drect, bounds);
+  if (NSIsEmptyRect(drect))
+    return;
+  dest_im = RGetXImage(context, draw, NSMinX(drect), NSMinY(drect), 
+		       NSWidth(drect), NSHeight(drect));
+  
+  /* Get the image from the source gstate */
+  source_win = [XGContext _windowWithTag: source->window];
+  if (!source_win)
+    DPS_ERROR(DPSinvalidid, @"Invalid composite source gstate");
+  if (source_win->buffer == 0 && source_win->map_state != IsViewable)
+    {
+      /* Can't get pixel information from a window that isn't mapped */
+      DPS_ERROR(DPSinvalidaccess, @"Invalid gstate buffer");
+    }
+
+  /* Get source image from new source rect. Doesn't account for
+   scaling, but we can't handle scaled images anyway */
+  sr = XGViewRectToX(source, aRect);
+  srect = NSMakeRect(sr.x + NSMinX(drect)-aPoint.x, 
+		     sr.y + NSMinY(drect)-aPoint.y,
+		     NSWidth(drect), NSHeight(drect));
+  source_im = RGetXImage(context, 
+		 (source_win->buffer) ? source_win->buffer : source_win->ident,
+		 NSMinX(srect), NSMinY(srect), 
+		 NSWidth(srect), NSHeight(srect));
+
+  /* Force creation of our alpha buffer */
+  if (dest_win->alpha_buffer == 0 
+      && dest_win->type != NSBackingStoreNonretained)
+    GS_NEW_ALPHA_BUFFER(dest_win->alpha_buffer, draw, xgcntxt, 
+			NSWidth(dest_win->xframe), NSHeight(dest_win->xframe),
+			dest_win->depth);
+  if (shouldDrawAlpha && dest_win->alpha_buffer != 0)
+    drawingAlpha = YES;
+  alpha_buffer = dest_win->alpha_buffer;
+
+  /* Composite it */
+  source_alpha = RGetXImage(context, source_win->alpha_buffer, 
+  		    NSMinX(srect), NSMinY(srect), 
+  		    NSWidth(srect), NSHeight(srect));
+  dest_alpha = NULL;
+  if (alpha_buffer)
+    dest_alpha = RGetXImage(context, alpha_buffer,
+			    NSMinX(drect), NSMinY(drect), 
+			    NSWidth(drect), NSHeight(drect));
+  _pixmap_combine_alpha(context, source_im, source_alpha, 
+			dest_im, dest_alpha, 0, 0, drect,
+			op, [xgcontext drawMechanism]);
+  /* Draw into the window/buffer */
+  RPutXImage(context, draw, xgcntxt, dest_im, 0, 0, 
+	     NSMinX(drect), NSMinY(drect), NSWidth(drect), NSHeight(drect));
+  if (dest_alpha)
+    {
+      RPutXImage(context, dest_win->alpha_buffer, xgcntxt, dest_alpha, 0, 0, 
+	       NSMinX(drect), NSMinY(drect), NSWidth(drect), NSHeight(drect));
+      RDestroyXImage(context, dest_alpha);
+    }
+  RDestroyXImage(context, dest_im);
+  RDestroyXImage(context, source_im);
+  if (source_alpha)
+    RDestroyXImage(context, source_alpha);
+}
+
+- (void) compositeGState: (XGGState *)source 
+                fromRect: (NSRect)aRect
+                 toPoint: (NSPoint)aPoint
+                      op: (NSCompositingOperation)op
+{
+  BOOL do_copy, source_alpha;
+  XGCValues comp_gcv;
+
+  if (!source)
+    source = self;
+
+  /* If we have no drawable, we can't proceed. */
+  if (draw == 0)
+    DPS_ERROR(DPSinvalidid, @"No Drawable defined");
+
+  /* Check alpha */
+#define CHECK_ALPHA						\
+  do {								\
+    gswindow_device_t *source_win;				\
+    source_win = [XGContext _windowWithTag: [source window]];	\
+    source_alpha = (source_win && source_win->alpha_buffer);	\
+  } while (0)
+
+  do_copy = NO;
+  switch (op)
+    {
+    case   NSCompositeClear:
+      do_copy = YES;
+      comp_gcv.function = GXclear;
+      break;
+    case   NSCompositeCopy:
+      do_copy = YES;
+      comp_gcv.function = GXcopy;
+      break;
+    case   NSCompositeSourceOver:
+      CHECK_ALPHA;
+      if (source_alpha == NO)
+	do_copy = YES;
+      else
+	do_copy = NO;
+      comp_gcv.function = GXcopy;
+      break;
+    case   NSCompositeSourceIn:
+      CHECK_ALPHA;
+      if (source_alpha == NO && drawingAlpha == NO)
+	do_copy = YES;
+      else
+	do_copy = NO;
+      comp_gcv.function = GXcopy;
+      break;
+    case   NSCompositeSourceOut:
+      do_copy = NO;
+      comp_gcv.function = GXcopy;
+      break;
+    case   NSCompositeSourceAtop:
+      CHECK_ALPHA;
+      if (source_alpha == NO && drawingAlpha == NO)
+	do_copy = YES;
+      else
+	do_copy = NO;
+      comp_gcv.function = GXcopy;
+      break;
+    case   NSCompositeDestinationOver:
+      CHECK_ALPHA;
+      if (drawingAlpha == NO)
+	return;
+      else
+	do_copy = NO;
+      comp_gcv.function = GXcopy;
+      break;
+    case   NSCompositeDestinationIn:
+      CHECK_ALPHA;
+      if (source_alpha == NO && drawingAlpha == NO)
+	return;
+      else
+	do_copy = NO;
+      comp_gcv.function = GXcopy;
+      break;
+    case   NSCompositeDestinationOut:
+      do_copy = NO;
+      comp_gcv.function = GXcopy;
+      break;
+    case   NSCompositeDestinationAtop:
+      CHECK_ALPHA;
+      if (source_alpha == NO && drawingAlpha == NO)
+	return;
+      else
+	do_copy = NO;
+      comp_gcv.function = GXcopy;
+      break;
+    case   NSCompositeXOR:
+      do_copy = NO;
+      comp_gcv.function = GXxor;
+      break;
+    case   NSCompositePlusDarker:
+      do_copy = NO;
+      comp_gcv.function = GXcopy;
+      break;
+    case   NSCompositeHighlight:
+      do_copy = NO;
+      comp_gcv.function = GXxor;
+      break;
+    case   NSCompositePlusLighter:
+      do_copy = NO;
+      comp_gcv.function = GXcopy;
+      break;
+    }
+
+  if (comp_gcv.function != GXcopy)
+    [self setGCValues: comp_gcv withMask: GCFunction];
+
+  if (shouldDrawAlpha == NO)
+    do_copy = YES;
+
+  if (do_copy)
+    [self copyBits: source fromRect: aRect toPoint: aPoint];
+  else
+    [self _compositeGState: source 
+                fromRect: aRect
+                 toPoint: aPoint
+                      op: op];
+
+  if (comp_gcv.function != GXcopy)
+    {
+      comp_gcv.function = GXcopy;
+      [self setGCValues: comp_gcv withMask: GCFunction];
+    }
+}
+
+- (void)copyImage: (RXImage *)image ofSize: (NSSize)size
 {
   NSRect	r;
   XRectangle	x;
@@ -439,7 +718,6 @@ static	Region	emptyRegion;
 			image, 	0, 0,
 			x.x, x.y,
 			x.width, x.height);
-  return self;
 }
 
 - (NSPoint) pointInMatrixSpace: (NSPoint)aPoint
@@ -470,6 +748,16 @@ static	Region	emptyRegion;
     {
     case path_stroke:
       XDrawLines(XDPY, draw, xgcntxt, pts, count, CoordModeOrigin);
+      if (drawingAlpha)
+	{
+	  xr_device_color_t old_color;
+	  NSAssert(alpha_buffer, NSInternalInconsistencyException);
+	  
+	  old_color = color;
+	  [self DPSsetgray: color.field[AINDEX]];
+	  XDrawLines(XDPY, alpha_buffer, xgcntxt, pts, count, CoordModeOrigin);
+	  [self setColor: old_color];
+	}
       break;
     case path_eofill:
       gcv.fill_rule = EvenOddRule;
@@ -478,6 +766,17 @@ static	Region	emptyRegion;
     case path_fill:
       XFillPolygon(XDPY, draw, xgcntxt, pts, count, Complex, 
 		   CoordModeOrigin);
+      if (drawingAlpha)
+	{
+	  xr_device_color_t old_color;
+	  NSAssert(alpha_buffer, NSInternalInconsistencyException);
+	  
+	  old_color = color;
+	  [self DPSsetgray: color.field[AINDEX]];
+	  XFillPolygon(XDPY, alpha_buffer, xgcntxt, pts, count, Complex, 
+		       CoordModeOrigin);
+	  [self setColor: old_color];
+	}
       
       if (gcv.fill_rule == EvenOddRule)
 	{
@@ -510,9 +809,9 @@ static	Region	emptyRegion;
       {
 	Region region;
 	region = XPolygonRegion(pts, count, fill_rule);
-	if (context->viewclip)
-	  XDestroyRegion(context->viewclip);
-	context->viewclip = region;
+	if (xgcontext->viewclip)
+	  XDestroyRegion(xgcontext->viewclip);
+	xgcontext->viewclip = region;
 	[self setClipMask];
       }
       break;
@@ -814,8 +1113,10 @@ static	Region	emptyRegion;
     XDestroyRegion(clipregion);
   clipregion = 0;
   /* FIXME: reset the GC */
-  color.space = gray_colorspace; color.field[0] = 0.0;
+  color.space = gray_colorspace; 
+  color.field[0] = 0.0;
   [self setColor: color];
+  color.field[AINDEX] = 1.0;
 }
 
 - (void)DPSinitmatrix 
@@ -978,6 +1279,16 @@ typedef enum {
       xp = XGViewPointToX(self, point);
       XDrawString(XDPY, draw, xgcntxt, xp.x, xp.y, s, 1);
 
+      if (drawingAlpha)
+	{
+	  xr_device_color_t old_color;
+	  NSAssert(alpha_buffer, NSInternalInconsistencyException);
+	  
+	  old_color = color;
+	  [self DPSsetgray: color.field[AINDEX]];
+	  XDrawString(XDPY, alpha_buffer, xgcntxt, xp.x, xp.y, s, 1);
+	  [self setColor: old_color];
+	}
       /* Note we update the current point according to the current 
 	 transformation scaling, although the text isn't currently
 	 scaled (FIXME). */
@@ -1058,11 +1369,118 @@ typedef enum {
   [self _paintPath];
 }
 
-- (void)DPSimage 
+- (void)DPSimage: dictionary
 {
+  id obj;
+  BOOL uses_alpha, is_planar;
+  NSRect rect;
+  XRectangle dr, cr;
+  NSRect drect, crect;
+  int width, height, colors;
+  id data;
+  const unsigned char *raw_data;
+  RXImage *dest_im, *dest_alpha;
+  gswindow_device_t *dest_win;
+
+  rect = NSZeroRect;
+  uses_alpha = NO;
+  colors = 1; /* Default is grey-scale */
+  width = height = 0;
+  obj = [dictionary objectForKey: @"Width"];
+  if (obj)
+    width = [obj doubleValue];
+  rect.size.width = width;
+  obj = [dictionary objectForKey: @"Height"];
+  if (obj)
+    height = [obj doubleValue];
+  rect.size.height = height;
+  obj = [dictionary objectForKey: @"SamplesPerPixel"];
+  if (obj)
+    colors = [obj intValue];
+  obj = [dictionary objectForKey: @"Alpha"];
+  if (obj)
+    uses_alpha = [obj boolValue];
+  obj = [dictionary objectForKey: @"MultipleDataSources"];
+  if (obj)
+    is_planar = [obj boolValue];
+  data = [dictionary objectForKey: @"DataSource"];
+  if ([data respondsToSelector: @selector(cString)])
+    raw_data = [data cString];
+  else
+    {
+      unsigned char **rdata;
+      rdata = (unsigned char **)[data pointerValue];
+      raw_data = rdata[0];
+    }
+
+  if (is_planar)
+    {
+      NSLog(@"XGContext (DPSimage): Cannot handle planar images\n");
+      return;
+    }
+
+  /* Get our drawable info */
+  dest_win = [XGContext _windowWithTag: window];
+  if (!dest_win)
+    DPS_ERROR(DPSinvalidid, @"Invalid image gstate");
+
+  /*
+   * Find the current view clipregion and see if the image is inside it.
+   */
+  XClipBox(xgcontext->viewclip, &cr);
+  rect.origin = point;
+  if (viewIsFlipped)
+    rect.origin.y -= rect.size.height;
+  dr = [self viewRectToX: rect];
+
+  /* Intersect the rects to find what part of the image we need to get */
+  drect = NSMakeRect(dr.x, dr.y, dr.width, dr.height);
+  crect = NSMakeRect(cr.x, cr.y, cr.width, cr.height);
+  drect = NSIntersectionRect(drect, crect);
+  if (NSIsEmptyRect(drect))
+    return;
+
+  if (dest_win->buffer == 0 && dest_win->map_state != IsViewable)
+    {
+      /* Why bother drawing? */
+      return;
+    }
+  else
+    dest_im = RGetXImage(context, draw, NSMinX(drect), NSMinY(drect), 
+			 NSWidth(drect), NSHeight(drect));
+  
+  /* Force creation of our alpha buffer */
+  if (uses_alpha && dest_win->alpha_buffer == 0
+      && dest_win->type != NSBackingStoreNonretained)
+    GS_NEW_ALPHA_BUFFER(dest_win->alpha_buffer, draw, xgcntxt, 
+			NSWidth(dest_win->xframe), NSHeight(dest_win->xframe),
+			dest_win->depth);
+  if (shouldDrawAlpha && dest_win->alpha_buffer != 0)
+    drawingAlpha = YES;
+  alpha_buffer = dest_win->alpha_buffer;
+
+  /* Composite it */
+  dest_alpha = 0;
+  if (uses_alpha && alpha_buffer != 0)
+    dest_alpha = RGetXImage(context, alpha_buffer,
+  		    NSMinX(drect), NSMinY(drect), 
+  		    NSWidth(drect), NSHeight(drect));
+  _bitmap_combine_alpha(context, raw_data, width, height, colors, uses_alpha,
+			dest_im, dest_alpha, drect,
+			0, [xgcontext drawMechanism]);
+  /* Draw into the window/buffer */
+  RPutXImage(context, draw, xgcntxt, dest_im, 0, 0, 
+	     NSMinX(drect), NSMinY(drect), NSWidth(drect), NSHeight(drect));
+  if (dest_alpha)
+    {
+      RPutXImage(context, dest_win->alpha_buffer, xgcntxt, dest_alpha, 0, 0, 
+	       NSMinX(drect), NSMinY(drect), NSWidth(drect), NSHeight(drect));
+      RDestroyXImage(context, dest_alpha);
+    }
+  RDestroyXImage(context, dest_im);
 }
 
-- (void)DPSimagemask 
+- (void)DPSimagemask: dictionary 
 {
 }
 
@@ -1081,6 +1499,19 @@ typedef enum {
   bounds = XGViewRectToX(self, NSMakeRect(x, y, w, h));
   XFillRectangle(XDPY, draw, xgcntxt,
 		 bounds.x, bounds.y, bounds.width, bounds.height);
+
+  if (drawingAlpha)
+    {
+      /* Fill alpha also */
+      xr_device_color_t old_color;
+      NSAssert(alpha_buffer, NSInternalInconsistencyException);
+      
+      old_color = color;
+      [self DPSsetgray: color.field[AINDEX]];
+      XFillRectangle(XDPY, alpha_buffer, xgcntxt,
+		 bounds.x, bounds.y, bounds.width, bounds.height);
+      [self setColor: old_color];
+    }
 }
 
 - (void)DPSrectstroke: (float)x : (float)y : (float)w : (float)h 
@@ -1096,6 +1527,19 @@ typedef enum {
   bounds.height--;
   XDrawRectangle(XDPY, draw, xgcntxt,
 		 bounds.x, bounds.y, bounds.width, bounds.height);
+
+  if (drawingAlpha)
+    {
+      /* Fill alpha also */
+      xr_device_color_t old_color;
+      NSAssert(alpha_buffer, NSInternalInconsistencyException);
+
+      old_color = color;
+      [self DPSsetgray: color.field[AINDEX]];
+      XDrawRectangle(XDPY, alpha_buffer, xgcntxt,
+		 bounds.x, bounds.y, bounds.width, bounds.height);
+      [self setColor: old_color];
+    }
 }
 
 - (void)DPSshow: (const char *)s 
@@ -1119,6 +1563,16 @@ typedef enum {
   xp = XGViewPointToX(self, point);
   XDrawString(XDPY, draw, xgcntxt, xp.x, xp.y, s, len);
 
+  if (drawingAlpha)
+    {
+      xr_device_color_t old_color;
+      NSAssert(alpha_buffer, NSInternalInconsistencyException);
+
+      old_color = color;
+      [self DPSsetgray: color.field[AINDEX]];
+      XDrawString(XDPY, alpha_buffer, xgcntxt, xp.x, xp.y, s, len);
+      [self setColor: old_color];
+    }
   /* Note we update the current point according to the current 
      transformation scaling, although the text isn't currently
      scaled (FIXME). */
@@ -1232,6 +1686,20 @@ typedef enum {
 
 - (void)DPScurveto: (float)x1 : (float)y1 : (float)x2 : (float)y2 : (float)x3 : (float)y3 
 {
+  NSPoint p;
+  double x, y, t, k = .025;
+
+  CHECK_PATH;
+  p.x = point.x;
+  p.y = point.y;
+  for(t = k; t <= 1+k; t += k)
+    {
+      x = (p.x+t*(-p.x*3+t*(3*p.x-p.x*t)))
+        +t*(3*x1+t*(-6*x1+x1*3*t))+t*t*(x2*3-x2*3*t)+x3*t*t*t;
+      y = (p.y+t*(-p.y*3+t*(3*p.y-p.y*t)))
+        +t*(3*y1+t*(-6*y1+y1*3*t))+t*t*(y2*3-y2*3*t)+y3*t*t*t;
+      [self DPSlineto: x : y];
+    }
 }
 
 - (void)DPSeoclip 
@@ -1262,9 +1730,9 @@ typedef enum {
 
 - (void)DPSinitviewclip 
 {
-  if (context->viewclip)
-    XDestroyRegion(context->viewclip);
-  context->viewclip = 0;
+  if (xgcontext->viewclip)
+    XDestroyRegion(xgcontext->viewclip);
+  xgcontext->viewclip = 0;
   [self setClipMask];
 }
 
@@ -1349,10 +1817,10 @@ typedef enum {
   	      xrect.x, xrect.y, xrect.width,  xrect.height);
     }
 
-  if (context->viewclip == 0)
-    context->viewclip = XCreateRegion();
-  XIntersectRegion(context->viewclip, emptyRegion, context->viewclip);
-  XUnionRectWithRegion(&xrect, context->viewclip, context->viewclip);
+  if (xgcontext->viewclip == 0)
+    xgcontext->viewclip = XCreateRegion();
+  XIntersectRegion(xgcontext->viewclip, emptyRegion, xgcontext->viewclip);
+  XUnionRectWithRegion(&xrect, xgcontext->viewclip, xgcontext->viewclip);
   [self setClipMask];
 }
 
@@ -1415,5 +1883,32 @@ typedef enum {
 {
 }
 
+- (void) DPSsetalpha: (float)a
+{
+  gswindow_device_t *gs_win;
+  color.field[AINDEX] = a;
+  gs_win = [XGContext _windowWithTag: window];
+  if (!gs_win)
+    return;
+  if (gs_win->alpha_buffer == 0 && a < 1.0
+      && gs_win->type != NSBackingStoreNonretained)
+    {
+      GS_NEW_ALPHA_BUFFER(gs_win->alpha_buffer, draw, xgcntxt,
+			  NSWidth(gs_win->xframe), NSHeight(gs_win->xframe),
+			  gs_win->depth);
+    }
+  drawingAlpha = NO;
+  if (shouldDrawAlpha && gs_win->alpha_buffer != 0)
+    drawingAlpha = YES;
+  alpha_buffer = gs_win->alpha_buffer;
+}
+
+- (void) DPScurrentalpha: (float *)alpha
+{
+  if (alpha)
+    *alpha = color.field[AINDEX];
+}
+
+
 @end
 
