@@ -885,8 +885,9 @@ NSMutableDictionary	*pasteboards = nil;
 {
   NSMutableArray	*permenant;
 }
-- (NSConnection*) connection: ancestor didConnect: newConn;
-- connectionBecameInvalid: notification;
+- (BOOL) connection: (NSConnection*)ancestor
+  shouldMakeNewConnection: (NSConnection*)newConn;
+- (id) connectionBecameInvalid: (NSNotification*)notification;
 
 - (id<GSPasteboardObj>) pasteboardByFilteringData: (NSData*)data
 					   ofType: (NSString*)type
@@ -900,7 +901,8 @@ NSMutableDictionary	*pasteboards = nil;
 
 @implementation PasteboardServer
 
-- (NSConnection*) connection: ancestor didConnect: newConn
+- (BOOL) connection: (NSConnection*)ancestor
+  shouldMakeNewConnection: (NSConnection*)newConn;
 {
   [[NSNotificationCenter defaultCenter]
     addObserver: self
@@ -908,7 +910,7 @@ NSMutableDictionary	*pasteboards = nil;
 	   name: NSConnectionDidDieNotification
 	 object: newConn];
   [newConn setDelegate: self];
-  return newConn;
+  return YES;
 }
 
 - (id) connectionBecameInvalid: (NSNotification*)notification
@@ -1086,10 +1088,12 @@ init(int argc, char** argv)
 
 
 int
-main(int argc, char** argv)
+main(int argc, char** argv, char **env)
 {
   CREATE_AUTORELEASE_POOL(pool);
 
+  [NSProcessInfo initializeWithArguments:argv count:argc environment:env];
+
   init(argc, argv);
 
   // [NSObject enableDoubleReleaseCheck: YES];
@@ -1103,10 +1107,9 @@ main(int argc, char** argv)
     }
 
   /* Register a connection that provides the server object to the network */
-  conn = [NSConnection newRegisteringAtName: PBSNAME
-		 withRootObject: server];
-
-  if (conn == nil)
+  conn = [NSConnection defaultConnection];
+  [conn setRootObject: server];
+  if ([conn registerName: PBSNAME] == NO)
     {
       NSLog(@"Unable to register with name server.\n");
       exit(1);
