@@ -26,7 +26,7 @@
 #include <AppKit/NSPasteboard.h>
 #include <AppKit/GSPasteboardServer.h>
 
-#include <wraster.h>
+#include <xrtools.h>
 #include <xdnd.h>
 #include <X11/Xatom.h>
 
